"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard")["default"];

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault")["default"];

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.addDataSetField = addDataSetField;
exports["default"] = void 0;
exports.initDataSetField = initDataSetField;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _objectSpread3 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _assertThisInitialized2 = _interopRequireDefault(require("@babel/runtime/helpers/assertThisInitialized"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _createSuper2 = _interopRequireDefault(require("@babel/runtime/helpers/createSuper"));

var _tslib = require("tslib");

var _mobx = require("mobx");

var _axios = _interopRequireDefault(require("axios"));

var _omit = _interopRequireDefault(require("lodash/omit"));

var _flatMap = _interopRequireDefault(require("lodash/flatMap"));

var _isNumber = _interopRequireDefault(require("lodash/isNumber"));

var _isArray = _interopRequireDefault(require("lodash/isArray"));

var _isObject = _interopRequireDefault(require("lodash/isObject"));

var _isNil = _interopRequireDefault(require("lodash/isNil"));

var _defer = _interopRequireDefault(require("lodash/defer"));

var _isString = _interopRequireDefault(require("lodash/isString"));

var _isPlainObject = _interopRequireDefault(require("lodash/isPlainObject"));

var _isFunction = _interopRequireDefault(require("lodash/isFunction"));

var _debounce = _interopRequireDefault(require("lodash/debounce"));

var _union = _interopRequireDefault(require("lodash/union"));

var _utils = require("../utils");

var _configure = require("../configure");

var _localeContext = _interopRequireWildcard(require("../locale-context"));

var _axios2 = _interopRequireDefault(require("../axios"));

var _Record = _interopRequireDefault(require("./Record"));

var _Field = _interopRequireDefault(require("./Field"));

var _formatter = require("../formatter");

var _utils2 = require("./utils");

var _eventManager = _interopRequireDefault(require("../event-manager"));

var _DataSetSnapshot = _interopRequireDefault(require("./DataSetSnapshot"));

var _enum = require("./enum");

var ObjectChainValue = _interopRequireWildcard(require("../object-chain-value"));

var _Transport = _interopRequireDefault(require("./Transport"));

var _promiseQueue = _interopRequireDefault(require("../promise-queue"));

var _DataSetRequestError = _interopRequireDefault(require("./DataSetRequestError"));

var _FeedBack = _interopRequireDefault(require("./FeedBack"));

var _iteratorHelper = require("../iterator-helper");

var _Validator = _interopRequireDefault(require("../validator/Validator"));

var _treeHelper = require("../tree-helper");

var ALL_PAGE_SELECTION = '__ALL_PAGE_SELECTION__'; // TODO:Symbol

var QUERY_PARAMETER = '__QUERY_PARAMETER__'; // TODO:Symbol

var TOTAL_KEY = '__TOTAL__'; // TODO:Symbol

function addDataSetField(dataSet, name) {
  var field = new _Field["default"]({
    name: name
  }, dataSet);
  dataSet.fields.set(name, field);
  return field;
}

function initDataSetField(dataSet, name, fieldProps) {
  return (0, _utils2.processIntlField)(name, function (langProps) {
    return new _Field["default"](langProps, dataSet);
  }, fieldProps, dataSet);
}

function processOneData(dataSet) {
  var data = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var status = arguments.length > 2 ? arguments[2] : undefined;
  var childrenField = arguments.length > 3 ? arguments[3] : undefined;
  var parent = arguments.length > 4 ? arguments[4] : undefined;
  var all = arguments.length > 5 ? arguments[5] : undefined;

  if (data instanceof _Record["default"]) {
    if (data.dataSet !== dataSet) {
      data.dataSet = dataSet;
      data.status = status;
    }

    if (childrenField) {
      var children = data.children;

      if (children) {
        children.forEach(function (r) {
          return processOneData(dataSet, r, status, childrenField);
        });
      }
    }

    if (all) {
      all.push(data);
    }

    return data;
  }

  var self = new _Record["default"](data, dataSet, status);

  if (parent) {
    self.parent = parent;
  }

  if (all) {
    all.push(self);
  }

  if (childrenField) {
    var _children = data[childrenField];

    if (_children) {
      // eslint-disable-next-line no-use-before-define
      var normalData = processNormalData(dataSet, _children, status, childrenField, self, all);
      self.children = normalData;
    }
  }

  return self;
}

function processTreeData(dataSet, allData, status, parentField, idField) {
  var allMap = new Map();
  allData.forEach(function (data, index) {
    var record = processOneData(dataSet, data, status);
    var id = !(0, _isNil["default"])(data[idField]) ? data[idField] : "__empty_".concat(index);
    allMap.set(String(id), record);
  });
  allMap.forEach(function (record) {
    var parent = allMap.get(String(record.get(parentField)));

    if (parent) {
      if (parent.children) {
        parent.children.push(record);
      } else {
        parent.children = [record];
      }

      record.parent = parent;
    }
  });
  return Array.from(allMap.values());
}

function processNormalData(dataSet, allData) {
  var status = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : _enum.RecordStatus.sync;
  var childrenField = arguments.length > 3 ? arguments[3] : undefined;
  var parent = arguments.length > 4 ? arguments[4] : undefined;
  var all = arguments.length > 5 ? arguments[5] : undefined;
  return allData.map(function (data) {
    return processOneData(dataSet, data, status, childrenField, parent, all);
  });
}

function updateCachedRecords(dataSet, cachedRecords, status) {
  if (dataSet.cacheSelectionKeys) {
    dataSet.cachedRecords = (0, _iteratorHelper.iteratorSliceToArray)(dataSet.cachedRecords.reduce(function (set, record) {
      if (!set.has(record) && (status && record.status !== status || (dataSet.isAllPageSelection ? !record.isSelected : record.isSelected))) {
        set.add(record);
      }

      return set;
    }, new Set(cachedRecords)).values());
  } else {
    dataSet.cachedRecords = cachedRecords;
  }
}

var DataSet = /*#__PURE__*/function (_EventManager) {
  (0, _inherits2["default"])(DataSet, _EventManager);

  var _super = (0, _createSuper2["default"])(DataSet);

  function DataSet(props, context) {
    var _this;

    (0, _classCallCheck2["default"])(this, DataSet);
    _this = _super.call(this);
    _this.children = {};
    _this.pending = new _promiseQueue["default"]();
    _this.performance = {
      timing: {
        fetchStart: 0,
        fetchEnd: 0,
        loadStart: 0,
        loadEnd: 0
      }
    };
    _this.originalData = [];
    _this.resetInBatch = false;
    _this.validating = false;
    _this.inBatchSelection = false;
    _this.syncChildrenRemote = (0, _debounce["default"])(function (remoteKeys, current) {
      var _assertThisInitialize = (0, _assertThisInitialized2["default"])(_this),
          children = _assertThisInitialize.children;

      remoteKeys.forEach(function (childName) {
        return _this.syncChild(children[childName], current, childName);
      });
    }, 300);

    if (context && !(0, _isFunction["default"])(context.getConfig)) {
      throw new Error('The type of the second parameter of the DataSet constructor is wrong. Please not pass the parameter or pass a DataSetContext type parameter instead.');
    }

    (0, _mobx.runInAction)(function () {
      props = (0, _objectSpread3["default"])((0, _objectSpread3["default"])({}, DataSet.defaultProps), props);
      _this.props = props;
      _this.context = context;
      var _props = props,
          data = _props.data,
          fields = _props.fields,
          queryFields = _props.queryFields,
          queryDataSet = _props.queryDataSet,
          autoQuery = _props.autoQuery,
          autoCreate = _props.autoCreate,
          pageSize = _props.pageSize,
          selection = _props.selection,
          events = _props.events,
          id = _props.id,
          name = _props.name,
          children = _props.children,
          _props$queryParameter = _props.queryParameter,
          queryParameter = _props$queryParameter === void 0 ? {} : _props$queryParameter,
          dataToJSON = _props.dataToJSON,
          selectionStrategy = _props.selectionStrategy,
          _props$status = _props.status,
          status = _props$status === void 0 ? _enum.DataSetStatus.ready : _props$status;
      _this.name = name;
      _this.selectionStrategy = selectionStrategy;
      _this.dataToJSON = dataToJSON;
      _this.records = [];
      _this.state = _mobx.observable.map();
      _this.fields = _mobx.observable.map(fields ? _this.initFields(fields) : undefined);
      _this.status = status;
      _this.currentPage = 1;
      _this.cachedRecords = [];
      _this.queryParameter = queryParameter;
      _this.pageSize = pageSize;
      _this.selection = selection;

      _this.initCombineSort();

      _this.processListener();

      if (id) {
        _this.id = id;
      }

      if (children) {
        _this.initChildren(children);
      }

      if (events) {
        _this.initEvents(events);
      }

      _this.initQueryDataSet(queryDataSet, queryFields);

      if (data) {
        var length = data.length;

        if (length) {
          _this.loadData(data, length);
        }
      } // ssr do not auto query


      if (autoQuery && typeof window !== 'undefined') {
        _this.query();
      } else if (autoCreate && _this.records.length === 0) {
        _this.create();
      }
    });
    return _this;
  }

  (0, _createClass2["default"])(DataSet, [{
    key: "queryParameter",
    get: function get() {
      var queryParameterMap = this.getState(QUERY_PARAMETER);

      if (queryParameterMap) {
        return queryParameterMap.toPOJO();
      }

      return {};
    },
    set: function set(queryParameter) {
      this.setState(QUERY_PARAMETER, _mobx.observable.map(queryParameter));
    }
  }, {
    key: "totalCount",
    get: function get() {
      var total = this.getState(TOTAL_KEY);

      if (total !== undefined && isFinite(total)) {
        return total;
      }

      var paging = this.paging,
          _this$props = this.props,
          idField = _this$props.idField,
          parentField = _this$props.parentField,
          childrenField = _this$props.childrenField;

      if (paging === 'server' && (idField && parentField || childrenField)) {
        return this.treeData.length;
      }

      return this.length;
    },
    set: function set(totalCount) {
      this.setState(TOTAL_KEY, totalCount);
    }
  }, {
    key: "realTotalCount",
    get: function get() {
      return this.getState(TOTAL_KEY);
    }
  }, {
    key: "cachedSelected",
    get: function get() {
      var isAllPageSelection = this.isAllPageSelection;
      return this.cachedRecords.filter(function (r) {
        return isAllPageSelection ? !r.isSelected : r.isSelected;
      });
    },
    set: function set(cachedSelected) {
      this.setCachedSelected(cachedSelected);
    }
  }, {
    key: "cachedUnSelected",
    get: function get() {
      return this.cachedRecords.filter(function (r) {
        return !r.isSelected;
      });
    }
  }, {
    key: "cachedDirtyRecords",
    get: function get() {
      var created = [];
      var updated = [];
      var destroyed = [];
      this.cachedRecords.forEach(function (record) {
        switch (record.status) {
          case _enum.RecordStatus.add:
            created.push(record);
            break;

          case _enum.RecordStatus.update:
            updated.push(record);
            break;

          case _enum.RecordStatus["delete"]:
            destroyed.push(record);
            break;

          default:
            {
              if (record.dirty) {
                updated.push(record);
              }
            }
        }
      });
      return [created, updated, destroyed];
    }
  }, {
    key: "cachedCreated",
    get: function get() {
      return this.cachedDirtyRecords[0];
    }
  }, {
    key: "cachedUpdated",
    get: function get() {
      return this.cachedDirtyRecords[1];
    }
  }, {
    key: "cachedDestroyed",
    get: function get() {
      return this.cachedDirtyRecords[2];
    }
  }, {
    key: "cachedModified",
    get: function get() {
      return this.cachedCreated.concat(this.cachedUpdated, this.cachedDestroyed);
    },
    set: function set(cachedModified) {
      this.setCachedModified(cachedModified);
    }
  }, {
    key: "isAllPageSelection",
    get: function get() {
      return this.getState(ALL_PAGE_SELECTION) === true;
    }
  }, {
    key: "cascadeRecords",
    get: function get() {
      var parent = this.parent,
          parentName = this.parentName;

      if (parent && parentName) {
        return parent.cascadeRecords.reduce(function (array, record) {
          return array.concat.apply(array, (0, _toConsumableArray2["default"])(record.getCascadeRecordsIncludeDelete(parentName) || []));
        }, []);
      }

      return this.records;
    }
  }, {
    key: "axios",
    get: function get() {
      return this.props.axios || this.getConfig('axios') || _axios2["default"];
    }
  }, {
    key: "dataKey",
    get: function get() {
      var _this$props$dataKey = this.props.dataKey,
          dataKey = _this$props$dataKey === void 0 ? this.getConfig('dataKey') : _this$props$dataKey;
      return dataKey;
    }
  }, {
    key: "totalKey",
    get: function get() {
      return this.props.totalKey || this.getConfig('totalKey');
    }
  }, {
    key: "countKey",
    get: function get() {
      return this.props.countKey || this.getConfig('countKey');
    }
  }, {
    key: "lang",
    get: function get() {
      return (0, _mobx.get)(this.props, 'lang') || _localeContext["default"].locale.lang;
    },
    set: function set(lang) {
      var _this2 = this;

      (0, _mobx.runInAction)(function () {
        (0, _mobx.set)(_this2.props, 'lang', lang);
      });
    }
  }, {
    key: "queryDataSet",
    get: function get() {
      return (0, _mobx.get)(this.props, 'queryDataSet');
    }
    /**
     * 设置查询的DataSet.
     * @param {DataSet} ds DataSet.
     */
    ,
    set: function set(ds) {
      var _this3 = this;

      (0, _mobx.runInAction)(function () {
        (0, _mobx.set)(_this3.props, 'queryDataSet', ds);

        if (ds) {
          // 初始化时如果直接执行create，mobx会报错，所以使用了defer
          ds.pending.add(new Promise(function (reslove) {
            (0, _defer["default"])(function () {
              if (ds.records.length === 0) {
                ds.create();
              } else if (!ds.current) {
                ds.first();
              }

              reslove();
            });
          }));
        }
      });
    }
  }, {
    key: "queryUrl",
    get: function get() {
      return (0, _mobx.get)(this.props, 'queryUrl') || this.name && "/dataset/".concat(this.name, "/queries");
    }
    /**
     * 设置提交的Url.
     * @param {String} url 提交的Url.
     */
    ,
    set: function set(url) {
      var _this4 = this;

      (0, _mobx.runInAction)(function () {
        (0, _mobx.set)(_this4.props, 'queryUrl', url);
      });
    }
  }, {
    key: "submitUrl",
    get: function get() {
      return (0, _mobx.get)(this.props, 'submitUrl') || this.name && "/dataset/".concat(this.name, "/mutations");
    }
    /**
     * 设置查询的Url.
     * @param {String} url 查询的Url.
     */
    ,
    set: function set(url) {
      var _this5 = this;

      (0, _mobx.runInAction)(function () {
        (0, _mobx.set)(_this5.props, 'submitUrl', url);
      });
    }
  }, {
    key: "tlsUrl",
    get: function get() {
      return (0, _mobx.get)(this.props, 'tlsUrl') || this.name && "/dataset/".concat(this.name, "/languages");
    }
    /**
     * 设置多语言的Url.
     * @param {String} url 多语言的Url.
     */
    ,
    set: function set(url) {
      var _this6 = this;

      (0, _mobx.runInAction)(function () {
        (0, _mobx.set)(_this6.props, 'tlsUrl', url);
      });
    }
  }, {
    key: "validateUrl",
    get: function get() {
      return (0, _mobx.get)(this.props, 'validateUrl') || this.name && "/dataset/".concat(this.name, "/validate");
    }
    /**
     * 设置远程校验查询请求的url.
     * @param {String} url 远程校验查询请求的url.
     */
    ,
    set: function set(url) {
      var _this7 = this;

      (0, _mobx.runInAction)(function () {
        (0, _mobx.set)(_this7.props, 'validateUrl', url);
      });
    }
  }, {
    key: "exportUrl",
    get: function get() {
      return (0, _mobx.get)(this.props, 'exportUrl') || this.name && "/dataset/".concat(this.name, "/export");
    }
    /**
     * 设置导出请求的url.
     * @param {String} url 远程校验查询请求的url.
     */
    ,
    set: function set(url) {
      var _this8 = this;

      (0, _mobx.runInAction)(function () {
        (0, _mobx.set)(_this8.props, 'exportUrl', url);
      });
    }
    /**
     * 服务端导出还是客户端导出
     */

  }, {
    key: "exportMode",
    get: function get() {
      return this.props.exportMode || this.getConfig('exportMode') || _enum.ExportMode.server;
    }
  }, {
    key: "transport",
    get: function get() {
      return new _Transport["default"](this.props.transport, this);
    },
    set: function set(transport) {
      var _this9 = this;

      (0, _mobx.runInAction)(function () {
        _this9.props.transport = transport instanceof _Transport["default"] ? transport.props : transport;
      });
    }
  }, {
    key: "feedback",
    get: function get() {
      return (0, _objectSpread3["default"])((0, _objectSpread3["default"])({}, this.getConfig('feedback')), this.props.feedback);
    }
  }, {
    key: "data",
    get: function get() {
      return this.records.filter(function (record) {
        return !record.isRemoved;
      });
    },
    set: function set(records) {
      this.loadData(records);
    }
  }, {
    key: "dirtyRecords",
    get: function get() {
      var created = [];
      var updated = [];
      var destroyed = [];
      this.all.forEach(function (record) {
        switch (record.status) {
          case _enum.RecordStatus.add:
            created.push(record);
            break;

          case _enum.RecordStatus.update:
            updated.push(record);
            break;

          case _enum.RecordStatus["delete"]:
            destroyed.push(record);
            break;

          default:
            {
              if (record.dirty) {
                updated.push(record);
              }
            }
        }
      });
      return [created, updated, destroyed];
    }
    /**
     * 获取新建的记录集
     * @return 记录集
     */

  }, {
    key: "created",
    get: function get() {
      return this.dirtyRecords[0];
    }
    /**
     * 获取变更的记录集
     * @return 记录集
     */

  }, {
    key: "updated",
    get: function get() {
      return this.dirtyRecords[1];
    }
    /**
     * 获取删除的记录集
     * @return 记录集
     */

  }, {
    key: "destroyed",
    get: function get() {
      return this.dirtyRecords[2];
    }
    /**
     * 获取选中的记录集
     * @return 记录集
     */

  }, {
    key: "selected",
    get: function get() {
      return this.currentSelected.concat(this.cachedSelected.filter(function (record) {
        return record.isSelected;
      }));
    }
    /**
     * 获取未选中的记录集， 在 isAllPageSelection 为 true 时有效
     * @return 记录集
     */

  }, {
    key: "unSelected",
    get: function get() {
      return this.currentUnSelected.concat(this.cachedSelected.filter(function (record) {
        return !record.isSelected;
      }));
    }
  }, {
    key: "currentSelected",
    get: function get() {
      return this.records.filter(function (record) {
        return record.isSelected;
      });
    }
  }, {
    key: "currentUnSelected",
    get: function get() {
      return this.records.filter(function (record) {
        return !record.isSelected;
      });
    }
  }, {
    key: "treeSelected",
    get: function get() {
      var selected = this.selected,
          selectionStrategy = this.selectionStrategy;

      if (selectionStrategy === _enum.CheckedStrategy.SHOW_CHILD) {
        return selected.filter(function (record) {
          return !record.children;
        });
      }

      if (selectionStrategy === _enum.CheckedStrategy.SHOW_PARENT) {
        return selected.filter(function (record) {
          var parent = record.parent;
          return !parent || parent.isSelectionIndeterminate ? !record.isSelectionIndeterminate : false;
        });
      }

      return selected.filter(function (record) {
        return !record.isSelectionIndeterminate;
      });
    }
  }, {
    key: "totalPage",
    get: function get() {
      return this.paging ? Math.ceil(this.totalCount / this.pageSize) : 1;
    } // 如果paging为server 返回root父节点的排序

  }, {
    key: "currentIndex",
    get: function get() {
      var current = this.current,
          pageSize = this.pageSize,
          currentPage = this.currentPage;

      if (current) {
        var index = this.indexOf(current);

        if (index !== -1) {
          if (this.paging === 'server') {
            var currentParent = (0, _utils2.findRootParent)(current);
            return this.treeData.findIndex(function (item) {
              return item.index === currentParent.index;
            });
          }

          return index + (currentPage - 1) * pageSize;
        }
      }

      return -1;
    },
    set: function set(index) {
      this.locate(index);
    }
    /**
     * 记录数
     */

  }, {
    key: "length",
    get: function get() {
      // Fix mobx error caused by chrome browser console
      if (this === DataSet.prototype) {
        return 0;
      }

      return this.data.length;
    }
  }, {
    key: "hasChildren",
    get: function get() {
      return Object.keys(this.children).length > 0;
    }
  }, {
    key: "treeRecords",
    get: function get() {
      return this.records.filter(function (record) {
        return !record.parent;
      });
    }
  }, {
    key: "treeData",
    get: function get() {
      return this.filter(function (record) {
        return !record.parent;
      });
    }
  }, {
    key: "paging",
    get: function get() {
      var _this$props2 = this.props,
          idField = _this$props2.idField,
          parentField = _this$props2.parentField,
          childrenField = _this$props2.childrenField,
          paging = _this$props2.paging;
      return paging === "server" && (parentField && idField || childrenField) ? paging : (parentField === undefined || idField === undefined) && childrenField === undefined && !!paging;
    },
    set: function set(paging) {
      var _this10 = this;

      (0, _mobx.runInAction)(function () {
        _this10.props.paging = paging;
      });
    }
  }, {
    key: "groups",
    get: function get() {
      return (0, _iteratorHelper.iteratorReduce)(this.fields.entries(), function (arr, _ref) {
        var _ref2 = (0, _slicedToArray2["default"])(_ref, 2),
            name = _ref2[0],
            field = _ref2[1];

        var group = field.get('group');

        if ((0, _isNumber["default"])(group)) {
          arr[group] = name;
        } else if (group === true && !arr[0]) {
          arr[0] = name;
        }

        return arr;
      }, []).filter(function (group) {
        return group !== undefined;
      });
    }
  }, {
    key: "groupedRecords",
    get: function get() {
      var groups = this.groups,
          records = this.records;
      return (0, _utils2.normalizeGroups)(groups, [], records);
    }
  }, {
    key: "groupedTreeRecords",
    get: function get() {
      var groups = this.groups,
          treeRecords = this.treeRecords;
      return (0, _utils2.normalizeGroups)(groups, [], treeRecords);
    }
    /**
     * 获取当前索引的记录
     * @return record 记录
     */

  }, {
    key: "current",
    get: function get() {
      return this.all.find(function (record) {
        return record.isCurrent;
      });
    }
    /**
     * 将记录设定为当前索引
     * @param record 记录
     */
    ,
    set: function set(record) {
      var _this11 = this;

      var currentRecord = this.current;

      if (currentRecord !== record && (!record || record.dataSet === this)) {
        (0, _mobx.runInAction)(function () {
          if (currentRecord) {
            currentRecord.isCurrent = false;
          }

          if (record && record.dataSet === _this11) {
            record.isCurrent = true;
          }

          _this11.fireEvent(_enum.DataSetEvents.indexChange, {
            dataSet: _this11,
            record: record,
            previous: currentRecord
          });
        });
      }
    }
  }, {
    key: "uniqueKeys",
    get: function get() {
      var primaryKey = this.props.primaryKey;

      if (primaryKey) {
        return [primaryKey];
      }

      var keys = (0, _utils2.getUniqueFieldNames)(this);

      if (keys.length) {
        return keys;
      }

      return undefined;
    }
  }, {
    key: "cacheKeys",
    get: function get() {
      var cacheRecords = this.getConfig('cacheRecords');

      if (cacheRecords) {
        return this.uniqueKeys;
      }

      return undefined;
    }
  }, {
    key: "cacheSelectionKeys",
    get: function get() {
      var cacheSelection = this.props.cacheSelection;

      if (cacheSelection) {
        return this.uniqueKeys;
      }

      if (cacheSelection !== false) {
        return this.cacheKeys;
      }
    }
  }, {
    key: "cacheModifiedKeys",
    get: function get() {
      var cacheModified = this.props.cacheModified;

      if (cacheModified) {
        return this.uniqueKeys;
      }

      if (cacheModified !== false) {
        return this.cacheKeys;
      }
    }
    /**
     * 获取所有记录包括缓存的选择记录
     * @param index 索引
     * @returns {Record}
     */

  }, {
    key: "all",
    get: function get() {
      return this.records.concat(this.cachedRecords);
    }
  }, {
    key: "dirty",
    get: function get() {
      return this.all.some(_utils2.isDirtyRecord);
    }
  }, {
    key: "processListener",
    value: function processListener() {
      this.addEventListener(_enum.DataSetEvents.indexChange, this.handleCascade);
    }
  }, {
    key: "destroy",
    value: function destroy() {
      this.clear();
    }
  }, {
    key: "setState",
    value: function setState(item, value) {
      if ((0, _isString["default"])(item)) {
        this.state.set(item, value);
      } else if ((0, _isPlainObject["default"])(item)) {
        this.state.merge(item);
      }

      return this;
    }
  }, {
    key: "getState",
    value: function getState(key) {
      return this.state.get(key);
    }
  }, {
    key: "snapshot",
    value: function snapshot() {
      return new _DataSetSnapshot["default"](this);
    }
  }, {
    key: "restore",
    value: function restore(snapshot) {
      if (snapshot.dataSet !== this) {
        this.events = {};
      } else if (snapshot.events) {
        this.events = snapshot.events;
      }

      this.records = snapshot.records;
      this.originalData = snapshot.originalData;
      this.totalCount = snapshot.totalCount;
      this.currentPage = snapshot.currentPage;
      this.pageSize = snapshot.pageSize;
      this.cachedRecords = snapshot.cachedRecords;
      this.dataToJSON = snapshot.dataToJSON;
      this.children = snapshot.children;
      this.current = snapshot.current;
      return this;
    }
  }, {
    key: "setAllPageSelection",
    value: function setAllPageSelection(enable) {
      if (this.selection === _enum.DataSetSelection.multiple) {
        var processAfterAllPageSelection = [];

        if (enable) {
          this.records.forEach(function (record) {
            if (record.selectable) {
              if (record.isSelected) {
                record.isSelected = false;
              }
            } else if (!record.isSelected) {
              processAfterAllPageSelection.push(function () {
                record.isSelected = false;
              });
            }
          });
        } else {
          this.records.forEach(function (record) {
            if (record.selectable) {
              if (!record.isSelected) {
                record.isSelected = true;
              }
            } else if (record.isSelected) {
              processAfterAllPageSelection.push(function () {
                record.isSelected = true;
              });
            }
          });
        }

        this.setCachedSelected([]);
        this.setState(ALL_PAGE_SELECTION, enable);
        processAfterAllPageSelection.forEach(function (cb) {
          return cb();
        });
        this.fireEvent(enable ? _enum.DataSetEvents.selectAllPage : _enum.DataSetEvents.unSelectAllPage, {
          dataSet: this
        });
      }
    }
  }, {
    key: "toData",
    value: function toData() {
      if (_mobx._isComputingDerivation && (0, _mobx._isComputingDerivation)()) {
        (0, _utils.warning)(false, 'Please do not use the `dataSet.toData` method during the rendering phase.');
      }

      return (0, _utils2.generateData)([].concat((0, _toConsumableArray2["default"])(this.cachedModified), (0, _toConsumableArray2["default"])(this.records))).data;
    }
    /**
     * 对应参数后续会废弃
     * @param isSelected
     * @param noCascade
     */

  }, {
    key: "toJSONData",
    value: function toJSONData(isSelected, noCascade) {
      if (_mobx._isComputingDerivation && (0, _mobx._isComputingDerivation)()) {
        (0, _utils.warning)(false, 'Please do not use the `dataSet.toJSONData` method during the rendering phase.');
      }

      var dataToJSON = (0, _utils2.adapterDataToJSON)(isSelected, noCascade) || this.dataToJSON;
      var records = (0, _utils2.useSelected)(dataToJSON) ? this.selected : this.all;
      return (0, _utils2.generateJSONData)(this, records).data;
    }
    /**
     * 等待选中或者所有记录准备就绪
     * @returns Promise
     */

  }, {
    key: "ready",
    value: function ready(_isSelect) {
      // return Promise.all([
      //   this.pending.ready(),
      //   ...(isSelect || useSelected(this.dataToJSON) ? this.selected : this.data).map(record =>
      //     record.ready(),
      //   ),
      //   ...[...this.fields.values()].map(field => field.ready()),
      // ]);
      return this.pending.ready();
    }
    /**
     * 查询记录
     * @param page 页码
     * @param params 查询参数
     * @param cache 是否保留 cachedRecords
     * @return Promise
     */

  }, {
    key: "query",
    value: function query(page, params, cache) {
      return this.pending.add(this.doQuery(page, params, cache));
    }
    /**
     * 查询更多记录，查询到的结果会拼接到原有数据之后
     * @param page 页码
     * @param params 查询参数
     * @return Promise
     */

  }, {
    key: "queryMore",
    value: function queryMore(page, params) {
      return this.pending.add(this.doQueryMore(page, params));
    }
  }, {
    key: "queryMoreChild",
    value: function queryMoreChild(parent, page) {
      var params = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
      var _this$props3 = this.props,
          idField = _this$props3.idField,
          parentField = _this$props3.parentField;

      if (idField && parentField && !parent.children) {
        var id = parent.get(idField);

        if (!(0, _utils.isEmpty)(id)) {
          if (parentField) {
            params[parentField] = id;
          }

          return this.pending.add(this.doQueryMoreChild(parent, page, (0, _objectSpread3["default"])((0, _objectSpread3["default"])({}, params), {}, (0, _defineProperty2["default"])({}, parentField, id))));
        }
      }

      return Promise.resolve();
    }
  }, {
    key: "doQuery",
    value: function () {
      var _doQuery = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(page, params, cache, paging) {
        var data, counting, totalKey, count;
        return _regenerator["default"].wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return this.read(page, params, undefined, paging);

              case 2:
                data = _context.sent;
                this.loadDataFromResponse(data, cache);

                if (!data) {
                  _context.next = 11;
                  break;
                }

                counting = this.counting, totalKey = this.totalKey;

                if (!(totalKey && counting)) {
                  _context.next = 11;
                  break;
                }

                _context.next = 9;
                return counting;

              case 9:
                count = _context.sent;

                if (count !== undefined) {
                  data[totalKey] = count;
                }

              case 11:
                return _context.abrupt("return", data);

              case 12:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function doQuery(_x, _x2, _x3, _x4) {
        return _doQuery.apply(this, arguments);
      }

      return doQuery;
    }()
  }, {
    key: "doQueryMore",
    value: function () {
      var _doQueryMore = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2(page, params) {
        var data, counting, totalKey, count;
        return _regenerator["default"].wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.next = 2;
                return this.read(page, params, true);

              case 2:
                data = _context2.sent;
                this.appendDataFromResponse(data);

                if (!data) {
                  _context2.next = 11;
                  break;
                }

                counting = this.counting, totalKey = this.totalKey;

                if (!(totalKey && counting)) {
                  _context2.next = 11;
                  break;
                }

                _context2.next = 9;
                return counting;

              case 9:
                count = _context2.sent;

                if (count !== undefined) {
                  data[totalKey] = count;
                }

              case 11:
                return _context2.abrupt("return", data);

              case 12:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function doQueryMore(_x5, _x6) {
        return _doQueryMore.apply(this, arguments);
      }

      return doQueryMore;
    }()
  }, {
    key: "doQueryMoreChild",
    value: function () {
      var _doQueryMoreChild = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee3(parent, page, params) {
        var data, counting, totalKey, count;
        return _regenerator["default"].wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.next = 2;
                return this.read(page, params, true);

              case 2:
                data = _context3.sent;
                this.appendDataFromResponse(data, parent);

                if (!data) {
                  _context3.next = 11;
                  break;
                }

                counting = this.counting, totalKey = this.totalKey;

                if (!(totalKey && counting)) {
                  _context3.next = 11;
                  break;
                }

                _context3.next = 9;
                return counting;

              case 9:
                count = _context3.sent;

                if (count !== undefined) {
                  data[totalKey] = count;
                }

              case 11:
                return _context3.abrupt("return", data);

              case 12:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function doQueryMoreChild(_x7, _x8, _x9) {
        return _doQueryMoreChild.apply(this, arguments);
      }

      return doQueryMoreChild;
    }()
    /**
     * TODO 参数废弃
     * 将数据集中的增删改的记录进行远程提交
     * @param isSelect 如果为true，则只提交选中记录
     * @param noCascade 如果为true，则不提交级联数据
     * @return Promise
     */

  }, {
    key: "submit",
    value: function () {
      var _submit = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee4(isSelect, noCascade) {
        var dataToJSON;
        return _regenerator["default"].wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                dataToJSON = (0, _utils2.adapterDataToJSON)(isSelect, noCascade) || this.dataToJSON;
                _context4.next = 3;
                return this.ready();

              case 3:
                _context4.next = 5;
                return this.validate();

              case 5:
                if (!_context4.sent) {
                  _context4.next = 7;
                  break;
                }

                return _context4.abrupt("return", this.pending.add(this.write((0, _utils2.useSelected)(dataToJSON) ? this.selected : this.all)));

              case 7:
                return _context4.abrupt("return", false);

              case 8:
              case "end":
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function submit(_x10, _x11) {
        return _submit.apply(this, arguments);
      }

      return submit;
    }()
    /**
     * 强制将数据集中的增删改的记录进行远程提交, 绕过校验
     * @return Promise
     */

  }, {
    key: "forceSubmit",
    value: function forceSubmit() {
      var _this12 = this;

      var dataToJSON = this.dataToJSON;
      return this.ready().then(function () {
        return _this12.pending.add(_this12.write((0, _utils2.useSelected)(dataToJSON) ? _this12.selected : _this12.all));
      });
    }
    /**
     * 导出数据
     * @param object columns 导出的列
     * @param number exportQuantity 导出数量
     */

  }, {
    key: "export",
    value: function () {
      var _export2 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee5() {
        var columns,
            exportQuantity,
            data,
            totalCount,
            totalKey,
            selected,
            params,
            newConfig,
            ExportQuantity,
            _args5 = arguments;
        return _regenerator["default"].wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                columns = _args5.length > 0 && _args5[0] !== undefined ? _args5[0] : {};
                exportQuantity = _args5.length > 1 && _args5[1] !== undefined ? _args5[1] : 0;

                if (!this.checkReadable(this.parent)) {
                  _context5.next = 24;
                  break;
                }

                _context5.next = 5;
                return this.ready();

              case 5:
                _context5.next = 7;
                return this.generateQueryParameter();

              case 7:
                data = _context5.sent;
                data._HAP_EXCEL_EXPORT_COLUMNS = columns;
                totalCount = this.totalCount, totalKey = this.totalKey, selected = this.selected;
                params = (0, _objectSpread3["default"])({
                  _r: Date.now()
                }, this.generateOrderQueryString());
                ObjectChainValue.set(params, totalKey, this.getState('__EXPORT-STRATEGY__') === 'ALL' ? totalCount : selected.length);
                newConfig = (0, _utils2.axiosConfigAdapter)('exports', this, data, params);

                if (!(newConfig.url || this.exportMode === _enum.ExportMode.client)) {
                  _context5.next = 23;
                  break;
                }

                _context5.next = 16;
                return this.fireEvent(_enum.DataSetEvents["export"], {
                  dataSet: this,
                  params: newConfig.params,
                  data: newConfig.data
                });

              case 16:
                _context5.t0 = _context5.sent;

                if (!(_context5.t0 !== false)) {
                  _context5.next = 21;
                  break;
                }

                ExportQuantity = exportQuantity > 1000 ? 1000 : exportQuantity;

                if (this.exportMode !== _enum.ExportMode.client) {
                  (0, _utils2.doExport)(this.axios.getUri(newConfig), newConfig.data, newConfig.method);
                }

                return _context5.abrupt("return", this.doClientExport(data, ExportQuantity, false));

              case 21:
                _context5.next = 24;
                break;

              case 23:
                (0, _utils.warning)(false, 'Unable to execute the export method of dataset, please check the ');

              case 24:
              case "end":
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      function _export() {
        return _export2.apply(this, arguments);
      }

      return _export;
    }()
    /**
     * 可以把json数组通过ds配置转化成可以直接浏览的数据信息
     * @param result 需要转化内容
     * @param columnsExport 表头信息
     */

  }, {
    key: "displayDataTransform",
    value: function displayDataTransform(result, columnsExport) {
      var _this13 = this;

      var newResult = [];

      if (result && result.length > 0) {
        // check: 这里做性能优化去掉实例化为record 从demo来看没啥问题
        // toJS(this.processData(result)).map((item) => item.data);
        var processData = result;
        processData.forEach(function (itemValue) {
          var dataItem = {};
          var columnsExportkeys = Object.keys(columnsExport);

          for (var i = 0; i < columnsExportkeys.length; i += 1) {
            var firstRecord = _this13.records[0];

            var exportField = _this13.getField(columnsExportkeys[i]);

            var processItemValue = (0, _utils2.getSplitValue)((0, _mobx.toJS)(itemValue), columnsExportkeys[i]); // 处理bind 情况 没有需要链式bind 没有处理反向bind 后面可能根据需求增加

            if (exportField && (0, _isNil["default"])(processItemValue) && exportField.get('bind', firstRecord)) {
              processItemValue = (0, _utils2.getSplitValue)((0, _utils2.getSplitValue)((0, _mobx.toJS)(itemValue), exportField.get('bind', firstRecord)), columnsExportkeys[i], false);
            }

            dataItem[columnsExportkeys[i]] = (0, _utils2.processExportValue)(processItemValue, exportField);
          }

          newResult.push(dataItem);
        });
      }

      return newResult;
    }
    /**
     * 客户端导出方法
     * @param data 表头数据
     * @param quantity 输入一次导出数量
     * @param isFile 是否导出为文件
     */

  }, {
    key: "doClientExport",
    value: function () {
      var _doClientExport = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee6(data, quantity) {
        var _this14 = this;

        var isFile,
            columnsExport,
            totalCount,
            selected,
            exportStrategy,
            newResult,
            queryTime,
            queryExportList,
            _loop,
            i,
            _args6 = arguments;

        return _regenerator["default"].wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                isFile = _args6.length > 2 && _args6[2] !== undefined ? _args6[2] : true;
                columnsExport = data._HAP_EXCEL_EXPORT_COLUMNS;
                delete data._HAP_EXCEL_EXPORT_COLUMNS;
                totalCount = this.totalCount, selected = this.selected;
                exportStrategy = this.getState('__EXPORT-STRATEGY__');
                (0, _mobx.runInAction)(function () {
                  _this14.exportProgress = 0;
                  _this14.exportStatus = _enum.DataSetExportStatus.start;
                });
                newResult = [];

                if (!(totalCount > 0)) {
                  _context6.next = 16;
                  break;
                }

                (0, _mobx.runInAction)(function () {
                  _this14.exportStatus = _enum.DataSetExportStatus.exporting;
                });

                if (!(exportStrategy === 'ALL')) {
                  _context6.next = 15;
                  break;
                }

                queryTime = Math.ceil(totalCount / quantity); // 处理超并发问题 在超大数据量下一口气发出了几千个请求造成数据丢失

                queryExportList = [];

                _loop = function _loop(i) {
                  var params = (0, _objectSpread3["default"])({}, _this14.generateQueryString(1 + i, quantity));
                  var newConfig = (0, _utils2.axiosConfigAdapter)('read', _this14, data, params);
                  queryExportList.push({
                    getPromise: function getPromise() {
                      return _this14.axios(newConfig);
                    }
                  });
                };

                for (i = 0; i < queryTime; i++) {
                  _loop(i);
                }

                return _context6.abrupt("return", (0, _utils2.concurrentPromise)(queryExportList, function (currentPromiseIndex) {
                  // 下面还有一段处理所以设置最大值为99
                  (0, _mobx.runInAction)(function () {
                    _this14.exportProgress = Math.min(99, Math.floor(currentPromiseIndex / queryExportList.length * 100));
                  });
                  return _this14.exportStatus === undefined;
                }).then(function (resultValue) {
                  var reducer = function reducer(accumulator, currentValue) {
                    return [].concat((0, _toConsumableArray2["default"])(accumulator), (0, _toConsumableArray2["default"])(currentValue));
                  };

                  var todataList = function todataList(item) {
                    return item ? item[_this14.dataKey] : [];
                  };

                  (0, _mobx.runInAction)(function () {
                    _this14.exportStatus = _enum.DataSetExportStatus.progressing;
                  });
                  var exportAlldate = resultValue.map(todataList).reduce(reducer);
                  newResult = _this14.displayDataTransform(exportAlldate, columnsExport);
                  newResult.unshift(columnsExport);
                  (0, _mobx.runInAction)(function () {
                    _this14.exportStatus = _enum.DataSetExportStatus.success;
                  });

                  if (isFile) {
                    (0, _utils2.exportExcel)(newResult, _this14.name, _this14.getConfig('xlsx'));
                  } else {
                    return newResult;
                  }
                })["catch"](function (err) {
                  console.warn(err);
                  (0, _mobx.runInAction)(function () {
                    _this14.exportStatus = _enum.DataSetExportStatus.failed;
                    _this14.exportProgress = 0;
                  });
                }));

              case 15:
                return _context6.abrupt("return", new Promise(function (resolve) {
                  (0, _mobx.runInAction)(function () {
                    _this14.exportProgress = 10;
                  });
                  setTimeout(function () {
                    resolve(true);
                  }, 1000);
                }).then(function () {
                  (0, _mobx.runInAction)(function () {
                    _this14.exportStatus = _enum.DataSetExportStatus.progressing;
                  });
                  var jsonData = selected.map(function (x) {
                    return (0, _objectSpread3["default"])({}, x.toData());
                  });
                  newResult = _this14.displayDataTransform(jsonData, columnsExport);
                  newResult.unshift(columnsExport);
                  (0, _mobx.runInAction)(function () {
                    _this14.exportStatus = _enum.DataSetExportStatus.success;
                  });

                  if (isFile) {
                    (0, _utils2.exportExcel)(newResult, _this14.name, _this14.getConfig('xlsx'));
                  } else {
                    return newResult;
                  }
                })["catch"](function (err) {
                  console.warn(err);
                  (0, _mobx.runInAction)(function () {
                    _this14.exportStatus = _enum.DataSetExportStatus.failed;
                    _this14.exportProgress = 0;
                  });
                }));

              case 16:
              case "end":
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      function doClientExport(_x12, _x13) {
        return _doClientExport.apply(this, arguments);
      }

      return doClientExport;
    }()
    /**
     * 重置更改
     */

  }, {
    key: "reset",
    value: function reset() {
      this.resetInBatch = true;

      try {
        this.records = this.records.filter(function (record) {
          var status = record.status,
              parent = record.parent;

          if (status === _enum.RecordStatus.add) {
            if (parent && parent.children) {
              var index = parent.children.indexOf(record);

              if (index > -1) {
                parent.children.splice(index, 1);
                if (!parent.children.length) parent.children = undefined;
              }
            }

            record.cancelReportValidity();
            return false;
          }

          return record.reset();
        });
      } finally {
        this.resetInBatch = false;
      }

      if (this.props.autoCreate && this.records.length === 0) {
        this.create();
      }

      this.clearValidationError();
      this.fireEvent(_enum.DataSetEvents.reset, {
        dataSet: this,
        records: this.records
      });
      return this;
    }
    /**
     * 定位到指定页码，如果paging为true或`server`，则做远程查询，约定当为Tree 状态的server时候 跳转到下一页也就是index为当前的index加上1
     * @param page 页码
     * @return Promise
     */

  }, {
    key: "page",
    value: function page(_page) {
      if (_page > 0 && this.paging) {
        return this.locate((_page - 1) * this.pageSize + (_page > this.currentPage ? this.created.length - this.cachedCreated.length - this.destroyed.length + this.cachedDestroyed.length : 0));
      }

      (0, _utils.warning)(_page > 0, 'Page number is incorrect.');
      (0, _utils.warning)(!!this.paging, 'Can not paging query util the property<paging> of DataSet is true or `server`.');
      return Promise.resolve();
    }
    /**
     * 变更检查，当有变更时会弹确认框
     * @param message 提示信息或者是confirm的参数
     * @param dataSet 触发查询数据源
     * @param source 修改提示信息来源 查询条：'query' 翻页： undefined
     * @return Promise
     */

  }, {
    key: "modifiedCheck",
    value: function modifiedCheck(message, dataSet, source) {
      var _this$props4 = this.props,
          modifiedCheck = _this$props4.modifiedCheck,
          modifiedCheckMessage = _this$props4.modifiedCheckMessage;

      if (this.cacheModifiedKeys || !modifiedCheck || !this.dirty) {
        return Promise.resolve(true);
      }

      return this.getConfig('confirm')(message || modifiedCheckMessage || (0, _localeContext.$l)('DataSet', 'unsaved_data_confirm'), dataSet, source);
    }
    /**
     * 定位记录
     * @param index 索引
     * @return Promise
     */

  }, {
    key: "locate",
    value: function () {
      var _locate = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee7(index) {
        var paging, pageSize, totalCount, autoLocateFirst, currentRecord;
        return _regenerator["default"].wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                paging = this.paging, pageSize = this.pageSize, totalCount = this.totalCount;
                autoLocateFirst = this.props.autoLocateFirst;
                currentRecord = this.findInAllPage(index);

                if (!currentRecord) {
                  _context7.next = 6;
                  break;
                }

                this.current = currentRecord;
                return _context7.abrupt("return", currentRecord);

              case 6:
                if (!(paging === true || paging === 'server')) {
                  _context7.next = 18;
                  break;
                }

                if (index > this.currentIndex) {
                  index -= this.created.length - this.cachedCreated.length - this.destroyed.length + this.cachedDestroyed.length;
                }

                if (!(index >= 0 && index < totalCount)) {
                  _context7.next = 18;
                  break;
                }

                _context7.next = 11;
                return this.modifiedCheck();

              case 11:
                if (!_context7.sent) {
                  _context7.next = 18;
                  break;
                }

                _context7.next = 14;
                return this.pending.add(this.doQuery(Math.floor(index / pageSize) + 1, undefined, true, true));

              case 14:
                currentRecord = this.findInAllPage(index);

                if (!currentRecord) {
                  _context7.next = 18;
                  break;
                }

                this.current = autoLocateFirst ? currentRecord : undefined;
                return _context7.abrupt("return", currentRecord);

              case 18:
                (0, _utils.warning)(false, 'Located index of Record is out of boundary.');
                return _context7.abrupt("return", Promise.resolve(undefined));

              case 20:
              case "end":
                return _context7.stop();
            }
          }
        }, _callee7, this);
      }));

      function locate(_x14) {
        return _locate.apply(this, arguments);
      }

      return locate;
    }()
    /**
     * 定位到第一条记录
     * @return Promise
     */

  }, {
    key: "first",
    value: function first() {
      return this.locate(0);
    }
    /**
     * 定位到最后一条记录
     * @return Promise
     */

  }, {
    key: "last",
    value: function last() {
      return this.locate((this.paging ? this.totalCount : this.length) - 1);
    }
    /**
     * 定位到当前记录的上一条记录
     * 若当前页中当前记录为第一条记录且有上一页，则会查询上一页并定位到上一页的最后一条记录
     * @return Promise
     */

  }, {
    key: "pre",
    value: function pre() {
      return this.locate(this.currentIndex - 1);
    }
    /**
     * 定位到当前记录的下一条记录
     * 若当前页中当前记录为最后一条记录且有下一页，则会查询下一页并定位到下一页的第一条记录
     * @return Promise
     */

  }, {
    key: "next",
    value: function next() {
      return this.locate(this.currentIndex + 1);
    }
    /**
     * 定位到首页
     * @return Promise
     */

  }, {
    key: "firstPage",
    value: function firstPage() {
      return this.page(1);
    }
    /**
     * 定位到上一页
     * @return Promise
     */

  }, {
    key: "prePage",
    value: function prePage() {
      return this.page(this.currentPage - 1);
    }
    /**
     * 定位到下一页
     * @return Promise
     */

  }, {
    key: "nextPage",
    value: function nextPage() {
      return this.page(this.currentPage + 1);
    }
    /**
     * 定位到尾页
     * @return Promise
     */

  }, {
    key: "lastPage",
    value: function lastPage() {
      return this.page(this.totalPage);
    }
    /**
     * 创建一条记录
     * @param data 数据对象
     * @param dataIndex 记录所在的索引
     * @return 新建的记录
     */

  }, {
    key: "create",
    value: function create() {
      var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var dataIndex = arguments.length > 1 ? arguments[1] : undefined;

      if (data === null) {
        data = {};
      }

      var record = new _Record["default"](data, this);
      var objectFieldsList = [];
      var normalFields = [];
      this.fields.forEach(function (field, name) {
        var fieldDefaultValue = field.get('defaultValue', record);
        var multiple = field.get('multiple', record);
        var defaultValue = multiple && (0, _isNil["default"])(fieldDefaultValue) ? [] : fieldDefaultValue;

        if (!(0, _isNil["default"])(defaultValue) && (0, _isNil["default"])(record.get(name))) {
          var type = field.get('type', record);

          if (type === _enum.FieldType.object) {
            var level = name.split('.').length - 1;
            objectFieldsList[level] = (objectFieldsList[level] || []).concat([[name, defaultValue]]);
          } else {
            normalFields.push([name, defaultValue]);
          }
        }
      });
      [].concat(objectFieldsList, [normalFields]).forEach(function (items) {
        if (items) {
          items.forEach(function (_ref3) {
            var _ref4 = (0, _slicedToArray2["default"])(_ref3, 2),
                name = _ref4[0],
                defaultValue = _ref4[1];

            return record.init(name, (0, _mobx.toJS)(defaultValue));
          });
        }
      });
      var _this$props5 = this.props,
          parentField = _this$props5.parentField,
          idField = _this$props5.idField,
          childrenField = _this$props5.childrenField,
          validationRules = _this$props5.validationRules;

      if (!childrenField && parentField && idField) {
        var parentId = record.get(parentField);

        if (parentId) {
          var parent = this.find(function (r) {
            return r.get(idField) === parentId;
          });

          if (parent) {
            record.parent = parent;
            var children = parent.children;

            if (children) {
              if ((0, _isNumber["default"])(dataIndex)) {
                children.splice(dataIndex, 0, record);
                dataIndex += parent.index + 1;
              } else {
                children.push(record);
                dataIndex = parent.index + children.length;
              }
            } else {
              parent.children = [record];
              dataIndex = parent.index + 1;
            }
          }
        }
      }

      if ((0, _isNumber["default"])(dataIndex)) {
        this.splice(dataIndex, 0, record);
      } else {
        this.push(record);
      }

      if (this.props.autoLocateAfterCreate) {
        this.current = record;
      }

      if (validationRules && this.validationSelfErrors) {
        var error = this.validationSelfErrors.find(function (item) {
          return item.name === _enum.ValidationSelfType.minLength;
        });

        if (error && error.value <= this.length) {
          this.clearValidationError();
        }
      }

      this.fireEvent(_enum.DataSetEvents.create, {
        dataSet: this,
        record: record
      });
      return record;
    }
    /**
     * 立即删除记录
     * @param records 记录或者记录数组，默认当前记录
     * @param confirmMessage 提示信息或弹窗的属性
     * @return Promise
     */

  }, {
    key: "delete",
    value: function () {
      var _delete2 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee8(records, confirmMessage) {
        var res, current, record;
        return _regenerator["default"].wrap(function _callee8$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                if (!records) {
                  _context8.next = 24;
                  break;
                }

                records = [].concat(records);
                _context8.t1 = records.length > 0;

                if (!_context8.t1) {
                  _context8.next = 8;
                  break;
                }

                _context8.next = 6;
                return this.fireEvent(_enum.DataSetEvents.beforeDelete, {
                  dataSet: this,
                  records: records
                });

              case 6:
                _context8.t2 = _context8.sent;
                _context8.t1 = _context8.t2 !== false;

              case 8:
                _context8.t0 = _context8.t1;

                if (!_context8.t0) {
                  _context8.next = 16;
                  break;
                }

                _context8.t3 = confirmMessage === false;

                if (_context8.t3) {
                  _context8.next = 15;
                  break;
                }

                _context8.next = 14;
                return this.getConfig('confirm')(confirmMessage && confirmMessage !== true ? confirmMessage : (0, _localeContext.$l)('DataSet', 'delete_selected_row_confirm'));

              case 14:
                _context8.t3 = _context8.sent;

              case 15:
                _context8.t0 = _context8.t3;

              case 16:
                if (!_context8.t0) {
                  _context8.next = 24;
                  break;
                }

                this.remove(records, false, false);
                _context8.next = 20;
                return this.pending.add(this.write(this.destroyed, true));

              case 20:
                res = _context8.sent;
                // 处理自动定位
                current = this.current;

                if (current) {
                  if (this.props.autoLocateAfterRemove) {
                    record = this.get(0);

                    if (record) {
                      (0, _mobx.runInAction)(function () {
                        record.isCurrent = true;
                      });
                    }
                  }

                  if (current !== record) {
                    this.fireEvent(_enum.DataSetEvents.indexChange, {
                      dataSet: this,
                      record: record,
                      previous: current
                    });
                  }
                }

                return _context8.abrupt("return", res);

              case 24:
              case "end":
                return _context8.stop();
            }
          }
        }, _callee8, this);
      }));

      function _delete(_x15, _x16) {
        return _delete2.apply(this, arguments);
      }

      return _delete;
    }()
    /**
     * 临时删除记录
     * @param records 记录或者记录数组
     * @param locate 是否需要进行定位操作
     */

  }, {
    key: "remove",
    value: function remove(records, forceRemove, locate) {
      var _this15 = this;

      if (records) {
        var data = (0, _mobx.isArrayLike)(records) ? records.slice() : [records];

        if (data.length && this.fireEventSync(_enum.DataSetEvents.beforeRemove, {
          dataSet: this,
          records: data
        }) !== false) {
          var current = this.current;
          data.forEach(function (record) {
            return _this15.deleteRecord(record, forceRemove);
          });

          if (this.props.validationRules && this.validationSelfErrors) {
            var error = this.validationSelfErrors.find(function (item) {
              return item.name === _enum.ValidationSelfType.maxLength;
            });

            if (error && error.value >= this.length) {
              this.clearValidationError();
            }
          }

          this.fireEvent(_enum.DataSetEvents.remove, {
            dataSet: this,
            records: data
          });

          if (!this.current) {
            var record;

            if (locate !== false && this.props.autoLocateAfterRemove) {
              record = this.get(0);

              if (record) {
                record.isCurrent = true;
              }
            }

            if (locate !== false && current !== record) {
              this.fireEvent(_enum.DataSetEvents.indexChange, {
                dataSet: this,
                record: record,
                previous: current
              });
            }
          }
        }
      }
    }
    /**
     * 临时删除所有记录
     */

  }, {
    key: "removeAll",
    value: function removeAll(forceRemove) {
      var _this16 = this;

      var current = this.current,
          data = this.data;

      if (data.length) {
        data.forEach(function (record) {
          return _this16.deleteRecord(record, forceRemove);
        });
        this.fireEvent(_enum.DataSetEvents.remove, {
          dataSet: this,
          records: data
        });

        if (current) {
          this.fireEvent(_enum.DataSetEvents.indexChange, {
            dataSet: this,
            previous: current
          });
        }
      }
    }
    /**
     * 删除所有记录
     * @param confirmMessage 提示信息或弹窗的属性
     */

  }, {
    key: "deleteAll",
    value: function () {
      var _deleteAll = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee9(confirmMessage) {
        return _regenerator["default"].wrap(function _callee9$(_context9) {
          while (1) {
            switch (_context9.prev = _context9.next) {
              case 0:
                _context9.t0 = this.records.length > 0;

                if (!_context9.t0) {
                  _context9.next = 8;
                  break;
                }

                _context9.t1 = confirmMessage === false;

                if (_context9.t1) {
                  _context9.next = 7;
                  break;
                }

                _context9.next = 6;
                return this.getConfig('confirm')(confirmMessage && confirmMessage !== true ? confirmMessage : (0, _localeContext.$l)('DataSet', 'delete_all_row_confirm'));

              case 6:
                _context9.t1 = _context9.sent;

              case 7:
                _context9.t0 = _context9.t1;

              case 8:
                if (!_context9.t0) {
                  _context9.next = 11;
                  break;
                }

                this.removeAll();
                return _context9.abrupt("return", this.pending.add(this.write(this.destroyed, true)));

              case 11:
              case "end":
                return _context9.stop();
            }
          }
        }, _callee9, this);
      }));

      function deleteAll(_x17) {
        return _deleteAll.apply(this, arguments);
      }

      return deleteAll;
    }()
    /**
     * 将若干数据记录插入记录堆栈顶部
     * @param records 数据集
     * @return 堆栈数量
     */

  }, {
    key: "push",
    value: function push() {
      var _this$records;

      (0, _utils2.checkParentByInsert)(this);

      for (var _len = arguments.length, records = new Array(_len), _key = 0; _key < _len; _key++) {
        records[_key] = arguments[_key];
      }

      return (_this$records = this.records).push.apply(_this$records, (0, _toConsumableArray2["default"])(this.transferRecords(records)));
    }
    /**
     * 将若干数据记录插入记录堆栈底部
     * @param records 数据集
     * @return 堆栈数量
     */

  }, {
    key: "unshift",
    value: function unshift() {
      var _this$records2;

      (0, _utils2.checkParentByInsert)(this);

      for (var _len2 = arguments.length, records = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        records[_key2] = arguments[_key2];
      }

      return (_this$records2 = this.records).unshift.apply(_this$records2, (0, _toConsumableArray2["default"])(this.transferRecords(records)));
    }
    /**
     * 从记录堆栈顶部获取记录
     * @return 记录
     */

  }, {
    key: "pop",
    value: function pop() {
      return this.deleteRecord(this.data.pop());
    }
    /**
     * 从记录堆栈底部获取记录
     * @return 记录
     */

  }, {
    key: "shift",
    value: function shift() {
      return this.deleteRecord(this.data.shift());
    }
    /**
     * 删除指定索引的若干记录，并可插入若干新记录
     * @param from 索引开始的位置
     * @default 0
     * @param deleteCount 删除的数量
     * @default 0
     * @param records 插入的若干新记录
     * @return 被删除的记录集
     */

  }, {
    key: "splice",
    value: function splice(from, deleteCount) {
      var _this17 = this;

      var fromRecord = this.get(from);
      var deleted = this.slice(from, from + deleteCount).map(function (record) {
        return _this17.deleteRecord(record);
      });

      for (var _len3 = arguments.length, items = new Array(_len3 > 2 ? _len3 - 2 : 0), _key3 = 2; _key3 < _len3; _key3++) {
        items[_key3 - 2] = arguments[_key3];
      }

      if (items.length) {
        (0, _utils2.checkParentByInsert)(this);
        var records = this.records;
        var spliceRecord = (0, _utils2.getSpliceRecord)(records, items, fromRecord);
        var transformedRecords = this.transferRecords(items);

        if (spliceRecord) {
          records.splice.apply(records, [records.indexOf(spliceRecord), 0].concat((0, _toConsumableArray2["default"])(transformedRecords)));
        } else {
          records.push.apply(records, (0, _toConsumableArray2["default"])(transformedRecords));
        }
      }

      return deleted;
    }
    /**
     * 切换记录的顺序
     */

  }, {
    key: "move",
    value: function move(from, to) {
      (0, _utils2.arrayMove)(this.records, from, to);
    }
    /**
     * 截取指定索引范围的记录集，不改变原记录堆栈
     * @param start 开始索引
     * @default 0
     * @param end 结束索引
     * @default 记录堆栈长度
     * @return 被删除的记录集
     */

  }, {
    key: "slice",
    value: function slice() {
      var start = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
      var end = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.length;
      return this.data.slice(start, end);
    }
    /**
     * 获取记录所在的索引
     * @param record 记录
     * @param fromIndex 开始检索的索引
     * @return 索引
     */

  }, {
    key: "indexOf",
    value: function indexOf(record, fromIndex) {
      return this.data.indexOf(record, fromIndex);
    }
    /**
     * 根据函数查找记录
     * @param fn 查询函数
     * @returns 记录
     */

  }, {
    key: "find",
    value: function find(fn) {
      return this.data.find(fn);
    }
    /**
     * 根据函数查找记录所在的索引
     * @param fn 查询函数
     * @returns 索引
     */

  }, {
    key: "findIndex",
    value: function findIndex(fn) {
      return this.data.findIndex(fn);
    }
    /**
     * 根据函数遍历
     * @param fn 遍历函数
     * @param thisArg this对象
     */

  }, {
    key: "forEach",
    value: function forEach(fn, thisArg) {
      this.data.forEach(fn, thisArg);
    }
    /**
     * 根据函数遍历并输出新数组
     * @param fn 遍历函数
     * @param thisArg this对象
     * @returns 输出新数组
     */

  }, {
    key: "map",
    value: function map(fn, thisArg) {
      return this.data.map(fn, thisArg);
    }
    /**
     * 根据函数遍历，当有返回值为true时，输出true
     * @param fn 遍历函数
     * @param thisArg this对象
     * @returns boolean
     */

  }, {
    key: "some",
    value: function some(fn, thisArg) {
      return this.data.some(fn, thisArg);
    }
    /**
     * 根据函数遍历，当有返回值为false时，输出false
     * @param fn 遍历函数
     * @param thisArg this对象
     * @returns boolean
     */

  }, {
    key: "every",
    value: function every(fn, thisArg) {
      return this.data.every(fn, thisArg);
    }
    /**
     * 根据函数过滤并返回记录集
     * @param fn 过滤函数
     * @param thisArg this对象
     * @returns {Record[]}
     */

  }, {
    key: "filter",
    value: function filter(fn, thisArg) {
      return this.data.filter(fn, thisArg);
    }
    /**
     * 为数组中的所有元素调用指定的回调函数。 回调函数的返回值是累计结果，并在下次调用回调函数时作为参数提供。
     * @param fn 累计函数
     * @param initialValue 初始值
     * @returns {U}
     */

  }, {
    key: "reduce",
    value: function reduce(fn, initialValue) {
      return this.data.reduce(fn, initialValue);
    }
    /**
     * 按降序调用数组中所有元素的指定回调函数。 回调函数的返回值是累计结果，并在下次调用回调函数时作为参数提供。
     * @param fn 累计函数
     * @param initialValue 初始值
     * @returns {U}
     */

  }, {
    key: "reduceRight",
    value: function reduceRight(fn, initialValue) {
      return this.data.reduceRight(fn, initialValue);
    }
    /**
     * 反转记录的顺序。
     */

  }, {
    key: "reverse",
    value: function reverse() {
      return this.records = this.records.reverse();
    }
    /**
     * 服务端排序
     *
     * @param sortInfo 字段名 或 有顺序的字段列表
     */

  }, {
    key: "sort",
    value: function sort(sortInfo) {
      var _this18 = this;

      var combineSort = this.props.combineSort;
      var nowCombineSort = this.combineSort;

      if (typeof sortInfo === 'string') {
        var field = this.getField(sortInfo);

        if (field) {
          this.fields.forEach(function (current) {
            if (current.order && (current !== field || nowCombineSort)) {
              current.order = undefined;
            }
          });
          this.combineSort = false;
          var order = field.order;

          switch (order) {
            case _enum.SortOrder.asc:
              field.order = _enum.SortOrder.desc;
              break;

            case _enum.SortOrder.desc:
              field.order = undefined;
              break;

            default:
              field.order = _enum.SortOrder.asc;
          }
        }
      } else if (combineSort && sortInfo) {
        this.fields.forEach(function (current) {
          current.order = undefined;
        });
        this.combineSort = true;
        this.combineSortFieldNames = _mobx.observable.map(sortInfo);
        sortInfo.forEach(function (sortOrder, fieldName) {
          var field = _this18.getField(fieldName);

          if (field) {
            field.order = sortOrder;
          }
        });
      } else {
        return;
      }

      if (this.paging) {
        this.query();
      } else {
        var orderFields = (0, _utils2.getOrderFields)(this);

        if (orderFields.length) {
          this.records = (0, _utils2.sortTree)(this.records, orderFields, true);
        } else {
          this.query();
        }
      }
    }
    /**
     * 选中记录
     * @param recordOrIndex 记录或记录索引
     */

  }, {
    key: "select",
    value: function select(recordOrIndex) {
      var selection = this.selection;

      if (selection) {
        var record = recordOrIndex;

        if ((0, _isNumber["default"])(recordOrIndex)) {
          record = this.get(recordOrIndex);
        }

        if (record && record.selectable && !record.isSelected) {
          var previous;

          if (selection === _enum.DataSetSelection.single) {
            this.selected.forEach(function (selected) {
              selected.isSelected = false;
              previous = selected;
            });
          }

          record.isSelected = true;

          if (this.isAllPageSelection && (!this.cacheModifiedKeys || !(0, _utils2.isDirtyRecord)(record))) {
            var cachedIndex = this.cachedRecords.indexOf(record);

            if (cachedIndex !== -1) {
              this.cachedRecords.splice(cachedIndex, 1);
            }
          }

          if (!this.inBatchSelection) {
            this.fireEvent(_enum.DataSetEvents.select, {
              dataSet: this,
              record: record,
              previous: previous
            });
            this.fireEvent(_enum.DataSetEvents.batchSelect, {
              dataSet: this,
              records: [record]
            });
          }
        }
      }
    }
    /**
     * 取消选中记录
     * @param recordOrIndex 记录或记录索引
     */

  }, {
    key: "unSelect",
    value: function unSelect(recordOrIndex) {
      if (this.selection) {
        var record = recordOrIndex;

        if ((0, _isNumber["default"])(recordOrIndex)) {
          record = this.get(recordOrIndex);
        }

        if (record && record.selectable && record.isSelected) {
          record.isSelected = false;

          if (!this.isAllPageSelection && (!this.cacheModifiedKeys || !(0, _utils2.isDirtyRecord)(record))) {
            var cachedIndex = this.cachedRecords.indexOf(record);

            if (cachedIndex !== -1) {
              this.cachedRecords.splice(cachedIndex, 1);
            }
          }

          if (!this.inBatchSelection) {
            this.fireEvent(_enum.DataSetEvents.unSelect, {
              dataSet: this,
              record: record
            });
            this.fireEvent(_enum.DataSetEvents.batchUnSelect, {
              dataSet: this,
              records: [record]
            });
          }
        }
      }
    }
    /**
     * 全选
     */

  }, {
    key: "selectAll",
    value: function selectAll(filter) {
      var _this19 = this;

      var selection = this.selection;

      if (selection) {
        this.inBatchSelection = true;

        try {
          var records = [];

          if (selection === _enum.DataSetSelection.single) {
            if (!this.currentSelected.length) {
              var record = filter ? this.filter(filter)[0] : this.get(0);

              if (record) {
                this.select(record);
                records.push(record);
              }
            }
          } else {
            this.records.forEach(function (record) {
              if (!filter || filter(record) !== false) {
                _this19.select(record);

                records.push(record);
              }
            });
          }

          this.fireEvent(_enum.DataSetEvents.selectAll, {
            dataSet: this,
            records: records
          });
          this.fireEvent(_enum.DataSetEvents.batchSelect, {
            dataSet: this,
            records: records
          });
        } finally {
          this.inBatchSelection = false;
        }
      }
    }
    /**
     * 取消全选
     */

  }, {
    key: "unSelectAll",
    value: function unSelectAll() {
      var _this20 = this;

      if (this.selection) {
        this.inBatchSelection = true;

        try {
          var records = [];
          this.currentSelected.forEach(function (record) {
            _this20.unSelect(record);

            records.push(record);
          });
          this.fireEvent(_enum.DataSetEvents.unSelectAll, {
            dataSet: this,
            records: records
          });
          this.fireEvent(_enum.DataSetEvents.batchUnSelect, {
            dataSet: this,
            records: records
          });
        } finally {
          this.inBatchSelection = false;
        }
      }
    }
    /**
     * 批量勾选
     */

  }, {
    key: "batchSelect",
    value: function batchSelect(recordOrId) {
      var _this21 = this;

      var selection = this.selection;

      if (selection) {
        var records = recordOrId.reduce(function (list, r) {
          var record = r;

          if ((0, _isNumber["default"])(r)) {
            record = _this21.findRecordById(r);
          }

          if (record && record.selectable && !record.isSelected) {
            list.push(record);
          }

          return list;
        }, []);

        if (records.length) {
          this.inBatchSelection = true;

          try {
            if (selection === _enum.DataSetSelection.single) {
              if (!this.currentSelected.length) {
                this.select(records[0]);
              }
            } else {
              records.forEach(function (record) {
                if (record.selectable) {
                  _this21.select(record);
                }
              });
            }

            this.fireEvent(_enum.DataSetEvents.batchSelect, {
              dataSet: this,
              records: records
            });
          } finally {
            this.inBatchSelection = false;
          }
        }
      }
    }
    /**
     * 批量取消勾选
     */

  }, {
    key: "batchUnSelect",
    value: function batchUnSelect(recordOrId) {
      var _this22 = this;

      var selection = this.selection;

      if (selection) {
        this.inBatchSelection = true;

        try {
          var records = recordOrId.reduce(function (list, r) {
            var record = r;

            if ((0, _isNumber["default"])(r)) {
              record = _this22.findRecordById(r);
            }

            if (record && record.selectable && record.isSelected) {
              _this22.unSelect(record);

              list.push(record);
            }

            return list;
          }, []);
          this.fireEvent(_enum.DataSetEvents.batchUnSelect, {
            dataSet: this,
            records: records
          });
        } finally {
          this.inBatchSelection = false;
        }
      }
    }
  }, {
    key: "treeSelect",
    value: function treeSelect(record) {
      var selection = this.selection;

      if (selection && selection === _enum.DataSetSelection.multiple) {
        this.inBatchSelection = true;

        try {
          var records = [];
          (0, _utils2.treeSelect)(this, record, records);
          (0, _utils2.treeSelectParent)(this, record, records);
          this.fireEvent(_enum.DataSetEvents.batchSelect, {
            dataSet: this,
            records: records
          });
        } finally {
          this.inBatchSelection = false;
        }
      }
    }
  }, {
    key: "treeUnSelect",
    value: function treeUnSelect(record) {
      var selection = this.selection;

      if (selection && selection === _enum.DataSetSelection.multiple) {
        this.inBatchSelection = true;

        try {
          var records = [];
          (0, _utils2.treeUnSelect)(this, record, records);
          (0, _utils2.treeUnSelectParent)(this, record, records);
          this.fireEvent(_enum.DataSetEvents.batchUnSelect, {
            dataSet: this,
            records: records
          });
        } finally {
          this.inBatchSelection = false;
        }
      }
    }
  }, {
    key: "clearCachedRecords",
    value: function clearCachedRecords() {
      var cachedRecords = this.cachedRecords;
      var cachedSelected = cachedRecords.filter(function (record) {
        return record.isSelected;
      });
      this.cachedRecords = [];
      this.fireEvent(_enum.DataSetEvents.batchUnSelect, {
        dataSet: this,
        records: cachedSelected
      });
    }
  }, {
    key: "clearCachedSelected",
    value: function clearCachedSelected() {
      var cachedSelected = this.cachedSelected.slice();
      this.setCachedSelected([]);
      this.fireEvent(_enum.DataSetEvents.batchUnSelect, {
        dataSet: this,
        records: cachedSelected
      });
    }
  }, {
    key: "setCachedSelected",
    value: function setCachedSelected(cachedSelected) {
      var _this23 = this;

      if (this.cacheModifiedKeys) {
        var cachedRecords = this.cachedRecords;
        this.cachedRecords = (0, _iteratorHelper.iteratorSliceToArray)(cachedRecords.reduce(function (set, record) {
          if (!set.has(record) && (0, _utils2.isDirtyRecord)(record)) {
            record.isSelected = false;
            set.add(record);
          }

          return set;
        }, new Set(cachedSelected)).values());
      } else {
        this.cachedRecords = cachedSelected;
      }

      var selectionStrategy = this.selectionStrategy;

      if (selectionStrategy === _enum.CheckedStrategy.SHOW_PARENT) {
        cachedSelected.forEach(function (record) {
          return (0, _utils2.treeSelect)(_this23, record, []);
        });
      } else if (selectionStrategy === _enum.CheckedStrategy.SHOW_CHILD) {
        cachedSelected.forEach(function (record) {
          return (0, _utils2.treeSelectParent)(_this23, record, []);
        });
      }
    }
  }, {
    key: "setCachedCreated",
    value: function setCachedCreated(cachedCreated) {
      updateCachedRecords(this, cachedCreated, _enum.RecordStatus.add);
    }
  }, {
    key: "setCachedUpdated",
    value: function setCachedUpdated(cachedUpdated) {
      updateCachedRecords(this, cachedUpdated, _enum.RecordStatus.update);
    }
  }, {
    key: "setCachedDestroyed",
    value: function setCachedDestroyed(cachedDestroyed) {
      updateCachedRecords(this, cachedDestroyed, _enum.RecordStatus["delete"]);
    }
  }, {
    key: "clearCachedModified",
    value: function clearCachedModified() {
      this.setCachedModified([]);
    }
  }, {
    key: "setCachedModified",
    value: function setCachedModified(cachedModified) {
      updateCachedRecords(this, cachedModified);
    }
    /**
     * 获取指定索引的记录
     * @param index 索引
     * @returns {Record}
     */

  }, {
    key: "get",
    value: function get(index) {
      var data = this.data;
      return data.length ? data[index] : undefined;
    }
    /**
     * 从树形数据中获取指定索引的根节点记录
     * @param index 索引
     * @returns {Record}
     */

  }, {
    key: "getFromTree",
    value: function getFromTree(index) {
      var treeData = this.treeData;
      return treeData.length ? treeData[index] : undefined;
    }
    /**
     * 判断是否有新增、变更或者删除的记录
     * @deprecated
     * @return true | false
     */

  }, {
    key: "isModified",
    value: function isModified() {
      return this.dirty;
    }
    /**
     * 获取指定分页的记录集
     * @param page 如果page为空或者paging为server，则获取当前分页的记录集
     * @return 记录集
     */

    /**
     * 根据记录ID查找记录
     * @param id 记录ID
     * @return 记录
     */

  }, {
    key: "findRecordById",
    value: function findRecordById(id) {
      if (id !== undefined) {
        return this.records.find(function (record) {
          return String(record.id) === String(id);
        });
      }
    }
    /**
     * 清除校验结果
     */

  }, {
    key: "clearValidationError",
    value: function clearValidationError() {
      this.validationSelfErrors = undefined;
    }
    /**
     * 校验数据记录是否有效 对应参数后续会废弃
     * @param isSelected 是否只校验选中记录
     * @param noCascade 是否级联校验
     * @return true | false
     */

  }, {
    key: "validate",
    value: function () {
      var _validate = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee10(isSelected, noCascade) {
        var _this24 = this;

        var dataToJSON, cascade, validateResult, valid;
        return _regenerator["default"].wrap(function _callee10$(_context10) {
          while (1) {
            switch (_context10.prev = _context10.next) {
              case 0:
                this.validating = true;
                _context10.prev = 1;

                if (this.validateSelf()) {
                  _context10.next = 4;
                  break;
                }

                return _context10.abrupt("return", false);

              case 4:
                dataToJSON = (0, _utils2.adapterDataToJSON)(isSelected, noCascade) || this.dataToJSON;
                cascade = noCascade === undefined && dataToJSON ? (0, _utils2.useCascade)(dataToJSON) : !noCascade;
                validateResult = Promise.all(((0, _utils2.useSelected)(dataToJSON) ? this.selected : this.all).map(function (record) {
                  return record.isRemoved || record.validate(_this24.props.forceValidate || (0, _utils2.useAll)(dataToJSON), !cascade);
                })).then(function (results) {
                  return results.every(function (result) {
                    return result;
                  });
                });
                _context10.next = 9;
                return validateResult;

              case 9:
                valid = _context10.sent;
                this.reportValidityImmediately(valid);
                return _context10.abrupt("return", valid);

              case 12:
                _context10.prev = 12;
                this.validating = false;
                return _context10.finish(12);

              case 15:
              case "end":
                return _context10.stop();
            }
          }
        }, _callee10, this, [[1,, 12, 15]]);
      }));

      function validate(_x18, _x19) {
        return _validate.apply(this, arguments);
      }

      return validate;
    }()
    /**
     * 校验dataSet是否有效
     * @return true | false
     */

  }, {
    key: "validateSelf",
    value: function validateSelf() {
      if (this.status === _enum.DataSetStatus.ready) {
        var errors = this.getValidationSelfErrors();
        var valid = !errors.length;
        this.validationSelfErrors = errors;
        this.reportSelfValidityImmediately(valid, errors);
        return valid;
      }

      return true;
    }
  }, {
    key: "reportValidityImmediately",
    value: function reportValidityImmediately(valid) {
      var errors = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.getValidationErrors();
      var fromField = arguments.length > 2 ? arguments[2] : undefined;
      this.fireEvent(_enum.DataSetEvents.validate, {
        dataSet: this,
        result: Promise.resolve(valid),
        valid: valid,
        errors: errors,
        noLocate: fromField
      });

      _Validator["default"].reportAll(errors);
    }
  }, {
    key: "reportSelfValidityImmediately",
    value: function reportSelfValidityImmediately(valid) {
      var errors = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.getValidationSelfErrors();
      this.fireEvent(_enum.DataSetEvents.validateSelf, {
        dataSet: this,
        result: Promise.resolve(valid),
        valid: valid,
        errors: errors
      });

      _Validator["default"].reportDataSet(errors);
    }
  }, {
    key: "reportValidity",
    value: function reportValidity(result, fromField) {
      var _this25 = this;

      var prepareForReport = (0, _utils2.getIf)(this, 'prepareForReport', {
        result: [],
        valid: true
      });

      if (!result.valid) {
        prepareForReport.valid = false;
        prepareForReport.result.push(result);
      }

      if (prepareForReport.timeout) {
        window.clearTimeout(prepareForReport.timeout);
      }

      prepareForReport.timeout = window.setTimeout(function () {
        _this25.reportValidityImmediately(prepareForReport.valid, prepareForReport.result, fromField);

        delete _this25.prepareForReport;
      }, 200);
    }
  }, {
    key: "getValidationErrors",
    value: function getValidationErrors() {
      var dataToJSON = this.dataToJSON;
      var data = (0, _utils2.useSelected)(dataToJSON) ? this.selected : this.all;
      return data.reduce(function (results, record) {
        if (!record.isRemoved) {
          var validationResults = record.getValidationErrors();

          if (validationResults.length) {
            results.push({
              record: record,
              errors: validationResults,
              valid: false
            });
          }
        }

        return results;
      }, []);
    }
  }, {
    key: "getValidationSelfErrors",
    value: function getValidationSelfErrors() {
      var _this26 = this;

      var validationRules = this.props.validationRules;
      var result = [];

      if (validationRules) {
        validationRules.forEach(function (item) {
          var message = item.message,
              value = item.value,
              name = item.name,
              disabled = item.disabled;

          if (!disabled || typeof disabled === 'function' && disabled({
            dataSet: _this26
          }) !== true) {
            switch (name) {
              case _enum.ValidationSelfType.minLength:
                if (_this26.length < value) {
                  result.push(_this26.getValidationSelfError((0, _objectSpread3["default"])((0, _objectSpread3["default"])({}, item), {}, {
                    message: message || (0, _formatter.formatTemplate)((0, _localeContext.$l)('DataSet', 'data_length_too_short'), {
                      length: value
                    })
                  })));
                }

                break;

              case _enum.ValidationSelfType.maxLength:
                if (_this26.length > value) {
                  result.push(_this26.getValidationSelfError((0, _objectSpread3["default"])((0, _objectSpread3["default"])({}, item), {}, {
                    message: message || (0, _formatter.formatTemplate)((0, _localeContext.$l)('DataSet', 'data_length_too_long'), {
                      length: value
                    })
                  })));
                }

                break;

              default:
                break;
            }
          }
        });
      }

      return result;
    }
  }, {
    key: "getValidationSelfError",
    value: function getValidationSelfError(result) {
      return (0, _objectSpread3["default"])((0, _objectSpread3["default"])({}, result), {}, {
        dataSet: this,
        valid: false
      });
    }
  }, {
    key: "getAllValidationErrors",
    value: function getAllValidationErrors() {
      return {
        dataSet: this.getValidationSelfErrors(),
        records: this.getValidationErrors()
      };
    }
    /**
     * 根据字段名获取字段
     * @param fieldName 字段名
     * @returns 字段
     */

  }, {
    key: "getField",
    value: function getField(fieldName) {
      if (fieldName) {
        return this.fields.get(fieldName);
      }
    }
    /**
     * 获取分组字段名
     * @returns 字段名列表
     */

  }, {
    key: "getGroups",
    value: function getGroups() {
      return this.groups;
    }
  }, {
    key: "initFields",
    value: function initFields(fieldProps) {
      var _this27 = this;

      var normalFields = [];
      var objectBindFields = [];
      var bindFields = [];
      var transformResponseField = [];
      var dynamicFields = [];
      var dynamicObjectBindFields = [];
      var dynamicBindFields = [];
      fieldProps.forEach(function (props) {
        var name = props.name;

        if (name) {
          var _initDataSetField = initDataSetField(_this27, name, props),
              _initDataSetField2 = (0, _slicedToArray2["default"])(_initDataSetField, 2),
              field = _initDataSetField2[0],
              intlFields = _initDataSetField2[1];

          var entry = [name, field];
          var computedProps = props.computedProps,
              _props$dynamicProps = props.dynamicProps,
              dynamicProps = _props$dynamicProps === void 0 ? computedProps : _props$dynamicProps,
              type = props.type,
              bind = props.bind,
              transformResponse = props.transformResponse;

          if (dynamicProps) {
            if (dynamicProps.bind) {
              if (type === _enum.FieldType.object) {
                dynamicObjectBindFields.push(entry);
              } else {
                dynamicBindFields.push(entry);
              }
            } else {
              dynamicFields.push(entry);
            }
          } else if (bind) {
            var targetNames = bind.split('.');
            targetNames.pop();

            if (targetNames.some(function (targetName) {
              var target = fieldProps.find(function (fieldProp) {
                return fieldProp.name === targetName;
              });
              return target && (target.computedProps || target.dynamicProps);
            })) {
              if (type === _enum.FieldType.object) {
                dynamicObjectBindFields.push(entry);
              } else {
                dynamicBindFields.push(entry);
              }
            } else if (transformResponse) {
              transformResponseField.push(entry);
            } else if (type === _enum.FieldType.object) {
              objectBindFields.push(entry);
            } else {
              bindFields.push(entry);
            }
          } else {
            normalFields.push(entry);
          }

          if (intlFields) {
            dynamicBindFields.push.apply(dynamicBindFields, (0, _toConsumableArray2["default"])(intlFields.entries()));
          }
        } else {
          (0, _utils.warning)(false, 'DataSet create field failed. Please check if property name is exists on field.');
        }
      });
      return [].concat(normalFields, objectBindFields, bindFields, transformResponseField, dynamicFields, dynamicObjectBindFields, dynamicBindFields);
    }
    /*
     * 增加新字段
     * @param name 字段名
     * @param field 字段属性
     * @return 新增字段
     */

  }, {
    key: "addField",
    value: function addField(name, fieldProps) {
      this.$needToSortFields = true;
      var fields = this.fields;
      var oldField = fields.get(name);

      if (oldField) {
        return oldField.replace(fieldProps);
      }

      var _initDataSetField3 = initDataSetField(this, name, fieldProps),
          _initDataSetField4 = (0, _slicedToArray2["default"])(_initDataSetField3, 2),
          field = _initDataSetField4[0],
          intlFields = _initDataSetField4[1];

      fields.set(name, field);

      if (intlFields) {
        fields.merge(intlFields);
      }

      this.records.forEach(function (r) {
        if (!r.ownerFields.has(name)) {
          var data = (0, _mobx.toJS)(r.data);
          var newData = (0, _objectSpread3["default"])({}, data);
          r.processFieldValue(name, field, fields, newData, data);
        }

        field.processForLookupAndLovConfig(r);
      });
      return field;
    }
  }, {
    key: "commitData",
    value: function commitData(allData, total, onlyDelete) {
      var _this28 = this;

      var _this$props6 = this.props,
          autoQueryAfterSubmit = _this$props6.autoQueryAfterSubmit,
          primaryKey = _this$props6.primaryKey,
          strictPageSize = _this$props6.strictPageSize;

      if (strictPageSize) {
        var paging = this.paging;

        if (paging === 'server') {
          var _this$props7 = this.props,
              idField = _this$props7.idField,
              parentField = _this$props7.parentField,
              childrenField = _this$props7.childrenField;

          if (idField && parentField || childrenField) {
            this.records = (0, _treeHelper.treeReduce)(this.treeRecords.slice(0, this.pageSize), function (pre, node) {
              return pre.concat(node);
            }, []);
          }
        } else if (paging) {
          this.records = this.records.slice(0, this.pageSize);
        }
      }

      if (this.dataToJSON === _enum.DataToJSON.normal) {
        (0, _flatMap["default"])(this.dirtyRecords).forEach(function (record) {
          return record.commit((0, _omit["default"])(record.toData(), ['__dirty']), _this28);
        }); // 若有响应数据，进行数据回写
      } else if (allData.length) {
        var statusKey = this.getConfig('statusKey');
        var status = this.getConfig('status');
        var restCreatedData = [];
        var restUpdatedData = [];
        allData.forEach(function (data) {
          var dataStatus = data[statusKey]; // 若有数据中含有__id，根据__id回写记录，否则根据主键回写非新增的记录

          var record = data.__id ? _this28.findRecordById(data.__id) : primaryKey && dataStatus !== status[_enum.RecordStatus.add] && _this28.records.find(function (r) {
            return r.get(primaryKey) === data[primaryKey];
          });

          if (record) {
            record.commit(data, _this28);
          } else if (dataStatus === status[_enum.RecordStatus.add]) {
            restCreatedData.push(data);
          } else if (dataStatus === status[_enum.RecordStatus.update]) {
            restUpdatedData.push(data);
          }
        });
        var created = this.created,
            updated = this.updated,
            destroyed = this.destroyed; // 没有回写成功的新增数据按顺序回写

        if (restCreatedData.length === created.length) {
          created.forEach(function (r, index) {
            return r.commit(restCreatedData[index], _this28);
          });
        } else if (autoQueryAfterSubmit) {
          // 若有新增数据没有回写成功， 必须重新查询来获取主键
          this.query();
          return this;
        } // 剩下未回写的非新增数据使用原数据进行回写


        if (restUpdatedData.length === updated.length) {
          updated.forEach(function (r, index) {
            return r.commit(restUpdatedData[index], _this28);
          });
        } else if (onlyDelete) {// onlyDelete状态不做其他处理 可以与else的处理合并
          // updated.forEach(r => r.commit(r.toData(), this));
        } else {
          updated.filter(function (r) {
            return restUpdatedData.some(function (data) {
              var dataStatus = data[statusKey];
              return data.__id ? r.id === data.__id : primaryKey && dataStatus !== status[_enum.RecordStatus.add] && r.get(primaryKey) === data[primaryKey];
            });
          }).forEach(function (r) {
            return r.commit((0, _omit["default"])(r.toData(), ['__dirty']), _this28);
          });
        }

        destroyed.forEach(function (r) {
          return r.commit(undefined, _this28);
        });

        if ((0, _isNumber["default"])(total)) {
          this.totalCount = total;
        }
      } else if (autoQueryAfterSubmit) {
        // 无回写数据时自动进行查询
        (0, _utils.warning)(false, "The primary key which generated by database is not exists in each created records,\nbecause of no data `".concat(this.dataKey, "` from the response by `submit` or `delete` method.\nThen the query method will be auto invoke."));
        this.query();
      }

      return this;
    }
    /**
     * 数据集头行级联绑定
     * @param ds 头数据集
     * @param name 头数据集字段名
     */

  }, {
    key: "bind",
    value: function bind(ds, name) {
      if (!name) {
        (0, _utils.warning)(false, 'DataSet: cascade binding need a name');
        return;
      }

      if (ds.children[name]) {
        (0, _utils.warning)(false, "DataSet: duplicate cascade binding of name<".concat(name, ">"));
        return;
      }

      ds.children[name] = this;
      this.parent = ds;
      this.parentName = name;
      var current = ds.current;

      if (current) {
        ds.syncChild(this, current, name);
      } else if (this.length) {
        this.loadData([]);
      }
    }
    /**
     * 设置查询的参数.
     * @param {string} para 参数名.
     * @param {any} value 参数值.
     */

  }, {
    key: "setQueryParameter",
    value: function setQueryParameter(para, value) {
      var queryParameter = this.getState(QUERY_PARAMETER);

      if (queryParameter) {
        if ((0, _isNil["default"])(value)) {
          queryParameter["delete"](para);
        } else {
          queryParameter.set(para, value);
        }
      }
    }
    /**
     * 获取查询的参数.
     * @param {string} para 参数名.
     * @return {any} 参数值.
     */

  }, {
    key: "getQueryParameter",
    value: function getQueryParameter(para) {
      var queryParameter = this.getState(QUERY_PARAMETER);

      if (queryParameter) {
        return queryParameter.get(para);
      }
    }
  }, {
    key: "appendData",
    value: function appendData() {
      var allData = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
      var parent = arguments.length > 1 ? arguments[1] : undefined;
      var sortedData = (0, _utils2.sortData)(allData, this);
      this.fireEvent(_enum.DataSetEvents.beforeAppend, {
        dataSet: this,
        data: sortedData
      });
      (0, _utils2.appendRecords)(this, this.processData(sortedData, undefined, parent), parent);
      this.fireEvent(_enum.DataSetEvents.append, {
        dataSet: this
      });
      this.releaseCachedRecords();
      return this;
    }
  }, {
    key: "loadData",
    value: function loadData() {
      var allData = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
      var total = arguments.length > 1 ? arguments[1] : undefined;
      var cache = arguments.length > 2 ? arguments[2] : undefined;
      this.performance.timing.loadStart = Date.now();
      var cacheRecords = this.getConfig('cacheRecords');
      var cacheSelection = this.props.cacheSelection;

      if (cacheRecords) {
        if (cache) {
          this.storeRecords();
        } else if (cache !== false && cacheSelection) {
          this.storeRecords(false);
        }
      } else {
        this.storeRecords(cache === true);
      }

      var paging = this.paging,
          pageSize = this.pageSize,
          _this$props8 = this.props,
          autoLocateFirst = _this$props8.autoLocateFirst,
          idField = _this$props8.idField,
          parentField = _this$props8.parentField,
          childrenField = _this$props8.childrenField,
          strictPageSize = _this$props8.strictPageSize;

      if (strictPageSize) {
        switch (paging) {
          case true:
            allData = allData.slice(0, pageSize);
            break;

          case 'server':
            allData = idField && parentField && !childrenField ? (0, _utils2.sliceTree)(idField, parentField, allData, pageSize) : allData.slice(0, pageSize);
            break;

          default:
            allData = allData.slice();
            break;
        }
      } else {
        allData = allData.slice();
      }

      var sortedData = (0, _utils2.sortData)(allData, this);
      this.fireEvent(_enum.DataSetEvents.beforeLoad, {
        dataSet: this,
        data: sortedData
      });
      var originalData = this.processData(sortedData, _enum.RecordStatus.sync);
      this.originalData = originalData;
      this.records = originalData;

      if (total !== undefined && (paging === true || paging === 'server')) {
        this.totalCount = total;
      }

      if (cacheRecords) {
        if (cache) {
          this.releaseCachedRecords();
        } else if (cache === false || !cacheSelection) {
          this.clearCachedRecords();
        } else {
          this.releaseCachedSelected();
        }
      } else {
        this.releaseCachedRecords(cache);

        if (!cache) {
          this.clearCachedModified();
        }
      }

      var nextRecord = autoLocateFirst && (idField && parentField || childrenField ? this.getFromTree(0) : this.get(0));

      if (nextRecord) {
        nextRecord.isCurrent = true;
      }

      this.performance.timing.loadEnd = Date.now();
      this.fireEvent(_enum.DataSetEvents.indexChange, {
        dataSet: this,
        record: nextRecord
      });
      this.fireEvent(_enum.DataSetEvents.load, {
        dataSet: this
      });
      return this;
    }
  }, {
    key: "processData",
    value: function processData(allData) {
      var status = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _enum.RecordStatus.sync;
      var parent = arguments.length > 2 ? arguments[2] : undefined;
      var _this$props9 = this.props,
          childrenField = _this$props9.childrenField,
          parentField = _this$props9.parentField,
          idField = _this$props9.idField;

      if (parentField && idField && !childrenField) {
        return processTreeData(this, allData, status, parentField, idField);
      }

      if (childrenField) {
        var all = [];
        processNormalData(this, allData, status, childrenField, parent, all);
        return all;
      }

      return processNormalData(this, allData, status);
    }
  }, {
    key: "deleteRecord",
    value: function deleteRecord(record, forceRemove) {
      if (record) {
        record.isSelected = false;
        record.isCurrent = false;
        var selected = this.selected,
            records = this.records,
            cachedRecords = this.cachedRecords;
        var selectedIndex = selected.indexOf(record);

        if (selectedIndex !== -1) {
          selected.splice(selectedIndex, 1);
        }

        if (record.isNew || forceRemove) {
          if (record.isCached) {
            var index = cachedRecords.indexOf(record);

            if (index !== -1) {
              cachedRecords.splice(index, 1);
            }
          } else {
            var _index = records.indexOf(record);

            if (_index !== -1) {
              records.splice(_index, 1);
            }
          }

          var parent = record.parent;

          if (parent) {
            var children = parent.children;

            if (children) {
              var _index2 = children.indexOf(record);

              if (_index2 !== -1) {
                children.splice(_index2, 1);
              }

              if (!children.length) {
                parent.children = undefined;
              }
            }

            record.parent = undefined;
          }
        } else if (!record.isRemoved) {
          record.status = _enum.RecordStatus["delete"];
        }
      }

      return record;
    } // 查询在所有页面的对应位置

  }, {
    key: "findInAllPage",
    value: function findInAllPage(index) {
      var paging = this.paging;
      var indexRecord;

      if (paging === true) {
        indexRecord = this.data[this.getIndexInCurrentPage(index)];
      } else if (paging === 'server') {
        indexRecord = this.treeData[this.getIndexInCurrentPage(index)];
      } else {
        indexRecord = this.data[index];
      }

      return indexRecord;
    }
  }, {
    key: "getIndexInCurrentPage",
    value: function getIndexInCurrentPage() {
      var index = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.currentIndex;
      var currentPage = this.currentPage,
          pageSize = this.pageSize;
      return index - (currentPage - 1) * pageSize;
    }
  }, {
    key: "transferRecords",
    value: function transferRecords(data) {
      var _this29 = this;

      return data.map(function (record) {
        var _record = record,
            dataSet = _record.dataSet;

        if (dataSet === _this29) {
          var records = _this29.records;
          var index = records.indexOf(record);

          if (index !== -1) {
            records.splice(index, 1);
          }

          return record;
        }

        dataSet.remove(record);
        record = new _Record["default"](record.toData(), _this29);
        record.status = _enum.RecordStatus.add;
        return record;
      });
    }
  }, {
    key: "initChildren",
    value: function initChildren(children) {
      var _this30 = this;

      if ((0, _isArray["default"])(children)) {
        children.forEach(function (childDs) {
          if (childDs instanceof DataSet) {
            var name = childDs.name;

            if (name) {
              childDs.bind(_this30, name);
            } else {
              (0, _utils.warning)(false, 'cascade DataSet need a name');
            }
          }
        });
      } else {
        Object.keys(children).forEach(function (childName) {
          var child = children[childName];

          if (child instanceof DataSet) {
            child.bind(_this30, childName);
          } else {
            (0, _utils.warning)(false, "cascade child<".concat(childName, "> must be instance of DataSet."));
          }
        });
      }
    }
  }, {
    key: "initQueryDataSet",
    value: function initQueryDataSet(queryDataSet, queryFields) {
      if (queryFields) {
        queryDataSet = new DataSet({
          fields: queryFields
        }, this.context);
      }

      if (queryDataSet) {
        this.queryDataSet = queryDataSet;
      }
    }
  }, {
    key: "initEvents",
    value: function initEvents(events) {
      var _this31 = this;

      Object.keys(events).forEach(function (event) {
        return _this31.addEventListener(event, events[event]);
      });
    }
  }, {
    key: "loadDataFromResponse",
    value: function loadDataFromResponse(resp, cache) {
      if (resp) {
        var dataKey = this.dataKey,
            totalKey = this.totalKey;
        var data = (0, _utils2.generateResponseData)(resp, dataKey);
        var total = ObjectChainValue.get(resp, totalKey);
        this.loadData(data, total, cache);
      }

      return this;
    }
  }, {
    key: "initCombineSort",
    value: function initCombineSort() {
      var combineSort = this.props.combineSort;
      this.combineSort = combineSort;
      var orderFieldNames = new Map();
      this.fields.forEach(function (field) {
        if (field.order && field.name) {
          orderFieldNames.set(field.name, field.order);
        }
      });

      if (orderFieldNames && orderFieldNames.size > 0) {
        this.combineSortFieldNames = _mobx.observable.map(orderFieldNames);
      }
    }
  }, {
    key: "appendDataFromResponse",
    value: function appendDataFromResponse(resp, parent) {
      if (resp) {
        var dataKey = this.dataKey;
        var data = (0, _utils2.generateResponseData)(resp, dataKey);
        this.appendData(data, parent);
      }

      return this;
    } // private groupData(allData: object[]): object[] {
    //   return this.getGroups().reverse()
    //     .reduce((arr, name) => arr.sort(
    //       (item1, item2) => String(item1[name]).localeCompare(String(item2[name])),
    //     ), allData);
    // }

  }, {
    key: "write",
    value: function () {
      var _write = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee11(records, onlyDelete) {
        var _this32 = this;

        var _prepareSubmitData, _prepareSubmitData2, created, updated, destroyed, axiosConfigs, submitData, submitEventResult, result;

        return _regenerator["default"].wrap(function _callee11$(_context11) {
          while (1) {
            switch (_context11.prev = _context11.next) {
              case 0:
                if (!records.length) {
                  _context11.next = 25;
                  break;
                }

                _prepareSubmitData = (0, _utils2.prepareSubmitData)(records, this.dataToJSON), _prepareSubmitData2 = (0, _slicedToArray2["default"])(_prepareSubmitData, 3), created = _prepareSubmitData2[0], updated = _prepareSubmitData2[1], destroyed = _prepareSubmitData2[2];
                axiosConfigs = [];
                submitData = [].concat((0, _toConsumableArray2["default"])((0, _utils2.prepareForSubmit)('create', created, axiosConfigs, this)), (0, _toConsumableArray2["default"])((0, _utils2.prepareForSubmit)('update', updated, axiosConfigs, this)), (0, _toConsumableArray2["default"])((0, _utils2.prepareForSubmit)('destroy', destroyed, axiosConfigs, this)));
                (0, _utils2.prepareForSubmit)('submit', submitData, axiosConfigs, this);

                if (!axiosConfigs.length) {
                  _context11.next = 25;
                  break;
                }

                _context11.prev = 6;
                this.changeSubmitStatus(_enum.DataSetStatus.submitting);
                _context11.next = 10;
                return this.fireEvent(_enum.DataSetEvents.submit, {
                  dataSet: this,
                  data: [].concat((0, _toConsumableArray2["default"])(created), (0, _toConsumableArray2["default"])(updated), (0, _toConsumableArray2["default"])(destroyed))
                });

              case 10:
                submitEventResult = _context11.sent;

                if (!submitEventResult) {
                  _context11.next = 16;
                  break;
                }

                _context11.next = 14;
                return _axios["default"].all(axiosConfigs.map(function (config) {
                  return _this32.axios(config);
                }));

              case 14:
                result = _context11.sent;
                return _context11.abrupt("return", this.handleSubmitSuccess(result, onlyDelete));

              case 16:
                _context11.next = 22;
                break;

              case 18:
                _context11.prev = 18;
                _context11.t0 = _context11["catch"](6);
                this.handleSubmitFail(_context11.t0);
                throw new _DataSetRequestError["default"](_context11.t0);

              case 22:
                _context11.prev = 22;
                this.changeSubmitStatus(_enum.DataSetStatus.ready);
                return _context11.finish(22);

              case 25:
              case "end":
                return _context11.stop();
            }
          }
        }, _callee11, this, [[6, 18, 22, 25]]);
      }));

      function write(_x20, _x21) {
        return _write.apply(this, arguments);
      }

      return write;
    }()
  }, {
    key: "read",
    value: function () {
      var _read = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee12() {
        var _this33 = this;

        var page,
            params,
            more,
            paging,
            data,
            newConfig,
            queryEventResult,
            result,
            _args12 = arguments;
        return _regenerator["default"].wrap(function _callee12$(_context12) {
          while (1) {
            switch (_context12.prev = _context12.next) {
              case 0:
                page = _args12.length > 0 && _args12[0] !== undefined ? _args12[0] : 1;
                params = _args12.length > 1 ? _args12[1] : undefined;
                more = _args12.length > 2 ? _args12[2] : undefined;
                paging = _args12.length > 3 ? _args12[3] : undefined;

                if (!this.checkReadable(this.parent)) {
                  _context12.next = 33;
                  break;
                }

                _context12.prev = 5;

                if (!more) {
                  this.changeStatus(_enum.DataSetStatus.loading);
                }

                _context12.next = 9;
                return this.generateQueryParameter(params);

              case 9:
                data = _context12.sent;
                newConfig = (0, _utils2.axiosConfigAdapter)('read', this, data, this.generateQueryString(page, undefined, undefined, paging));

                if (!newConfig.url) {
                  _context12.next = 24;
                  break;
                }

                _context12.next = 14;
                return this.fireEvent(_enum.DataSetEvents.query, {
                  dataSet: this,
                  params: newConfig.params,
                  data: newConfig.data
                });

              case 14:
                queryEventResult = _context12.sent;

                if (!queryEventResult) {
                  _context12.next = 24;
                  break;
                }

                this.performance.timing.fetchStart = Date.now();
                this.performance.url = newConfig.url;
                _context12.next = 20;
                return this.axios((0, _utils2.fixAxiosConfig)(newConfig));

              case 20:
                result = _context12.sent;
                this.performance.timing.fetchEnd = Date.now();
                (0, _mobx.runInAction)(function () {
                  if (page >= 0) {
                    _this33.currentPage = page;
                  }

                  var countKey = _this33.countKey;
                  var needCount = ObjectChainValue.get(result, countKey) === 'Y';

                  if (needCount) {
                    _this33.counting = _this33.count(page, params).then((0, _mobx.action)(function (countResult) {
                      var totalKey = _this33.totalKey;
                      var total = ObjectChainValue.get(countResult, totalKey);

                      if (total !== undefined) {
                        _this33.totalCount = total;
                      }

                      return total;
                    }))["finally"]((0, _mobx.action)(function () {
                      _this33.counting = undefined;
                    }));
                  }
                });
                return _context12.abrupt("return", this.handleLoadSuccess(result));

              case 24:
                _context12.next = 30;
                break;

              case 26:
                _context12.prev = 26;
                _context12.t0 = _context12["catch"](5);
                this.handleLoadFail(_context12.t0);
                throw new _DataSetRequestError["default"](_context12.t0);

              case 30:
                _context12.prev = 30;

                if (!more) {
                  this.changeStatus(_enum.DataSetStatus.ready);
                }

                return _context12.finish(30);

              case 33:
              case "end":
                return _context12.stop();
            }
          }
        }, _callee12, this, [[5, 26, 30, 33]]);
      }));

      function read() {
        return _read.apply(this, arguments);
      }

      return read;
    }()
  }, {
    key: "count",
    value: function () {
      var _count = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee13() {
        var page,
            params,
            data,
            newConfig,
            countEventResult,
            _args13 = arguments;
        return _regenerator["default"].wrap(function _callee13$(_context13) {
          while (1) {
            switch (_context13.prev = _context13.next) {
              case 0:
                page = _args13.length > 0 && _args13[0] !== undefined ? _args13[0] : 1;
                params = _args13.length > 1 ? _args13[1] : undefined;
                _context13.next = 4;
                return this.generateQueryParameter(params);

              case 4:
                data = _context13.sent;
                newConfig = (0, _utils2.axiosConfigAdapter)('read', this, data, this.generateQueryString(page, undefined, true));

                if (!newConfig.url) {
                  _context13.next = 12;
                  break;
                }

                _context13.next = 9;
                return this.fireEvent(_enum.DataSetEvents.count, {
                  dataSet: this,
                  params: newConfig.params,
                  data: newConfig.data
                });

              case 9:
                countEventResult = _context13.sent;

                if (!countEventResult) {
                  _context13.next = 12;
                  break;
                }

                return _context13.abrupt("return", this.axios((0, _utils2.fixAxiosConfig)(newConfig)));

              case 12:
              case "end":
                return _context13.stop();
            }
          }
        }, _callee13, this);
      }));

      function count() {
        return _count.apply(this, arguments);
      }

      return count;
    }()
  }, {
    key: "storeRecords",
    value: function storeRecords() {
      var cacheModified = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;
      var cacheSelectionKeys = this.cacheSelectionKeys,
          cacheModifiedKeys = this.cacheModifiedKeys,
          records = this.records,
          cachedRecords = this.cachedRecords,
          isAllPageSelection = this.isAllPageSelection;

      if (cacheSelectionKeys || cacheModified && cacheModifiedKeys) {
        this.cachedRecords = cachedRecords.concat(records).reduce(function (list, record) {
          if (cacheSelectionKeys && (isAllPageSelection ? !record.isSelected : record.isSelected) || cacheModified && cacheModifiedKeys && (0, _utils2.isDirtyRecord)(record)) {
            record.isCurrent = false;
            record.isCached = true;
            list.push(record);
          }

          return list;
        }, []);
      }
    }
  }, {
    key: "releaseCachedRecords",
    value: function releaseCachedRecords(cache) {
      var cacheSelectionKeys = this.cacheSelectionKeys,
          cacheModifiedKeys = this.cacheModifiedKeys;

      if (cacheModifiedKeys || cacheSelectionKeys) {
        var cacheRecords = this.getConfig('cacheRecords');
        var cachedKeys = (0, _union["default"])(cacheModifiedKeys, cacheSelectionKeys);
        var records = this.records;
        var cachedRecords = this.cachedRecords.slice();
        this.records = records.map(function (record) {
          var index = cachedRecords.findIndex(function (cached) {
            return cachedKeys.every(function (key) {
              return record.get(key) === cached.get(key);
            });
          });

          if (index !== -1) {
            var cached = cachedRecords.splice(index, 1)[0];

            if (cacheSelectionKeys) {
              record.isSelected = cached.isSelected;

              if (!(0, _isNil["default"])(cached.selectedTimestamp) && cached.isSelected) {
                record.selectedTimestamp = cached.selectedTimestamp;
              }
            }

            if ((cache || cacheRecords) && cacheModifiedKeys || cache && !cacheRecords && cacheSelectionKeys) {
              var dirtyData = cached.dirtyData;

              if (dirtyData) {
                dirtyData.forEach(function (_, key) {
                  record.set(key, cached.get(key));
                });
              }
            }
          }

          return record;
        });
        this.cachedRecords = cachedRecords;
      }
    }
  }, {
    key: "releaseCachedModified",
    value: function releaseCachedModified() {
      this.releaseCachedRecords();
    }
  }, {
    key: "releaseCachedSelected",
    value: function releaseCachedSelected() {
      this.releaseCachedRecords();
    }
  }, {
    key: "changeStatus",
    value: function changeStatus(status) {
      this.status = status;
    }
  }, {
    key: "changeSubmitStatus",
    value: function changeSubmitStatus(status) {
      this.status = status;
      Object.values(this.children).forEach(function (ds) {
        if (ds instanceof DataSet) {
          ds.changeSubmitStatus(status);
        }
      });
    }
  }, {
    key: "handleCascade",
    value: function handleCascade(_ref5) {
      var dataSet = _ref5.dataSet,
          record = _ref5.record,
          previous = _ref5.previous;

      if (dataSet.hasChildren) {
        dataSet.syncChildren(record, previous);
      }
    }
  }, {
    key: "handleLoadSuccess",
    value: function handleLoadSuccess(resp) {
      var _this$feedback$loadSu = this.feedback.loadSuccess,
          loadSuccess = _this$feedback$loadSu === void 0 ? _FeedBack["default"].loadSuccess : _this$feedback$loadSu;
      loadSuccess(resp);
      return resp;
    }
  }, {
    key: "handleLoadFail",
    value: function handleLoadFail(e) {
      var _this$feedback$loadFa = this.feedback.loadFailed,
          loadFailed = _this$feedback$loadFa === void 0 ? _FeedBack["default"].loadFailed : _this$feedback$loadFa;
      this.fireEvent(_enum.DataSetEvents.loadFailed, {
        dataSet: this
      });
      loadFailed(e);
    }
  }, {
    key: "handleSubmitSuccess",
    value: function handleSubmitSuccess(resp, onlyDelete) {
      var dataKey = this.dataKey,
          totalKey = this.totalKey;
      var _this$feedback$submit = this.feedback.submitSuccess,
          submitSuccess = _this$feedback$submit === void 0 ? _FeedBack["default"].submitSuccess : _this$feedback$submit;
      var data = [];
      var total;
      resp.forEach(function (item) {
        data.push.apply(data, (0, _toConsumableArray2["default"])((0, _utils2.generateResponseData)(item, dataKey)));

        if (totalKey && (0, _isObject["default"])(item)) {
          var myTotal = ObjectChainValue.get(item, totalKey);

          if (!(0, _isNil["default"])(myTotal)) {
            total = myTotal;
          }
        }
      });
      var result = dataKey ? {
        success: true
      } : data;

      if (dataKey) {
        ObjectChainValue.set(result, dataKey, data);

        if (totalKey) {
          ObjectChainValue.set(result, totalKey, total);
        }
      }

      this.fireEvent(_enum.DataSetEvents.submitSuccess, {
        dataSet: this,
        data: result
      }); // 针对 204 的情况进行特殊处理
      // 不然在设置了 primaryKey 的情况 下,在先新增一条再使用delete的情况下，会将204这个请求内容填入到record中

      if (!(data[0] && data[0].status === 204 && data[0].statusText === 'No Content')) {
        this.commitData(data, total, onlyDelete);
      } else {
        this.commitData([], total);
      }

      submitSuccess(result);
      this.clearCachedModified();
      return result;
    }
  }, {
    key: "handleSubmitFail",
    value: function handleSubmitFail(e) {
      var _this34 = this;

      var current = this.current;
      var _this$feedback$submit2 = this.feedback.submitFailed,
          submitFailed = _this$feedback$submit2 === void 0 ? _FeedBack["default"].submitFailed : _this$feedback$submit2;
      this.fireEvent(_enum.DataSetEvents.submitFailed, {
        dataSet: this
      });
      submitFailed(e);

      if (this.props.autoLocateAfterRemove && current && this.destroyed.length) {
        current.isCurrent = false;
      }

      this.destroyed.forEach(function (record, index) {
        record.reset();
        record.isSelected = true;

        if (_this34.props.autoLocateAfterRemove && index === 0) {
          record.isCurrent = true;
        }
      });
    }
  }, {
    key: "syncChildren",
    value: function syncChildren(current, previous) {
      var _this35 = this;

      var children = this.children;
      var keys = Object.keys(children);
      var remoteKeys = [];
      keys.forEach(function (childName) {
        var ds = children[childName];

        if (previous && ds.status === _enum.DataSetStatus.ready) {
          var dataSetSnapshot = previous.dataSetSnapshot;

          if (dataSetSnapshot && dataSetSnapshot[childName]) {
            dataSetSnapshot[childName] = ds.snapshot();
            ds.current = undefined;
          }
        }

        if (current) {
          var _dataSetSnapshot = current.dataSetSnapshot;
          var snapshot = _dataSetSnapshot && _dataSetSnapshot[childName];

          if (snapshot instanceof _DataSetSnapshot["default"]) {
            ds.restore(snapshot);
          } else if (!_this35.syncChild(ds, current, childName, true)) {
            ds.loadData([]);
            remoteKeys.push(childName);
          }
        } else {
          ds.loadData([]);
        }
      });

      if (current && remoteKeys.length) {
        this.syncChildrenRemote(remoteKeys, current);
      }
    }
  }, {
    key: "syncChild",
    value: function syncChild(ds, currentRecord, childName, onlyClient) {
      var _this36 = this;

      var cascadeRecordsMap = currentRecord.cascadeRecordsMap;
      var cascadeRecords = cascadeRecordsMap && cascadeRecordsMap[childName];
      var childRecords = cascadeRecords || currentRecord.get(childName);

      if (currentRecord.isNew || (0, _mobx.isArrayLike)(childRecords)) {
        if (cascadeRecords && cascadeRecordsMap) {
          delete cascadeRecordsMap[childName];
        }

        ds.clearCachedRecords();
        ds.loadData(childRecords ? childRecords.slice() : []);

        if (currentRecord.isNew) {
          if (ds.length) {
            ds.forEach(function (record) {
              return record.status = _enum.RecordStatus.add;
            });
          } else if (ds.props.autoCreate) {
            ds.create();
          }
        }

        var dataSetSnapshot = (0, _utils2.getIf)(currentRecord, 'dataSetSnapshot', {});
        dataSetSnapshot[childName] = ds.snapshot();
        return true;
      }

      if (!onlyClient) {
        var oldSnapshot = ds.snapshot();
        ds.read(1).then(function (resp) {
          var current = _this36.current;

          if (current !== currentRecord) {
            ds = new DataSet(undefined, _this36.context).restore(oldSnapshot);
          }

          ds.clearCachedRecords();
          var dataSetSnapshot = (0, _utils2.getIf)(currentRecord, 'dataSetSnapshot', {});
          dataSetSnapshot[childName] = ds.loadDataFromResponse(resp).snapshot();
        });
      }

      return false;
    }
  }, {
    key: "checkReadable",
    value: function checkReadable(parent) {
      if (parent) {
        var current = parent.current;

        if (!current || current.isNew) {
          return false;
        }
      }

      return true;
    }
    /**
     * page相关请求设置
     * @param page 在那个页面, 小于0时不分页
     * @param pageSizeInner 页面大小
     * @param onlyCount 只做计数
     */

  }, {
    key: "generatePageQueryString",
    value: function generatePageQueryString(page, pageSizeInner, onlyCount, usePaging) {
      var params = {};

      if (page >= 0) {
        var paging = this.paging,
            pageSize = this.pageSize,
            autoCount = this.props.autoCount;
        var defaultCount = this.getConfig('autoCount');

        if ((0, _isNumber["default"])(pageSizeInner)) {
          params.page = page;
          params.pagesize = pageSizeInner;
        } else if (paging === true || paging === 'server') {
          params.page = page;
          params.pagesize = pageSize;
        }

        if (autoCount === false) {
          params.count = 'N';
        } else if (autoCount === true) {
          params.count = 'Y';
        } else if (defaultCount === false) {
          params.defaultCount = 'N';
        } else if (defaultCount === true) {
          params.defaultCount = 'Y';
        }

        if (onlyCount) {
          params.onlyCount = 'Y';
        } else if (usePaging && (autoCount === false || defaultCount === false)) {
          var realTotalCount = this.realTotalCount;

          if (realTotalCount !== undefined && isFinite(realTotalCount)) {
            params.totalCount = realTotalCount;
          }
        }
      }

      return params;
    }
  }, {
    key: "generateOrderQueryString",
    value: function generateOrderQueryString() {
      var _this37 = this;

      var combineSort = this.props.combineSort;
      var orderFields = (0, _utils2.getOrderFields)(this);

      if (combineSort) {
        return orderFields.map(function (orderField) {
          var sortname = orderField.name;

          if (orderField.type === _enum.FieldType.object) {
            var bindField = (0, _utils2.findBindFieldBy)(orderField, _this37.fields, 'valueField');

            if (bindField) {
              sortname = bindField.name;
            }
          }

          return "".concat(sortname, ",").concat(orderField.order);
        });
      }

      if (orderFields.length) {
        var orderField = orderFields[0];
        var param = {
          sortname: orderField.name,
          sortorder: orderField.order
        };

        if (orderField.type === _enum.FieldType.object) {
          var bindField = (0, _utils2.findBindFieldBy)(orderField, this.fields, 'valueField');

          if (bindField) {
            param.sortname = bindField.name;
          }
        }

        return param;
      }

      return {};
    }
    /**
     * 返回configure 配置的值
     * @param page 在那个页面, 小于0时不分页
     * @param pageSizeInner 页面大小
     * @param onlyCount 只做计数
     */

  }, {
    key: "generateQueryString",
    value: function generateQueryString(page, pageSizeInner, onlyCount, paging) {
      var combineSort = this.props.combineSort;
      var order = this.generateOrderQueryString();
      var pageQuery = this.generatePageQueryString(page, pageSizeInner, onlyCount, paging);
      var generatePageQuery = this.getConfig('generatePageQuery');
      var sortParams = combineSort && order.length ? {
        sort: order
      } : {
        sortName: order.sortname,
        sortOrder: order.sortorder
      };

      if (typeof generatePageQuery === 'function') {
        return generatePageQuery((0, _objectSpread3["default"])((0, _objectSpread3["default"])({}, sortParams), {}, {
          pageSize: pageQuery.pagesize,
          page: pageQuery.page,
          count: pageQuery.count,
          defaultCount: pageQuery.defaultCount,
          totalCount: pageQuery.totalCount,
          onlyCount: pageQuery.onlyCount
        }));
      }

      return (0, _objectSpread3["default"])((0, _objectSpread3["default"])({}, pageQuery), sortParams);
    }
  }, {
    key: "getParentParams",
    value: function getParentParams() {
      var parent = this.parent,
          cascadeParams = this.props.cascadeParams;

      if (parent) {
        var primaryKey = parent.props.primaryKey,
            current = parent.current;

        if (current) {
          return cascadeParams(current, primaryKey);
        }
      }

      return {};
    }
  }, {
    key: "generateQueryParameter",
    value: function () {
      var _generateQueryParameter = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee14(params) {
        var queryDataSet, validateBeforeQuery, parentParams, validationMessage, data, current;
        return _regenerator["default"].wrap(function _callee14$(_context14) {
          while (1) {
            switch (_context14.prev = _context14.next) {
              case 0:
                queryDataSet = this.queryDataSet, validateBeforeQuery = this.props.validateBeforeQuery;
                parentParams = this.getParentParams();

                if (!queryDataSet) {
                  _context14.next = 13;
                  break;
                }

                _context14.next = 5;
                return queryDataSet.ready();

              case 5:
                _context14.t0 = validateBeforeQuery && queryDataSet && queryDataSet.current;

                if (!_context14.t0) {
                  _context14.next = 10;
                  break;
                }

                _context14.next = 9;
                return queryDataSet.current.validate();

              case 9:
                _context14.t0 = !_context14.sent;

              case 10:
                if (!_context14.t0) {
                  _context14.next = 13;
                  break;
                }

                validationMessage = queryDataSet.current.getValidationErrors().map(function (error) {
                  return error.errors[0].validationMessage;
                }).join(' ');
                throw new Error("".concat((0, _localeContext.$l)('DataSet', 'invalid_query_dataset'), ": ").concat(validationMessage));

              case 13:
                data = {};

                if (queryDataSet) {
                  current = queryDataSet.current;

                  if (current) {
                    data = (0, _omit["default"])(current.toData(true), ['__dirty']);
                  }
                }

                data = (0, _objectSpread3["default"])((0, _objectSpread3["default"])((0, _objectSpread3["default"])((0, _objectSpread3["default"])({}, data), this.queryParameter), parentParams), params);
                return _context14.abrupt("return", Object.keys(data).reduce(function (p, key) {
                  var value = data[key];

                  if (!(0, _utils.isEmpty)(value)) {
                    p[key] = value;
                  }

                  return p;
                }, {}));

              case 17:
              case "end":
                return _context14.stop();
            }
          }
        }, _callee14, this);
      }));

      function generateQueryParameter(_x22) {
        return _generateQueryParameter.apply(this, arguments);
      }

      return generateQueryParameter;
    }()
  }, {
    key: "getConfig",
    value: function getConfig(key) {
      var context = this.context;

      if (context) {
        return context.getConfig(key);
      }

      return (0, _configure.getConfig)(key);
    }
  }]);
  return DataSet;
}(_eventManager["default"]);

exports["default"] = DataSet;
DataSet.defaultProps = {
  autoCreate: false,
  autoQuery: false,
  autoQueryAfterSubmit: true,
  autoLocateFirst: true,
  autoLocateAfterCreate: true,
  autoLocateAfterRemove: true,
  validateBeforeQuery: true,
  selection: _enum.DataSetSelection.multiple,
  modifiedCheck: true,
  pageSize: 10,
  paging: true,
  strictPageSize: true,
  combineSort: false,
  dataToJSON: _enum.DataToJSON.dirty,
  cascadeParams: function cascadeParams(parent, primaryKey) {
    if (primaryKey) {
      return (0, _defineProperty2["default"])({}, primaryKey, parent.get(primaryKey));
    }

    return (0, _omit["default"])(parent.toData(), ['__dirty']);
  }
};
(0, _tslib.__decorate)([_mobx.computed], DataSet.prototype, "queryParameter", null);
(0, _tslib.__decorate)([_mobx.observable], DataSet.prototype, "lookupCaches", void 0);
(0, _tslib.__decorate)([_mobx.observable], DataSet.prototype, "selectionStrategy", void 0);
(0, _tslib.__decorate)([_mobx.observable], DataSet.prototype, "parent", void 0);
(0, _tslib.__decorate)([_mobx.observable], DataSet.prototype, "name", void 0);
(0, _tslib.__decorate)([_mobx.observable], DataSet.prototype, "parentName", void 0);
(0, _tslib.__decorate)([_mobx.observable], DataSet.prototype, "records", void 0);
(0, _tslib.__decorate)([_mobx.observable], DataSet.prototype, "fields", void 0);
(0, _tslib.__decorate)([_mobx.observable], DataSet.prototype, "props", void 0);
(0, _tslib.__decorate)([_mobx.observable], DataSet.prototype, "pageSize", void 0);
(0, _tslib.__decorate)([_mobx.observable], DataSet.prototype, "combineSort", void 0);
(0, _tslib.__decorate)([_mobx.observable], DataSet.prototype, "combineSortFieldNames", void 0);
(0, _tslib.__decorate)([_mobx.computed], DataSet.prototype, "totalCount", null);
(0, _tslib.__decorate)([_mobx.observable], DataSet.prototype, "status", void 0);
(0, _tslib.__decorate)([_mobx.observable], DataSet.prototype, "counting", void 0);
(0, _tslib.__decorate)([_mobx.observable], DataSet.prototype, "exportStatus", void 0);
(0, _tslib.__decorate)([_mobx.observable], DataSet.prototype, "exportProgress", void 0);
(0, _tslib.__decorate)([_mobx.observable], DataSet.prototype, "currentPage", void 0);
(0, _tslib.__decorate)([_mobx.observable], DataSet.prototype, "selection", void 0);
(0, _tslib.__decorate)([_mobx.computed], DataSet.prototype, "cachedSelected", null);
(0, _tslib.__decorate)([_mobx.computed], DataSet.prototype, "cachedUnSelected", null);
(0, _tslib.__decorate)([_mobx.computed], DataSet.prototype, "cachedCreated", null);
(0, _tslib.__decorate)([_mobx.computed], DataSet.prototype, "cachedUpdated", null);
(0, _tslib.__decorate)([_mobx.computed], DataSet.prototype, "cachedDestroyed", null);
(0, _tslib.__decorate)([_mobx.computed], DataSet.prototype, "cachedModified", null);
(0, _tslib.__decorate)([_mobx.observable], DataSet.prototype, "dataToJSON", void 0);
(0, _tslib.__decorate)([_mobx.observable], DataSet.prototype, "state", void 0);
(0, _tslib.__decorate)([_mobx.observable], DataSet.prototype, "validationSelfErrors", void 0);
(0, _tslib.__decorate)([_mobx.computed], DataSet.prototype, "cascadeRecords", null);
(0, _tslib.__decorate)([_mobx.computed], DataSet.prototype, "transport", null);
(0, _tslib.__decorate)([_mobx.computed], DataSet.prototype, "data", null);
(0, _tslib.__decorate)([_mobx.computed], DataSet.prototype, "dirtyRecords", null);
(0, _tslib.__decorate)([_mobx.computed], DataSet.prototype, "selected", null);
(0, _tslib.__decorate)([_mobx.computed], DataSet.prototype, "unSelected", null);
(0, _tslib.__decorate)([_mobx.computed], DataSet.prototype, "currentSelected", null);
(0, _tslib.__decorate)([_mobx.computed], DataSet.prototype, "currentUnSelected", null);
(0, _tslib.__decorate)([_mobx.computed], DataSet.prototype, "treeSelected", null);
(0, _tslib.__decorate)([_mobx.computed], DataSet.prototype, "currentIndex", null);
(0, _tslib.__decorate)([_mobx.computed], DataSet.prototype, "treeRecords", null);
(0, _tslib.__decorate)([_mobx.computed], DataSet.prototype, "treeData", null);
(0, _tslib.__decorate)([_mobx.computed], DataSet.prototype, "groups", null);
(0, _tslib.__decorate)([_mobx.computed], DataSet.prototype, "groupedRecords", null);
(0, _tslib.__decorate)([_mobx.computed], DataSet.prototype, "groupedTreeRecords", null);
(0, _tslib.__decorate)([_mobx.computed], DataSet.prototype, "current", null);
(0, _tslib.__decorate)([_mobx.computed], DataSet.prototype, "uniqueKeys", null);
(0, _tslib.__decorate)([_mobx.observable], DataSet.prototype, "cachedRecords", void 0);
(0, _tslib.__decorate)([_mobx.computed], DataSet.prototype, "all", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "setState", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "restore", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "setAllPageSelection", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "doClientExport", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "reset", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "create", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "remove", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "removeAll", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "deleteAll", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "push", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "unshift", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "pop", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "shift", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "splice", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "move", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "reverse", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "sort", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "select", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "unSelect", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "selectAll", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "unSelectAll", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "batchSelect", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "batchUnSelect", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "treeSelect", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "treeUnSelect", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "clearCachedRecords", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "clearCachedSelected", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "setCachedSelected", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "setCachedCreated", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "setCachedUpdated", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "setCachedDestroyed", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "setCachedModified", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "clearValidationError", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "validateSelf", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "addField", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "commitData", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "bind", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "setQueryParameter", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "appendData", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "loadData", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "processData", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "initCombineSort", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "storeRecords", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "releaseCachedRecords", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "changeStatus", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "changeSubmitStatus", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "handleSubmitSuccess", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "handleSubmitFail", null);
(0, _tslib.__decorate)([_mobx.action], DataSet.prototype, "syncChild", null);
//# sourceMappingURL=DataSet.js.map
