"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;
exports.iteratorEvery = iteratorEvery;
exports.iteratorFilterToArray = iteratorFilterToArray;
exports.iteratorFind = iteratorFind;
exports.iteratorFindIndex = iteratorFindIndex;
exports.iteratorReduce = iteratorReduce;
exports.iteratorSliceToArray = iteratorSliceToArray;
exports.iteratorSome = iteratorSome;

function iteratorSome(iterator, callbackfn) {
  var result;
  var index = 0;

  while (true) {
    result = iterator.next();

    if (result.done) {
      return false;
    }

    if (callbackfn(result.value, index)) {
      return true;
    }

    index += 1;
  }
}

function iteratorEvery(iterator, callbackfn) {
  var result;
  var index = 0;

  while (true) {
    result = iterator.next();

    if (result.done) {
      return true;
    }

    if (!callbackfn(result.value, index)) {
      return false;
    }

    index += 1;
  }
}

function iteratorFind(iterator, callbackfn) {
  var result;
  var index = 0;

  while (true) {
    result = iterator.next();

    if (result.done) {
      return undefined;
    }

    if (callbackfn(result.value, index)) {
      return result.value;
    }

    index += 1;
  }
}

function iteratorFindIndex(iterator, callbackfn) {
  var result;
  var index = 0;

  while (true) {
    result = iterator.next();

    if (result.done) {
      return -1;
    }

    if (callbackfn(result.value, index)) {
      return index;
    }

    index += 1;
  }
}

function iteratorFilterToArray(iterator, callbackfn) {
  var result;
  var index = 0;
  var ret = [];

  while (true) {
    result = iterator.next();

    if (result.done) {
      return ret;
    }

    if (callbackfn(result.value, index)) {
      ret.push(result.value);
    }

    index += 1;
  }
}

function iteratorReduce(iterator, callbackfn, initialValue) {
  var result;
  var index = 0;

  while (true) {
    result = iterator.next();

    if (result.done) {
      return initialValue;
    }

    initialValue = callbackfn(initialValue, result.value, index);
    index += 1;
  }
}

function iteratorSliceToArray(iterator) {
  var start = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
  var end = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : Infinity;
  var ret = [];
  var index = 0;

  while (true) {
    var result = iterator.next();

    if (result.done || index > end) {
      return ret;
    }

    if (index >= start) {
      ret.push(result.value);
    }

    index++;
  }
}

var _default = {
  iteratorSome: iteratorSome,
  iteratorEvery: iteratorEvery,
  iteratorFind: iteratorFind,
  iteratorFindIndex: iteratorFindIndex,
  iteratorFilterToArray: iteratorFilterToArray,
  iteratorReduce: iteratorReduce,
  iteratorSliceToArray: iteratorSliceToArray
};
exports["default"] = _default;
//# sourceMappingURL=index.js.map
