import BigNumber from 'bignumber.js';
export interface BigNumberOptions {
    /**
     * 强制返回BigNumber, 否则根据结果是否是大数字返回BigNumber还是number
     */
    strict?: boolean;
}
/**
 * 当 BigNumber 能转换为普通数字则返回普通数字
 * @param n1 BigNumber
 * @return BigNumber | number
 */
declare function fix(number: BigNumber): BigNumber | number;
/**
 * 大数字加
 * @param n1 string | number | BigNumber
 * @param n2 string | number | BigNumber
 * @param options Options
 * @return BigNumber | number
 */
declare function plus(n1: BigNumber.Value, n2: BigNumber.Value, options?: BigNumberOptions): BigNumber | number;
/**
 * 大数字减
 * @param n1 string | number | BigNumber
 * @param n2 string | number | BigNumber
 * @param options Options
 * @return BigNumber | number
 */
declare function minus(n1: BigNumber.Value, n2: BigNumber.Value, options?: BigNumberOptions): BigNumber | number;
/**
 * 大数字乘以
 * @param n1 string | number | BigNumber
 * @param n2 string | number | BigNumber
 * @param options Options
 * @return BigNumber | number
 */
declare function multipliedBy(n1: BigNumber.Value, n2: BigNumber.Value, options?: BigNumberOptions): BigNumber | number;
/**
 * 大数字除以
 * @param n1 string | number | BigNumber
 * @param n2 string | number | BigNumber
 * @param options Options
 * @return BigNumber | number
 */
declare function div(n1: BigNumber.Value, n2: BigNumber.Value, options?: BigNumberOptions): BigNumber | number;
/**
 * 大数字模
 * @param n1 string | number | BigNumber
 * @param n2 string | number | BigNumber
 * @param options Options
 * @return BigNumber | number
 */
declare function mod(n1: BigNumber.Value, n2: BigNumber.Value, options?: BigNumberOptions): BigNumber | number;
/**
 * 大数字幂
 * @param n1 string | number | BigNumber
 * @param n2 string | number | BigNumber
 * @param options Options
 * @return BigNumber | number
 */
declare function pow(n1: BigNumber.Value, n2: BigNumber.Value, options?: BigNumberOptions): BigNumber | number;
/**
 * 大数字平方根
 * @param n1 string | number | BigNumber
 * @param options Options
 * @return BigNumber | number
 */
declare function sqrt(n1: BigNumber.Value, options?: BigNumberOptions): BigNumber | number;
/**
 * 格式化小数精度
 * @param n1 string | number | BigNumber
 * @param decimalPlaces number
 * @return string
 */
declare function toFixed(n1: BigNumber.Value, decimalPlaces: number): string;
/**
 * 大数字小于
 * @param n1 string | number | BigNumber
 * @param n2 string | number | BigNumber
 * @return boolean
 */
declare function lt(n1: BigNumber.Value, n2: BigNumber.Value): boolean;
/**
 * 大数字小于等于
 * @param n1 string | number | BigNumber
 * @param n2 string | number | BigNumber
 * @return BigNumber
 */
declare function lte(n1: BigNumber.Value, n2: BigNumber.Value): boolean;
/**
 * 大数字大于
 * @param n1 string | number | BigNumber
 * @param n2 string | number | BigNumber
 * @return boolean
 */
declare function gt(n1: BigNumber.Value, n2: BigNumber.Value): boolean;
/**
 * 大数字大于等于
 * @param n1 string | number | BigNumber
 * @param n2 string | number | BigNumber
 * @return boolean
 */
declare function gte(n1: BigNumber.Value, n2: BigNumber.Value): boolean;
/**
 * 大数字等于
 * @param n1 string | number | BigNumber
 * @param n2 string | number | BigNumber
 * @return boolean
 */
declare function eq(n1: BigNumber.Value, n2: BigNumber.Value): boolean;
/**
 * 最大值
 * @param n1 A numeric value.
 * @return BigNumber | number
 */
declare function max(...n1: BigNumber.Value[]): BigNumber | number;
/**
 * 最小值
 * @param n1 A numeric value.
 * @return BigNumber | number
 */
declare function min(...n1: BigNumber.Value[]): BigNumber | number;
/**
 * 总和
 * @param n1 A numeric value.
 * @return BigNumber | number
 */
declare function sum(...n1: BigNumber.Value[]): BigNumber | number;
/**
 * 随机数
 * @param decimalPlaces A numeric value.
 * @param options Options
 * @return BigNumber | number
 */
declare function random(decimalPlaces?: number, options?: BigNumberOptions): BigNumber | number;
/**
 * 类似 Math.round
 * @param n1 string | number | BigNumber
 * @param options Options
 * @return BigNumber | number
 */
declare function round(n1: BigNumber.Value, options?: BigNumberOptions): BigNumber | number;
/**
 * 类似 Math.floor
 * @param n1 string | number | BigNumber
 * @param options Options
 * @return BigNumber | number
 */
declare function floor(n1: BigNumber.Value, options?: BigNumberOptions): BigNumber | number;
/**
 * 类似 Math.ceil
 * @param n1 string | number | BigNumber
 * @param options Options
 * @return BigNumber | number
 */
declare function ceil(n1: BigNumber.Value, options?: BigNumberOptions): BigNumber | number;
/**
 * 取负
 * @param n1 string | number | BigNumber
 * @param options Options
 * @return BigNumber | number
 */
declare function negated(n1: BigNumber.Value, options?: BigNumberOptions): BigNumber | number;
/**
 * 大数字小数位
 * @param n1
 * @return number
 */
declare function dp(n1: BigNumber.Value): number;
/**
 * 绝对值
 * @param n1
 * @param options Options
 * @return BigNumber | number
 */
declare function abs(n1: BigNumber.Value, options?: BigNumberOptions): BigNumber | number;
/**
 * 判断是不是 BigNumber
 * @param n1 any
 * @return boolean
 */
declare function isBigNumber(v1: any): v1 is BigNumber;
/**
 * 判断 BigNumber 是不是有限可用数
 * @param n1 BigNumber
 * @return boolean
 */
declare function isValidBigNumber(n1: BigNumber): boolean;
/**
 * 判断数字是不是有限可用数
 * @param n1 BigNumber.Value
 * @return boolean
 */
declare function isValidNumber(n1: BigNumber.Value): boolean;
/**
 * 判断是不是有限数
 * @param n1 string | number | BigNumber
 * @return boolean
 */
declare function isFinite(n1: BigNumber.Value): boolean;
/**
 * 判断是不是 NaN
 * @param n1 string | number | BigNumber
 * @return boolean
 */
declare function isNaN(n1: BigNumber.Value): boolean;
/**
 * 判断是不是负数
 * @param n1 string | number | BigNumber
 * @return boolean
 */
declare function isNegative(n1: BigNumber.Value): boolean;
/**
 * 判断是不是0
 * @param n1 string | number | BigNumber
 * @return boolean
 */
declare function isZero(n1: BigNumber.Value): boolean;
/**
 * 判断是不是-0
 * @param n1 string | number | BigNumber
 * @return boolean
 */
declare function isNegativeZero(n1: BigNumber.Value): boolean;
/**
 * 转换成字符串， 永远不会转换成科学计数法
 * @param n1 string | number | BigNumber
 * @return string
 */
declare function toString(n1: BigNumber.Value): string;
declare const _default: {
    fix: typeof fix;
    plus: typeof plus;
    minus: typeof minus;
    multipliedBy: typeof multipliedBy;
    div: typeof div;
    mod: typeof mod;
    pow: typeof pow;
    sqrt: typeof sqrt;
    lt: typeof lt;
    lte: typeof lte;
    gt: typeof gt;
    gte: typeof gte;
    eq: typeof eq;
    round: typeof round;
    floor: typeof floor;
    ceil: typeof ceil;
    dp: typeof dp;
    max: typeof max;
    min: typeof min;
    abs: typeof abs;
    negated: typeof negated;
    random: typeof random;
    sum: typeof sum;
    isFinite: typeof isFinite;
    isNaN: typeof isNaN;
    isNegative: typeof isNegative;
    isZero: typeof isZero;
    isNegativeZero: typeof isNegativeZero;
    isBigNumber: typeof isBigNumber;
    isValidNumber: typeof isValidNumber;
    isValidBigNumber: typeof isValidBigNumber;
    toFixed: typeof toFixed;
    toString: typeof toString;
};
export default _default;
