import { Moment } from 'moment';
import { BigNumber } from 'bignumber.js';
import { TimeStep } from '../interface';
import Field from '../data-set/Field';
import { FieldType } from '../data-set/enum';
import Record from '../data-set/Record';
import { Config, ConfigKeys, DefaultConfig } from '../configure';
import { formatFileSize } from '../formatter';
import { BigNumberOptions } from '../math';
export declare const MAX_SAFE_INTEGER: number;
export declare const MIN_SAFE_INTEGER: number;
export declare function warning(valid: boolean, message: string): void;
export declare function isEmpty(value: any): value is '' | null | undefined;
export declare function isSame(newValue: any, oldValue: any): boolean;
export declare function isSameLike(newValue: any, oldValue: any): boolean;
export declare function parseNumber(value: BigNumber.Value | boolean, precision?: number, strict?: boolean): number | BigNumber;
export declare function parseBigNumber(value: BigNumber.Value, precision?: number): BigNumber;
export declare function getNearStepMomentValues(value: Moment, step: BigNumber.Value | TimeStep): Moment[] | undefined;
export declare function getNearStepValues(value: BigNumber.Value, step: BigNumber.Value | TimeStep, min?: BigNumber.Value | null, max?: BigNumber.Value | null, options?: BigNumberOptions): BigNumber.Value[] | undefined;
export declare function getGlobalConfig<T extends ConfigKeys>(key: T, field?: Field): T extends keyof DefaultConfig ? DefaultConfig[T] : Config[T];
export declare function getDateFormatByFieldType(type: FieldType, field?: Field): string;
export declare function getDateFormatByField(field?: Field, type?: FieldType, record?: Record): string;
export declare function getNumberFormatOptions(type: FieldType, options?: Intl.NumberFormatOptions): Intl.NumberFormatOptions;
export declare function toRangeValue(value: any, range?: boolean | [string, string]): [any, any];
export declare function normalizeLanguage(language?: string): string | undefined;
declare const _default: {
    MAX_SAFE_INTEGER: number;
    MIN_SAFE_INTEGER: number;
    warning: typeof warning;
    isEmpty: typeof isEmpty;
    isSame: typeof isSame;
    isSameLike: typeof isSameLike;
    parseNumber: typeof parseNumber;
    getNearStepValues: typeof getNearStepValues;
    getDateFormatByFieldType: typeof getDateFormatByFieldType;
    getDateFormatByField: typeof getDateFormatByField;
    getNumberFormatOptions: typeof getNumberFormatOptions;
    toRangeValue: typeof toRangeValue;
    normalizeLanguage: typeof normalizeLanguage;
    parseBigNumber: typeof parseBigNumber;
    formatFileSize: typeof formatFileSize;
};
export default _default;
