import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
import _extends from "@babel/runtime/helpers/extends";
import _objectSpread from "@babel/runtime/helpers/objectSpread2";
import _classCallCheck from "@babel/runtime/helpers/classCallCheck";
import _createClass from "@babel/runtime/helpers/createClass";
import _assertThisInitialized from "@babel/runtime/helpers/assertThisInitialized";
import _inherits from "@babel/runtime/helpers/inherits";
import _createSuper from "@babel/runtime/helpers/createSuper";
var _excluded = ["depth", "rowIndex", "isHeaderRow"],
    _excluded2 = ["children", "columns", "className", "data", "style", "isTree", "hover", "bordered", "cellBordered", "wordWrap", "classPrefix", "loading", "showHeader", "queryBar"];
import _regeneratorRuntime from "@babel/runtime/regenerator";
import * as React from 'react';
import { get, isArrayLike, runInAction } from 'mobx';
import classNames from 'classnames';
import isFunction from 'lodash/isFunction';
import flatten from 'lodash/flatten';
import debounce from 'lodash/debounce';
import isEqual from 'lodash/isEqual';
import defaultTo from 'lodash/defaultTo';
import eq from 'lodash/eq';
import omit from 'lodash/omit';
import merge from 'lodash/merge';
import uniq from 'lodash/uniq';
import isNil from 'lodash/isNil';
import BScroll from '@better-scroll/core';
import bindElementResize, { unbind as unbindElementResize } from 'element-resize-event';
import { getTranslateDOMPositionXY } from "dom-lib/es/transition/translateDOMPositionXY";
import { addStyle, getHeight, getOffset, getWidth, on, scrollLeft, scrollTop, WheelHandler } from 'dom-lib';
import { DragDropContext, Draggable, Droppable } from 'react-beautiful-dnd';
import isPromise from 'is-promise';
import ConfigContext from '../../../es/config-provider/ConfigContext';
import { toPx } from '../../../es/_util/UnitConvertor';
import LocaleReceiver from '../../../es/locale-provider/LocaleReceiver';
import defaultLocale from '../../../es/locale-provider/default';
import warning from '../../../es/_util/warning';
import { stopPropagation } from '../_util/EventManager';
import ModalProvider from '../modal-provider/ModalProvider';
import Row from './Row';
import CellGroup from './CellGroup';
import Scrollbar from './Scrollbar';
import SelectionBox from './SelectionBox';
import SelectionCheckboxAll from './SelectionCheckboxAll';
import TableContext from './TableContext';
import { CELL_PADDING_HEIGHT, SCROLLBAR_LARGE_WIDTH, SCROLLBAR_WIDTH } from './constants';
import { TableQueryBarType } from './enum';
import { cancelAnimationTimeout, defaultClassPrefix, findAllParents, findRowKeys, flattenData, getTotalByColumns, getUnhandledProps, isNumberOrTrue, isRTL as _isRTL, mergeCells, prefix, requestAnimationTimeout, resetLeftForCells, shouldShowRowByExpanded, toggleClass } from './utils';
import isMobile from '../_util/isMobile';
import { transformZoomData } from '../_util/DocumentUtils';
import ColumnGroup from './ColumnGroup';
import Column from './Column';
import Cell from './Cell';
import HeaderCell from './HeaderCell';
import Spin from '../spin';
import PerformanceTableQueryBar from './query-bar';
import ProfessionalBar from './query-bar/TableProfessionalBar';
import DynamicFilterBar from './query-bar/TableDynamicFilterBar';
import TableStore from './TableStore';
import Toolbar from './tool-bar';
import { TableAutoHeightType, TableColumnResizeTriggerType, TableHeightType } from '../table/enum';
import { isDropresult } from '../table/utils';
import { arrayMove } from '../data-set/utils';
import { $l } from '../locale-context';
export { TableQueryBarType };
var SORT_TYPE = {
  DESC: 'desc',
  ASC: 'asc'
};
var propTypeKeys = ['columns', 'autoHeight', 'affixHeader', 'affixHorizontalScrollbar', 'bordered', 'bodyRef', 'className', 'classPrefix', 'children', 'cellBordered', 'clickScrollLength', 'data', 'defaultExpandAllRows', 'defaultExpandedRowKeys', 'defaultSortType', 'disabledScroll', 'expandedRowKeys', 'hover', 'height', 'headerHeight', 'locale', 'loading', 'loadAnimation', 'minHeight', 'rowKey', 'rowHeight', 'renderTreeToggle', 'renderRowExpanded', 'rowExpandedHeight', 'renderEmpty', 'renderLoading', 'rowClassName', 'rtl', 'style', 'sortColumn', 'sortType', 'showHeader', 'showScrollArrow', 'shouldUpdateScroll', 'translate3d', 'wordWrap', 'width', 'virtualized', 'isTree', 'onRowClick', 'onRowContextMenu', 'onScroll', 'onSortColumn', 'onExpandChange', 'onTouchStart', 'onTouchMove', 'onDataUpdated', 'highLightRow', 'queryBar', 'customizedCode', 'customizable', 'columnDraggable', 'columnTitleEditable', 'columnsDragRender', 'rowSelection', 'rowDraggable', 'onDragEndBefore', 'onDragEnd', 'onDragStart'];
export var CUSTOMIZED_KEY = '__customized-column__'; // TODO:Symbol

function getRowSelection(props) {
  return props.rowSelection || {};
}

var PerformanceTable = /*#__PURE__*/function (_React$Component) {
  _inherits(PerformanceTable, _React$Component);

  var _super = _createSuper(PerformanceTable);

  function PerformanceTable(props, context) {
    var _this;

    _classCallCheck(this, PerformanceTable);

    _this = _super.call(this, props, context);
    _this.translateDOMPositionXY = null;
    _this.scrollListener = null;
    _this.bscroll = null;
    _this.tableRows = {};
    _this.mounted = false;
    _this.disableEventsTimeoutId = null;
    _this.scrollY = 0;
    _this.scrollX = 0;
    _this.nextRowZIndex = [];
    _this.calcStartRowSpan = {
      rowIndex: 0,
      rowSpan: 0,
      height: 0
    };
    _this._cacheCalcStartRowSpan = []; // 缓存合并行的计算结果

    _this._cacheCells = null;
    _this._cacheScrollX = 0;
    _this._cacheRenderCols = [];
    _this._cacheChildrenSize = 0;
    _this._visibleRows = [];
    _this.tableStore = new TableStore(_assertThisInitialized(_this));

    _this.setSelectionColumn = function (props) {
      var rowSelection = props.rowSelection,
          _props$columns = props.columns,
          columns = _props$columns === void 0 ? [] : _props$columns,
          _props$children = props.children,
          children = _props$children === void 0 ? [] : _props$children;
      var index = columns.findIndex(function (column) {
        return column.key === 'rowSelection';
      });

      if (rowSelection) {
        if (index > -1) columns.splice(index, 1);
        var rowSelectionFixed = 'left';

        if ('fixed' in rowSelection) {
          rowSelectionFixed = rowSelection.fixed;
        }

        if (columns && columns.length) {
          var columnsWithRowSelectionProps = {
            title: $l('Table', 'select_current_page'),
            key: 'rowSelection',
            width: 50,
            align: 'center',
            fixed: rowSelectionFixed
          };
          columns.splice(rowSelection.columnIndex || 0, 0, columnsWithRowSelectionProps);
        }

        if (children && children.length) {
          var columnsWithRowSelection = _this.renderRowSelection(rowSelectionFixed);

          if (columnsWithRowSelection) {
            if (rowSelectionFixed) {
              children.splice(rowSelectionFixed === true || 'left' ? rowSelection.columnIndex || 0 : rowSelection.columnIndex || children.length, 0, columnsWithRowSelection);

              _this.setState({
                shouldFixedColumn: true
              });
            } else {
              children.splice(rowSelection.columnIndex || 0, 0, columnsWithRowSelection);
            }
          }
        }
      }

      _this.tableStore.originalColumns = columns;
      _this.tableStore.originalChildren = children;
    };

    _this.listenWheel = function (deltaX, deltaY) {
      _this.handleWheel(deltaX, deltaY);

      var xRef = _this.scrollbarXRef.current;

      if (xRef && xRef.onWheelScroll) {
        xRef.onWheelScroll(deltaX);
      }

      var yRef = _this.scrollbarYRef.current;

      if (yRef && yRef.onWheelScroll) {
        yRef.onWheelScroll(deltaY);
      }
    };

    _this.getRecordKey = function (record, index) {
      var rowKey = _this.props.rowKey;
      var recordKey = record[rowKey];
      warning(recordKey !== undefined, 'Each record in dataSource of table should have a unique `key` prop, ' + 'or set `rowKey` of Table to an unique primary key.');
      return recordKey === undefined ? index : recordKey;
    };

    _this.getCheckboxPropsByItem = function (item, index) {
      var rowSelection = getRowSelection(_this.props);

      if (!rowSelection.getCheckboxProps) {
        return {};
      }

      var key = _this.getRecordKey(item, index); // Cache checkboxProps


      if (!_this.tableStore.checkboxPropsCache[key]) {
        _this.tableStore.checkboxPropsCache[key] = rowSelection.getCheckboxProps(item) || {};
        var checkboxProps = _this.tableStore.checkboxPropsCache[key];
        warning(!('checked' in checkboxProps) && !('defaultChecked' in checkboxProps), 'Do not set `checked` or `defaultChecked` in `getCheckboxProps`. Please use `selectedRowKeys` instead.');
      }

      return _this.tableStore.checkboxPropsCache[key];
    };

    _this.handleSelect = function (record, rowIndex, e) {
      var checked = e.target.checked;
      var nativeEvent = e.nativeEvent;
      var defaultSelection = _this.tableStore.selectionDirty ? [] : _this.getDefaultSelection(); // @ts-ignore

      var selectedRowKeys = _this.tableStore.selectedRowKeys.concat(defaultSelection);

      var key = _this.getRecordKey(record, rowIndex);

      var _this$state = _this.state,
          pivot = _this$state.pivot,
          data = _this$state.data;

      var rows = _objectSpread({}, data);

      var realIndex = rowIndex;

      if (_this.props.expandedRowRender) {
        realIndex = rows.findIndex(function (row) {
          return _this.getRecordKey(row, rowIndex) === key;
        });
      }

      if (nativeEvent.shiftKey && pivot !== undefined && realIndex !== pivot) {
        var changeRowKeys = [];
        var direction = Math.sign(pivot - realIndex);
        var dist = Math.abs(pivot - realIndex);
        var step = 0;

        var _loop = function _loop() {
          var i = realIndex + step * direction;
          step += 1;
          var row = rows[i];

          var rowKey = _this.getRecordKey(row, i);

          var checkboxProps = _this.getCheckboxPropsByItem(row, i);

          if (!checkboxProps.disabled) {
            if (selectedRowKeys.includes(rowKey)) {
              if (!checked) {
                selectedRowKeys = selectedRowKeys.filter(function (j) {
                  return rowKey !== j;
                });
                changeRowKeys.push(rowKey);
              }
            } else if (checked) {
              selectedRowKeys.push(rowKey);
              changeRowKeys.push(rowKey);
            }
          }
        };

        while (step <= dist) {
          _loop();
        }

        _this.setState({
          pivot: realIndex
        });

        _this.tableStore.selectionDirty = true;

        _this.setSelectedRowKeys(selectedRowKeys, {
          selectWay: 'onSelectMultiple',
          record: record,
          checked: checked,
          changeRowKeys: changeRowKeys,
          nativeEvent: nativeEvent
        });
      } else {
        if (checked) {
          selectedRowKeys.push(_this.getRecordKey(record, realIndex));
        } else {
          selectedRowKeys = selectedRowKeys.filter(function (i) {
            return key !== i;
          });
        }

        _this.setState({
          pivot: realIndex
        });

        _this.setSelectedRowKeys(selectedRowKeys, {
          selectWay: 'onSelect',
          record: record,
          checked: checked,
          changeRowKeys: undefined,
          nativeEvent: nativeEvent
        });
      }
    };

    _this.handleSelectRow = function (selectionKey, index, onSelectFunc) {
      var data = _this.state.data;
      var defaultSelection = _this.tableStore.selectionDirty ? [] : _this.getDefaultSelection(); //@ts-ignore

      var selectedRowKeys = _this.tableStore.selectedRowKeys.concat(defaultSelection);

      var changeableRowKeys = data.filter(function (item, i) {
        return !_this.getCheckboxPropsByItem(item, i).disabled;
      }).map(function (item, i) {
        return _this.getRecordKey(item, i);
      });
      var changeRowKeys = [];
      var selectWay = 'onSelectAll';
      var checked; // handle default selection

      switch (selectionKey) {
        case 'all':
          changeableRowKeys.forEach(function (key) {
            if (selectedRowKeys.indexOf(key) < 0) {
              selectedRowKeys.push(key);
              changeRowKeys.push(key);
            }
          });
          selectWay = 'onSelectAll';
          checked = true;
          break;

        case 'removeAll':
          changeableRowKeys.forEach(function (key) {
            if (selectedRowKeys.indexOf(key) >= 0) {
              selectedRowKeys.splice(selectedRowKeys.indexOf(key), 1);
              changeRowKeys.push(key);
            }
          });
          selectWay = 'onSelectAll';
          checked = false;
          break;

        case 'invert':
          changeableRowKeys.forEach(function (key) {
            if (selectedRowKeys.indexOf(key) < 0) {
              selectedRowKeys.push(key);
            } else {
              selectedRowKeys.splice(selectedRowKeys.indexOf(key), 1);
            }

            changeRowKeys.push(key);
            selectWay = 'onSelectInvert';
          });
          break;

        default:
          break;
      }

      _this.tableStore.selectionDirty = true; // when select custom selection, callback selections[n].onSelect

      var rowSelection = _this.props.rowSelection;
      var customSelectionStartIndex = 2;

      if (rowSelection && rowSelection.hideDefaultSelections) {
        customSelectionStartIndex = 0;
      }

      if (index >= customSelectionStartIndex && typeof onSelectFunc === 'function') {
        return onSelectFunc(changeableRowKeys);
      }

      _this.setSelectedRowKeys(selectedRowKeys, {
        selectWay: selectWay,
        checked: checked,
        changeRowKeys: changeRowKeys
      });
    };

    _this.handleRadioSelect = function (record, rowIndex, e) {
      var checked = e.target.checked;
      var nativeEvent = e.nativeEvent;

      var key = _this.getRecordKey(record, rowIndex);

      var selectedRowKeys = [key];
      _this.tableStore.selectionDirty = true;

      _this.setSelectedRowKeys(selectedRowKeys, {
        selectWay: 'onSelect',
        record: record,
        checked: checked,
        changeRowKeys: undefined,
        nativeEvent: nativeEvent
      });
    };

    _this.renderSelectionBox = function (type, rowData, rowIndex) {
      var rowKey = _this.getRecordKey(rowData, rowIndex);

      var props = _this.getCheckboxPropsByItem(rowData, rowIndex);

      var handleChange = function handleChange(e) {
        return type === 'radio' ? _this.handleRadioSelect(rowData, rowIndex, e) : _this.handleSelect(rowData, rowIndex, e);
      };

      return /*#__PURE__*/React.createElement("span", {
        onClick: stopPropagation
      }, /*#__PURE__*/React.createElement(SelectionBox, _extends({
        store: _this.tableStore,
        type: type,
        rowIndex: rowKey,
        onChange: handleChange,
        defaultSelection: _this.getDefaultSelection()
      }, props)));
    };

    _this.setOffsetByAffix = function () {
      var _this$props = _this.props,
          affixHeader = _this$props.affixHeader,
          affixHorizontalScrollbar = _this$props.affixHorizontalScrollbar;

      var _assertThisInitialize = _assertThisInitialized(_this),
          headerWrapperRef = _assertThisInitialize.headerWrapperRef,
          tableRef = _assertThisInitialize.tableRef;

      var headerNode = headerWrapperRef && headerWrapperRef.current;

      if (isNumberOrTrue(affixHeader) && headerNode) {
        _this.setState(function () {
          return {
            headerOffset: getOffset(headerNode)
          };
        });
      }

      var tableNode = tableRef && tableRef.current;

      if (isNumberOrTrue(affixHorizontalScrollbar) && tableNode) {
        _this.setState(function () {
          return {
            tableOffset: getOffset(tableNode)
          };
        });
      }
    };

    _this.handleWindowScroll = function () {
      var _this$props2 = _this.props,
          affixHeader = _this$props2.affixHeader,
          affixHorizontalScrollbar = _this$props2.affixHorizontalScrollbar;

      if (isNumberOrTrue(affixHeader)) {
        _this.affixTableHeader();
      }

      if (isNumberOrTrue(affixHorizontalScrollbar)) {
        _this.affixHorizontalScrollbar();
      }
    };

    _this.affixHorizontalScrollbar = function () {
      var scrollY = window.scrollY || window.pageYOffset;
      var windowHeight = getHeight(window);

      var height = _this.getTableHeight();

      var _this$state2 = _this.state,
          tableOffset = _this$state2.tableOffset,
          fixedHorizontalScrollbar = _this$state2.fixedHorizontalScrollbar;
      var affixHorizontalScrollbar = _this.props.affixHorizontalScrollbar;

      var headerHeight = _this.getTableHeaderHeight();

      var bottom = typeof affixHorizontalScrollbar === 'number' ? affixHorizontalScrollbar : 0;
      var offset = defaultTo(tableOffset && tableOffset.top, 0) + bottom;
      var fixedScrollbar = // @ts-ignore
      scrollY + windowHeight < height + offset && // @ts-ignore
      scrollY + windowHeight - headerHeight > offset;

      var _assertThisInitialize2 = _assertThisInitialized(_this),
          scrollbarXRef = _assertThisInitialize2.scrollbarXRef;

      if (scrollbarXRef) {
        var current = scrollbarXRef.current;

        if (current) {
          var barRef = current.barRef;

          if (barRef && barRef.current && fixedHorizontalScrollbar !== fixedScrollbar) {
            _this.setState({
              fixedHorizontalScrollbar: fixedScrollbar
            });
          }
        }
      }
    };

    _this.affixTableHeader = function () {
      var affixHeader = _this.props.affixHeader;
      var top = typeof affixHeader === 'number' ? affixHeader : 0;
      var _this$state3 = _this.state,
          headerOffset = _this$state3.headerOffset,
          contentHeight = _this$state3.contentHeight;
      var scrollY = window.scrollY || window.pageYOffset;
      var fixedHeader = // @ts-ignore
      scrollY - (headerOffset.top - top) >= 0 && scrollY < headerOffset.top - top + contentHeight;

      if (_this.affixHeaderWrapperRef.current) {
        toggleClass(_this.affixHeaderWrapperRef.current, 'fixed', fixedHeader);
      }
    };

    _this.handleSortColumn = function (dataKey) {
      var sortType = _this.getSortType();

      var _this$props3 = _this.props,
          onSortColumn = _this$props3.onSortColumn,
          sortColumn = _this$props3.sortColumn;

      if (sortColumn === dataKey) {
        switch (sortType) {
          case SORT_TYPE.ASC:
            sortType = SORT_TYPE.DESC;
            break;

          case SORT_TYPE.DESC:
            sortType = undefined;
            break;

          default:
            sortType = SORT_TYPE.ASC;
        }

        _this.setState({
          sortType: sortType
        });
      }

      if (onSortColumn) {
        onSortColumn(dataKey, sortType);
      }
    };

    _this.handleShowMouseArea = function (width, left, fixed) {
      var tableColumnResizeTrigger = _this.tableStore.tableColumnResizeTrigger;
      if (tableColumnResizeTrigger !== TableColumnResizeTriggerType.hover) return;

      _this.handleColumnResizeMove(width, left, !!fixed);
    };

    _this.handleHideMouseArea = function () {
      var tableColumnResizeTrigger = _this.tableStore.tableColumnResizeTrigger;
      if (tableColumnResizeTrigger !== TableColumnResizeTriggerType.hover) return;
      addStyle(_this.mouseAreaRef.current, {
        display: 'none'
      });
    };

    _this.handleColumnResizeEnd = function (columnWidth, _cursorDelta, dataKey, index) {
      _this._cacheCells = null;

      if (_this.tableStore.customizable) {
        _this.tableStore.changeCustomizedColumnValue(dataKey, {
          width: columnWidth
        });
      }

      _this.setState(_defineProperty({
        isColumnResizing: false
      }, "".concat(dataKey, "_").concat(index, "_width"), columnWidth));

      addStyle(_this.mouseAreaRef.current, {
        display: 'none'
      });

      _this.scrollLeft(-_this.scrollX);
    };

    _this.handleColumnResizeStart = function (width, left, fixed) {
      _this.setState({
        isColumnResizing: true
      });

      _this.handleColumnResizeMove(width, left, fixed);
    };

    _this.handleColumnResizeMove = function (width, left, fixed) {
      var mouseAreaLeft = width + left;
      var x = mouseAreaLeft;
      var dir = 'left';

      if (_this.isRTL()) {
        mouseAreaLeft += _this.minScrollX + _this.getScrollBarYWidth;
        dir = 'right';
      }

      if (!fixed) {
        x = mouseAreaLeft + (_this.isRTL() ? -_this.scrollX : _this.scrollX);
      }

      addStyle(_this.mouseAreaRef.current, _defineProperty({
        display: 'block'
      }, dir, "".concat(x, "px")));
    };

    _this.handleTreeToggle = function (rowKey, _rowIndex, rowData) {
      var expandedRowKeys = _this.getExpandedRowKeys();

      var open = false;
      var nextExpandedRowKeys = [];

      for (var i = 0; i < expandedRowKeys.length; i++) {
        var key = expandedRowKeys[i];

        if (key === rowKey) {
          open = true;
        } else {
          // @ts-ignore
          nextExpandedRowKeys.push(key);
        }
      }

      if (!open) {
        // @ts-ignore
        nextExpandedRowKeys.push(rowKey);
      }

      _this.setState({
        expandedRowKeys: nextExpandedRowKeys
      });

      var onExpandChange = _this.props.onExpandChange;

      if (onExpandChange) {
        onExpandChange(!open, rowData);
      }
    };

    _this.handleScrollX = function (delta) {
      _this.handleWheel(delta, 0);
    };

    _this.handleScrollY = function (delta) {
      _this.handleWheel(0, delta);
    };

    _this.handleWheel = function (deltaX, deltaY) {
      var _this$props4 = _this.props,
          onScroll = _this$props4.onScroll,
          virtualized = _this$props4.virtualized;
      var _this$state4 = _this.state,
          contentWidth = _this$state4.contentWidth,
          width = _this$state4.width;

      if (!_this.tableRef.current) {
        return;
      }

      var nextScrollX = contentWidth <= width ? 0 : _this.scrollX - deltaX;
      var nextScrollY = _this.scrollY - deltaY;
      _this.scrollY = Math.min(0, nextScrollY < _this.minScrollY ? _this.minScrollY : nextScrollY);
      _this.scrollX = Math.min(0, nextScrollX < _this.minScrollX ? _this.minScrollX : nextScrollX);

      _this.updatePosition();

      if (onScroll) {
        onScroll(_this.scrollX, _this.scrollY);
      }

      if (virtualized) {
        _this.setState({
          isScrolling: true,
          scrollY: _this.scrollY,
          scrollX: _this.scrollX
        });

        if (_this.disableEventsTimeoutId) {
          // @ts-ignore
          cancelAnimationTimeout(_this.disableEventsTimeoutId);
        } // @ts-ignore


        _this.disableEventsTimeoutId = requestAnimationTimeout(_this.debounceScrollEndedCallback, 150);
      }
    };

    _this.debounceScrollEndedCallback = function () {
      _this.disableEventsTimeoutId = null;

      _this.setState({
        isScrolling: false
      });
    }; // 处理移动端 Touch 事件,  Start 的时候初始化 x,y


    _this.handleTouchStart = function (event) {
      if (event.touches) {
        var _event$touches$ = event.touches[0],
            pageX = _event$touches$.pageX,
            pageY = _event$touches$.pageY;
        _this.touchX = transformZoomData(pageX);
        _this.touchY = transformZoomData(pageY);
      }

      var onTouchStart = _this.props.onTouchStart;

      if (onTouchStart) {
        onTouchStart(event);
      }
    }; // 处理移动端 Touch 事件, Move 的时候初始化，更新 scroll


    _this.handleTouchMove = function (_ref) {
      var e = _ref.e;
      var _this$props5 = _this.props,
          autoHeight = _this$props5.autoHeight,
          onTouchMove = _this$props5.onTouchMove;

      if (e.touches) {
        var eTouch = e.touches[0];
        var pageX = transformZoomData(eTouch.pageX);
        var pageY = transformZoomData(eTouch.pageY);
        var deltaX = _this.touchX - pageX;
        var deltaY = autoHeight ? 0 : _this.touchY - pageY;

        if (!_this.shouldHandleWheelY(deltaY) && !_this.shouldHandleWheelX(deltaX)) {
          return;
        }

        if (e.preventDefault) {
          e.preventDefault();
        }

        _this.handleWheel(deltaX, deltaY);

        var xRef = _this.scrollbarXRef.current;

        if (xRef && xRef.onWheelScroll) {
          xRef.onWheelScroll(deltaX);
        }

        var yRef = _this.scrollbarYRef.current;

        if (yRef && yRef.onWheelScroll) {
          yRef.onWheelScroll(deltaY);
        }

        _this.touchX = pageX;
        _this.touchY = pageY;
      }

      if (onTouchMove) {
        onTouchMove(e);
      }
    };
    /**
     * 当用户在 Table 内使用 tab 键，触发了 onScroll 事件，这个时候应该更新滚动条位置
     * https://github.com/rsuite/rsuite/issues/234
     */


    _this.handleBodyScroll = function (event) {
      if (event.target !== _this.tableBodyRef.current) {
        return;
      }

      var left = scrollLeft(event.target);
      var top = scrollTop(event.target);

      if (top === 0 && left === 0) {
        return;
      }

      _this.listenWheel(left, top);

      scrollLeft(event.target, 0);
      scrollTop(event.target, 0);
    };

    _this.handleDragStart = function (initial, provided) {
      var onDragStart = _this.props.onDragStart;
      var draggableId = initial.draggableId;

      _this.setState({
        dragRowIndex: draggableId
      });

      if (isFunction(onDragStart)) {
        onDragStart(initial, provided);
      }
    };

    _this.handleDragEnd = /*#__PURE__*/function () {
      var _ref2 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(resultDrag, provided) {
        var _this$props6, onDragEnd, onDragEndBefore, data, resultBefore, resultStatus, result, resData;

        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _this$props6 = _this.props, onDragEnd = _this$props6.onDragEnd, onDragEndBefore = _this$props6.onDragEndBefore;
                data = _this.state.data;

                _this.setState({
                  dragRowIndex: ''
                });

                resultBefore = resultDrag;

                if (!onDragEndBefore) {
                  _context.next = 17;
                  break;
                }

                resultStatus = onDragEndBefore(resultDrag, provided);

                if (!isPromise(resultStatus)) {
                  _context.next = 12;
                  break;
                }

                _context.next = 9;
                return resultStatus;

              case 9:
                _context.t0 = _context.sent;
                _context.next = 13;
                break;

              case 12:
                _context.t0 = resultStatus;

              case 13:
                result = _context.t0;

                if (result) {
                  _context.next = 16;
                  break;
                }

                return _context.abrupt("return");

              case 16:
                if (isDropresult(result)) {
                  resultBefore = result;
                }

              case 17:
                if (resultBefore && resultBefore.destination) {
                  resData = _toConsumableArray(data);
                  arrayMove(resData, resultBefore.source.index, resultBefore.destination.index); // 使setState变成同步处理

                  setTimeout(function () {
                    _this.setState({
                      data: resData
                    });
                  });

                  if (onDragEnd) {
                    onDragEnd(resultBefore, provided, resData);
                  }
                }

              case 18:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      }));

      return function (_x, _x2) {
        return _ref2.apply(this, arguments);
      };
    }();

    _this.shouldHandleWheelX = function (delta) {
      var _this$props7 = _this.props,
          disabledScroll = _this$props7.disabledScroll,
          loading = _this$props7.loading;

      if (delta === 0 || disabledScroll || loading) {
        return false;
      }

      return true;
    };

    _this.shouldHandleWheelY = function (delta) {
      var _this$props8 = _this.props,
          disabledScroll = _this$props8.disabledScroll,
          loading = _this$props8.loading;

      if (delta === 0 || disabledScroll || loading) {
        return false;
      }

      return delta >= 0 && _this.scrollY > _this.minScrollY || delta < 0 && _this.scrollY < 0;
    };

    _this.addPrefix = function (name) {
      return prefix(_this.props.classPrefix)(name);
    };

    _this.calculateTableWidth = function () {
      var table = _this.tableRef && _this.tableRef.current;
      var width = _this.state.width;

      if (table) {
        var nextWidth = getWidth(table);

        if (width !== nextWidth) {
          _this.scrollX = 0;
          var current = _this.scrollbarXRef && _this.scrollbarXRef.current;
          current && current.resetScrollBarPosition();
          _this._cacheCells = null;
        }

        if (nextWidth !== 0) {
          _this.setState({
            width: nextWidth
          });
        }
      }

      _this.setOffsetByAffix();
    };
    /**
     * public method
     */


    _this.scrollTop = function () {
      var top = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;

      var _this$getControlledSc = _this.getControlledScrollTopValue(top),
          _this$getControlledSc2 = _slicedToArray(_this$getControlledSc, 2),
          scrollY = _this$getControlledSc2[0],
          handleScrollY = _this$getControlledSc2[1];

      _this.scrollY = scrollY;
      var current = _this.scrollbarYRef && _this.scrollbarYRef.current;
      current && current.resetScrollBarPosition && current.resetScrollBarPosition(handleScrollY);

      _this.updatePosition();
      /**
       * 当开启 virtualized，调用 scrollTop 后会出现白屏现象，
       * 原因是直接操作 DOM 的坐标，但是组件没有重新渲染，需要调用 forceUpdate 重新进入 render。
       * Fix: rsuite#1044
       */


      if (_this.props.virtualized && _this.state.contentHeight > _this.getTableHeight()) {
        _this.forceUpdate();
      }
    }; // public method


    _this.scrollLeft = function () {
      var left = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;

      var _this$getControlledSc3 = _this.getControlledScrollLeftValue(left),
          _this$getControlledSc4 = _slicedToArray(_this$getControlledSc3, 2),
          scrollX = _this$getControlledSc4[0],
          handleScrollX = _this$getControlledSc4[1];

      _this.scrollX = scrollX;
      var current = _this.scrollbarXRef && _this.scrollbarXRef.current;
      current && current.resetScrollBarPosition && current.resetScrollBarPosition(handleScrollX);

      _this.updatePosition();
      /**
       * 这里的条件与 scrollTop 触发强制更新的条件相反，避免重复的渲染。
       */


      if (_this.props.virtualized && _this.state.contentHeight <= _this.getTableHeight()) {
        _this.forceUpdate();
      }
    };

    _this.scrollTo = function (coord) {
      var _ref3 = coord || {},
          x = _ref3.x,
          y = _ref3.y;

      if (typeof x === 'number') {
        _this.scrollLeft(x);
      }

      if (typeof y === 'number') {
        _this.scrollTop(y);
      }
    };

    _this.bindTableRowsRef = function (index, rowData, provided) {
      return function (ref) {
        if (ref) {
          _this.tableRows[index] = [ref, rowData];

          if (provided) {
            provided.innerRef(ref);
          }
        }
      };
    };

    _this.bindRowClick = function (rowIndex, index, rowData) {
      return function (event) {
        _this.onRowClick(rowData, event, rowIndex, index);
      };
    };

    _this.bindRowContextMenu = function (rowData) {
      return function (event) {
        var onRowContextMenu = _this.props.onRowContextMenu;

        if (onRowContextMenu) {
          onRowContextMenu(rowData, event);
        }
      };
    };

    var width = props.width,
        height = props.height,
        data = props.data,
        rowKey = props.rowKey,
        defaultExpandAllRows = props.defaultExpandAllRows,
        renderRowExpanded = props.renderRowExpanded,
        defaultExpandedRowKeys = props.defaultExpandedRowKeys,
        _props$children2 = props.children,
        children = _props$children2 === void 0 ? [] : _props$children2,
        _props$columns2 = props.columns,
        columns = _props$columns2 === void 0 ? [] : _props$columns2,
        isTree = props.isTree,
        defaultSortType = props.defaultSortType;
    var expandedRowKeys = defaultExpandAllRows ? findRowKeys(data, rowKey, isFunction(renderRowExpanded)) : defaultExpandedRowKeys || [];
    var shouldFixedColumn = Array.from(children).some(function (child) {
      return child && child.props && child.props.fixed;
    });

    if (columns && columns.length) {
      shouldFixedColumn = Array.from(columns).some(function (child) {
        return child && child.fixed;
      });
    }

    if (isTree && !rowKey) {
      throw new Error('The `rowKey` is required when set isTree');
    }

    _this.state = {
      isTree: isTree,
      expandedRowKeys: expandedRowKeys,
      shouldFixedColumn: shouldFixedColumn,
      cacheData: data,
      data: isTree ? flattenData(data) : data,
      width: width || 0,
      height: height || 0,
      columnWidth: 0,
      dataKey: 0,
      contentHeight: 0,
      contentWidth: 0,
      tableRowsMaxHeight: [],
      sortType: defaultSortType,
      scrollY: 0,
      isScrolling: false,
      fixedHeader: false,
      searchText: '',
      pivot: undefined,
      selectedRowKeys: [],
      dragRowIndex: ''
    };
    _this.scrollY = 0;
    _this.scrollX = 0;
    _this._cacheScrollX = 0;
    _this._cacheRenderCols = [];
    _this.wheelHandler = new WheelHandler(_this.listenWheel, _this.shouldHandleWheelX, _this.shouldHandleWheelY, false);
    _this._cacheChildrenSize = flatten(children || columns).length;
    _this.translateDOMPositionXY = getTranslateDOMPositionXY({
      enable3DTransform: props.translate3d
    });
    _this.tableRef = /*#__PURE__*/React.createRef();
    _this.scrollbarYRef = /*#__PURE__*/React.createRef();
    _this.scrollbarXRef = /*#__PURE__*/React.createRef();
    _this.tableBodyRef = /*#__PURE__*/React.createRef();
    _this.affixHeaderWrapperRef = /*#__PURE__*/React.createRef();
    _this.mouseAreaRef = /*#__PURE__*/React.createRef();
    _this.headerWrapperRef = /*#__PURE__*/React.createRef();
    _this.wheelWrapperRef = /*#__PURE__*/React.createRef();
    _this.tableHeaderRef = /*#__PURE__*/React.createRef();
    runInAction(function () {
      return _this.setSelectionColumn(props);
    });
    return _this;
  }

  _createClass(PerformanceTable, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      this.calculateTableWidth();
      this.calculateTableContextHeight();
      this.calculateRowMaxHeight();
      this.setOffsetByAffix();
      this.initPosition();
      bindElementResize(this.tableRef.current, debounce(this.calculateTableWidth, 400));
      var options = {
        passive: false
      };
      var tableBody = this.tableBodyRef.current;

      if (tableBody) {
        if (isMobile()) {
          this.initBScroll(tableBody);
        }

        this.wheelListener = on(tableBody, 'wheel', this.wheelHandler.onWheel, options); // this.touchStartListener = on(tableBody, 'touchstart', this.handleTouchStart, options);
        // this.touchMoveListener = on(tableBody, 'touchmove', this.handleTouchMove, options);
      }

      var _this$props9 = this.props,
          affixHeader = _this$props9.affixHeader,
          affixHorizontalScrollbar = _this$props9.affixHorizontalScrollbar,
          bodyRef = _this$props9.bodyRef;

      if (isNumberOrTrue(affixHeader) || isNumberOrTrue(affixHorizontalScrollbar)) {
        this.scrollListener = on(window, 'scroll', this.handleWindowScroll);
      }

      if (bodyRef) {
        bodyRef(this.wheelWrapperRef.current);
      }
    }
  }, {
    key: "shouldComponentUpdate",
    value: function shouldComponentUpdate(nextProps, nextState) {
      var _this2 = this;

      var _cacheChildrenSize = flatten(nextProps.children || nextProps.columns || []).length;
      /**
       * 单元格列的信息在初始化后会被缓存，在某些属性被更新以后，需要清除缓存。
       */

      if (_cacheChildrenSize !== this._cacheChildrenSize) {
        this._cacheChildrenSize = _cacheChildrenSize;
        this._cacheCells = null;
        this.tableStore.updateProps(nextProps, this);
      } else if (this.props.children !== nextProps.children || this.props.columns !== nextProps.columns || this.props.sortColumn !== nextProps.sortColumn || this.props.sortType !== nextProps.sortType) {
        this._cacheCells = null;
        this.tableStore.updateProps(nextProps, this);
      } else if (this.props.data !== nextProps.data && this._cacheCells) {
        var rowSelection = nextProps.rowSelection;

        if (rowSelection && rowSelection.type !== 'radio') {
          var flatData = nextState.data.filter(function (item, index) {
            if (rowSelection.getCheckboxProps) {
              return !_this2.getCheckboxPropsByItem(item, index).disabled;
            }

            return true;
          });
          var checkboxAllDisabled = flatData.every(function (item, index) {
            return _this2.getCheckboxPropsByItem(item, index).disabled;
          });
          var checkboxAllHeaderCell = /*#__PURE__*/React.cloneElement(this._cacheCells.headerCells[0], {
            children: /*#__PURE__*/React.cloneElement(this._cacheCells.headerCells[0].props.children, {
              disabled: checkboxAllDisabled,
              data: flatData
            })
          });
          this._cacheCells.headerCells[0] = checkboxAllHeaderCell;
        }
      }

      var flag = this.props.columns !== nextProps.columns || this.props.children !== nextProps.children || this.props.rowSelection !== nextProps.rowSelection;

      if (flag) {
        runInAction(function () {
          return _this2.setSelectionColumn(nextProps);
        });
      }

      return !eq(this.props, nextProps) || !isEqual(this.state, nextState);
    }
  }, {
    key: "componentDidUpdate",
    value: function componentDidUpdate(prevProps, prevState) {
      var _this3 = this;

      var rowHeight = prevProps.rowHeight,
          data = prevProps.data,
          autoHeight = prevProps.autoHeight,
          height = prevProps.height,
          virtualized = prevProps.virtualized,
          children = prevProps.children,
          columns = prevProps.columns;
      var props = this.props,
          state = this.state;
      var nextData = props.data,
          nextAutoHeight = props.autoHeight,
          onDataUpdated = props.onDataUpdated,
          shouldUpdateScroll = props.shouldUpdateScroll,
          nextColumns = props.columns,
          rowSelection = props.rowSelection,
          nextChildren = props.children;

      if (data !== nextData) {
        this.calculateRowMaxHeight();

        if (onDataUpdated) {
          onDataUpdated(nextData, this.scrollTo);
        }

        var maxHeight = nextData.length * (typeof rowHeight === 'function' ? rowHeight({}) : rowHeight); // 当开启允许更新滚动条，或者滚动条位置大于表格的最大高度，则初始滚动条位置

        if (shouldUpdateScroll || Math.abs(this.scrollY) > maxHeight) {
          this.scrollTo({
            x: 0,
            y: 0
          });
        }
      } else {
        this.updatePosition();
      }

      if (columns !== nextColumns || children !== nextChildren || this.tableStore.customizable) {
        var shouldFixedColumn = false;

        if ((!columns || columns.length === 0) && rowSelection && nextColumns && nextColumns.length) {
          var rowSelectionFixed = 'left';

          if ('fixed' in rowSelection) {
            rowSelectionFixed = rowSelection.fixed;
          }

          var columnsWithRowSelectionProps = {
            title: $l('Table', 'select_current_page'),
            key: 'rowSelection',
            width: 50,
            align: 'center',
            fixed: rowSelectionFixed
          };
          runInAction(function () {
            _this3.tableStore.originalColumns = nextColumns.splice(rowSelection.columnIndex || 0, 0, columnsWithRowSelectionProps);
          });
        }

        if (rowSelection) {
          runInAction(function () {
            _this3.tableStore.selectedRowKeys = rowSelection.selectedRowKeys || [];
          });
        }

        if (nextChildren) {
          shouldFixedColumn = Array.from(nextChildren).some(function (child) {
            return child && child.props && child.props.fixed;
          });
        }

        var originalColumns = this.tableStore.originalColumns;

        if (originalColumns && originalColumns.length) {
          shouldFixedColumn = Array.from(originalColumns).some(function (child) {
            return child && child.fixed;
          });
        }

        this.setState({
          shouldFixedColumn: shouldFixedColumn
        });
      }

      if ( // 当 Table 的 data 发生变化，需要重新计算高度
      data !== nextData || // 当 Table 内容区的高度发生变化需要重新计算
      height !== props.height || autoHeight !== nextAutoHeight || // 当 Table 内容区的高度发生变化需要重新计算
      prevState.contentHeight !== state.contentHeight || // 当 expandedRowKeys 发生变化，需要重新计算 Table 高度，如果重算会导致滚动条不显示。
      prevState.expandedRowKeys !== state.expandedRowKeys || prevProps.expandedRowKeys !== props.expandedRowKeys) {
        this.calculateTableContextHeight(prevProps);
      }

      this.calculateTableContentWidth(prevProps);

      if (virtualized) {
        this.calculateTableWidth();
      }

      var tableBody = this.tableBodyRef.current;

      if (!this.wheelListener && tableBody) {
        var options = {
          passive: false
        };

        if (isMobile()) {
          this.initBScroll(tableBody);
        }

        this.wheelListener = on(tableBody, 'wheel', this.wheelHandler.onWheel, options);
      }
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      this.wheelHandler = null;
      var current = this.tableRef.current;

      if (current) {
        unbindElementResize(current);
      }

      var wheelListener = this.wheelListener,
          touchStartListener = this.touchStartListener,
          touchMoveListener = this.touchMoveListener,
          scrollListener = this.scrollListener;

      if (wheelListener) {
        wheelListener.off();
      }

      if (touchStartListener) {
        touchStartListener.off();
      }

      if (touchMoveListener) {
        touchMoveListener.off();
      }

      if (scrollListener) {
        scrollListener.off();
      }
    }
  }, {
    key: "getExpandedRowKeys",
    value: function getExpandedRowKeys() {
      var expandedRowKeys = this.props.expandedRowKeys;
      return typeof expandedRowKeys === 'undefined' ? this.state.expandedRowKeys : expandedRowKeys;
    }
  }, {
    key: "getSortType",
    value: function getSortType() {
      var sortType = this.props.sortType;
      return typeof sortType === 'undefined' ? this.state.sortType : sortType;
    }
  }, {
    key: "getScrollCellGroups",
    value: function getScrollCellGroups() {
      var current = this.tableRef.current;
      return current && current.querySelectorAll(".".concat(this.addPrefix('cell-group-scroll')));
    }
  }, {
    key: "getFixedLeftCellGroups",
    value: function getFixedLeftCellGroups() {
      var current = this.tableRef.current;
      return current && current.querySelectorAll(".".concat(this.addPrefix('cell-group-fixed-left')));
    }
  }, {
    key: "getFixedRightCellGroups",
    value: function getFixedRightCellGroups() {
      var current = this.tableRef.current;
      return current && current.querySelectorAll(".".concat(this.addPrefix('cell-group-fixed-right')));
    }
  }, {
    key: "isRTL",
    value: function isRTL() {
      return this.props.rtl || _isRTL();
    }
  }, {
    key: "getRowHeight",
    value: function getRowHeight() {
      var rowData = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var rowHeight = this.props.rowHeight;
      return typeof rowHeight === 'function' ? rowHeight(rowData) : rowHeight;
    }
  }, {
    key: "getScrollBarYWidth",
    get: function get() {
      var contentHeight = this.state.contentHeight;
      var showScrollArrow = this.props.showScrollArrow;
      var height = this.getTableHeight();

      if (contentHeight > height && !!this.scrollbarYRef) {
        return showScrollArrow ? SCROLLBAR_LARGE_WIDTH : SCROLLBAR_WIDTH;
      }

      return 0;
    }
    /**
     * 获取表头高度
     */

  }, {
    key: "getTableHeaderHeight",
    value: function getTableHeaderHeight() {
      var _this$props10 = this.props,
          headerHeight = _this$props10.headerHeight,
          showHeader = _this$props10.showHeader;
      return showHeader ? headerHeight : 0;
    }
    /**
     * Table 个性化高度变更
     */

  }, {
    key: "handleHeightTypeChange",
    value: function handleHeightTypeChange() {
      this.calculateTableContextHeight();
    }
    /**
     * 获取 Table 需要渲染的高度
     */

  }, {
    key: "getTableHeight",
    value: function getTableHeight() {
      var contentHeight = this.state.contentHeight;
      var _this$props11 = this.props,
          minHeight = _this$props11.minHeight,
          height = _this$props11.height,
          data = _this$props11.data,
          showScrollArrow = _this$props11.showScrollArrow;
      var headerHeight = this.getTableHeaderHeight();
      var _this$tableStore = this.tableStore,
          _this$tableStore$cust = _this$tableStore.customized,
          heightType = _this$tableStore$cust.heightType,
          cusHeight = _this$tableStore$cust.height,
          heightDiff = _this$tableStore$cust.heightDiff,
          tempCustomized = _this$tableStore.tempCustomized,
          autoHeight = _this$tableStore.autoHeight;
      var tableHeight = Math.max(height, minHeight);

      if (this.tableStore.customizable) {
        var tempHeightType = get(tempCustomized, 'heightType');

        if (tempHeightType) {
          if (tempHeightType === TableHeightType.fixed) {
            tableHeight = get(tempCustomized, 'height');
          }

          if (tempHeightType === TableHeightType.flex) {
            tableHeight = document.documentElement.clientHeight - (get(tempCustomized, 'heightDiff') || 0);
          }
        }

        if (heightType) {
          if (heightType === TableHeightType.fixed) {
            tableHeight = cusHeight;
          }

          if (heightType === TableHeightType.flex) {
            tableHeight = document.documentElement.clientHeight - (heightDiff || 0);
          }
        }

        this.setState({
          height: tableHeight
        });
      }

      if (data.length === 0 && !autoHeight) {
        return tableHeight;
      }

      if (autoHeight) {
        if (typeof autoHeight === 'boolean') {
          return Math.max(headerHeight + contentHeight, minHeight) + (showScrollArrow ? SCROLLBAR_LARGE_WIDTH : SCROLLBAR_WIDTH);
        } else {
          var tableRef = this.tableRef && this.tableRef.current;
          if (!tableRef) return tableHeight;
          var parentNode = tableRef.parentNode;
          var offsetHeight = parentNode.offsetHeight;
          var type = autoHeight.type,
              diff = autoHeight.diff;

          if (type === TableAutoHeightType.minHeight) {
            return offsetHeight - diff;
          } else {
            return Math.min(offsetHeight, headerHeight + contentHeight + (showScrollArrow ? SCROLLBAR_LARGE_WIDTH : SCROLLBAR_WIDTH) + diff) - diff;
          }
        }
      }

      return tableHeight;
    }
    /**
     * 处理 column props
     * @param column
     */

  }, {
    key: "getColumnProps",
    value: function getColumnProps(column) {
      return omit(column, ['title', 'dataIndex', 'key', 'render']);
    }
    /**
     * 处理columns json -> reactNode
     * @param columns
     */

  }, {
    key: "processTableColumns",
    value: function processTableColumns(columns) {
      var _this4 = this;

      var visibleColumn = columns.filter(function (col) {
        return !col.hidden;
      });
      return visibleColumn.map(function (column) {
        var dataKey = column.dataIndex;

        if (column.type === 'ColumnGroup') {
          return /*#__PURE__*/React.createElement(ColumnGroup, _extends({}, _this4.getColumnProps(column)), _this4.processTableColumns(column.children));
        }

        if (column.key === 'rowSelection') {
          return _this4.renderRowSelection(column.fixed);
        }

        return (
          /*#__PURE__*/
          // @ts-ignore
          React.createElement(Column, _extends({}, _this4.getColumnProps(column), {
            dataKey: dataKey
          }), /*#__PURE__*/React.createElement(HeaderCell, null, typeof column.title === 'function' ? column.title() : column.title), typeof column.render === 'function' ? /*#__PURE__*/React.createElement(Cell, {
            dataKey: dataKey
          }, function (rowData, rowIndex) {
            return column.render({
              rowData: rowData,
              rowIndex: rowIndex,
              dataIndex: dataKey
            });
          }) : /*#__PURE__*/React.createElement(Cell, {
            dataKey: dataKey
          }))
        );
      });
    }
  }, {
    key: "getFlattenColumn",
    value: function getFlattenColumn(column, cellProps, array) {
      var _column$props = column.props,
          header = _column$props.header,
          childColumns = _column$props.children,
          align = _column$props.align,
          fixed = _column$props.fixed,
          verticalAlign = _column$props.verticalAlign;

      for (var index = 0; index < childColumns.length; index += 1) {
        var childColumn = childColumns[index];

        if (childColumn) {
          var _childColumn$props = childColumn.props,
              childVerticalAlign = _childColumn$props.verticalAlign,
              childAlign = _childColumn$props.align;

          var parentProps = _objectSpread({
            align: childAlign || align,
            fixed: fixed,
            verticalAlign: childVerticalAlign || verticalAlign
          }, cellProps);

          var groupCellProps = _objectSpread(_objectSpread({
            parent: column
          }, childColumn.props), parentProps);

          if (index === 0) {
            groupCellProps.groupCount = childColumns.length;
            groupCellProps.groupHeader = header;
          }

          if (childColumn.type && childColumn.type.__PRO_TABLE_COLUMN_GROUP) {
            var res = this.getFlattenColumn(childColumn, _objectSpread(_objectSpread({}, parentProps), {}, {
              parent: column
            }), array);
            array.concat(res);
          } else {
            array.push( /*#__PURE__*/React.cloneElement(childColumn, groupCellProps));
          }
        }
      }

      return array;
    }
    /**
     * 获取 columns ReactElement 数组
     * - 处理 children 中存在 <Column> 数组的情况
     * - 过滤 children 中的空项
     */

  }, {
    key: "getTableColumns",
    value: function getTableColumns() {
      var _this5 = this;

      var _this$tableStore2 = this.tableStore,
          originalColumns = _this$tableStore2.originalColumns,
          originalChildren = _this$tableStore2.originalChildren;
      var children = originalChildren;

      if (originalColumns && originalColumns.length) {
        children = this.processTableColumns(originalColumns);
      }

      if (!Array.isArray(children) && !isArrayLike(children)) {
        return children;
      } // Fix that the `ColumnGroup` array cannot be rendered in the Table


      var flattenColumns = flatten(children).map(function (column) {
        if (column) {
          var columnChildren = column.props.children;
          var cellProps = {
            dataIndex: columnChildren.length > 1 ? columnChildren[1].props.dataKey : columnChildren[0].props.dataKey
          };
          cellProps.hidden = column.props.hidden;

          if (column.type && column.type.__PRO_TABLE_COLUMN_GROUP) {
            return _this5.getFlattenColumn(column, cellProps, []);
          }

          return /*#__PURE__*/React.cloneElement(column, cellProps);
        }

        return column;
      }); // 把 Columns 中的数组，展平为一维数组，计算 lastColumn 与 firstColumn。

      var flatColumns = flatten(flattenColumns).filter(function (col) {
        return col && col.props && !col.props.hidden;
      });
      /**
       * 将左固定列、右固定列和其他列提取出来
       * 排列成正常显示列的顺序
       * 提供给后面bodyCell使用
       */

      var _this$calculateFixedA = this.calculateFixedAndScrollColumn(flatColumns),
          fixedLeftCells = _this$calculateFixedA.fixedLeftCells,
          fixedRightCells = _this$calculateFixedA.fixedRightCells,
          scrollCells = _this$calculateFixedA.scrollCells; // 重新计算列的时候需清除在虚拟滚动时候的渲染列缓存


      this._cacheRenderCols = [];
      return [].concat(_toConsumableArray(fixedLeftCells), _toConsumableArray(scrollCells), _toConsumableArray(fixedRightCells));
    }
  }, {
    key: "getDefaultSelection",
    value: function getDefaultSelection() {
      var _this6 = this;

      var rowSelection = getRowSelection(this.props);

      if (!rowSelection.getCheckboxProps) {
        return [];
      }

      return this.state.data.filter(function (item, rowIndex) {
        return _this6.getCheckboxPropsByItem(item, rowIndex).defaultChecked;
      }).map(function (record, rowIndex) {
        return _this6.getRecordKey(record, rowIndex);
      });
    }
  }, {
    key: "renderRowSelection",
    value: function renderRowSelection(fixed) {
      var _this7 = this;

      var _this$props12 = this.props,
          rowSelection = _this$props12.rowSelection,
          classPrefix = _this$props12.classPrefix,
          rowKey = _this$props12.rowKey;

      if (rowSelection) {
        var flatData = this.state.data.filter(function (item, index) {
          if (rowSelection.getCheckboxProps) {
            return !_this7.getCheckboxPropsByItem(item, index).disabled;
          }

          return true;
        });
        var selectionColumn = {
          key: 'selection-column',
          fixed: fixed,
          width: rowSelection.columnWidth || 50,
          title: rowSelection.columnTitle
        };
        var selectionCheckboxAll = null;

        if (rowSelection.type !== 'radio') {
          var checkboxAllDisabled = flatData.every(function (item, index) {
            return _this7.getCheckboxPropsByItem(item, index).disabled;
          });
          selectionCheckboxAll = rowSelection.columnTitle || /*#__PURE__*/React.createElement(SelectionCheckboxAll, {
            store: this.tableStore,
            data: flatData,
            getCheckboxPropsByItem: this.getCheckboxPropsByItem,
            getRecordKey: this.getRecordKey,
            disabled: checkboxAllDisabled,
            prefixCls: classPrefix,
            onSelect: this.handleSelectRow,
            selections: rowSelection.selections,
            hideDefaultSelections: rowSelection.hideDefaultSelections
          });
        }

        return /*#__PURE__*/React.createElement(Column, {
          key: selectionColumn.key,
          width: selectionColumn.width,
          align: "center",
          fixed: fixed
        }, /*#__PURE__*/React.createElement(HeaderCell, null, selectionCheckboxAll), /*#__PURE__*/React.createElement(Cell, {
          dataKey: rowKey
        }, function (rowData, rowIndex) {
          return _this7.renderSelectionBox(rowSelection.type, rowData, rowIndex);
        }));
      }
    }
  }, {
    key: "getCellDescriptor",
    value: function getCellDescriptor() {
      var _this8 = this;

      if (this._cacheCells) {
        return this._cacheCells;
      }

      var hasCustomTreeCol = false;
      var left = 0; // Cell left margin

      var headerCells = []; // Table header cell

      var bodyCells = []; // Table body cell

      var _this$props13 = this.props,
          children = _this$props13.children,
          columnsJson = _this$props13.columns;
      var columnHack = children;

      if (columnsJson && columnsJson.length) {
        columnHack = columnsJson;
      }

      if (!columnHack) {
        this._cacheCells = {
          headerCells: headerCells,
          bodyCells: bodyCells,
          hasCustomTreeCol: hasCustomTreeCol,
          allColumnsWidth: left
        };
        return this._cacheCells;
      }

      var columns = this.getTableColumns();
      var tableWidth = this.state.width;
      var _this$props14 = this.props,
          sortColumn = _this$props14.sortColumn,
          rowHeight = _this$props14.rowHeight,
          showHeader = _this$props14.showHeader;

      var _getTotalByColumns = getTotalByColumns(columns, this.state),
          totalFlexGrow = _getTotalByColumns.totalFlexGrow,
          totalWidth = _getTotalByColumns.totalWidth;

      var headerHeight = this.getTableHeaderHeight();
      React.Children.forEach(columns, function (column, index) {
        if ( /*#__PURE__*/React.isValidElement(column)) {
          var columnChildren = column.props.children;
          var _column$props2 = column.props,
              width = _column$props2.width,
              resizable = _column$props2.resizable,
              flexGrow = _column$props2.flexGrow,
              minWidth = _column$props2.minWidth,
              onResize = _column$props2.onResize,
              treeCol = _column$props2.treeCol;

          if (treeCol) {
            hasCustomTreeCol = true;
          }

          if (resizable && flexGrow) {
            console.warn("Cannot set 'resizable' and 'flexGrow' together in <Column>, column index: ".concat(index));
          }

          if (columnChildren.length !== 2) {
            throw new Error("Component <HeaderCell> and <Cell> is required, column index: ".concat(index, " "));
          }

          var nextWidth = _this8.state["".concat(columnChildren[1].props.dataKey, "_").concat(index, "_width")] || width || 0;

          if (tableWidth && flexGrow && totalFlexGrow) {
            nextWidth = Math.max((tableWidth - totalWidth) / totalFlexGrow * flexGrow, minWidth || 60);
          }

          var cellProps = _objectSpread(_objectSpread({}, omit(column.props, ['children'])), {}, {
            'aria-colindex': index + 1,
            left: left,
            index: index,
            headerHeight: headerHeight,
            key: index,
            width: nextWidth,
            height: rowHeight,
            firstColumn: index === 0,
            lastColumn: index === columns.length - 1,
            onCell: column.props.onCell
          });

          if (showHeader && headerHeight) {
            var headerCellProps = {
              // index 用于拖拽列宽时候（Resizable column），定义的序号
              index: index,
              dataKey: columnChildren[1].props.dataKey,
              isHeaderCell: true,
              minWidth: column.props.minWidth,
              sortable: column.props.sortable,
              onSortColumn: _this8.handleSortColumn,
              sortType: _this8.getSortType(),
              sortColumn: sortColumn,
              flexGrow: flexGrow
            };

            if (resizable) {
              merge(headerCellProps, {
                onResize: onResize,
                onColumnResizeEnd: _this8.handleColumnResizeEnd,
                onColumnResizeStart: _this8.handleColumnResizeStart,
                onColumnResizeMove: _this8.handleColumnResizeMove,
                onMouseEnterHandler: _this8.handleShowMouseArea,
                onMouseLeaveHandler: _this8.handleHideMouseArea
              });
            }

            headerCells.push(
            /*#__PURE__*/
            // @ts-ignore
            React.cloneElement(columnChildren[0], _objectSpread(_objectSpread({}, cellProps), headerCellProps)));
          } // @ts-ignore


          bodyCells.push( /*#__PURE__*/React.cloneElement(columnChildren[1], cellProps));
          left += nextWidth;
        }
      });

      if (this.tableStore.customizable) {
        var customizationHeaderProps = {
          'aria-colindex': 999,
          left: left - 30,
          headerHeight: headerHeight,
          key: CUSTOMIZED_KEY,
          width: 14,
          height: rowHeight,
          fixed: 'right',
          className: this.addPrefix('customization-header'),
          isHeaderCell: true
        };
        var customizedColumnHeader = this.tableStore.customizedColumnHeader; // @ts-ignore

        headerCells.push( /*#__PURE__*/React.createElement(HeaderCell, _extends({}, customizationHeaderProps), customizedColumnHeader()));
      }

      return this._cacheCells = {
        headerCells: headerCells,
        bodyCells: bodyCells,
        allColumnsWidth: left,
        hasCustomTreeCol: hasCustomTreeCol
      };
    }
  }, {
    key: "setSelectedRowKeys",
    value: function setSelectedRowKeys(selectedRowKeys, selectionInfo) {
      var _this9 = this;

      var selectWay = selectionInfo.selectWay,
          record = selectionInfo.record,
          checked = selectionInfo.checked,
          changeRowKeys = selectionInfo.changeRowKeys,
          nativeEvent = selectionInfo.nativeEvent;
      var rowSelection = getRowSelection(this.props);

      if (rowSelection) {
        runInAction(function () {
          _this9.tableStore.selectedRowKeys = selectedRowKeys;
        });
      }

      var data = this.state.data;

      if (!rowSelection.onChange && !rowSelection[selectWay]) {
        return;
      }

      var selectedRows = data.filter(function (row, i) {
        return selectedRowKeys.indexOf(_this9.getRecordKey(row, i)) >= 0;
      });

      if (rowSelection.onChange) {
        rowSelection.onChange(selectedRowKeys, selectedRows);
      }

      if (selectWay === 'onSelect' && rowSelection.onSelect) {
        rowSelection.onSelect(record, checked, selectedRows, nativeEvent);
      } else if (selectWay === 'onSelectMultiple' && rowSelection.onSelectMultiple) {
        var changeRows = data.filter(function (row, i) {
          return changeRowKeys.indexOf(_this9.getRecordKey(row, i)) >= 0;
        });
        rowSelection.onSelectMultiple(checked, selectedRows, changeRows);
      } else if (selectWay === 'onSelectAll' && rowSelection.onSelectAll) {
        var _changeRows = data.filter(function (row, i) {
          return changeRowKeys.indexOf(_this9.getRecordKey(row, i)) >= 0;
        });

        rowSelection.onSelectAll(checked, selectedRows, _changeRows);
      } else if (selectWay === 'onSelectInvert' && rowSelection.onSelectInvert) {
        rowSelection.onSelectInvert(selectedRowKeys);
      }
    }
  }, {
    key: "initPosition",
    value: function initPosition() {
      var _this10 = this;

      if (this.isRTL()) {
        setTimeout(function () {
          var _this10$state = _this10.state,
              contentWidth = _this10$state.contentWidth,
              width = _this10$state.width;
          _this10.scrollX = width - contentWidth - _this10.getScrollBarYWidth;

          _this10.updatePosition();

          var scrollbarXRef = _this10.scrollbarXRef;

          if (scrollbarXRef) {
            var current = scrollbarXRef.current;

            if (current && current.resetScrollBarPosition) {
              current.resetScrollBarPosition(-_this10.scrollX);
            }
          }
        }, 0);
      }
    }
  }, {
    key: "initBScroll",
    value: function initBScroll(tableBody) {
      var _this11 = this;

      this.bscroll = new BScroll(tableBody, {
        disableMouse: false,
        disableTouch: false,
        useTransition: false,
        scrollbar: false,
        probeType: 3,
        scrollX: true,
        click: true,
        momentumLimitTime: 500
      });
      var hooks = this.bscroll.scroller.actions.hooks;
      hooks.on('start', this.handleTouchStart);
      var thHooks = this.bscroll.scroller.actionsHandler.hooks;
      thHooks.on('move', this.handleTouchMove);
      this.bscroll.on('scroll', function (pos) {
        _this11.handleScrollY(_this11.scrollY - pos.y);

        _this11.scrollY = pos.y;
      });
    }
  }, {
    key: "updatePosition",
    value: function updatePosition() {
      /**
       * 当存在锁定列情况处理
       */
      if (this.state.shouldFixedColumn) {
        this.updatePositionByFixedCell();
      } else {
        var wheelStyle = {};
        var headerStyle = {}; // @ts-ignore

        this.translateDOMPositionXY(wheelStyle, this.scrollX, this.scrollY); // @ts-ignore

        this.translateDOMPositionXY(headerStyle, this.scrollX, 0);
        var wheelElement = this.wheelWrapperRef && this.wheelWrapperRef.current;
        var headerElement = this.headerWrapperRef && this.headerWrapperRef.current;
        var affixHeaderElement = this.affixHeaderWrapperRef && this.affixHeaderWrapperRef.current;
        wheelElement && addStyle(wheelElement, wheelStyle);
        headerElement && addStyle(headerElement, headerStyle);

        if (affixHeaderElement && affixHeaderElement.hasChildNodes && affixHeaderElement.hasChildNodes()) {
          addStyle(affixHeaderElement.firstChild, headerStyle);
        }
      }

      var header = this.tableHeaderRef && this.tableHeaderRef.current;

      if (header) {
        toggleClass(header, this.addPrefix('cell-group-shadow'), this.scrollY < 0);
      }
    }
  }, {
    key: "updatePositionByFixedCell",
    value: function updatePositionByFixedCell() {
      var wheelGroupStyle = {};
      var wheelStyle = {};
      var scrollGroups = this.getScrollCellGroups();
      var fixedLeftGroups = this.getFixedLeftCellGroups();
      var fixedRightGroups = this.getFixedRightCellGroups();
      var _this$state5 = this.state,
          contentWidth = _this$state5.contentWidth,
          width = _this$state5.width; // @ts-ignore

      this.translateDOMPositionXY(wheelGroupStyle, this.scrollX, 0); // @ts-ignore

      this.translateDOMPositionXY(wheelStyle, 0, this.scrollY);
      var scrollArrayGroups = Array.from(scrollGroups);

      for (var i = 0; i < scrollArrayGroups.length; i++) {
        var group = scrollArrayGroups[i];
        addStyle(group, wheelGroupStyle);
      }

      var header = this.wheelWrapperRef && this.wheelWrapperRef.current;

      if (header) {
        addStyle(header, wheelStyle);
      }

      var leftShadowClassName = this.addPrefix('cell-group-left-shadow');
      var rightShadowClassName = this.addPrefix('cell-group-right-shadow');
      var showLeftShadow = this.scrollX < 0;
      var showRightShadow = width - contentWidth - this.getScrollBarYWidth !== this.scrollX;
      toggleClass(fixedLeftGroups, leftShadowClassName, showLeftShadow);
      toggleClass(fixedRightGroups, rightShadowClassName, showRightShadow);
    }
  }, {
    key: "shouldRenderExpandedRow",
    value: function shouldRenderExpandedRow(rowData) {
      var _this$props15 = this.props,
          rowKey = _this$props15.rowKey,
          renderRowExpanded = _this$props15.renderRowExpanded,
          isTree = _this$props15.isTree;
      var expandedRowKeys = this.getExpandedRowKeys() || [];
      return isFunction(renderRowExpanded) && !isTree && expandedRowKeys.some(function (key) {
        return key === rowData[rowKey];
      });
    }
  }, {
    key: "calculateRowMaxHeight",
    value: function calculateRowMaxHeight() {
      var wordWrap = this.props.wordWrap;

      if (wordWrap) {
        var tableRowsMaxHeight = [];
        var tableRows = Object.values(this.tableRows);

        for (var i = 0; i < tableRows.length; i++) {
          var _tableRows$i = _slicedToArray(tableRows[i], 1),
              row = _tableRows$i[0];

          if (row) {
            var cells = row.querySelectorAll(".".concat(this.addPrefix('cell-wrap'))) || [];
            var cellArray = Array.from(cells);
            var maxHeight = 0;

            for (var j = 0; j < cellArray.length; j++) {
              var cell = cellArray[j];
              var h = getHeight(cell);
              maxHeight = Math.max(maxHeight, h);
            } // @ts-ignore


            tableRowsMaxHeight.push(maxHeight);
          }
        }

        this.setState({
          tableRowsMaxHeight: tableRowsMaxHeight
        });
      }
    }
  }, {
    key: "calculateTableContentWidth",
    value: function calculateTableContentWidth(prevProps) {
      var table = this.tableRef && this.tableRef.current;
      var row = table.querySelector(".".concat(this.addPrefix('row'), ":not(.virtualized)"));
      var contentWidth = row ? getWidth(row) : 0;
      this.setState({
        contentWidth: contentWidth
      }); // 这里 -SCROLLBAR_WIDTH 是为了让滚动条不挡住内容部分

      this.minScrollX = -(contentWidth - this.state.width) - this.getScrollBarYWidth;
      /**
       * 1.判断 Table 列数是否发生变化
       * 2.判断 Table 内容区域是否宽度有变化
       *
       * 满足 1 和 2 则更新横向滚动条位置
       */

      if (flatten(this.props.children).length !== flatten(prevProps.children).length && this.state.contentWidth !== contentWidth) {
        this.scrollLeft(0);
      }
    }
  }, {
    key: "calculateTableContextHeight",
    value: function calculateTableContextHeight(prevProps) {
      var _this12 = this;

      var table = this.tableRef.current;
      var rows = table.querySelectorAll(".".concat(this.addPrefix('row'))) || [];
      var affixHeader = this.props.affixHeader;
      var height = this.getTableHeight();
      var headerHeight = this.getTableHeaderHeight();
      var contentHeight = rows.length ? Array.from(rows).map(function (row) {
        return Math.max(getHeight(row), Number(toPx(row.style.height)), _this12.getRowHeight()) || _this12.getRowHeight();
      }).reduce(function (x, y) {
        return x + y;
      }) : 0; // 当设置 affixHeader 属性后要减掉两个 header 的高度

      var nextContentHeight = contentHeight - (affixHeader ? headerHeight * 2 : headerHeight);

      if (nextContentHeight !== this.state.contentHeight) {
        this.setState({
          contentHeight: nextContentHeight
        });
      }

      if (prevProps && ( // 当 data 更新，或者表格高度更新，则更新滚动条
      prevProps.height !== height || prevProps.data !== this.props.data) && this.scrollY !== 0) {
        this.scrollTop(Math.abs(this.scrollY));
        this.updatePosition();
      }

      this.minScrollY = -(contentHeight - height) - this.getScrollBarYWidth; // 如果内容区域的高度小于表格的高度，则重置 Y 坐标滚动条

      if (contentHeight < height) {
        this.scrollTop(0);
      } // 如果 scrollTop 的值大于可以滚动的范围 ，则重置 Y 坐标滚动条
      // 当 Table 为 virtualized 时， wheel 事件触发每次都会进入该逻辑， 避免在滚动到底部后滚动条重置, +SCROLLBAR_WIDTH


      if (Math.abs(this.scrollY) + height - headerHeight > nextContentHeight + this.getScrollBarYWidth) {
        this.scrollTop(this.scrollY);
      }
    }
  }, {
    key: "getControlledScrollTopValue",
    value: function getControlledScrollTopValue(value) {
      if (this.props.autoHeight) {
        return [0, 0];
      }

      var contentHeight = this.state.contentHeight;
      var headerHeight = this.getTableHeaderHeight();
      var height = this.getTableHeight(); // 滚动值的最大范围判断

      value = Math.min(value, Math.max(0, contentHeight - (height - headerHeight))); // value 值是表格理论滚动位置的一个值，通过 value 计算出 scrollY 坐标值与滚动条位置的值

      return [-value, value / contentHeight * (height - headerHeight)];
    }
  }, {
    key: "getControlledScrollLeftValue",
    value: function getControlledScrollLeftValue(value) {
      var _this$state6 = this.state,
          contentWidth = _this$state6.contentWidth,
          width = _this$state6.width; // 滚动值的最大范围判断

      value = Math.min(value, Math.max(0, contentWidth - width));
      return [-value, value / contentWidth * width];
    }
  }, {
    key: "onRowClick",
    value: function onRowClick(rowData, event, rowIndex, index) {
      var _this$props16 = this.props,
          highLightRow = _this$props16.highLightRow,
          rowKey = _this$props16.rowKey,
          rowDraggable = _this$props16.rowDraggable,
          isTree = _this$props16.isTree,
          onRowClick = _this$props16.onRowClick,
          virtualized = _this$props16.virtualized;
      var useRowKey = rowDraggable || isTree || virtualized;
      var rowNum = useRowKey ? rowData[rowKey] : rowIndex;

      if (highLightRow && (!useRowKey || useRowKey && !isNil(rowNum))) {
        var tableRows = Object.values(this.tableRows);
        if (tableRows[0][0].className.includes("".concat(this.addPrefix('row-header')))) tableRows.shift();
        var ref = useRowKey ? this.tableRows[rowNum] && this.tableRows[rowNum][0] : tableRows[index] && tableRows[index][0];

        if (this._lastRowIndex !== rowNum && ref) {
          if (this._lastRowIndex || this._lastRowIndex === 0) {
            var row = useRowKey ? this.tableRows[this._lastRowIndex] : tableRows[this._lastRowIndex];

            if (row && row[0]) {
              row[0].className = ref.className.replace(" ".concat(this.addPrefix('row-highLight')), '');
            }
          }

          ref.className = "".concat(ref.className, " ").concat(this.addPrefix('row-highLight'));
        }

        this._lastRowIndex = rowNum;
      }

      if (onRowClick) {
        onRowClick(rowData, event);
      }
    }
  }, {
    key: "renderRowData",
    value: function renderRowData(bodyCells, rowData, props, shouldRenderExpandedRow) {
      var _this$props17 = this.props,
          renderTreeToggle = _this$props17.renderTreeToggle,
          rowKey = _this$props17.rowKey,
          wordWrap = _this$props17.wordWrap,
          isTree = _this$props17.isTree;
      var hasChildren = isTree && rowData.children && Array.isArray(rowData.children);
      var nextRowKey = typeof rowData[rowKey] !== 'undefined' ? rowData[rowKey] : props.key;

      var rowProps = _objectSpread(_objectSpread({}, props), {}, {
        // Fixed Row missing custom rowKey
        key: nextRowKey,
        'aria-rowindex': props.key + 2,
        rowRef: this.bindTableRowsRef(props.key, rowData),
        onClick: this.bindRowClick(props.rowIndex, props.key, rowData),
        onContextMenu: this.bindRowContextMenu(rowData)
      });

      var expandedRowKeys = this.getExpandedRowKeys() || [];
      var expanded = expandedRowKeys.some(function (key) {
        return key === rowData[rowKey];
      });
      var cells = [];

      for (var i = 0; i < bodyCells.length; i++) {
        var cell = bodyCells[i];
        cells.push(
        /*#__PURE__*/
        // @ts-ignore
        React.cloneElement(cell, {
          hasChildren: hasChildren,
          rowData: rowData,
          wordWrap: wordWrap,
          renderTreeToggle: renderTreeToggle,
          height: props.height,
          rowIndex: props.rowIndex,
          depth: props.depth,
          onTreeToggle: this.handleTreeToggle,
          rowKey: nextRowKey,
          expanded: expanded
        }));
      }

      return this.renderRow(rowProps, cells, shouldRenderExpandedRow, rowData);
    }
  }, {
    key: "calculateFixedAndScrollColumn",
    value: function calculateFixedAndScrollColumn(cells) {
      var fixedLeftCells = [];
      var fixedRightCells = [];
      var scrollCells = [];
      var fixedLeftCellGroupWidth = 0;
      var fixedRightCellGroupWidth = 0;

      for (var i = 0; i < cells.length; i++) {
        var cell = cells[i];
        var _cell$props = cell.props,
            fixed = _cell$props.fixed,
            width = _cell$props.width;
        var isFixedStart = fixed === 'left' || fixed === true;
        var isFixedEnd = fixed === 'right';

        if (this.isRTL()) {
          isFixedStart = fixed === 'right';
          isFixedEnd = fixed === 'left' || fixed === true;
        }

        if (isFixedStart) {
          // @ts-ignore
          fixedLeftCells.push(cell);
          fixedLeftCellGroupWidth += width;
        } else if (isFixedEnd) {
          // @ts-ignore
          fixedRightCells.push(cell);

          if (cell.key !== CUSTOMIZED_KEY) {
            fixedRightCellGroupWidth += width;
          }
        } else {
          // @ts-ignore
          scrollCells.push(cell);
        }
      }

      return {
        fixedLeftCells: fixedLeftCells,
        fixedRightCells: fixedRightCells,
        scrollCells: scrollCells,
        fixedLeftCellGroupWidth: fixedLeftCellGroupWidth,
        fixedRightCellGroupWidth: fixedRightCellGroupWidth
      };
    }
  }, {
    key: "renderRow",
    value: function renderRow(props, cells, shouldRenderExpandedRow, rowData) {
      var _this13 = this;

      var _this$props18 = this.props,
          rowClassName = _this$props18.rowClassName,
          highLightRow = _this$props18.highLightRow,
          virtualized = _this$props18.virtualized,
          rowDraggable = _this$props18.rowDraggable;
      var _this$state7 = this.state,
          shouldFixedColumn = _this$state7.shouldFixedColumn,
          width = _this$state7.width,
          contentWidth = _this$state7.contentWidth,
          dragRowIndex = _this$state7.dragRowIndex;

      var depth = props.depth,
          rowIndex = props.rowIndex,
          isHeaderRow = props.isHeaderRow,
          restRowProps = _objectWithoutProperties(props, _excluded);

      var rowKey = rowData && this.getRecordKey(rowData, rowIndex);

      if (typeof rowClassName === 'function') {
        restRowProps.className = rowClassName(rowData);
      } else {
        restRowProps.className = rowClassName;
      }

      if (rowKey === this._lastRowIndex && virtualized && highLightRow && !isHeaderRow) {
        restRowProps.className = "".concat(rowClassName, " ").concat(this.addPrefix('row-highLight'));
      }

      var rowStyles = {};
      var rowRight = 0;

      if (this.isRTL() && contentWidth > width) {
        rowRight = width - contentWidth;
        rowStyles.right = rowRight;
      }

      var currnetRowIndex = ''; // 修复合并行的最后一行没有borderBottom 和 合并行后的单元格被遮挡的问题

      for (var i = 0; i < cells.length; i++) {
        var cellUnit = cells[i];
        var _cellUnit$props = cellUnit.props,
            onCell = _cellUnit$props.onCell,
            dataIndex = _cellUnit$props.dataIndex,
            fixed = _cellUnit$props.fixed;

        if (onCell && typeof onCell === 'function') {
          var cellExternalProps = onCell({
            rowData: rowData,
            dataIndex: dataIndex,
            rowIndex: rowIndex
          }) || {};
          var _cellExternalProps$ro = cellExternalProps.rowSpan,
              rowSpan = _cellExternalProps$ro === void 0 ? 1 : _cellExternalProps$ro;

          if (rowSpan > 1 && !currnetRowIndex) {
            currnetRowIndex = "".concat(rowIndex);
            rowStyles.zIndex = fixed ? 1 : 0;
          } else if (rowSpan < 1 && !isHeaderRow) {
            restRowProps.width = undefined;
          }
        }
      } // 优化拖拽行被 rowSpan 合并行覆盖的问题


      if (dragRowIndex === "".concat(rowIndex)) {
        rowStyles.zIndex = 1;
      } // IF there are fixed columns, add a fixed group


      if (shouldFixedColumn && contentWidth > width) {
        if (rowData && uniq(this.tableStore.rowZIndex.slice()).includes(rowIndex)) {
          rowStyles.zIndex = 0;
        }

        var _this$calculateFixedA2 = this.calculateFixedAndScrollColumn(cells),
            _this$calculateFixedA3 = _this$calculateFixedA2.fixedLeftCells,
            fixedLeftCells = _this$calculateFixedA3 === void 0 ? [] : _this$calculateFixedA3,
            _this$calculateFixedA4 = _this$calculateFixedA2.fixedRightCells,
            fixedRightCells = _this$calculateFixedA4 === void 0 ? [] : _this$calculateFixedA4,
            _this$calculateFixedA5 = _this$calculateFixedA2.scrollCells,
            scrollCells = _this$calculateFixedA5 === void 0 ? [] : _this$calculateFixedA5,
            _this$calculateFixedA6 = _this$calculateFixedA2.fixedLeftCellGroupWidth,
            fixedLeftCellGroupWidth = _this$calculateFixedA6 === void 0 ? 0 : _this$calculateFixedA6,
            _this$calculateFixedA7 = _this$calculateFixedA2.fixedRightCellGroupWidth,
            fixedRightCellGroupWidth = _this$calculateFixedA7 === void 0 ? 0 : _this$calculateFixedA7;

        if (rowDraggable && !isHeaderRow) {
          return /*#__PURE__*/React.createElement(Draggable, {
            draggableId: String(rowKey),
            index: rowIndex,
            key: rowKey
          }, function (provided, snapshot) {
            return /*#__PURE__*/React.createElement(Row, _extends({}, restRowProps, {
              "data-depth": depth,
              style: rowStyles,
              rowDraggable: rowDraggable,
              isHeaderRow: isHeaderRow,
              provided: provided,
              snapshot: snapshot,
              rowRef: _this13.bindTableRowsRef(props.key, rowData, provided)
            }), fixedLeftCellGroupWidth ? /*#__PURE__*/React.createElement(CellGroup, {
              provided: provided,
              snapshot: snapshot,
              rowDraggable: rowDraggable,
              fixed: "left",
              height: props.isHeaderRow ? props.headerHeight : props.height,
              width: fixedLeftCellGroupWidth,
              // @ts-ignore
              style: _this13.isRTL() ? {
                right: width - fixedLeftCellGroupWidth - rowRight
              } : null
            }, mergeCells(resetLeftForCells(fixedLeftCells))) : null, /*#__PURE__*/React.createElement(CellGroup, {
              provided: provided,
              snapshot: snapshot,
              rowDraggable: rowDraggable
            }, mergeCells(scrollCells)), fixedRightCellGroupWidth || fixedRightCells.length ? /*#__PURE__*/React.createElement(CellGroup, {
              provided: provided,
              snapshot: snapshot,
              rowDraggable: rowDraggable,
              fixed: "right",
              style: _this13.isRTL() ? {
                right: 0 - rowRight
              } : {
                left: width - fixedRightCellGroupWidth - _this13.getScrollBarYWidth
              },
              height: props.isHeaderRow ? props.headerHeight : props.height,
              width: fixedRightCellGroupWidth + _this13.getScrollBarYWidth
            }, mergeCells(resetLeftForCells(fixedRightCells, _this13.getScrollBarYWidth))) : null, shouldRenderExpandedRow && _this13.renderRowExpanded(rowData));
          });
        }

        return /*#__PURE__*/React.createElement(Row, _extends({}, restRowProps, {
          isHeaderRow: isHeaderRow,
          "data-depth": depth,
          style: rowStyles,
          rowRef: this.bindTableRowsRef(props.key, rowData)
        }), fixedLeftCellGroupWidth ? /*#__PURE__*/React.createElement(CellGroup, {
          fixed: "left",
          height: props.isHeaderRow ? props.headerHeight : props.height,
          width: fixedLeftCellGroupWidth,
          // @ts-ignore
          style: this.isRTL() ? {
            right: width - fixedLeftCellGroupWidth - rowRight
          } : null
        }, mergeCells(resetLeftForCells(fixedLeftCells))) : null, /*#__PURE__*/React.createElement(CellGroup, null, mergeCells(scrollCells, fixedLeftCells.length)), fixedRightCellGroupWidth || fixedRightCells.length ? /*#__PURE__*/React.createElement(CellGroup, {
          fixed: "right",
          style: this.isRTL() ? {
            right: 0 - rowRight
          } : {
            left: width - fixedRightCellGroupWidth - this.getScrollBarYWidth
          },
          height: props.isHeaderRow ? props.headerHeight : props.height,
          width: fixedRightCellGroupWidth + this.getScrollBarYWidth
        }, mergeCells(resetLeftForCells(fixedRightCells, this.getScrollBarYWidth))) : null, shouldRenderExpandedRow && this.renderRowExpanded(rowData));
      }

      if (rowDraggable && !isHeaderRow) {
        return /*#__PURE__*/React.createElement(Draggable, {
          draggableId: String(rowKey),
          index: rowIndex,
          key: rowKey
        }, function (provided, snapshot) {
          return /*#__PURE__*/React.createElement(Row, _extends({}, restRowProps, {
            "data-depth": depth,
            style: rowStyles,
            rowDraggable: rowDraggable,
            provided: provided,
            snapshot: snapshot,
            isHeaderRow: isHeaderRow,
            rowRef: _this13.bindTableRowsRef(props.key, rowData, provided)
          }), /*#__PURE__*/React.createElement(CellGroup, {
            provided: provided,
            snapshot: snapshot,
            rowDraggable: rowDraggable
          }, mergeCells(cells)), shouldRenderExpandedRow && _this13.renderRowExpanded(rowData));
        });
      }

      return /*#__PURE__*/React.createElement(Row, _extends({}, restRowProps, {
        isHeaderRow: isHeaderRow,
        "data-depth": depth,
        style: rowStyles,
        rowRef: this.bindTableRowsRef(props.key, rowData)
      }), /*#__PURE__*/React.createElement(CellGroup, null, mergeCells(cells)), shouldRenderExpandedRow && this.renderRowExpanded(rowData));
    }
  }, {
    key: "renderRowExpanded",
    value: function renderRowExpanded(rowData) {
      var _this$props19 = this.props,
          renderRowExpanded = _this$props19.renderRowExpanded,
          rowExpandedHeight = _this$props19.rowExpandedHeight;
      var styles = {
        height: rowExpandedHeight
      };

      if (isFunction(renderRowExpanded)) {
        return /*#__PURE__*/React.createElement("div", {
          className: this.addPrefix('row-expanded'),
          style: styles
        }, renderRowExpanded(rowData));
      }

      return null;
    }
  }, {
    key: "renderMouseArea",
    value: function renderMouseArea() {
      var headerHeight = this.getTableHeaderHeight();
      var styles = {
        height: this.getTableHeight()
      };
      var spanStyles = {
        height: headerHeight - 1
      };
      return /*#__PURE__*/React.createElement("div", {
        ref: this.mouseAreaRef,
        className: this.addPrefix('mouse-area'),
        style: styles
      }, /*#__PURE__*/React.createElement("span", {
        style: spanStyles
      }));
    }
  }, {
    key: "renderTableHeader",
    value: function renderTableHeader(headerCells, rowWidth) {
      var affixHeader = this.props.affixHeader;
      var tableWidth = this.state.width;
      var top = typeof affixHeader === 'number' ? affixHeader : 0;
      var headerHeight = this.getTableHeaderHeight();
      var rowProps = {
        'aria-rowindex': 1,
        rowRef: this.tableHeaderRef,
        width: rowWidth,
        height: this.getRowHeight(),
        headerHeight: headerHeight,
        isHeaderRow: true,
        top: 0
      };
      var fixedStyle = {
        position: 'fixed',
        overflow: 'hidden',
        height: this.getTableHeaderHeight(),
        width: tableWidth,
        top: top
      }; // Affix header

      var header = /*#__PURE__*/React.createElement("div", {
        className: classNames(this.addPrefix('affix-header')),
        style: fixedStyle,
        ref: this.affixHeaderWrapperRef
      }, this.renderRow(rowProps, headerCells));
      return /*#__PURE__*/React.createElement(React.Fragment, null, (affixHeader === 0 || affixHeader) && header, /*#__PURE__*/React.createElement("div", {
        role: "rowgroup",
        className: this.addPrefix('header-row-wrapper'),
        ref: this.headerWrapperRef
      }, this.renderRow(rowProps, headerCells)));
    }
  }, {
    key: "renderTableBody",
    value: function renderTableBody(bodyCells, rowWidth) {
      var _this14 = this;

      var _this$props20 = this.props,
          rowExpandedHeight = _this$props20.rowExpandedHeight,
          renderRowExpanded = _this$props20.renderRowExpanded,
          isTree = _this$props20.isTree,
          rowKey = _this$props20.rowKey,
          wordWrap = _this$props20.wordWrap,
          virtualized = _this$props20.virtualized,
          rowHeight = _this$props20.rowHeight,
          rowDraggable = _this$props20.rowDraggable;
      var headerHeight = this.getTableHeaderHeight();
      var _this$state8 = this.state,
          tableRowsMaxHeight = _this$state8.tableRowsMaxHeight,
          isScrolling = _this$state8.isScrolling,
          data = _this$state8.data,
          contentWidth = _this$state8.contentWidth,
          shouldFixedColumn = _this$state8.shouldFixedColumn,
          width = _this$state8.width;
      var height = this.getTableHeight();
      var bodyHeight = height - headerHeight;
      var minLeft = Math.abs(this.scrollX);
      var bodyStyles = {
        top: headerHeight,
        height: bodyHeight
      };
      var contentHeight = 0;
      var topHideHeight = 0;
      var bottomHideHeight = 0;
      this._visibleRows = [];
      this.nextRowZIndex = [];
      this._cacheCalcStartRowSpan = [];

      if (data) {
        var top = 0; // Row position

        var renderCols = bodyCells; // Render Col

        var minTop = Math.abs(this.scrollY); // @ts-ignore

        var maxTop = minTop + height + rowExpandedHeight;
        var isCustomRowHeight = isFunction(rowHeight);
        var isUncertainHeight = !!(renderRowExpanded || isCustomRowHeight || isTree);
        /**
         * 如果开启了虚拟滚动 则计算列显示
         * 判断是否有缓存，如果minLeft 没有变化，就取缓存的值，有变化就重新计算
         */

        if (virtualized && contentWidth > width && (this._cacheScrollX !== minLeft || !this._cacheRenderCols.length)) {
          // 计算渲染列数量
          var colIndex = 0; // 列索引

          var displayColWidth = 0; // 显示列的宽度

          var renderLeftFixedCol = []; // 需要渲染的固定左边列

          var renderRightFixedCol = []; // 需要渲染的固定右边列

          var showNum = 0; // 显示列数

          var divideLeftFixedCol = 0;
          var divideRightFixedCol = 0; // 找到左固定列

          if (shouldFixedColumn) {
            for (var i = 0; i < bodyCells.length; i++) {
              var bc = bodyCells[i];
              var fixed = bc.props.fixed;

              if (fixed && fixed !== 'right' || fixed === 'left') {
                renderLeftFixedCol.push(bc);
              } else {
                break;
              }
            } // 找到右固定列


            for (var _i = bodyCells.length - 1; _i >= 0; _i--) {
              var _bc = bodyCells[_i];
              var _fixed = _bc.props.fixed;

              if (_fixed === 'right') {
                renderRightFixedCol.push(_bc);
              } else {
                break;
              }
            }

            renderRightFixedCol = renderRightFixedCol.reverse(); // 计算需要减去左右固定列的宽度和

            divideLeftFixedCol = renderLeftFixedCol.reduce(function (val, item) {
              return val + item.props.width;
            }, 0);
            divideRightFixedCol = renderRightFixedCol.reduce(function (val, item) {
              return val + item.props.width;
            }, 0);
          } // 遍历显示列的总宽度与x滚动条关系


          for (var _i2 = 0; _i2 < bodyCells.length; _i2++) {
            var elem = bodyCells[_i2];
            displayColWidth += elem.props.width;

            if (displayColWidth - divideLeftFixedCol > minLeft) {
              colIndex = _i2;
              break;
            }
          } // 计算显示列开始下标


          var colStartIndex = colIndex > renderLeftFixedCol.length ? colIndex - 1 : colIndex; //
          // 判断当前容器宽度能容纳列数

          var currentDisplayColWidth = 0; // 总宽度减去左右固定列的宽度

          var divideWidth = width - divideLeftFixedCol - divideRightFixedCol; // 遍历列宽度 与 容器宽度 得出该容器下显示的列数

          for (var _i3 = colIndex + 1 + renderRightFixedCol.length; _i3 < bodyCells.length; _i3++) {
            var _elem = bodyCells[_i3];
            currentDisplayColWidth += _elem.props.width;

            if (currentDisplayColWidth > divideWidth) {
              showNum = _i3 - colIndex;
              break;
            }
          } // 计算列显示的结束下标


          var colEndIndex = (showNum ? colIndex + showNum + 1 : bodyCells.length) - renderRightFixedCol.length; //
          // 最后slice 需要渲染的部分列

          if (renderLeftFixedCol.length + renderRightFixedCol.length === bodyCells.length) {
            renderCols = [].concat(renderLeftFixedCol, _toConsumableArray(renderRightFixedCol));
          } else {
            renderCols = [].concat(renderLeftFixedCol, _toConsumableArray(bodyCells.slice(colStartIndex, colEndIndex)), _toConsumableArray(renderRightFixedCol));
          }

          this._cacheScrollX = minLeft;
          this._cacheRenderCols = renderCols;
        } else {
          renderCols = this._cacheRenderCols.length ? this._cacheRenderCols : bodyCells;
        }
        /**
         如果开启了 virtualized  同时 Table 中的的行高是可变的，
         则需要循环遍历 data, 获取每一行的高度。
         */


        if (isUncertainHeight && virtualized || !virtualized) {
          // temp test
          var keyIndex = 0;

          for (var index = 0; index < data.length; index++) {
            var rowData = data[index];
            var maxHeight = tableRowsMaxHeight[index];
            var shouldRenderExpandedRow = this.shouldRenderExpandedRow(rowData);
            var nextRowHeight = 0;
            var depth = 0;

            if (typeof rowHeight === 'function') {
              nextRowHeight = rowHeight(rowData);
            } else {
              nextRowHeight = maxHeight ? Math.max(maxHeight + CELL_PADDING_HEIGHT, rowHeight) : rowHeight;

              if (shouldRenderExpandedRow) {
                // @ts-ignore
                nextRowHeight += rowExpandedHeight;
              }
            }

            if (isTree) {
              var parents = findAllParents(rowData, rowKey);
              var expandedRowKeys = this.getExpandedRowKeys();
              depth = parents.length; //  如果是 Tree Table,  判断当前的行是否展开/折叠，如果是折叠则不显示该行。
              // @ts-ignore

              if (!shouldShowRowByExpanded(expandedRowKeys, parents)) {
                continue;
              }
            }

            contentHeight += nextRowHeight;
            var rowProps = {
              key: keyIndex,
              rowIndex: index,
              top: top,
              width: rowWidth,
              depth: depth,
              height: nextRowHeight
            };
            top += nextRowHeight;

            if (virtualized && !wordWrap) {
              if (top + nextRowHeight < minTop) {
                topHideHeight += nextRowHeight;
                continue;
              } else if (top > maxTop) {
                bottomHideHeight += nextRowHeight;
                continue;
              }
            }

            this._visibleRows.push( // @ts-ignore
            this.renderRowData(renderCols, rowData, rowProps, shouldRenderExpandedRow));

            keyIndex++;
          }
        } else {
          /**
           如果 Table 的行高是固定的，则直接通过行高与行数进行计算，
           减少遍历所有 data 带来的性能消耗
           */
          var _nextRowHeight = this.getRowHeight();

          var startIndex = Math.max(Math.floor(minTop / _nextRowHeight), 0);
          var endIndex = Math.min(startIndex + Math.ceil(bodyHeight / _nextRowHeight), data.length);
          contentHeight = data.length * _nextRowHeight;
          topHideHeight = startIndex * _nextRowHeight;
          bottomHideHeight = (data.length - endIndex) * _nextRowHeight;
          /**
           * 判断行合并的情况，并遍历出合并行，在滚动的时候根据滚动高度判断是否显示
           */

          var hasSpanRow = [];
          var _keyIndex = 0;
          var rowSpanStartIndex;

          if (this.calcStartRowSpan.rowIndex > startIndex) {
            // 从缓存记录往上找
            var lastHistory = this._cacheCalcStartRowSpan.pop();

            this.calcStartRowSpan = lastHistory || {
              rowIndex: 0,
              rowSpan: 0,
              height: 0
            };
          }

          if (this.calcStartRowSpan.height >= minTop) {
            rowSpanStartIndex = this.calcStartRowSpan.rowIndex;
          } else {
            rowSpanStartIndex = this.calcStartRowSpan.rowIndex + this.calcStartRowSpan.rowSpan;
          }

          var hasOnCellCol = renderCols.filter(function (x) {
            return x.props.onCell;
          });

          var _loop2 = function _loop2(_i4) {
            var rowData = data[_i4];

            for (var j = 0; j < hasOnCellCol.length; j++) {
              var col = hasOnCellCol[j];
              var onCellInfo = col.props.onCell({
                rowData: rowData,
                dataIndex: col.props.dataIndex,
                rowIndex: _i4
              });
              var cellRowSpan = onCellInfo.rowSpan; // 12

              var calcHeight = (cellRowSpan + _i4) * _nextRowHeight;

              if (cellRowSpan > 1 && minTop < calcHeight) {
                var _rowProps = {
                  key: _keyIndex,
                  rowIndex: _i4,
                  top: _i4 * _nextRowHeight,
                  width: rowWidth,
                  height: _nextRowHeight
                };

                if (calcHeight > _this14.calcStartRowSpan.height && cellRowSpan + _i4 < endIndex && rowSpanStartIndex >= _i4) {
                  var tempCalc = {
                    rowIndex: _i4,
                    rowSpan: cellRowSpan,
                    height: calcHeight
                  };
                  _this14.calcStartRowSpan = tempCalc;

                  _this14._cacheCalcStartRowSpan.push(tempCalc);
                }

                var isExits = hasSpanRow.find(function (x) {
                  return x.rowIndex === _i4;
                });

                if (!isExits) {
                  hasSpanRow.push({
                    rowIndex: _i4,
                    render: _this14.renderRowData(renderCols, rowData, _rowProps, false)
                  });
                }

                _keyIndex++;
              }
            }
          };

          for (var _i4 = rowSpanStartIndex; _i4 < endIndex; _i4++) {
            _loop2(_i4);
          }

          var _loop3 = function _loop3(_index) {
            var rowData = data[_index];
            var rowProps = {
              key: _keyIndex,
              rowIndex: _index,
              top: _index * _nextRowHeight,
              width: rowWidth,
              height: _nextRowHeight
            };

            if (!hasSpanRow.length) {
              _this14._visibleRows.push(_this14.renderRowData(renderCols, rowData, rowProps, false));

              _keyIndex++;
            } else {
              var findHasSpanRow = hasSpanRow.find(function (x) {
                return x.rowIndex === _index;
              });

              if (!findHasSpanRow) {
                hasSpanRow.push({
                  rowIndex: _index,
                  render: _this14.renderRowData(renderCols, rowData, rowProps, false)
                });
                _keyIndex++;
              }
            }
          };

          for (var _index = startIndex; _index < endIndex; _index++) {
            _loop3(_index);
          }

          if (hasSpanRow.length) {
            var sortRow = hasSpanRow.sort(function (a, b) {
              return a.rowIndex - b.rowIndex;
            });
            this._visibleRows = sortRow.map(function (x) {
              return x.render;
            });
          }
        }
      }

      var wheelStyles = {
        position: 'absolute',
        height: contentHeight,
        minHeight: height,
        pointerEvents: isScrolling ? 'none' : undefined
      };
      var topRowStyles = {
        height: topHideHeight
      };
      var bottomRowStyles = {
        height: bottomHideHeight
      };
      var body = /*#__PURE__*/React.createElement(LocaleReceiver, {
        componentName: "PerformanceTable",
        defaultLocale: defaultLocale.PerformanceTable
      }, function (locale) {
        return /*#__PURE__*/React.createElement("div", {
          ref: _this14.tableBodyRef,
          role: "rowgroup",
          className: _this14.addPrefix('body-row-wrapper'),
          style: bodyStyles,
          onScroll: _this14.handleBodyScroll
        }, /*#__PURE__*/React.createElement("div", {
          style: wheelStyles,
          className: _this14.addPrefix('body-wheel-area'),
          ref: _this14.wheelWrapperRef
        }, topHideHeight ? /*#__PURE__*/React.createElement(Row, {
          style: topRowStyles,
          className: "virtualized"
        }) : null, _this14._visibleRows, bottomHideHeight ? /*#__PURE__*/React.createElement(Row, {
          style: bottomRowStyles,
          className: "virtualized"
        }) : null), _this14.renderInfo(locale), _this14.renderScrollbar(), _this14.renderLoading());
      });
      return rowDraggable ? /*#__PURE__*/React.createElement(DragDropContext, {
        onDragStart: this.handleDragStart,
        onDragEnd: this.handleDragEnd
      }, /*#__PURE__*/React.createElement(Droppable, {
        droppableId: "table",
        key: "table"
      }, function (droppableProvided) {
        return /*#__PURE__*/React.createElement("div", _extends({
          ref: droppableProvided.innerRef
        }, droppableProvided.droppableProps), body, droppableProvided.placeholder);
      })) : body;
    }
  }, {
    key: "renderInfo",
    value: function renderInfo(locale) {
      var _this$props21 = this.props,
          renderEmpty = _this$props21.renderEmpty,
          loading = _this$props21.loading;

      if (this._visibleRows.length || loading) {
        return null;
      }

      var emptyMessage = /*#__PURE__*/React.createElement("div", {
        className: this.addPrefix('body-info')
      }, locale.emptyMessage);
      return renderEmpty ? renderEmpty(emptyMessage) : /*#__PURE__*/React.createElement("div", {
        className: this.addPrefix('body-info')
      }, this.tableStore.getConfig('renderEmpty')('Table'));
    }
  }, {
    key: "renderScrollbar",
    value: function renderScrollbar() {
      var _this$props22 = this.props,
          disabledScroll = _this$props22.disabledScroll,
          affixHorizontalScrollbar = _this$props22.affixHorizontalScrollbar,
          id = _this$props22.id,
          showScrollArrow = _this$props22.showScrollArrow,
          clickScrollLength = _this$props22.clickScrollLength;
      var _this$state9 = this.state,
          contentWidth = _this$state9.contentWidth,
          contentHeight = _this$state9.contentHeight,
          width = _this$state9.width,
          fixedHorizontalScrollbar = _this$state9.fixedHorizontalScrollbar;
      var bottom = typeof affixHorizontalScrollbar === 'number' ? affixHorizontalScrollbar : 0;
      var headerHeight = this.getTableHeaderHeight();
      var height = this.getTableHeight();

      if (disabledScroll) {
        return null;
      }

      var scrollBarOffset = contentWidth <= this.state.width || contentHeight <= height - headerHeight ? 40 : 60; // 减去border的左右边框像素

      var decScrollBarOffset = showScrollArrow ? scrollBarOffset : 2;
      return /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement(Scrollbar, {
        tableId: id,
        showScrollArrow: showScrollArrow,
        clickScrollLength: clickScrollLength,
        className: classNames({
          fixed: fixedHorizontalScrollbar
        }),
        style: {
          width: width,
          bottom: fixedHorizontalScrollbar ? bottom : undefined
        },
        length: this.state.width - decScrollBarOffset,
        onScroll: this.handleScrollX,
        scrollLength: contentWidth - decScrollBarOffset,
        ref: this.scrollbarXRef,
        scrollBarOffset: decScrollBarOffset
      }), /*#__PURE__*/React.createElement(Scrollbar, {
        vertical: true,
        tableId: id,
        showScrollArrow: showScrollArrow,
        clickScrollLength: clickScrollLength,
        length: height - headerHeight - decScrollBarOffset,
        scrollLength: contentHeight - decScrollBarOffset,
        onScroll: this.handleScrollY,
        ref: this.scrollbarYRef,
        scrollBarOffset: decScrollBarOffset
      }));
    }
    /**
     *  show loading
     */

  }, {
    key: "renderLoading",
    value: function renderLoading() {
      var _this$props23 = this.props,
          loading = _this$props23.loading,
          loadAnimation = _this$props23.loadAnimation,
          renderLoading = _this$props23.renderLoading;

      if (!loadAnimation && !loading) {
        return null;
      }

      var loadingElement = /*#__PURE__*/React.createElement("div", {
        className: this.addPrefix('loader-wrapper')
      }, /*#__PURE__*/React.createElement("div", {
        className: this.addPrefix('loader')
      }, /*#__PURE__*/React.createElement(Spin, null)));
      return renderLoading ? renderLoading(loadingElement) : loadingElement;
    }
  }, {
    key: "renderTableToolbar",
    value: function renderTableToolbar() {
      var _this$props24 = this.props,
          toolbar = _this$props24.toolbar,
          toolBarRender = _this$props24.toolBarRender;
      if (toolBarRender === false || !toolbar) return null;
      var header = toolbar.header,
          buttons = toolbar.buttons,
          settings = toolbar.settings;
      /** 内置的工具栏 */

      return /*#__PURE__*/React.createElement(Toolbar, {
        header: header,
        hideToolbar: settings === false && !header && !toolBarRender && !toolbar,
        buttons: buttons,
        settings: settings,
        toolBarRender: toolBarRender
      });
    }
  }, {
    key: "render",
    value: function render() {
      var _classNames,
          _this15 = this;

      var _this$props25 = this.props,
          children = _this$props25.children,
          columns = _this$props25.columns,
          className = _this$props25.className,
          data = _this$props25.data,
          style = _this$props25.style,
          _this$props25$isTree = _this$props25.isTree,
          isTree = _this$props25$isTree === void 0 ? false : _this$props25$isTree,
          hover = _this$props25.hover,
          bordered = _this$props25.bordered,
          cellBordered = _this$props25.cellBordered,
          wordWrap = _this$props25.wordWrap,
          classPrefix = _this$props25.classPrefix,
          loading = _this$props25.loading,
          showHeader = _this$props25.showHeader,
          queryBar = _this$props25.queryBar,
          rest = _objectWithoutProperties(_this$props25, _excluded2);

      var _this$state10 = this.state,
          isColumnResizing = _this$state10.isColumnResizing,
          width = _this$state10.width;

      var _this$getCellDescript = this.getCellDescriptor(),
          headerCells = _this$getCellDescript.headerCells,
          bodyCells = _this$getCellDescript.bodyCells,
          allColumnsWidth = _this$getCellDescript.allColumnsWidth,
          hasCustomTreeCol = _this$getCellDescript.hasCustomTreeCol;

      var rowWidth = allColumnsWidth > width ? allColumnsWidth : width;
      var clesses = classNames(classPrefix, className, (_classNames = {}, _defineProperty(_classNames, this.addPrefix('word-wrap'), wordWrap), _defineProperty(_classNames, this.addPrefix('treetable'), isTree), _defineProperty(_classNames, this.addPrefix('bordered'), bordered), _defineProperty(_classNames, this.addPrefix('cell-bordered'), cellBordered), _defineProperty(_classNames, this.addPrefix('column-resizing'), isColumnResizing), _defineProperty(_classNames, this.addPrefix('hover'), hover), _defineProperty(_classNames, this.addPrefix('loading'), loading), _classNames));
      var height = this.getTableHeight();

      var styles = _objectSpread({
        width: width || 'auto',
        height: height,
        lineHeight: "".concat(toPx(this.getRowHeight()), "px")
      }, style);

      runInAction(function () {
        _this15.tableStore.totalHeight = height;
      });
      var unhandled = getUnhandledProps(propTypeKeys, rest);
      var tableStore = this.tableStore,
          translateDOMPositionXY = this.translateDOMPositionXY;
      return /*#__PURE__*/React.createElement(TableContext.Provider, {
        value: {
          translateDOMPositionXY: translateDOMPositionXY,
          rtl: this.isRTL(),
          isTree: isTree,
          hasCustomTreeCol: hasCustomTreeCol,
          tableStore: tableStore
        }
      }, /*#__PURE__*/React.createElement(ModalProvider, null, queryBar === false ? null : /*#__PURE__*/React.createElement(PerformanceTableQueryBar, null), this.renderTableToolbar(), /*#__PURE__*/React.createElement("div", _extends({
        role: isTree ? 'treegrid' : 'grid',
        "aria-rowcount": data.length + 1,
        "aria-colcount": this._cacheChildrenSize
      }, unhandled, {
        className: clesses,
        style: styles,
        ref: this.tableRef
      }), showHeader && this.renderTableHeader(headerCells, rowWidth), columns && columns.length ? this.renderTableBody(bodyCells, rowWidth) : children && this.renderTableBody(bodyCells, rowWidth), showHeader && this.renderMouseArea())));
    }
  }], [{
    key: "getDerivedStateFromProps",
    value: function getDerivedStateFromProps(props, state) {
      if (props.data !== state.cacheData || props.isTree !== state.isTree) {
        return {
          cacheData: props.data,
          isTree: props.isTree,
          data: props.isTree ? flattenData(props.data) : props.data
        };
      }

      return null;
    }
  }, {
    key: "contextType",
    get: function get() {
      return ConfigContext;
    }
  }]);

  return PerformanceTable;
}(React.Component);

export { PerformanceTable as default };
PerformanceTable.displayName = 'performance';
PerformanceTable.Column = Column;
PerformanceTable.Cell = Cell;
PerformanceTable.ColumnGroup = ColumnGroup;
PerformanceTable.HeaderCell = HeaderCell;
PerformanceTable.ProfessionalBar = ProfessionalBar;
PerformanceTable.DynamicFilterBar = DynamicFilterBar;
PerformanceTable.defaultProps = {
  classPrefix: defaultClassPrefix('performance-table'),
  data: [],
  defaultSortType: SORT_TYPE.ASC,
  height: 200,
  rowHeight: 33,
  headerHeight: 33,
  minHeight: 0,
  rowExpandedHeight: 100,
  hover: true,
  highLightRow: true,
  showHeader: true,
  showScrollArrow: false,
  bordered: true,
  rowKey: 'key',
  translate3d: true,
  shouldUpdateScroll: true,
  locale: {
    emptyMessage: 'No data found',
    loading: 'Loading...'
  },
  clickScrollLength: {
    horizontal: 100,
    vertical: 33
  }
};
//# sourceMappingURL=Table.js.map
