import React, { memo, useCallback, useContext } from 'react';
import { action } from 'mobx';
import TableContext from './TableContext';
import Dropdown from '../dropdown/Dropdown';
import Menu from '../menu';
import { $l } from '../locale-context';
import { stopEvent } from '../_util/EventManager';
import { EXPAND_KEY } from './TableStore';

var AggregationButton = function AggregationButton(props) {
  var expanded = props.expanded,
      record = props.record,
      rowGroup = props.rowGroup,
      headerGroup = props.headerGroup,
      aggregationExpandKey = props.aggregationExpandKey,
      isColumnGroup = props.isColumnGroup;

  var _useContext = useContext(TableContext),
      tableStore = _useContext.tableStore,
      prefixCls = _useContext.prefixCls;

  var handleMenuClick = useCallback(action(function (_ref) {
    var key = _ref.key;

    switch (key) {
      case 'cell':
        {
          if (record) {
            tableStore.setAggregationCellExpanded(record, aggregationExpandKey, !expanded);
          } else if (headerGroup) {
            headerGroup.setState(EXPAND_KEY, !expanded);
          }

          break;
        }

      case 'row':
        {
          if (rowGroup && (headerGroup || isColumnGroup)) {
            tableStore.columnGroups.allLeafs.forEach(function (_ref2) {
              var col = _ref2.column,
                  columnKey = _ref2.key;

              if (col.aggregation) {
                rowGroup.totalRecords.forEach(function (r) {
                  tableStore.setAggregationCellExpanded(r, columnKey, !expanded);
                });
              }
            });
          } else if (headerGroup) {
            tableStore.columnGroups.allLeafs.forEach(function (_ref3) {
              var $headerGroup = _ref3.headerGroup;

              if ($headerGroup) {
                $headerGroup.setState(EXPAND_KEY, !expanded);
              }
            });
          } else if (record) {
            tableStore.columnGroups.allLeafs.forEach(function (_ref4) {
              var col = _ref4.column,
                  columnKey = _ref4.key;
              return col.aggregation && tableStore.setAggregationCellExpanded(record, columnKey, !expanded);
            });
          }

          break;
        }

      case 'column':
        {
          if (record) {
            record.dataSet.forEach(function (r) {
              return tableStore.setAggregationCellExpanded(r, aggregationExpandKey, !expanded);
            });

            if (headerGroup) {
              headerGroup.setState(EXPAND_KEY, !expanded);
            }
          } else if (headerGroup) {
            headerGroup.totalRecords.forEach(function (r) {
              return tableStore.setAggregationCellExpanded(r, aggregationExpandKey, !expanded);
            });
            headerGroup.setState(EXPAND_KEY, !expanded);
          }

          break;
        }

      default:
    }
  }), [tableStore, record, aggregationExpandKey, expanded, headerGroup, rowGroup]);
  var handleClick = useCallback(function () {
    handleMenuClick({
      key: tableStore.aggregationExpandType
    });
  }, [handleMenuClick, tableStore]);
  var getOverlay = useCallback(function () {
    return /*#__PURE__*/React.createElement(Menu, {
      prefixCls: "".concat(prefixCls, "-dropdown-menu"),
      onClick: handleMenuClick
    }, /*#__PURE__*/React.createElement(Menu.Item, {
      key: "cell"
    }, $l('Table', expanded ? 'collapse_cell' : 'expand_cell')), /*#__PURE__*/React.createElement(Menu.Item, {
      key: "row"
    }, $l('Table', expanded ? 'collapse_row' : 'expand_row')), /*#__PURE__*/React.createElement(Menu.Item, {
      key: "column"
    }, $l('Table', expanded ? 'collapse_column' : 'expand_column')));
  }, [prefixCls, handleMenuClick, expanded]);
  return /*#__PURE__*/React.createElement("span", {
    className: "".concat(prefixCls, "-cell-expand"),
    onClick: stopEvent
  }, /*#__PURE__*/React.createElement("button", {
    type: "button",
    className: "".concat(prefixCls, "-cell-expand-btn"),
    onClick: handleClick
  }, $l('Table', expanded ? 'collapse' : 'expand_button')), /*#__PURE__*/React.createElement(Dropdown, {
    overlay: getOverlay
  }, /*#__PURE__*/React.createElement("span", {
    className: "".concat(prefixCls, "-cell-expand-menu-btn")
  })));
};

AggregationButton.displayName = 'AggregationButton';
export default /*#__PURE__*/memo(AggregationButton);
//# sourceMappingURL=AggregationButton.js.map
