import _extends from "@babel/runtime/helpers/extends";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
import _objectSpread from "@babel/runtime/helpers/objectSpread2";
import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _classCallCheck from "@babel/runtime/helpers/classCallCheck";
import _createClass from "@babel/runtime/helpers/createClass";
import _assertThisInitialized from "@babel/runtime/helpers/assertThisInitialized";
import _get from "@babel/runtime/helpers/get";
import _getPrototypeOf from "@babel/runtime/helpers/getPrototypeOf";
import _inherits from "@babel/runtime/helpers/inherits";
import _createSuper from "@babel/runtime/helpers/createSuper";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import { __decorate } from "tslib";
import React from 'react';
import classNames from 'classnames';
import ResizeObserver from 'resize-observer-polyfill';
import raf from 'raf';
import { observer } from 'mobx-react';
import pick from 'lodash/pick';
import omit from 'lodash/omit';
import isString from 'lodash/isString';
import isNil from 'lodash/isNil';
import isUndefined from 'lodash/isUndefined';
import noop from 'lodash/noop';
import { action, runInAction, toJS } from 'mobx';
import { DragDropContext } from 'react-beautiful-dnd';
import warning from '../../../es/_util/warning';
import { isCalcSize, isPercentSize, pxToRem, toPx } from '../../../es/_util/UnitConvertor';
import measureScrollbar from '../../../es/_util/measureScrollbar';
import KeyCode from '../../../es/_util/KeyCode';
import ReactResizeObserver from '../../../es/_util/resizeObserver';
import Column from './Column';
import TableRow from './TableRow';
import TableHeaderCell from './TableHeaderCell';
import TableStore, { CUSTOMIZED_KEY } from './TableStore';
import TableHeader from './TableHeader';
import autobind from '../_util/autobind';
import Pagination from '../pagination/Pagination';
import Spin from '../spin';
import DataSetComponent from '../data-set/DataSetComponent';
import { TableContextProvider } from './TableContext';
import TableWrapper from './TableWrapper';
import Profiler from './Profiler';
import TableTBody from './TableTBody';
import ExpandableTableTBody from './ExpandableTableTBody';
import TableFooter from './TableFooter';
import { ColumnLock, RowBoxPlacement, ScrollPosition, SelectionMode, TableHeightType, TableMode, TablePaginationPosition } from './enum';
import TableQueryBar from './query-bar';
import ToolBar from './query-bar/TableToolBar';
import FilterBar from './query-bar/TableFilterBar';
import AdvancedQueryBar from './query-bar/TableAdvancedQueryBar';
import ProfessionalBar from './query-bar/TableProfessionalBar';
import ComboBar from './query-bar/TableComboBar';
import DynamicFilterBar from './query-bar/TableDynamicFilterBar';
import FilterSelect from './query-bar/FilterSelect';
import { findCell, findIndexedSibling, findRow, getCellVerticalSize, getHeight, getPaginationPosition, isCanEdictingRow, isDropresult, isStickySupport, onlyCustomizedColumn } from './utils';
import TableBody from './TableBody';
import VirtualWrapper from './VirtualWrapper';
import SelectionTips from './SelectionTips';
import { DataSetEvents, DataSetSelection, RecordStatus } from '../data-set/enum';
import { Size } from '../core/enum';
import StickyShadow from './StickyShadow';
import { getUniqueFieldNames } from '../data-set/utils';
import mergeProps from '../_util/mergeProps';
import ErrorBar from './ErrorBar';
import TableSibling from './TableSibling';

var _instance; // 构造一个单例table来防止body下不能有table元素的报错


export var instance = function instance(wrapperClassName, prefixCls) {
  // Using a table as the portal so that we do not get react
  // warnings when mounting a tr element
  var _tableContain = function _tableContain() {
    var table = document.createElement('table');

    if (wrapperClassName) {
      table.className = wrapperClassName;
    }

    var thead = document.createElement('thead');
    thead.className = "".concat(prefixCls, "-thead");
    table.appendChild(thead);
    var headtr = document.createElement('tr');
    thead.appendChild(headtr);
    var tbody = document.createElement('tbody');
    tbody.className = "".concat(prefixCls, "-tbody");
    table.appendChild(tbody);

    if (!document.body) {
      throw new Error('document.body required a body to append');
    }

    document.body.appendChild(table);
    return {
      tbody: tbody,
      headtr: headtr
    };
  };

  if (_instance) {
    return _instance;
  }

  return _instance = _tableContain();
};

var Table = /*#__PURE__*/function (_DataSetComponent) {
  _inherits(Table, _DataSetComponent);

  var _super = _createSuper(Table);

  function Table() {
    var _this;

    _classCallCheck(this, Table);

    _this = _super.apply(this, arguments);
    _this.tableStore = new TableStore(_assertThisInitialized(_this));
    _this.wrapperWidth = [];
    return _this;
  }

  _createClass(Table, [{
    key: "currentRow",
    get: function get() {
      var prefixCls = this.prefixCls,
          element = this.element;
      return element ? element.querySelector(".".concat(prefixCls, "-row-current")) : null;
    }
  }, {
    key: "firstRow",
    get: function get() {
      var prefixCls = this.prefixCls,
          element = this.element;
      return element ? element.querySelector(".".concat(prefixCls, "-row:first-child")) : null;
    }
  }, {
    key: "lastRow",
    get: function get() {
      var prefixCls = this.prefixCls,
          element = this.element;
      return element ? element.querySelector(".".concat(prefixCls, "-row:last-child")) : null;
    }
  }, {
    key: "saveResizeRef",
    value: function saveResizeRef(node) {
      this.resizeLine = node;
    }
  }, {
    key: "saveContentRef",
    value: function saveContentRef(node) {
      this.tableContentWrap = node;
    }
  }, {
    key: "useFocusedClassName",
    value: function useFocusedClassName() {
      return false;
    }
  }, {
    key: "setCode",
    value: function setCode(props) {
      var customizedCode = props.customizedCode;

      if (customizedCode) {
        this.code = customizedCode;
      } else {
        _get(_getPrototypeOf(Table.prototype), "setCode", this).call(this, props);
      }
    }
  }, {
    key: "handleResize",
    value: function handleResize(width) {
      var element = this.element,
          tableStore = this.tableStore,
          wrapperWidth = this.wrapperWidth;

      if (width !== undefined) {
        var duplicate = wrapperWidth.includes(width);
        wrapperWidth.unshift(width);
        window.clearTimeout(this.wrapperWidthTimer);
        this.wrapperWidthTimer = window.setTimeout(function () {
          wrapperWidth.pop();
        }, 500);

        if (wrapperWidth.length > 2) {
          wrapperWidth.pop();
        }

        if (duplicate) {
          return;
        }
      }

      if (element && !element.offsetParent) {
        tableStore.styledHidden = true;
      } else if (!tableStore.hidden) {
        this.syncSizeInFrame(width);
      } else {
        tableStore.styledHidden = false;
      }
    }
  }, {
    key: "handleWindowResize",
    value: function handleWindowResize() {
      this.handleResize();
    }
  }, {
    key: "handleDataSetLoad",
    value: function handleDataSetLoad() {
      var tableStore = this.tableStore;

      if (tableStore.performanceEnabled) {
        tableStore.performanceOn = true;
      }

      this.initDefaultExpandedRows();
    }
  }, {
    key: "handleDataSetCreate",
    value: function handleDataSetCreate(_ref) {
      var record = _ref.record,
          dataSet = _ref.dataSet;
      var tableStore = this.tableStore;

      if (tableStore.inlineEdit) {
        if (tableStore.currentEditRecord) {
          tableStore.currentEditRecord.reset();
          dataSet.remove(record);
        } else {
          tableStore.currentEditRecord = record;
        }
      }
    }
  }, {
    key: "handleDataSetReset",
    value: function handleDataSetReset(_ref2) {
      var record = _ref2.record,
          dataSet = _ref2.dataSet;

      if (record) {
        var errors = dataSet.getAllValidationErrors();
        this.bubbleValidationReport(errors.dataSet.length > 0 || errors.records.length > 0);
      } else {
        this.bubbleValidationReport(false);
      }
    }
  }, {
    key: "handleDataSetRemove",
    value: function handleDataSetRemove(_ref3) {
      var records = _ref3.records,
          dataSet = _ref3.dataSet;

      if (records) {
        var errors = dataSet.getAllValidationErrors();
        this.bubbleValidationReport(errors.dataSet.length > 0 || errors.records.length > 0);
      }
    }
  }, {
    key: "handleDataSetValidateSelf",
    value: function handleDataSetValidateSelf(props) {
      var dataSet = props.dataSet;
      var errors = dataSet.getAllValidationErrors();
      this.bubbleValidationReport(errors.dataSet.length > 0 || errors.records.length > 0);
    }
  }, {
    key: "handleDataSetValidate",
    value: function handleDataSetValidate(props) {
      var valid = props.valid,
          dataSet = props.dataSet,
          validationErrors = props.errors,
          noLocate = props.noLocate;
      var autoValidationLocate = this.props.autoValidationLocate;
      var errors = dataSet.getAllValidationErrors();
      this.bubbleValidationReport(errors.dataSet.length > 0 || errors.records.length > 0);

      if (autoValidationLocate !== false && !noLocate && !valid) {
        var tableStore = this.tableStore;

        var _validationErrors = _slicedToArray(validationErrors, 1),
            firstInvalidRecord = _validationErrors[0];

        if (firstInvalidRecord) {
          var _errors = firstInvalidRecord.errors,
              record = firstInvalidRecord.record;

          if (_errors.length) {
            if (!tableStore.showCachedSelection) {
              if (dataSet.cachedRecords.includes(record)) {
                runInAction(function () {
                  tableStore.showCachedSelection = true;
                });
              }
            }

            var _errors2 = _slicedToArray(_errors, 1),
                name = _errors2[0].field.name;

            if (tableStore.virtual && !findRow(tableStore, record)) {
              var tableBodyWrap = this.tableBodyWrap;

              if (tableBodyWrap) {
                var rowMetaData = tableStore.rowMetaData;

                if (rowMetaData) {
                  var metaData = rowMetaData.find(function (m) {
                    return m.record === record;
                  });

                  if (metaData) {
                    tableBodyWrap.scrollTop = metaData.offset;
                  }
                } else {
                  tableBodyWrap.scrollTop = record.index * tableStore.virtualRowHeight;
                }
              }
            }

            raf(function () {
              var cell = findCell(tableStore, name, undefined, record);

              if (cell) {
                cell.focus();
              }
            });
          }
        }
      }
    }
  }, {
    key: "handleKeyDown",
    value: function handleKeyDown(e) {
      var tableStore = this.tableStore;
      var keyboard = tableStore.keyboard;

      if (!tableStore.editing) {
        try {
          var dataSet = this.props.dataSet;
          var ctrlKey = e.ctrlKey || e.metaKey;
          var altKey = e.altKey;
          var shiftKey = e.shiftKey;

          switch (e.keyCode) {
            case KeyCode.UP:
              if (shiftKey && keyboard) {
                this.handleKeyDownUpShift(e);
              } else {
                this.handleKeyDownUp(e);
              }

              break;

            case KeyCode.DOWN:
              if (shiftKey && keyboard) {
                this.handleKeyDownDownShift(e);
              } else {
                this.handleKeyDownDown(e);
              }

              break;

            case KeyCode.RIGHT:
              this.handleKeyDownRight(e);
              break;

            case KeyCode.LEFT:
              this.handleKeyDownLeft(e);
              break;

            case KeyCode.PAGE_UP:
              e.preventDefault();
              dataSet.prePage();
              break;

            case KeyCode.PAGE_DOWN:
              e.preventDefault();
              dataSet.nextPage();
              break;

            case KeyCode.HOME:
              this.handleKeyDownHome(e);
              break;

            case KeyCode.END:
              this.handleKeyDownEnd(e);
              break;

            case KeyCode.S:
              if (ctrlKey === true && keyboard) this.handleKeyDownCTRLS(e);
              break;

            case KeyCode.N:
              if (altKey === true && keyboard) this.handleKeyDownCTRLN(e);
              break;

            case KeyCode.D:
              if (ctrlKey === true && keyboard) this.handleKeyDownCTRLD(e);
              break;

            case KeyCode.DELETE:
              if (altKey === true && keyboard) this.handleKeyDownCTRLDELETE(e);
              break;

            default:
          }
        } catch (error) {
          warning(false, error.message);
        }
      }

      var _this$props$onKeyDown = this.props.onKeyDown,
          onKeyDown = _this$props$onKeyDown === void 0 ? noop : _this$props$onKeyDown;
      onKeyDown(e);
    }
  }, {
    key: "focusRow",
    value: function focusRow(row) {
      if (row) {
        var index = row.dataset.index;

        if (index) {
          var dataSet = this.props.dataSet;
          var record = dataSet.findRecordById(index);

          if (record) {
            dataSet.current = record;
            return record;
          }
        }
      }
    }
  }, {
    key: "handleKeyDownHome",
    value: function () {
      var _handleKeyDownHome = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(e) {
        var dataSet;
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                e.preventDefault();
                dataSet = this.props.dataSet;

                if (this.tableStore.isTree) {
                  _context.next = 5;
                  break;
                }

                _context.next = 5;
                return dataSet.first();

              case 5:
                this.focusRow(this.firstRow);

              case 6:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function handleKeyDownHome(_x) {
        return _handleKeyDownHome.apply(this, arguments);
      }

      return handleKeyDownHome;
    }()
  }, {
    key: "handleKeyDownEnd",
    value: function () {
      var _handleKeyDownEnd = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2(e) {
        var dataSet;
        return _regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                e.preventDefault();
                dataSet = this.props.dataSet;

                if (this.tableStore.isTree) {
                  _context2.next = 5;
                  break;
                }

                _context2.next = 5;
                return dataSet.last();

              case 5:
                this.focusRow(this.lastRow);

              case 6:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function handleKeyDownEnd(_x2) {
        return _handleKeyDownEnd.apply(this, arguments);
      }

      return handleKeyDownEnd;
    }()
  }, {
    key: "handleKeyDownCTRLS",
    value: function () {
      var _handleKeyDownCTRLS = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee3(e) {
        var dataSet;
        return _regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                e.preventDefault();
                dataSet = this.tableStore.dataSet;
                dataSet.submit();

              case 3:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function handleKeyDownCTRLS(_x3) {
        return _handleKeyDownCTRLS.apply(this, arguments);
      }

      return handleKeyDownCTRLS;
    }()
  }, {
    key: "handleKeyDownCTRLN",
    value: function () {
      var _handleKeyDownCTRLN = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee4(e) {
        var _this$tableStore, dataSet, editors;

        return _regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                _this$tableStore = this.tableStore, dataSet = _this$tableStore.dataSet, editors = _this$tableStore.editors;

                if (editors.size) {
                  e.preventDefault();
                  dataSet.create({}, 0);
                }

              case 2:
              case "end":
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function handleKeyDownCTRLN(_x4) {
        return _handleKeyDownCTRLN.apply(this, arguments);
      }

      return handleKeyDownCTRLN;
    }() // TODO: To be optimized

  }, {
    key: "handleKeyDownCTRLD",
    value: function () {
      var _handleKeyDownCTRLD = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee5(e) {
        var currentRow, tableStore, dataSet, currentElementSibling, previousElementSibling, index, currentIndex, record, currentRecord, cloneRecodData, dealCloneRecodData, editeColumn, uniqueFieldNames;
        return _regeneratorRuntime.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                e.preventDefault();
                currentRow = this.currentRow, tableStore = this.tableStore;
                dataSet = tableStore.dataSet;
                currentElementSibling = null; // to justice it can be change or standards compliant

                if (isCanEdictingRow(currentRow)) {
                  currentElementSibling = currentRow;
                }

                if (currentElementSibling && tableStore && dataSet) {
                  previousElementSibling = findIndexedSibling(currentElementSibling, -1);

                  if (previousElementSibling) {
                    index = previousElementSibling.dataset.index;
                    currentIndex = currentElementSibling.dataset.index;

                    if (index && currentIndex) {
                      record = dataSet.findRecordById(index);
                      currentRecord = dataSet.findRecordById(currentIndex); // exculde the primery key and merge has columns which has edictor

                      if (record && currentRecord && tableStore) {
                        cloneRecodData = record.clone().toData() || {};
                        dealCloneRecodData = {};
                        editeColumn = tableStore.columns.filter(function (column) {
                          return !!column.editor;
                        }).reduce(function (accumulator, nowValue) {
                          return [].concat(_toConsumableArray(accumulator), [nowValue.name]);
                        }, []).filter(function (v) {
                          return !(v === null || v === undefined || v === '');
                        });

                        if (editeColumn && editeColumn.length > 0) {
                          editeColumn.forEach(function (element) {
                            if (element) {
                              dealCloneRecodData[element] = cloneRecodData[element];
                            }
                          });
                        } // remove the unique name of fields


                        uniqueFieldNames = getUniqueFieldNames(dataSet);

                        if (uniqueFieldNames && uniqueFieldNames.length > 0) {
                          uniqueFieldNames.forEach(function (element) {
                            if (element) {
                              delete dealCloneRecodData[element];
                            }
                          });
                        }

                        currentRecord.set(dealCloneRecodData);
                      }
                    }
                  }
                }

              case 6:
              case "end":
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      function handleKeyDownCTRLD(_x5) {
        return _handleKeyDownCTRLD.apply(this, arguments);
      }

      return handleKeyDownCTRLD;
    }()
  }, {
    key: "handleKeyDownCTRLDELETE",
    value: function () {
      var _handleKeyDownCTRLDELETE = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee6(e) {
        var dataSet;
        return _regeneratorRuntime.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                e.preventDefault();
                dataSet = this.tableStore.dataSet;
                dataSet["delete"](dataSet.selected);

              case 3:
              case "end":
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      function handleKeyDownCTRLDELETE(_x6) {
        return _handleKeyDownCTRLDELETE.apply(this, arguments);
      }

      return handleKeyDownCTRLDELETE;
    }()
  }, {
    key: "handleKeyDownUp",
    value: function () {
      var _handleKeyDownUp = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee7(e) {
        var currentRow, returnRecod, previousElementSibling, dataSet;
        return _regeneratorRuntime.wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                e.preventDefault();
                currentRow = this.currentRow;

                if (!currentRow) {
                  _context7.next = 14;
                  break;
                }

                previousElementSibling = findIndexedSibling(currentRow, -1);

                if (!previousElementSibling) {
                  _context7.next = 8;
                  break;
                }

                returnRecod = this.focusRow(previousElementSibling);
                _context7.next = 12;
                break;

              case 8:
                dataSet = this.props.dataSet;
                _context7.next = 11;
                return dataSet.prePage();

              case 11:
                returnRecod = this.focusRow(this.lastRow);

              case 12:
                if (!returnRecod) {
                  _context7.next = 14;
                  break;
                }

                return _context7.abrupt("return", Promise.resolve(returnRecod));

              case 14:
                return _context7.abrupt("return", Promise.reject());

              case 15:
              case "end":
                return _context7.stop();
            }
          }
        }, _callee7, this);
      }));

      function handleKeyDownUp(_x7) {
        return _handleKeyDownUp.apply(this, arguments);
      }

      return handleKeyDownUp;
    }()
  }, {
    key: "handleKeyDownDown",
    value: function () {
      var _handleKeyDownDown = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee8(e) {
        var currentRow, returnRecod, nextElementSibling, dataSet;
        return _regeneratorRuntime.wrap(function _callee8$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                e.preventDefault();
                currentRow = this.currentRow;

                if (!currentRow) {
                  _context8.next = 14;
                  break;
                }

                nextElementSibling = findIndexedSibling(currentRow, 1);

                if (!nextElementSibling) {
                  _context8.next = 8;
                  break;
                }

                returnRecod = this.focusRow(nextElementSibling);
                _context8.next = 12;
                break;

              case 8:
                dataSet = this.props.dataSet;
                _context8.next = 11;
                return dataSet.nextPage();

              case 11:
                returnRecod = this.focusRow(this.firstRow);

              case 12:
                if (!returnRecod) {
                  _context8.next = 14;
                  break;
                }

                return _context8.abrupt("return", Promise.resolve(returnRecod));

              case 14:
                return _context8.abrupt("return", Promise.reject());

              case 15:
              case "end":
                return _context8.stop();
            }
          }
        }, _callee8, this);
      }));

      function handleKeyDownDown(_x8) {
        return _handleKeyDownDown.apply(this, arguments);
      }

      return handleKeyDownDown;
    }()
  }, {
    key: "handleKeyDownDownShift",
    value: function () {
      var _handleKeyDownDownShift = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee9(e) {
        var dataSet, currentRow, index, record, currentRecord;
        return _regeneratorRuntime.wrap(function _callee9$(_context9) {
          while (1) {
            switch (_context9.prev = _context9.next) {
              case 0:
                e.preventDefault();
                dataSet = this.tableStore.dataSet;
                currentRow = this.currentRow;

                if (!(currentRow && isCanEdictingRow(currentRow))) {
                  _context9.next = 22;
                  break;
                }

                index = currentRow.dataset.index;

                if (!index) {
                  _context9.next = 20;
                  break;
                }

                record = dataSet.findRecordById(index);

                if (!record) {
                  _context9.next = 20;
                  break;
                }

                if (record.selectable) {
                  _context9.next = 12;
                  break;
                }

                this.handleKeyDownDown(e);
                _context9.next = 20;
                break;

              case 12:
                if (record.isSelected) {
                  _context9.next = 16;
                  break;
                }

                dataSet.select(record);
                _context9.next = 20;
                break;

              case 16:
                _context9.next = 18;
                return this.handleKeyDownDown(e);

              case 18:
                currentRecord = _context9.sent;

                if (currentRecord && dataSet) {
                  dataSet.select(currentRecord);
                }

              case 20:
                _context9.next = 23;
                break;

              case 22:
                this.handleKeyDownDown(e);

              case 23:
              case "end":
                return _context9.stop();
            }
          }
        }, _callee9, this);
      }));

      function handleKeyDownDownShift(_x9) {
        return _handleKeyDownDownShift.apply(this, arguments);
      }

      return handleKeyDownDownShift;
    }()
  }, {
    key: "handleKeyDownUpShift",
    value: function () {
      var _handleKeyDownUpShift = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee10(e) {
        var dataSet, currentRow, index, record, currentRecord;
        return _regeneratorRuntime.wrap(function _callee10$(_context10) {
          while (1) {
            switch (_context10.prev = _context10.next) {
              case 0:
                e.preventDefault();
                dataSet = this.tableStore.dataSet;
                currentRow = this.currentRow;

                if (!(currentRow && isCanEdictingRow(currentRow))) {
                  _context10.next = 22;
                  break;
                }

                index = currentRow.dataset.index;

                if (!index) {
                  _context10.next = 20;
                  break;
                }

                record = dataSet.findRecordById(index);

                if (!record) {
                  _context10.next = 20;
                  break;
                }

                if (record.selectable) {
                  _context10.next = 12;
                  break;
                }

                this.handleKeyDownUp(e);
                _context10.next = 20;
                break;

              case 12:
                if (record.isSelected) {
                  _context10.next = 16;
                  break;
                }

                dataSet.select(record);
                _context10.next = 20;
                break;

              case 16:
                _context10.next = 18;
                return this.handleKeyDownUp(e);

              case 18:
                currentRecord = _context10.sent;

                if (currentRecord) {
                  dataSet.select(currentRecord);
                }

              case 20:
                _context10.next = 23;
                break;

              case 22:
                this.handleKeyDownUp(e);

              case 23:
              case "end":
                return _context10.stop();
            }
          }
        }, _callee10, this);
      }));

      function handleKeyDownUpShift(_x10) {
        return _handleKeyDownUpShift.apply(this, arguments);
      }

      return handleKeyDownUpShift;
    }()
  }, {
    key: "handleKeyDownRight",
    value: function handleKeyDownRight(e) {
      var tableStore = this.tableStore,
          _this$props = this.props,
          expandedRowRenderer = _this$props.expandedRowRenderer,
          dataSet = _this$props.dataSet;

      if (tableStore.isTree || expandedRowRenderer) {
        var current = dataSet.current;

        if (current) {
          e.preventDefault();
          tableStore.setRowExpanded(current, true);
        }
      }
    }
  }, {
    key: "handleKeyDownLeft",
    value: function handleKeyDownLeft(e) {
      var tableStore = this.tableStore,
          _this$props2 = this.props,
          expandedRowRenderer = _this$props2.expandedRowRenderer,
          dataSet = _this$props2.dataSet;

      if (tableStore.isTree || expandedRowRenderer) {
        var current = dataSet.current;

        if (current) {
          e.preventDefault();
          tableStore.setRowExpanded(current, false);
        }
      }
    }
  }, {
    key: "getOmitPropsKeys",
    value: function getOmitPropsKeys() {
      return _get(_getPrototypeOf(Table.prototype), "getOmitPropsKeys", this).call(this).concat(['columns', 'header', 'showHeader', 'footer', 'border', 'columnEditorBorder', 'selectionMode', 'rowBoxPlacement', 'alwaysShowRowBox', 'showSelectionTips', 'showSelectionCachedButton', 'showCachedSelection', 'onShowCachedSelectionChange', 'showAllPageSelectionButton', 'onRow', 'onColumnResize', 'rowRenderer', 'buttons', 'buttonsLimit', 'rowHeight', 'headerRowHeight', 'footerRowHeight', 'queryFields', 'queryFieldsLimit', 'summaryFieldsLimit', 'summaryBarFieldWidth', 'queryBar', 'queryBarProps', 'autoFocus', 'summaryBar', 'defaultRowExpanded', 'expandRowByClick', 'expandedRowRenderer', 'expandIconColumnIndex', 'indentSize', 'filter', 'treeFilter', 'mode', 'editMode', 'filterBarFieldName', 'filterBarPlaceholder', 'pagination', 'highLightRow', 'selectedHighLightRow', 'columnResizable', 'columnTitleEditable', 'pristine', 'expandIcon', 'spin', 'virtual', 'virtualCell', 'virtualSpin', 'columnBuffer', 'columnThreshold', 'autoWidth', 'autoHeight', 'autoFootHeight', 'useMouseBatchChoose', 'autoMaxWidth', 'dragColumnAlign', 'columnDraggable', 'rowDraggable', 'onDragEnd', 'rowDragRender', 'columnsDragRender', 'editorNextKeyEnterDown', 'onDragEndBefore', 'keyboard', 'dynamicFilterBar', 'parityRow', 'rowNumber', 'treeAsync', 'treeLoadData', 'customizable', 'customizedBtn', 'customizedCode', 'dragColumn', 'dragRow', 'clientExportQuantity', 'treeQueryExpanded', 'summary', 'aggregation', 'onAggregationChange', 'showRemovedRow', 'searchCode', 'groups', 'onScrollLeft', 'onScrollTop', 'bodyExpandable', 'defaultBodyExpanded', 'bodyExpanded', 'onBodyExpanded', 'selectionBoxRenderer', 'boxSizing', 'fullColumnWidth']);
    }
  }, {
    key: "getWrapperProps",
    value: function getWrapperProps() {
      var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var autoWidth = this.props.autoWidth;

      var wrapperProps = _objectSpread({}, props);

      if (autoWidth) {
        var _this$tableStore2 = this.tableStore,
            columnGroups = _this$tableStore2.columnGroups,
            overflowY = _this$tableStore2.overflowY,
            border = _this$tableStore2.border;
        wrapperProps.style = {
          width: columnGroups.width + (overflowY ? measureScrollbar() : 0) + (border ? 2 : 0)
        };
      }

      return _get(_getPrototypeOf(Table.prototype), "getWrapperProps", this).call(this, wrapperProps);
    }
  }, {
    key: "getOtherProps",
    value: function getOtherProps() {
      var otherProps = _get(_getPrototypeOf(Table.prototype), "getOtherProps", this).call(this);

      otherProps.onKeyDown = this.handleKeyDown;
      var rowHeight = this.tableStore.rowHeight;

      if (rowHeight === 'auto') {
        delete otherProps.style;
      } else {
        otherProps.style = {
          lineHeight: pxToRem(rowHeight)
        };
      }

      return otherProps;
    }
  }, {
    key: "getClassName",
    value: function getClassName() {
      var _get$call;

      var _this$tableStore3 = this.tableStore,
          border = _this$tableStore3.border,
          parityRow = _this$tableStore3.parityRow,
          aggregation = _this$tableStore3.aggregation,
          size = _this$tableStore3.size,
          isCombinedColumn = _this$tableStore3.isCombinedColumn,
          prefixCls = this.prefixCls;
      return _get(_getPrototypeOf(Table.prototype), "getClassName", this).call(this, (_get$call = {}, _defineProperty(_get$call, "".concat(prefixCls, "-").concat(size), size !== Size["default"]), _defineProperty(_get$call, "".concat(prefixCls, "-bordered"), border), _defineProperty(_get$call, "".concat(prefixCls, "-parity-row"), parityRow), _defineProperty(_get$call, "".concat(prefixCls, "-aggregation"), aggregation), _defineProperty(_get$call, "".concat(prefixCls, "-combined-column"), isCombinedColumn), _get$call));
    }
    /**
     * 获取传入的 Spin props
     */

  }, {
    key: "getSpinProps",
    value: function getSpinProps() {
      var _this$props3 = this.props,
          spin = _this$props3.spin,
          dataSet = _this$props3.dataSet;

      var spinProps = _objectSpread({}, spin);

      if (spinProps && !isUndefined(spinProps.spinning)) return _objectSpread({}, spinProps);
      var loading = this.tableStore.loading;

      if (loading) {
        return _objectSpread(_objectSpread({}, spinProps), {}, {
          spinning: true
        });
      }

      return _objectSpread(_objectSpread({}, spinProps), {}, {
        dataSet: dataSet
      });
    }
  }, {
    key: "componentWillMount",
    value: function componentWillMount() {
      this.initDefaultExpandedRows();
    }
  }, {
    key: "componentDidMount",
    value: function componentDidMount() {
      _get(_getPrototypeOf(Table.prototype), "componentDidMount", this).call(this);

      this.connect();
      this.syncSize();
      this.syncSizeInFrame();
    }
  }, {
    key: "componentDidUpdate",
    value: function componentDidUpdate() {
      var tableStore = this.tableStore;

      if (tableStore.currentEditorName) {
        var length = tableStore.columns.length;

        if (length !== this.columnsSize) {
          this.columnsSize = length;
          tableStore.alignEditor();
        }
      }
    }
  }, {
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(nextProps, nextContext) {
      _get(_getPrototypeOf(Table.prototype), "componentWillReceiveProps", this).call(this, nextProps, nextContext);

      this.disconnect();
      this.columnsSize = this.tableStore.columns.length;
      this.tableStore.updateProps(nextProps);
      this.connect();
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      this.disconnect();

      if (this.scrollId !== undefined) {
        raf.cancel(this.scrollId);
      }

      this.bubbleValidationReport(false);
      this.tableStore.dispose();
    }
  }, {
    key: "bubbleValidationReport",
    value: function bubbleValidationReport(showInvalid) {
      var pristine = this.props.pristine;

      if (!pristine) {
        var onComponentValidationReport = this.getContextConfig('onComponentValidationReport');

        if (onComponentValidationReport) {
          onComponentValidationReport({
            showInvalid: showInvalid,
            component: this
          });
        }
      }
    }
  }, {
    key: "syncParentSize",
    value: function syncParentSize(height, target) {
      var tableStore = this.tableStore,
          element = this.element,
          wrapper = this.wrapper;

      if (element) {
        var wrapperHeight = Math.round(wrapper.getBoundingClientRect().height);

        if (wrapperHeight !== height) {
          tableStore.parentHeight = height;
          tableStore.parentPaddingTop = Math.round(element.getBoundingClientRect().top) - Math.round(target.getBoundingClientRect().top);
        }
      }
    }
  }, {
    key: "handleParentResize",
    value: function handleParentResize(entries) {
      var _entries = _slicedToArray(entries, 1),
          entry = _entries[0];

      var height = Math.round(entry.contentRect.height);
      this.syncParentSize(height, entry.target);
    }
  }, {
    key: "connect",
    value: function connect() {
      this.processDataSetListener(true);
      var _this$tableStore4 = this.tableStore,
          styleMaxHeight = _this$tableStore4.styleMaxHeight,
          styleMinHeight = _this$tableStore4.styleMinHeight,
          styleHeight = _this$tableStore4.styleHeight,
          heightType = _this$tableStore4.heightType;

      if (isString(styleHeight) && isPercentSize(styleHeight) || isString(styleMaxHeight) && isPercentSize(styleMaxHeight) || isString(styleMinHeight) && isPercentSize(styleMinHeight)) {
        var wrapper = this.wrapper;

        if (wrapper) {
          var parentNode = wrapper.parentNode;

          if (parentNode) {
            var resizeObserver = new ResizeObserver(this.handleParentResize);
            resizeObserver.observe(parentNode);
            this.resizeObserver = resizeObserver;
            this.syncParentSize(parentNode.offsetHeight, parentNode);
          }
        }
      }

      if (heightType === TableHeightType.flex || isString(styleMaxHeight) && isCalcSize(styleMaxHeight) || isString(styleMinHeight) && isCalcSize(styleMinHeight)) {
        window.addEventListener('resize', this.handleWindowResize, false);
      }
    }
  }, {
    key: "disconnect",
    value: function disconnect() {
      var resizeObserver = this.resizeObserver;

      if (resizeObserver) {
        resizeObserver.disconnect();
        delete this.resizeObserver;
      }

      this.processDataSetListener(false);
      window.removeEventListener('resize', this.handleWindowResize, false);
    }
  }, {
    key: "processDataSetListener",
    value: function processDataSetListener(flag) {
      var _this$tableStore5 = this.tableStore,
          dataSet = _this$tableStore5.dataSet,
          inlineEdit = _this$tableStore5.inlineEdit;

      if (dataSet) {
        var handler = flag ? dataSet.addEventListener : dataSet.removeEventListener;
        handler.call(dataSet, DataSetEvents.load, this.handleDataSetLoad);

        if (inlineEdit) {
          handler.call(dataSet, DataSetEvents.create, this.handleDataSetCreate);
        }

        handler.call(dataSet, DataSetEvents.validate, this.handleDataSetValidate);
        handler.call(dataSet, DataSetEvents.validateSelf, this.handleDataSetValidateSelf);
        handler.call(dataSet, DataSetEvents.reset, this.handleDataSetReset);
        handler.call(dataSet, DataSetEvents.remove, this.handleDataSetRemove);
      }
    }
  }, {
    key: "render",
    value: function render() {
      var _this$tableStore6 = this.tableStore,
          overflowX = _this$tableStore6.overflowX,
          isAnyColumnsLeftLock = _this$tableStore6.isAnyColumnsLeftLock,
          isAnyColumnsRightLock = _this$tableStore6.isAnyColumnsRightLock,
          _this$props4 = this.props,
          dataSet = _this$props4.dataSet,
          treeQueryExpanded = _this$props4.treeQueryExpanded,
          virtualSpin = _this$props4.virtualSpin,
          buttons = _this$props4.buttons,
          buttonsLimit = _this$props4.buttonsLimit,
          queryFields = _this$props4.queryFields,
          queryFieldsLimit = _this$props4.queryFieldsLimit,
          summaryFieldsLimit = _this$props4.summaryFieldsLimit,
          summaryBarFieldWidth = _this$props4.summaryBarFieldWidth,
          filterBarFieldName = _this$props4.filterBarFieldName,
          filterBarPlaceholder = _this$props4.filterBarPlaceholder,
          summaryBar = _this$props4.summaryBar,
          dynamicFilterBar = _this$props4.dynamicFilterBar,
          clientExportQuantity = _this$props4.clientExportQuantity,
          indentSize = _this$props4.indentSize,
          selectionMode = _this$props4.selectionMode,
          rowRenderer = _this$props4.rowRenderer,
          onRow = _this$props4.onRow,
          onColumnResize = _this$props4.onColumnResize,
          expandedRowRenderer = _this$props4.expandedRowRenderer,
          expandRowByClick = _this$props4.expandRowByClick,
          rowDragRender = _this$props4.rowDragRender,
          columnsDragRender = _this$props4.columnsDragRender,
          mode = _this$props4.mode,
          pristine = _this$props4.pristine,
          showSelectionCachedButton = _this$props4.showSelectionCachedButton,
          onShowCachedSelectionChange = _this$props4.onShowCachedSelectionChange,
          autoMaxWidth = _this$props4.autoMaxWidth,
          summary = _this$props4.summary,
          searchCode = _this$props4.searchCode,
          boxSizing = _this$props4.boxSizing,
          fullColumnWidth = _this$props4.fullColumnWidth,
          tableStore = this.tableStore,
          prefixCls = this.prefixCls;
      var content = this.getTable();
      var pagination = this.getPagination(TablePaginationPosition.top);
      var globalTableSpinProps = this.getContextConfig('tableSpinProps');
      var localTableSpinProps = this.getSpinProps();
      var tableSpinProps = globalTableSpinProps ? mergeProps(mergeProps({}, globalTableSpinProps), localTableSpinProps) : localTableSpinProps;
      var tableButtonsLimit = isNil(buttonsLimit) ? this.getContextConfig('tableButtonsLimit') : buttonsLimit;
      return /*#__PURE__*/React.createElement(ReactResizeObserver, {
        resizeProp: "width",
        onResize: this.handleResize
      }, /*#__PURE__*/React.createElement("div", _extends({}, this.getWrapperProps()), /*#__PURE__*/React.createElement(TableContextProvider, {
        code: this.code,
        prefixCls: prefixCls,
        dataSet: dataSet,
        tableStore: tableStore,
        indentSize: indentSize,
        selectionMode: selectionMode,
        onRow: onRow,
        onColumnResize: onColumnResize,
        rowRenderer: rowRenderer,
        expandedRowRenderer: expandedRowRenderer,
        expandRowByClick: expandRowByClick,
        rowDragRender: rowDragRender,
        columnsDragRender: columnsDragRender,
        showSelectionCachedButton: showSelectionCachedButton,
        onShowCachedSelectionChange: onShowCachedSelectionChange,
        autoMaxWidth: autoMaxWidth,
        pristine: pristine,
        summary: summary,
        virtualSpin: virtualSpin,
        spinProps: tableSpinProps,
        fullColumnWidth: fullColumnWidth,
        isTree: mode === TableMode.tree
      }, /*#__PURE__*/React.createElement(TableSibling, {
        position: "before",
        boxSizing: boxSizing
      }, this.getHeader(), /*#__PURE__*/React.createElement(TableQueryBar, {
        buttons: buttons,
        buttonsLimit: tableButtonsLimit,
        pagination: pagination,
        queryFields: queryFields,
        clientExportQuantity: clientExportQuantity,
        summaryBar: summaryBar,
        dynamicFilterBar: dynamicFilterBar,
        queryFieldsLimit: queryFieldsLimit,
        summaryBarFieldWidth: summaryBarFieldWidth,
        summaryFieldsLimit: summaryFieldsLimit,
        filterBarFieldName: filterBarFieldName,
        filterBarPlaceholder: filterBarPlaceholder,
        treeQueryExpanded: treeQueryExpanded,
        searchCode: searchCode
      }), /*#__PURE__*/React.createElement(ErrorBar, {
        dataSet: dataSet,
        prefixCls: prefixCls
      })), /*#__PURE__*/React.createElement(Spin, _extends({}, tableSpinProps, {
        key: "content"
      }), /*#__PURE__*/React.createElement("div", _extends({}, this.getOtherProps()), /*#__PURE__*/React.createElement("div", {
        className: classNames("".concat(prefixCls, "-content"), _defineProperty({}, "".concat(prefixCls, "-content-overflow"), isStickySupport() && overflowX && tableStore.height === undefined)),
        ref: this.saveContentRef,
        onScroll: this.handleBodyScroll
      }, !isStickySupport() && isAnyColumnsLeftLock && overflowX && this.getLeftFixedTable(), content, !isStickySupport() && isAnyColumnsRightLock && overflowX && this.getRightFixedTable()), isStickySupport() && overflowX && /*#__PURE__*/React.createElement(StickyShadow, {
        position: "left"
      }), isStickySupport() && overflowX && /*#__PURE__*/React.createElement(StickyShadow, {
        position: "right"
      }), /*#__PURE__*/React.createElement("div", {
        ref: this.saveResizeRef,
        className: "".concat(prefixCls, "-split-line")
      }))), /*#__PURE__*/React.createElement(TableSibling, {
        position: "after",
        boxSizing: boxSizing
      }, this.getFooter(), this.getPagination(TablePaginationPosition.bottom)))));
    }
  }, {
    key: "reorderDataSet",
    value: function reorderDataSet(startIndex, endIndex) {
      var dataSet = this.tableStore.dataSet;
      dataSet.move(startIndex, endIndex);
    }
    /**
     * 触发组合或拖拽排序, 移除原纪录
     * @param currentRecord
     */

  }, {
    key: "removeSourceRecord",
    value: function removeSourceRecord(currentRecord) {
      var parent = currentRecord.parent;

      if (parent) {
        var children = parent.children;

        if (children) {
          var index = children.indexOf(currentRecord);

          if (index !== -1) {
            children.splice(index, 1);
          }

          if (!children.length) {
            parent.children = undefined;
          }
        }
      }
    }
  }, {
    key: "handleDragStart",
    value: function handleDragStart(initial, provided) {
      var dragDropContextProps = this.props.dragDropContextProps;
      var _this$tableStore7 = this.tableStore,
          rowMetaData = _this$tableStore7.rowMetaData,
          isTree = _this$tableStore7.isTree,
          currentEditorName = _this$tableStore7.currentEditorName;

      if (isTree && rowMetaData) {
        var source = initial.source;
        var currentRecord = rowMetaData[source.index].record;

        if (currentRecord && currentRecord.children && currentRecord.children.length) {
          currentRecord.isExpanded = false;
        }
      }

      if (dragDropContextProps && dragDropContextProps.onDragStart) {
        dragDropContextProps.onDragStart(initial, provided);
      }

      if (currentEditorName) {
        this.tableStore.blurEditor();
      }
    }
  }, {
    key: "handleDragEnd",
    value: function handleDragEnd(resultDrag, provided) {
      var _rowMetaData$resultBe, _rowMetaData$resultBe2;

      var _this$tableStore8 = this.tableStore,
          dataSet = _this$tableStore8.dataSet,
          rowMetaData = _this$tableStore8.rowMetaData,
          isTree = _this$tableStore8.isTree,
          showRemovedRow = _this$tableStore8.showRemovedRow;
      var _dataSet$props = dataSet.props,
          parentField = _dataSet$props.parentField,
          idField = _dataSet$props.idField,
          childrenField = _dataSet$props.childrenField;
      var _this$props5 = this.props,
          onDragEnd = _this$props5.onDragEnd,
          onDragEndBefore = _this$props5.onDragEndBefore,
          dragDropContextProps = _this$props5.dragDropContextProps;
      var resultBefore = resultDrag;
      var recordFromIndex = rowMetaData ? (_rowMetaData$resultBe = rowMetaData[resultBefore.source.index].record) === null || _rowMetaData$resultBe === void 0 ? void 0 : _rowMetaData$resultBe.index : undefined;
      var recordToIndex = rowMetaData && resultBefore.destination ? (_rowMetaData$resultBe2 = rowMetaData[resultBefore.destination.index].record) === null || _rowMetaData$resultBe2 === void 0 ? void 0 : _rowMetaData$resultBe2.index : undefined;

      if (onDragEndBefore) {
        var result = onDragEndBefore(this.tableStore.dataSet, toJS(this.tableStore.columns), resultDrag, provided, [recordFromIndex, recordToIndex]);

        if (result === false) {
          return;
        }

        if (isDropresult(result)) {
          resultBefore = result;
        }
      }

      if (resultBefore && isTree && rowMetaData) {
        var _resultBefore = resultBefore,
            destination = _resultBefore.destination,
            source = _resultBefore.source,
            combine = _resultBefore.combine;
        var currentRecord = rowMetaData[source.index].record;

        if (currentRecord) {
          // 平铺数据
          if (!childrenField && parentField && idField) {
            // 拖拽组合关联
            if (combine) {
              var parentRecord = dataSet.find(function (record) {
                return String(record.key) === combine.draggableId;
              });
              currentRecord.set(parentField, parentRecord.get(idField));
              parentRecord.isExpanded = true;
            } // 拖拽排序更新


            if (destination && destination.index !== source.index && rowMetaData) {
              var destinationRecord = rowMetaData[destination.index].record;

              if (destinationRecord) {
                var parent = destinationRecord.parent;

                if (parent) {
                  if (currentRecord.parent !== parent && parent !== currentRecord && parent.children) {
                    currentRecord.set(parentField, parent.get(idField));
                  } else if (parent.children) {
                    var childIndex = parent.children.indexOf(destinationRecord);
                    this.removeSourceRecord(currentRecord);
                    parent.children.splice(childIndex, 0, currentRecord);
                  }
                } else {
                  currentRecord.set(parentField, undefined);
                  dataSet.move(currentRecord.index, destinationRecord.index);
                }
              }
            }
          } // 树形数据


          if (childrenField) {
            // 拖拽组合关联
            if (combine) {
              var _parentRecord = dataSet.find(function (record) {
                return String(record.key) === combine.draggableId;
              });

              this.removeSourceRecord(currentRecord);
              currentRecord.parent = _parentRecord;

              if (_parentRecord.children && _parentRecord.children.length) {
                _parentRecord.children.unshift(currentRecord);
              } else {
                _parentRecord.children = [currentRecord];
              }

              _parentRecord.isExpanded = true;
            } // 拖拽排序更新


            if (destination && destination.index !== source.index && rowMetaData) {
              var _destinationRecord = rowMetaData[destination.index].record;

              if (_destinationRecord) {
                var _parent = _destinationRecord.parent;

                if (_parent) {
                  if (currentRecord.parent !== _parent && _parent !== currentRecord && _parent.children) {
                    var _childIndex = _parent.children.indexOf(_destinationRecord);

                    this.removeSourceRecord(currentRecord);
                    currentRecord.parent = _parent;

                    _parent.children.splice(_childIndex, 0, currentRecord);
                  } else if (_parent.children) {
                    var _childIndex2 = _parent.children.indexOf(_destinationRecord);

                    this.removeSourceRecord(currentRecord);

                    _parent.children.splice(_childIndex2, 0, currentRecord);
                  }
                } else {
                  if (currentRecord.parent) {
                    this.removeSourceRecord(currentRecord);
                    currentRecord.parent = undefined;
                  }

                  dataSet.move(currentRecord.index, _destinationRecord.index);
                }
              }
            }
          }
        }
      } else if (resultBefore && resultBefore.destination) {
        if (resultBefore.source.index !== resultBefore.destination.index) {
          // 非树形拖拽排序
          var records = dataSet.records,
              data = dataSet.data; // 若存在没有展示的 delete 数据，则需要对数据的起点落点做校正

          if (!showRemovedRow && records.length !== data.length) {
            var destinationIndex = resultBefore.destination.index; // 操作的数据在 records 中的真实起落点索引

            var recordsSourceIndex;
            var recordsToIndex;
            var findSourceFlag = false;
            var findDestinationFlag = false;
            var showedRecordsCount = 0;

            for (var i = 0; i < records.length; i++) {
              if (!findSourceFlag && records[i].key === resultBefore.draggableId) {
                recordsSourceIndex = i;
                findSourceFlag = true;
              }

              if (!findDestinationFlag) {
                if (records[i].status !== RecordStatus["delete"]) {
                  if (showedRecordsCount === destinationIndex) {
                    recordsToIndex = destinationIndex + (i - showedRecordsCount);
                    findDestinationFlag = true;
                  }

                  showedRecordsCount++;
                }
              }

              if (findSourceFlag && findDestinationFlag) {
                break;
              }
            }

            this.reorderDataSet(recordsSourceIndex, recordsToIndex);
          } else {
            this.reorderDataSet(resultBefore.source.index, resultBefore.destination.index);
          }
        }
      }
      /**
       * 相应变化后的数据
       */


      var onDragEndFun = onDragEnd || (dragDropContextProps ? dragDropContextProps.onDragEnd : undefined);

      if (onDragEndFun) {
        onDragEndFun(this.tableStore.dataSet, toJS(this.tableStore.columns), resultBefore, provided);
      }
    }
  }, {
    key: "handleBodyScroll",
    value: function handleBodyScroll(e) {
      var _this2 = this;

      var currentTarget = e.currentTarget;

      var handle = function handle() {
        _this2.handleBodyScrollTop(e, currentTarget);

        _this2.handleBodyScrollLeft(e, currentTarget);
      };

      if (isStickySupport()) {
        handle();
      } else {
        e.persist();
        this.scrollId = raf(handle);
      }
    }
  }, {
    key: "handleBodyScrollTop",
    value: function handleBodyScrollTop(e, currentTarget) {
      var target = e.target;
      var tableStore = this.tableStore;

      if ( // (isStickySupport() && !tableStore.virtual) ||
      // (![TableHeightType.fixed, TableHeightType.flex].includes(tableStore.heightType)) ||
      !tableStore.overflowY || currentTarget !== target || target === this.tableFootWrap) {
        return;
      }

      var scrollTop = target.scrollTop;
      this.setScrollTop(scrollTop, target);
    }
  }, {
    key: "handleBodyScrollLeft",
    value: function handleBodyScrollLeft(e, currentTarget) {
      var target = e.target;
      var tableStore = this.tableStore;

      if (!tableStore.overflowX || currentTarget !== target || target === this.fixedColumnsBodyRight || target === this.fixedColumnsBodyLeft) {
        return;
      }

      var scrollLeft = target.scrollLeft;
      this.setScrollLeft(scrollLeft, target);
    }
  }, {
    key: "setScrollPositionClassName",
    value: function setScrollPositionClassName(target) {
      var tableStore = this.tableStore;

      if (tableStore.isAnyColumnsLock && tableStore.overflowX) {
        var node = target || this.tableBodyWrap || this.tableContentWrap;

        if (node) {
          var scrollToLeft = node.scrollLeft === 0;
          var table = node.querySelector('table');
          var scrollToRight = table && node.scrollLeft >= table.offsetWidth - node.offsetWidth;

          if (scrollToLeft && scrollToRight) {
            this.setScrollPosition(ScrollPosition.both);
          } else if (scrollToLeft) {
            this.setScrollPosition(ScrollPosition.left);
          } else if (scrollToRight) {
            this.setScrollPosition(ScrollPosition.right);
          } else {
            this.setScrollPosition(ScrollPosition.middle);
          }
        }
      }
    }
  }, {
    key: "setScrollPosition",
    value: function setScrollPosition(position) {
      var tableStore = this.tableStore;

      if (tableStore.scrollPosition !== position) {
        tableStore.scrollPosition = position;
      }
    }
  }, {
    key: "renderTable",
    value: function renderTable(hasHeader, hasBody, hasFooter, lock, virtual) {
      var _this3 = this;

      var tableStore = this.tableStore;

      var columnGroups = function () {
        if (lock === ColumnLock.right) {
          return tableStore.rightColumnGroups;
        }

        if (lock) {
          return tableStore.leftColumnGroups;
        }

        return tableStore.columnGroups;
      }();
      /* eslint-disable react/no-this-in-sfc */


      var wrapper = function wrapper(snapshot, dragRowHeight) {
        return /*#__PURE__*/React.createElement(TableWrapper, {
          key: "tableWrapper",
          lock: lock,
          hasBody: hasBody,
          hasHeader: hasHeader,
          hasFooter: hasFooter,
          columnGroups: columnGroups
        }, hasHeader && _this3.getTableHeader(lock), hasBody && _this3.getTableBody(columnGroups, lock, snapshot, dragRowHeight), hasFooter && _this3.getTableFooter(columnGroups, lock));
      };
      /* eslint-enable react/no-this-in-sfc */


      var virtualWrapper = virtual ? /*#__PURE__*/React.createElement(VirtualWrapper, null, wrapper) : wrapper();

      if (hasBody && tableStore.rowDraggable) {
        var dragDropContextProps = this.props.dragDropContextProps;
        return /*#__PURE__*/React.createElement(DragDropContext, _extends({}, dragDropContextProps, {
          onDragStart: this.handleDragStart,
          onDragEnd: this.handleDragEnd
        }), virtualWrapper);
      }

      return virtualWrapper;
    }
  }, {
    key: "getHeader",
    value: function getHeader() {
      var _this$props6 = this.props,
          header = _this$props6.header,
          dataSet = _this$props6.dataSet;

      if (header) {
        var prefixCls = this.prefixCls;
        var data = dataSet ? dataSet.records : [];
        return /*#__PURE__*/React.createElement("div", {
          key: "header",
          className: "".concat(prefixCls, "-header")
        }, typeof header === 'function' ? header(data) : header);
      }
    }
  }, {
    key: "getFooter",
    value: function getFooter() {
      var _this$props7 = this.props,
          footer = _this$props7.footer,
          dataSet = _this$props7.dataSet;

      if (footer) {
        var prefixCls = this.prefixCls;
        var data = dataSet ? dataSet.records : [];
        return /*#__PURE__*/React.createElement("div", {
          key: "footer",
          className: "".concat(prefixCls, "-footer")
        }, typeof footer === 'function' ? footer(data) : footer);
      }
    }
  }, {
    key: "getPagination",
    value: function getPagination(position) {
      var _this$props8 = this.props,
          dataSet = _this$props8.dataSet,
          selectionMode = _this$props8.selectionMode,
          prefixCls = this.prefixCls,
          _this$tableStore9 = this.tableStore,
          pagination = _this$tableStore9.pagination,
          showSelectionTips = _this$tableStore9.showSelectionTips,
          showCachedTips = _this$tableStore9.showCachedTips;

      if (pagination !== false && dataSet && dataSet.paging) {
        var paginationPosition = getPaginationPosition(pagination);

        if (paginationPosition === TablePaginationPosition.both || paginationPosition === position) {
          var paginationProps = omit(pagination, 'position');
          return /*#__PURE__*/React.createElement(Pagination, _extends({
            key: "pagination-".concat(position)
          }, paginationProps, {
            className: classNames("".concat(prefixCls, "-pagination"), paginationProps.className, _defineProperty({}, "".concat(prefixCls, "-pagination-with-selection-tips"), showSelectionTips || showCachedTips)),
            dataSet: dataSet
          }), selectionMode !== SelectionMode.none && dataSet.selection === DataSetSelection.multiple && /*#__PURE__*/React.createElement(SelectionTips, null));
        }
      }
    }
  }, {
    key: "getTable",
    value: function getTable(lock) {
      var _this4 = this;

      var props = this.props,
          tableStore = this.tableStore;
      var overflowX = tableStore.overflowX,
          heightType = tableStore.heightType,
          footer = tableStore.hasFooter,
          virtual = tableStore.virtual;
      var tableHead;
      var tableBody;
      var tableFooter;

      if (virtual || !isStickySupport() && overflowX || [TableHeightType.flex, TableHeightType.fixed].includes(heightType) || tableStore.height !== undefined) {
        var prefixCls = this.prefixCls;
        var tableHeadRef;
        var tableBodyRef;
        var tableFootRef;

        if (!lock) {
          tableHeadRef = function tableHeadRef(node) {
            return _this4.tableHeadWrap = node;
          };

          tableFootRef = function tableFootRef(node) {
            return _this4.tableFootWrap = node;
          };

          tableBodyRef = function tableBodyRef(node) {
            return _this4.tableBodyWrap = node;
          };
        } else if (lock === ColumnLock.right) {
          tableBodyRef = function tableBodyRef(node) {
            _this4.fixedColumnsBodyRight = node;

            if (node) {
              _this4.syncSize();
            }
          };
        } else {
          tableBodyRef = function tableBodyRef(node) {
            _this4.fixedColumnsBodyLeft = node;

            if (node) {
              _this4.syncSize();
            }
          };
        }

        var style = lock ? {
          width: pxToRem(lock === ColumnLock.right ? tableStore.rightColumnGroups.width - 1 + (tableStore.overflowY ? measureScrollbar() : 0) : tableStore.leftColumnGroups.width, true),
          marginLeft: lock === ColumnLock.right ? pxToRem(1) : undefined
        } : undefined;
        tableHead = /*#__PURE__*/React.createElement("div", {
          key: "tableHead",
          ref: tableHeadRef,
          className: "".concat(prefixCls, "-head"),
          style: style
        }, this.renderTable(true, false, false, lock));

        if (lock !== ColumnLock.right || !onlyCustomizedColumn(tableStore)) {
          var body = this.renderTable(false, true, false, lock, virtual);
          tableBody = /*#__PURE__*/React.createElement(TableBody, {
            key: "tableBody",
            getRef: tableBodyRef,
            lock: lock,
            onScroll: this.handleBodyScroll,
            style: pick(props.style, ['maxHeight', 'minHeight'])
          }, body);
        }

        if (footer) {
          tableFooter = /*#__PURE__*/React.createElement("div", {
            key: "tableFooter",
            ref: tableFootRef,
            className: "".concat(prefixCls, "-foot"),
            style: style,
            onScroll: this.handleBodyScroll
          }, this.renderTable(false, false, true, lock));
        }
      } else {
        tableBody = this.renderTable(true, true, footer, lock);
      }

      return [tableHead, tableBody, tableFooter];
    }
  }, {
    key: "getLeftFixedTable",
    value: function getLeftFixedTable() {
      return /*#__PURE__*/React.createElement(StickyShadow, {
        position: "left"
      }, this.getTable(ColumnLock.left));
    }
  }, {
    key: "getRightFixedTable",
    value: function getRightFixedTable() {
      return /*#__PURE__*/React.createElement(StickyShadow, {
        position: "right"
      }, this.getTable(ColumnLock.right));
    }
  }, {
    key: "getTableBody",
    value: function getTableBody(columnGroups, lock, snapshot, dragRowHeight) {
      var performanceEnabled = this.tableStore.performanceEnabled,
          bodyExpandable = this.props.bodyExpandable;
      var tBodyProps = {
        key: 'tbody',
        lock: lock,
        columnGroups: columnGroups,
        snapshot: snapshot,
        dragRowHeight: dragRowHeight
      };
      var body = bodyExpandable ? /*#__PURE__*/React.createElement(ExpandableTableTBody, _extends({}, tBodyProps)) : /*#__PURE__*/React.createElement(TableTBody, _extends({}, tBodyProps));
      return performanceEnabled ? /*#__PURE__*/React.createElement(Profiler, null, body) : body;
    }
  }, {
    key: "getTableHeader",
    value: function getTableHeader(lock) {
      var _this$props9 = this.props,
          showHeader = _this$props9.showHeader,
          queryFields = _this$props9.queryFields;
      return showHeader || this.tableStore.customizable ? /*#__PURE__*/React.createElement(TableHeader, {
        key: "thead",
        lock: lock,
        queryFields: queryFields
      }) : undefined;
    }
  }, {
    key: "getTableFooter",
    value: function getTableFooter(columnGroups, lock) {
      return /*#__PURE__*/React.createElement(TableFooter, {
        key: "tfoot",
        lock: lock,
        columnGroups: columnGroups
      });
    }
  }, {
    key: "getStyleHeight",
    value: function getStyleHeight() {
      var style = this.props.style;

      if (style) {
        return toPx(style.height);
      }
    }
  }, {
    key: "syncSizeInFrame",
    value: function syncSizeInFrame() {
      var width = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.getWidth();

      if (this.nextFrameActionId !== undefined) {
        raf.cancel(this.nextFrameActionId);
      }

      this.nextFrameActionId = raf(this.syncSize.bind(this, width));
    }
  }, {
    key: "syncSize",
    value: function syncSize() {
      var width = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.getWidth();
      var element = this.element,
          tableStore = this.tableStore;

      if (tableStore.hidden || !element || !element.offsetParent) {
        return;
      }

      tableStore.width = Math.floor(width);
      var prefixCls = this.prefixCls;
      var tableHeader = tableStore.props.showHeader ? element.querySelector(".".concat(prefixCls, "-thead")) : null;
      var tableFooter = tableStore.hasFooter ? element.querySelector(".".concat(prefixCls, "-tfoot")) : null;

      if (tableStore.cellVerticalSize === undefined) {
        tableStore.cellVerticalSize = getCellVerticalSize(element, prefixCls);
      }

      tableStore.screenHeight = document.documentElement.clientHeight;
      tableStore.headerHeight = tableHeader ? getHeight(tableHeader) : 0;
      tableStore.footerHeight = tableFooter ? getHeight(tableFooter) : 0;
      this.setScrollPositionClassName();
    }
  }, {
    key: "initDefaultExpandedRows",
    value: function initDefaultExpandedRows() {
      var tableStore = this.tableStore,
          _this$props10 = this.props,
          dataSet = _this$props10.dataSet,
          defaultRowExpanded = _this$props10.defaultRowExpanded;

      if (tableStore.isTree && defaultRowExpanded) {
        dataSet.forEach(function (record) {
          if (record.children) {
            record.isExpanded = true;
          }
        });
      }
    }
  }, {
    key: "handleHeightTypeChange",
    value: function handleHeightTypeChange(immediate) {
      this.disconnect();

      if (immediate) {
        this.syncSize();
      } else {
        this.syncSizeInFrame();
      }

      this.connect();
    }
  }, {
    key: "getWidth",
    value: function getWidth() {
      var wrapper = this.wrapper;

      if (wrapper) {
        return wrapper.offsetWidth;
      }

      return 0;
    }
  }, {
    key: "getHeaderGroups",
    value: function getHeaderGroups() {
      return this.tableStore.groupedDataWithHeader;
    }
  }, {
    key: "getGroups",
    value: function getGroups() {
      return this.tableStore.groupedData;
    }
  }, {
    key: "setColumnWidth",
    value: function setColumnWidth(width, indexOrKeyOrName) {
      var saveToCustomization = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
      var tableStore = this.tableStore;
      var target = tableStore.findColumnGroup(indexOrKeyOrName);

      if (target) {
        tableStore.setColumnWidth(target, width, saveToCustomization);
      }
    }
  }, {
    key: "setScrollTop",
    value: function setScrollTop(scrollTop, target) {
      if (scrollTop !== this.lastScrollTop) {
        var tableStore = this.tableStore,
            fixedColumnsBodyLeft = this.fixedColumnsBodyLeft,
            tableBodyWrap = this.tableBodyWrap,
            fixedColumnsBodyRight = this.fixedColumnsBodyRight,
            tableContentWrap = this.tableContentWrap;

        if (fixedColumnsBodyLeft && target !== fixedColumnsBodyLeft) {
          fixedColumnsBodyLeft.scrollTop = scrollTop;
        }

        if (tableBodyWrap && target !== tableBodyWrap) {
          tableBodyWrap.scrollTop = scrollTop;
        }

        if (fixedColumnsBodyRight && target !== fixedColumnsBodyRight) {
          fixedColumnsBodyRight.scrollTop = scrollTop;
        }

        if (tableContentWrap && target !== tableContentWrap) {
          tableContentWrap.scrollTop = scrollTop;
        }

        this.lastScrollTop = scrollTop;
        tableStore.setLastScrollTop(scrollTop);

        if (target) {
          var onScrollTop = this.props.onScrollTop;

          if (onScrollTop) {
            onScrollTop(scrollTop, function () {
              var groupedData = tableStore.groupedData,
                  rowMetaData = tableStore.rowMetaData;
              var start = -1;
              var end = -1;
              var index = 0;

              if (rowMetaData) {
                rowMetaData.some(function (meta) {
                  var offset = meta.offset,
                      height = meta.height;

                  if (start === -1 && offset + height >= scrollTop) {
                    start = index;
                  }

                  if (end === -1 && offset >= target.clientHeight + scrollTop) {
                    end = index;
                    return true;
                  }

                  index += 1;
                  return false;
                });
              } else {
                var trs = Array.from(target.querySelectorAll('tr'));
                trs.some(function (tr) {
                  var offsetTop = tr.offsetTop,
                      offsetHeight = tr.offsetHeight;

                  if (start === -1 && offsetTop + offsetHeight >= scrollTop) {
                    start = index;
                  }

                  if (end === -1 && offsetTop >= target.clientHeight + scrollTop) {
                    end = index;
                    return true;
                  }

                  index += 1;
                  return false;
                });
              }

              if (end === -1) {
                end = index;
              }

              return {
                groups: groupedData,
                start: start,
                end: end
              };
            });
          }
        }
      }
    }
  }, {
    key: "setScrollLeft",
    value: function setScrollLeft(scrollLeft, target) {
      if (scrollLeft !== this.lastScrollLeft) {
        var tableStore = this.tableStore;
        var tableHeadWrap = this.tableHeadWrap,
            tableBodyWrap = this.tableBodyWrap,
            tableFootWrap = this.tableFootWrap,
            tableContentWrap = this.tableContentWrap;

        if (isStickySupport()) {
          tableStore.editors.forEach(function (editor) {
            if (editor.lock && editor.cellNode) {
              if (tableStore.inlineEdit) {
                editor.alignEditor(editor.cellNode);
              } else {
                editor.hideEditor();
              }
            }
          });
        }

        if (tableHeadWrap && target !== tableHeadWrap) {
          tableHeadWrap.scrollLeft = scrollLeft;
        }

        if (tableBodyWrap && target !== tableBodyWrap) {
          tableBodyWrap.scrollLeft = scrollLeft;
        }

        if (tableFootWrap && target !== tableFootWrap) {
          tableFootWrap.scrollLeft = scrollLeft;
        }

        if (tableContentWrap && target !== tableContentWrap) {
          tableContentWrap.scrollLeft = scrollLeft;
        }

        this.setScrollPositionClassName(target);
        this.lastScrollLeft = scrollLeft;
        tableStore.setLastScrollLeft(scrollLeft);

        if (target) {
          var onScrollLeft = this.props.onScrollLeft;

          if (onScrollLeft) {
            onScrollLeft(scrollLeft, function () {
              var groupedDataWithHeader = tableStore.groupedDataWithHeader,
                  columnGroups = tableStore.columnGroups;
              var start = -1;
              var end = -1;
              var leftLocks = 0;
              var leftWidth = 0;
              var index = 0;
              var rightWidth = tableStore.rightColumnGroups.columns.reduce(function (width, column) {
                return column.key === CUSTOMIZED_KEY ? width : width + column.width;
              }, 0);
              columnGroups.columns.some(function (colGroup) {
                if (colGroup.lock !== ColumnLock.right) {
                  if (colGroup.lock === ColumnLock.left) {
                    leftLocks += 1;
                    leftWidth += colGroup.width;
                  } else {
                    if (start === -1 && colGroup.left + colGroup.width - leftWidth >= scrollLeft) {
                      start = index;
                    }

                    if (end === -1 && colGroup.left >= target.clientWidth - rightWidth + scrollLeft) {
                      end = index;
                      return true;
                    }
                  }

                  index += 1;
                }

                return false;
              });

              if (end === -1) {
                end = index;
              }

              return {
                groups: groupedDataWithHeader,
                start: start - leftLocks,
                end: end - leftLocks
              };
            });
          }
        }
      }
    }
  }]);

  return Table;
}(DataSetComponent);

Table.displayName = 'Table';
Table.Column = Column;
Table.FilterBar = FilterBar;
Table.AdvancedQueryBar = AdvancedQueryBar;
Table.ProfessionalBar = ProfessionalBar;
Table.ComboBar = ComboBar;
Table.DynamicFilterBar = DynamicFilterBar;
Table.FilterSelect = FilterSelect;
Table.ToolBar = ToolBar;
Table.TableRow = TableRow;
Table.TableHeaderCell = TableHeaderCell;
Table.defaultProps = {
  suffixCls: 'table',
  tabIndex: 0,
  selectionMode: SelectionMode.rowbox,
  rowBoxPlacement: RowBoxPlacement.start,
  queryFields: {},
  defaultRowExpanded: false,
  expandRowByClick: false,
  indentSize: 15,
  summaryFieldsLimit: 3,
  summaryBarFieldWidth: 170,
  filterBarFieldName: 'params',
  virtualSpin: false,
  autoHeight: false,
  autoMaxWidth: true,
  autoFootHeight: false,
  clientExportQuantity: 100,
  showSelectionCachedButton: true,
  showHeader: true,
  fullColumnWidth: true
};

__decorate([autobind], Table.prototype, "saveResizeRef", null);

__decorate([autobind], Table.prototype, "saveContentRef", null);

__decorate([autobind, action], Table.prototype, "handleResize", null);

__decorate([autobind], Table.prototype, "handleWindowResize", null);

__decorate([autobind, action], Table.prototype, "handleDataSetLoad", null);

__decorate([autobind], Table.prototype, "handleDataSetCreate", null);

__decorate([autobind], Table.prototype, "handleDataSetReset", null);

__decorate([autobind], Table.prototype, "handleDataSetRemove", null);

__decorate([autobind], Table.prototype, "handleDataSetValidateSelf", null);

__decorate([autobind], Table.prototype, "handleDataSetValidate", null);

__decorate([autobind], Table.prototype, "handleKeyDown", null);

__decorate([action], Table.prototype, "syncParentSize", null);

__decorate([autobind], Table.prototype, "handleParentResize", null);

__decorate([action], Table.prototype, "reorderDataSet", null);

__decorate([action], Table.prototype, "removeSourceRecord", null);

__decorate([autobind, action], Table.prototype, "handleDragStart", null);

__decorate([autobind, action], Table.prototype, "handleDragEnd", null);

__decorate([autobind], Table.prototype, "handleBodyScroll", null);

__decorate([action], Table.prototype, "setScrollPosition", null);

__decorate([autobind, action], Table.prototype, "syncSize", null);

__decorate([action], Table.prototype, "initDefaultExpandedRows", null);

Table = __decorate([observer], Table);
export default Table;
//# sourceMappingURL=Table.js.map
