import _classCallCheck from "@babel/runtime/helpers/classCallCheck";
import _createClass from "@babel/runtime/helpers/createClass";
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";
import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _objectSpread from "@babel/runtime/helpers/objectSpread2";
import _extends from "@babel/runtime/helpers/extends";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import { __decorate } from "tslib";
import React, { Children, isValidElement } from 'react';
import { action, computed, get as _get, observable, reaction, runInAction, set } from 'mobx';
import sortBy from 'lodash/sortBy';
import debounce from 'lodash/debounce';
import isNil from 'lodash/isNil';
import isObject from 'lodash/isObject';
import isPlainObject from 'lodash/isPlainObject';
import isString from 'lodash/isString';
import isNumber from 'lodash/isNumber';
import defaultTo from 'lodash/defaultTo';
import { warning } from 'choerodon-ui/dataset/utils';
import measureScrollbar from '../../../es/_util/measureScrollbar';
import { isCalcSize, scaleSize, toPx } from '../../../es/_util/UnitConvertor';
import Icon from '../../../es/icon';
import isFunction from 'lodash/isFunction';
import omit from 'lodash/omit';
import noop from 'lodash/noop';
import { ColumnDefaultProps, defaultAggregationRenderer } from './Column';
import CustomizationSettings from './customization-settings/CustomizationSettings';
import isFragment from '../_util/isFragment';
import ObserverCheckBox from '../check-box/CheckBox';
import ObserverRadio from '../radio/Radio';
import { DataSetSelection, FieldType, RecordCachedType } from '../data-set/enum';
import { ColumnAlign, ColumnLock, DragColumnAlign, GroupType, RowBoxPlacement, ScrollPosition, SelectionMode, TableAutoHeightType, TableBoxSizing, TableColumnTooltip, TableEditMode, TableHeightType, TableMode, TableQueryBarType } from './enum';
import { stopPropagation } from '../_util/EventManager';
import { getCachedSelectableCounts, getCachedSelectableRecords, getColumnKey, getCurrentSelectableCounts, getHeader, isDisabledRow } from './utils';
import getReactNodeText from '../_util/getReactNodeText';
import ColumnGroups from './ColumnGroups';
import autobind from '../_util/autobind';
import { Size } from '../core/enum';
import { $l } from '../locale-context';
import CustomizationColumnHeader from './customization-settings/CustomizationColumnHeader';
import ComboCustomizationSettings from './combo-customization-settings';
import Dropdown from '../dropdown/Dropdown';
import Menu from '../menu';
import Modal from '../modal/Modal';
import { treeMap, treeSome } from '../_util/treeUtils';
import { getIf, mergeGroupStates, normalizeGroups } from '../data-set/utils';
import BatchRunner from '../_util/BatchRunner';
import { LabelLayout } from '../form/enum';
export var SELECTION_KEY = '__selection-column__'; // TODO:Symbol

export var COMBOBAR_KEY = '__combo-column__'; // TODO:Symbol

export var ROW_NUMBER_KEY = '__row-number-column__'; // TODO:Symbol

export var DRAG_KEY = '__drag-column__'; // TODO:Symbol

export var EXPAND_KEY = '__expand-column__'; // TODO:Symbol

export var CUSTOMIZED_KEY = '__customized-column__'; // TODO:Symbol

export var AGGREGATION_EXPAND_CELL_KEY = '__aggregation-expand-cell__'; // TODO:Symbol

export var BODY_EXPANDED = '__body_expanded__'; // TODO:Symbol

export var VIRTUAL_ROOT_MARGIN = 50;

function columnFilter(column) {
  return Boolean(column);
}

function getOverScanCount(tableStore, index, next) {
  var rowMetaData = tableStore.rowMetaData;

  if (rowMetaData) {
    var count = 0;
    var height = 0;

    while (height < VIRTUAL_ROOT_MARGIN) {
      index += next ? 1 : -1;
      var metaData = rowMetaData[index];

      if (!metaData) {
        return count;
      }

      height += metaData.height;
      count++;
    }

    return count;
  }

  return Math.ceil(VIRTUAL_ROOT_MARGIN / tableStore.virtualRowHeight);
}

function getItemMetadata(rowMetaData, index, tableStore) {
  var lastMeasuredIndex = tableStore.lastMeasuredIndex;

  if (index > lastMeasuredIndex) {
    tableStore.lastMeasuredIndex = index;
  }

  return rowMetaData[index];
}

function findNearestItemBinarySearch(rowMetaData, tableStore, high, low, offset) {
  while (low <= high) {
    var middle = low + Math.floor((high - low) / 2);
    var item = getItemMetadata(rowMetaData, middle, tableStore);

    if (!item) {
      return 0;
    }

    var currentOffset = item.offset;

    if (currentOffset === offset) {
      return middle;
    }

    if (currentOffset < offset) {
      low = middle + 1;
    }

    if (currentOffset > offset) {
      high = middle - 1;
    }
  }

  if (low > 0) {
    return low - 1;
  }

  return 0;
}

function overOffset(rowMetaData, tableStore, index, offset) {
  var item = getItemMetadata(rowMetaData, index, tableStore);

  if (item) {
    return item.offset < offset;
  }

  return false;
}

function findNearestItemExponentialSearch(rowMetaData, tableStore, index, offset) {
  var virtualEstimatedRows = tableStore.virtualEstimatedRows;
  var interval = 1;

  while (index < virtualEstimatedRows && overOffset(rowMetaData, tableStore, index, offset)) {
    index += interval;
    interval *= 2;
  }

  return findNearestItemBinarySearch(rowMetaData, tableStore, Math.min(index, virtualEstimatedRows - 1), Math.floor(index / 2), offset);
}

function getVisibleStartIndex(tableStore) {
  var getLastScrollTop = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : function () {
    return tableStore.lastScrollTop;
  };
  var height = tableStore.height,
      virtualEstimatedRows = tableStore.virtualEstimatedRows;

  if (height === undefined || !virtualEstimatedRows) {
    return 0;
  }

  var lastScrollTop = getLastScrollTop();
  var virtualRowHeight = tableStore.virtualRowHeight,
      rowMetaData = tableStore.rowMetaData;

  if (rowMetaData) {
    if (!rowMetaData.length) {
      return 0;
    }

    var lastMeasuredIndex = tableStore.lastMeasuredIndex;
    var lastRowMetaData = lastMeasuredIndex > 0 ? rowMetaData[lastMeasuredIndex] : undefined;
    var lastMeasuredItemOffset = lastRowMetaData ? lastRowMetaData.offset : 0;

    if (lastMeasuredItemOffset >= lastScrollTop) {
      return findNearestItemBinarySearch(rowMetaData, tableStore, lastMeasuredIndex, 0, lastScrollTop);
    }

    return findNearestItemExponentialSearch(rowMetaData, tableStore, Math.max(0, lastMeasuredIndex), lastScrollTop);
  }

  return Math.max(0, Math.min(virtualEstimatedRows, Math.floor(lastScrollTop / virtualRowHeight)));
}

function getVisibleEndIndex(tableStore) {
  var getVirtualVisibleStartIndex = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : function () {
    return tableStore.virtualVisibleStartIndex;
  };
  var getLastScrollTop = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : function () {
    return tableStore.lastScrollTop;
  };
  var height = tableStore.height,
      virtualEstimatedRows = tableStore.virtualEstimatedRows;

  if (height === undefined) {
    return virtualEstimatedRows;
  }

  var virtualVisibleStartIndex = getVirtualVisibleStartIndex();
  var virtualRowHeight = tableStore.virtualRowHeight,
      rowMetaData = tableStore.rowMetaData;

  if (rowMetaData) {
    if (!rowMetaData.length) {
      return 0;
    }

    var itemMetadata = getItemMetadata(rowMetaData, virtualVisibleStartIndex, tableStore);

    if (!itemMetadata) {
      return 0;
    }

    var maxOffset = getLastScrollTop() + height;
    var offset = itemMetadata.offset + itemMetadata.height;
    var stopIndex = virtualVisibleStartIndex;

    while (stopIndex < virtualEstimatedRows - 1 && offset < maxOffset) {
      stopIndex++;
      var item = getItemMetadata(rowMetaData, stopIndex, tableStore);

      if (item) {
        offset += item.height;
      }
    }

    return Math.max(0, Math.min(virtualEstimatedRows, stopIndex + 1));
  }

  var numVisibleItems = Math.ceil(height / virtualRowHeight);
  return Math.max(0, Math.min(virtualEstimatedRows, virtualVisibleStartIndex + numVisibleItems));
}

function getStartIndex(tableStore) {
  var getVirtualVisibleStartIndex = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : function () {
    return tableStore.virtualVisibleStartIndex;
  };
  var virtualVisibleStartIndex = getVirtualVisibleStartIndex();
  return Math.max(0, virtualVisibleStartIndex - getOverScanCount(tableStore, virtualVisibleStartIndex));
}

function getEndIndex(tableStore) {
  var getVirtualVisibleEndIndex = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : function () {
    return tableStore.virtualVisibleEndIndex;
  };
  var virtualVisibleEndIndex = getVirtualVisibleEndIndex();
  var virtualEstimatedRows = tableStore.virtualEstimatedRows;
  return Math.min(virtualEstimatedRows, virtualVisibleEndIndex + getOverScanCount(tableStore, virtualVisibleEndIndex, true));
}

export function getIdList(store) {
  var mouseBatchChooseStartId = store.mouseBatchChooseStartId,
      mouseBatchChooseEndId = store.mouseBatchChooseEndId,
      element = store.node.element,
      prefixCls = store.prefixCls;
  var rows = Array.from(element.querySelectorAll(".".concat(prefixCls, "-row")));
  var endId;
  var idList = [];
  rows.some(function (row) {
    var index = Number(row.dataset.index);

    if (!endId) {
      if (mouseBatchChooseStartId === index) {
        endId = mouseBatchChooseEndId;
      } else if (mouseBatchChooseEndId === index) {
        endId = mouseBatchChooseStartId;
      }
    }

    if (endId) {
      idList.push(index);
    }

    return endId === index;
  });
  return idList;
}

function getRowNumbers(record, dataSet, isTree) {
  if (record && dataSet) {
    if (record.isCached) {
      return [];
    }

    var paging = dataSet.paging,
        currentPage = dataSet.currentPage,
        pageSize = dataSet.pageSize;
    var pageIndex = (isTree ? paging === 'server' : paging) ? (currentPage - 1) * pageSize : 0;

    if (isTree) {
      return record.path.map(function (r, index) {
        return r.indexInParent + 1 + (index === 0 ? pageIndex : 0);
      });
    }

    return [record.index + 1 + pageIndex];
  }

  return [0];
}

function hasCheckField(_ref, checkField) {
  var editor = _ref.editor,
      name = _ref.name,
      hidden = _ref.hidden;
  return !hidden && !!editor && checkField === name;
}

function _renderSelectionBox(_ref2) {
  var record = _ref2.record,
      store = _ref2.store;
  var dataSet = record.dataSet;

  if (dataSet) {
    var selection = dataSet.selection;

    var handleChange = function handleChange(value) {
      if (store.props.selectionMode === SelectionMode.mousedown) {
        // 将处理逻辑交给 mousedown 的处理逻辑 不然会两次触发导致不被勾选上
        return;
      }

      if (value) {
        dataSet.select(record);
      } else {
        dataSet.unSelect(record);
      }
    };

    var handleClick = function handleClick(e) {
      stopPropagation(e);

      if (selection === DataSetSelection.multiple) {
        var lastSelected = store.lastSelected;

        if (lastSelected) {
          var nativeEvent = e.nativeEvent;
          var startIndex = -1;
          var endIndex = -1;

          if (nativeEvent.shiftKey) {
            var pointKeys = new Set([lastSelected.key, record.key]);
            dataSet.some(function (pointRecord, index) {
              if (pointKeys.has(pointRecord.key)) {
                if (startIndex === -1) {
                  startIndex = index;
                } else {
                  endIndex = index;
                  return true;
                }
              }

              return false;
            });
          }

          if (endIndex !== -1 && startIndex !== endIndex) {
            // Batch update selections
            var rangeRecords = dataSet.slice(startIndex, endIndex + 1);
            var changedRecords = [];
            var selectedKeys = new Set(dataSet.selected.map(function (selected) {
              return selected.key;
            }));

            if (record.isSelected) {
              rangeRecords.forEach(function (rangeRecord) {
                if (selectedKeys.has(rangeRecord.key)) {
                  changedRecords.push(rangeRecord);
                }
              });
              dataSet.batchUnSelect(changedRecords);
            } else {
              rangeRecords.forEach(function (rangeRecord) {
                if (!selectedKeys.has(rangeRecord.key)) {
                  changedRecords.push(rangeRecord);
                }
              });
              dataSet.batchSelect(changedRecords);
            }
          }
        }

        store.lastSelected = record;
      } else if (record.isSelected) {
        dataSet.unSelect(record);
      }
    };

    if (selection === DataSetSelection.multiple) {
      var batchSelectProps = {};
      var handleDragMouseUp = action(function () {
        var mouseBatchChooseIdList = store.mouseBatchChooseIdList;

        if (store.mouseBatchChooseState) {
          store.mouseBatchChooseState = false;
          store.changeMouseBatchChooseIdList([]);
          var mouseBatchChooseStartId = store.mouseBatchChooseStartId,
              mouseBatchChooseEndId = store.mouseBatchChooseEndId;

          if (mouseBatchChooseStartId === mouseBatchChooseEndId) {
            return;
          }

          var startRecord = dataSet.findRecordById(mouseBatchChooseStartId);

          var _ref3 = startRecord || {},
              isSelected = _ref3.isSelected;

          if (isSelected) {
            dataSet.batchUnSelect(mouseBatchChooseIdList);
          } else {
            dataSet.batchSelect(mouseBatchChooseIdList);
          }
        }

        document.removeEventListener('pointerup', handleDragMouseUp);
      });

      if (store.useMouseBatchChoose) {
        batchSelectProps.onMouseDown = action(function () {
          store.mouseBatchChooseStartId = record.id;
          store.mouseBatchChooseEndId = record.id;
          store.mouseBatchChooseState = true; // 为什么使用 pointerup
          // 因为需要对disabled的元素进行特殊处理
          // 因为状态的改变依赖 mouseup 而在disabled的元素上 无法触发mouseup事件
          // 导致状态无法进行修正
          // 以下两种方案通过 pointer-events:none 进行处理
          // https://stackoverflow.com/questions/322378/javascript-check-if-mouse-button-down
          // https://stackoverflow.com/questions/62081666/the-event-of-the-document-is-not-triggered-when-it-is-on-a-disabled-element
          // 而使用指针事件可以突破disabled的限制
          // https://stackoverflow.com/questions/62126515/how-to-get-the-state-of-the-mouse-through-javascript/62127845#62127845

          document.addEventListener('pointerup', handleDragMouseUp);
        });

        batchSelectProps.onMouseEnter = function () {
          if (store.mouseBatchChooseState) {
            store.mouseBatchChooseEndId = record.id;
            store.changeMouseBatchChooseIdList(getIdList(store));
          }
        };
      }

      return /*#__PURE__*/React.createElement(ObserverCheckBox, _extends({}, batchSelectProps, {
        checked: record.isSelected,
        onChange: handleChange,
        onClick: handleClick,
        disabled: !record.selectable,
        "data-selection-key": SELECTION_KEY,
        labelLayout: LabelLayout.none,
        value: true
      }));
    }

    if (selection === DataSetSelection.single) {
      return /*#__PURE__*/React.createElement(ObserverRadio, {
        checked: record.isSelected,
        onChange: handleChange,
        onClick: handleClick,
        disabled: !record.selectable,
        "data-selection-key": SELECTION_KEY,
        value: true
      });
    }
  }
}

function getCustomizedColumnByKey(key, customizedColumns) {
  if (customizedColumns) {
    return customizedColumns[key];
  }
}

function getCustomizedColumn(column, customizedColumns) {
  if (customizedColumns) {
    return getCustomizedColumnByKey(getColumnKey(column).toString(), customizedColumns);
  }
}

function mergeColumnLock(column, parent, customizedColumn) {
  if (parent) {
    column.lock = parent.lock;
  } else if (customizedColumn && 'lock' in customizedColumn) {
    column.lock = customizedColumn.lock;
  }
}

function mergeCustomizedColumn(column, tableStore, customizedColumn, isChildrenHideDisabled) {
  if (isChildrenHideDisabled) {
    column.hideable = false;
  } else {
    var field = tableStore.dataSet.getField(column.name);

    if (field) {
      var dynamicProps = field.get('dynamicProps');
      var computedProps = field.get('computedProps');

      if (dynamicProps && dynamicProps.required || computedProps && computedProps.required || field.get('required')) {
        column.hideable = false;
      }
    }
  }

  if (customizedColumn) {
    if (column.hideable === false || !tableStore.columnHideable) {
      delete customizedColumn.hidden;
    }

    if (column.resizable === false || !tableStore.columnResizable) {
      delete customizedColumn.width;
    }

    if (column.titleEditable === false || !tableStore.columnTitleEditable) {
      delete customizedColumn.title;
    }

    if (!tableStore.columnDraggable) {
      delete customizedColumn.sort;
    }

    _extends(column, customizedColumn);
  }
}

function findAndMergeCustomizedColumn(column, tableStore, customizedColumns, isChildrenHideDisabled) {
  var customizedColumn = getCustomizedColumn(column, customizedColumns);
  mergeCustomizedColumn(column, tableStore, customizedColumn, isChildrenHideDisabled);
}

function mergeDefaultProps(tableStore, originalColumns, tableAggregation, customizedColumns) {
  var parent = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : null;
  var defaultKey = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : [0];
  var columnSort = arguments.length > 6 && arguments[6] !== undefined ? arguments[6] : {
    left: 0,
    center: 0,
    right: 0
  };
  var columns = [];
  var leftColumns = [];
  var rightColumns = [];
  var hasAggregationColumn = false;
  var isHideDisabled = false;
  originalColumns.forEach(function (column) {
    if (isPlainObject(column)) {
      var newColumn = _objectSpread(_objectSpread({}, ColumnDefaultProps), column);

      if (isNil(getColumnKey(newColumn))) {
        newColumn.key = "anonymous-".concat(defaultKey[0]++);
      }

      var children = newColumn.children,
          aggregation = newColumn.aggregation;

      if (!hasAggregationColumn && aggregation) {
        hasAggregationColumn = true;
      }

      if (tableAggregation || !aggregation) {
        var customizedColumn = getCustomizedColumn(newColumn, customizedColumns);
        mergeColumnLock(newColumn, parent, customizedColumn);

        if (children) {
          var _mergeDefaultProps = mergeDefaultProps(tableStore, children, tableAggregation, customizedColumns, newColumn, defaultKey),
              _mergeDefaultProps2 = _slicedToArray(_mergeDefaultProps, 4),
              childrenColumns = _mergeDefaultProps2[1],
              _mergeDefaultProps2$ = _mergeDefaultProps2[3],
              childrenHasAggregationColumn = _mergeDefaultProps2$.hasAggregationColumn,
              childrenIsHideDisabled = _mergeDefaultProps2$.isHideDisabled;

          newColumn.children = childrenColumns;

          if (!hasAggregationColumn && childrenHasAggregationColumn) {
            hasAggregationColumn = true;
          }

          if (!isHideDisabled && childrenIsHideDisabled) {
            isHideDisabled = true;
          }

          mergeCustomizedColumn(newColumn, tableStore, customizedColumn, childrenIsHideDisabled);
        } else {
          mergeCustomizedColumn(newColumn, tableStore, customizedColumn);
        }

        if (!isHideDisabled && newColumn.hideable === false) {
          isHideDisabled = true;
        }

        if (parent || !newColumn.lock) {
          if (newColumn.sort === undefined) {
            newColumn.sort = columnSort.center;
          }

          columnSort.center++;
          columns.push(newColumn);
        } else if (newColumn.lock === true || newColumn.lock === ColumnLock.left) {
          if (newColumn.sort === undefined) {
            newColumn.sort = columnSort.left;
          }

          columnSort.left++;
          leftColumns.push(newColumn);
        } else {
          if (newColumn.sort === undefined) {
            newColumn.sort = columnSort.right;
          }

          columnSort.right++;
          rightColumns.push(newColumn);
        }
      } else if (children) {
        var _mergeDefaultProps3 = mergeDefaultProps(tableStore, children, tableAggregation, customizedColumns, parent, defaultKey, parent ? undefined : columnSort),
            _mergeDefaultProps4 = _slicedToArray(_mergeDefaultProps3, 4),
            leftNodes = _mergeDefaultProps4[0],
            nodes = _mergeDefaultProps4[1],
            rightNodes = _mergeDefaultProps4[2],
            _mergeDefaultProps4$ = _mergeDefaultProps4[3],
            _childrenHasAggregationColumn = _mergeDefaultProps4$.hasAggregationColumn,
            _childrenIsHideDisabled = _mergeDefaultProps4$.isHideDisabled;

        if (!hasAggregationColumn && _childrenHasAggregationColumn) {
          hasAggregationColumn = true;
        }

        if (!isHideDisabled && _childrenIsHideDisabled) {
          newColumn.hideable = false;
          isHideDisabled = true;
        }

        if (parent) {
          parent.children = [].concat(_toConsumableArray(leftNodes), _toConsumableArray(nodes), _toConsumableArray(rightNodes));
        } else {
          leftColumns.push.apply(leftColumns, _toConsumableArray(leftNodes));
          columns.push.apply(columns, _toConsumableArray(nodes));
          rightColumns.push.apply(rightColumns, _toConsumableArray(rightNodes));
        }
      }
    }
  }, []);

  if (parent) {
    return [[], sortBy(columns, function (_ref4) {
      var sort = _ref4.sort;
      return sort;
    }), [], {
      hasAggregationColumn: hasAggregationColumn,
      isHideDisabled: isHideDisabled
    }];
  }

  return [sortBy(leftColumns, function (_ref5) {
    var sort = _ref5.sort;
    return sort;
  }), sortBy(columns, function (_ref6) {
    var sort = _ref6.sort;
    return sort;
  }), sortBy(rightColumns, function (_ref7) {
    var sort = _ref7.sort;
    return sort;
  }), {
    hasAggregationColumn: hasAggregationColumn,
    isHideDisabled: isHideDisabled
  }];
}

function normalizeColumns(tableStore, elements, tableAggregation, customizedColumns) {
  var parent = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : null;
  var defaultKey = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : [0];
  var columnSort = arguments.length > 6 && arguments[6] !== undefined ? arguments[6] : {
    left: 0,
    center: 0,
    right: 0
  };
  var columns = [];
  var leftColumns = [];
  var rightColumns = [];
  var hasAggregationColumn = false;
  var isHideDisabled = false;

  var normalizeColumn = function normalizeColumn(element) {
    if ( /*#__PURE__*/isValidElement(element)) {
      var props = element.props,
          key = element.key,
          type = element.type;

      if (isFragment(element)) {
        var children = props.children;

        if (children) {
          Children.forEach(children, normalizeColumn);
        }
      } else if (type.__PRO_TABLE_COLUMN) {
        var column = _objectSpread({}, props);

        if (key) {
          column.key = key;
        } else if (isNil(getColumnKey(column))) {
          column.key = "anonymous-".concat(defaultKey[0]++);
        }

        var _children = column.children,
            aggregation = column.aggregation;

        if (!hasAggregationColumn && aggregation) {
          hasAggregationColumn = true;
        }

        if (tableAggregation || !aggregation) {
          var customizedColumn = getCustomizedColumn(column, customizedColumns);
          mergeColumnLock(column, parent, customizedColumn);

          var _normalizeColumns = normalizeColumns(tableStore, _children, tableAggregation, customizedColumns, column, defaultKey),
              _normalizeColumns2 = _slicedToArray(_normalizeColumns, 4),
              childrenColumns = _normalizeColumns2[1],
              _normalizeColumns2$ = _normalizeColumns2[3],
              childrenHasAggregationColumn = _normalizeColumns2$.hasAggregationColumn,
              childrenIsHideDisabled = _normalizeColumns2$.isHideDisabled;

          column.children = childrenColumns;

          if (!hasAggregationColumn && childrenHasAggregationColumn) {
            hasAggregationColumn = childrenHasAggregationColumn;
          }

          if (!isHideDisabled && childrenIsHideDisabled) {
            isHideDisabled = true;
          }

          mergeCustomizedColumn(column, tableStore, customizedColumn, childrenIsHideDisabled);

          if (!isHideDisabled && column.hideable === false) {
            isHideDisabled = true;
          }

          if (parent || !column.lock) {
            if (column.sort === undefined) {
              column.sort = columnSort.center;
            }

            columnSort.center++;
            columns.push(column);
          } else if (column.lock === true || column.lock === ColumnLock.left) {
            if (column.sort === undefined) {
              column.sort = columnSort.left;
            }

            columnSort.left++;
            leftColumns.push(column);
          } else {
            if (column.sort === undefined) {
              column.sort = columnSort.right;
            }

            columnSort.right++;
            rightColumns.push(column);
          }
        } else {
          var _normalizeColumns3 = normalizeColumns(tableStore, _children, tableAggregation, customizedColumns, parent, defaultKey, parent ? undefined : columnSort),
              _normalizeColumns4 = _slicedToArray(_normalizeColumns3, 4),
              leftNodes = _normalizeColumns4[0],
              nodes = _normalizeColumns4[1],
              rightNodes = _normalizeColumns4[2],
              _normalizeColumns4$ = _normalizeColumns4[3],
              _childrenHasAggregationColumn2 = _normalizeColumns4$.hasAggregationColumn,
              _childrenIsHideDisabled2 = _normalizeColumns4$.isHideDisabled;

          if (!hasAggregationColumn && _childrenHasAggregationColumn2) {
            hasAggregationColumn = _childrenHasAggregationColumn2;
          }

          if (!isHideDisabled && _childrenIsHideDisabled2) {
            column.hideable = false;
            isHideDisabled = true;
          }

          if (parent) {
            parent.children = [].concat(_toConsumableArray(leftNodes), _toConsumableArray(nodes), _toConsumableArray(rightNodes));
          } else {
            leftColumns.push.apply(leftColumns, _toConsumableArray(leftNodes));
            columns.push.apply(columns, _toConsumableArray(nodes));
            rightColumns.push.apply(rightColumns, _toConsumableArray(rightNodes));
          }
        }
      }
    }
  };

  Children.forEach(elements, normalizeColumn);

  if (parent) {
    return [[], sortBy(columns, function (_ref8) {
      var sort = _ref8.sort;
      return sort;
    }), [], {
      hasAggregationColumn: hasAggregationColumn,
      isHideDisabled: isHideDisabled
    }];
  }

  return [sortBy(leftColumns, function (_ref9) {
    var sort = _ref9.sort;
    return sort;
  }), sortBy(columns, function (_ref10) {
    var sort = _ref10.sort;
    return sort;
  }), sortBy(rightColumns, function (_ref11) {
    var sort = _ref11.sort;
    return sort;
  }), {
    hasAggregationColumn: hasAggregationColumn,
    isHideDisabled: isHideDisabled
  }];
}

function getColumnGroupedColumns(tableStore, groups, customizedColumns) {
  var leftGroupedColumns = [];
  var groupedColumns = [];
  var rightGroupedColumns = [];
  var hasAggregation = false;
  groups.forEach(function (group) {
    var name = group.name,
        type = group.type,
        columnProps = group.columnProps;

    if (type === GroupType.column) {
      var column = _objectSpread(_objectSpread(_objectSpread({}, ColumnDefaultProps), {}, {
        lock: ColumnLock.left
      }, columnProps), {}, {
        draggable: false,
        hideable: false,
        key: "__group-".concat(name),
        name: name,
        __tableGroup: group
      });

      column.children = columnProps && columnProps.children ? treeMap(columnProps.children, function (col, index, parentNode) {
        var newCol = _objectSpread(_objectSpread({}, col), {}, {
          __tableGroup: group,
          lock: column.lock
        });

        if (!getColumnKey(col)) {
          newCol.key = parentNode ? "".concat(parentNode.key, "-").concat(index) : "__group-".concat(name, "-").concat(index);
        }

        findAndMergeCustomizedColumn(newCol, tableStore, customizedColumns);
        return newCol;
      }, function (_ref12, _ref13) {
        var _ref12$sort = _ref12.sort,
            sort = _ref12$sort === void 0 ? Infinity : _ref12$sort;
        var _ref13$sort = _ref13.sort,
            sort2 = _ref13$sort === void 0 ? Infinity : _ref13$sort;
        return sort - sort2;
      }) : undefined;
      findAndMergeCustomizedColumn(column, tableStore, customizedColumns);

      if (!column.lock) {
        groupedColumns.push(column);
      } else if (column.lock === true || column.lock === ColumnLock.left) {
        leftGroupedColumns.push(column);
      } else {
        rightGroupedColumns.push(column);
      }

      if (columnProps && !hasAggregation && columnProps.aggregation) {
        hasAggregation = true;
      }
    }
  });
  return [leftGroupedColumns, groupedColumns, rightGroupedColumns, hasAggregation];
}

function getHeaderGroupedColumns(tableStore, groups, tableGroups, columns, dataSet, groupedColumns, customizedColumns, parentKey) {
  var generatedColumns = new Set();
  var headerUsed = false;
  groups.forEach(function (group) {
    var name = group.name,
        subGroups = group.subGroups,
        value = group.value;
    var key = parentKey ? "".concat(parentKey, "-").concat(value) : value;
    var subColumns = subGroups.length ? getHeaderGroupedColumns(tableStore, subGroups, tableGroups, columns, dataSet, groupedColumns, customizedColumns, key) : columns;
    var tableGroup = tableGroups.find(function ($tableGroup) {
      return name === $tableGroup.name;
    });

    if (tableGroup) {
      var columnProps = tableGroup.columnProps,
          groupName = tableGroup.name,
          hidden = tableGroup.hidden;

      if (hidden) {
        subColumns.forEach(function (col) {
          var __originalKey = getColumnKey(col);

          var colKey = "".concat(key, "-").concat(__originalKey);

          var newCol = _objectSpread(_objectSpread({}, col), {}, {
            key: colKey,
            __tableGroup: tableGroup,
            __group: group,
            __groups: groups,
            __originalKey: __originalKey
          });

          findAndMergeCustomizedColumn(newCol, tableStore, customizedColumns);
          newCol.lock = false;
          generatedColumns.add(newCol);
        });
      } else {
        var length = groupedColumns.length;

        if (length && !headerUsed) {
          headerUsed = true;
          var header = getHeader(_objectSpread(_objectSpread({}, columnProps), {}, {
            name: groupName,
            dataSet: dataSet,
            group: group,
            groups: groups
          }));

          if (header) {
            var oldColumn = groupedColumns[length - 1];
            var newKey = "".concat(key, "-").concat(getColumnKey(oldColumn));

            var newColumn = _objectSpread(_objectSpread({}, columnProps), {}, {
              lock: oldColumn.lock,
              titleEditable: false,
              draggable: false,
              hideable: false,
              key: newKey,
              header: header,
              children: [oldColumn],
              __tableGroup: tableGroup
            });

            var customizedColumn = getCustomizedColumnByKey(newKey, customizedColumns);
            mergeCustomizedColumn(newColumn, tableStore, customizedColumn);
            groupedColumns[length - 1] = newColumn;
          }
        }

        var renderer = columnProps && columnProps.renderer || defaultAggregationRenderer;

        var column = _objectSpread(_objectSpread({}, columnProps), {}, {
          titleEditable: false,
          key: key,
          headerStyle: columnProps && columnProps.style,
          header: function header() {
            return renderer({
              dataSet: dataSet,
              record: group.totalRecords[0],
              name: groupName,
              text: value,
              value: value,
              headerGroup: group
            });
          },
          children: treeMap(subColumns, function (col) {
            var __originalKey = getColumnKey(col);

            var colKey = "".concat(key, "-").concat(__originalKey);

            var newCol = _objectSpread(_objectSpread({}, col), {}, {
              key: colKey,
              __originalKey: __originalKey
            });

            if (customizedColumns) {
              _extends(newCol, customizedColumns[__originalKey], customizedColumns[colKey]);
            }

            newCol.lock = false;
            return newCol;
          }, function (_ref14, _ref15) {
            var _ref14$sort = _ref14.sort,
                sort = _ref14$sort === void 0 ? Infinity : _ref14$sort;
            var _ref15$sort = _ref15.sort,
                sort2 = _ref15$sort === void 0 ? Infinity : _ref15$sort;
            return sort - sort2;
          }),
          __tableGroup: tableGroup,
          __group: group,
          __groups: groups
        });

        findAndMergeCustomizedColumn(column, tableStore, customizedColumns);
        generatedColumns.add(column);
      }
    } else if (subColumns.length) {
      subColumns.forEach(function (column) {
        return generatedColumns.add(column);
      });
    }
  });
  return _toConsumableArray(generatedColumns);
}

export function normalizeGroupColumns(tableStore, columns, children, aggregation, customizedColumns) {
  var headerTableGroups = tableStore.headerTableGroups,
      groups = tableStore.groups,
      dataSet = tableStore.dataSet;
  var hasHeaderGroup = headerTableGroups.length > 0;
  var generatedColumns = columns ? mergeDefaultProps(tableStore, columns, aggregation, customizedColumns) : normalizeColumns(tableStore, children, aggregation, customizedColumns);

  var _generatedColumns = _slicedToArray(generatedColumns, 4),
      leftOriginalColumns = _generatedColumns[0],
      originalColumns = _generatedColumns[1],
      rightOriginalColumns = _generatedColumns[2],
      hasAggregationColumn = _generatedColumns[3].hasAggregationColumn;

  var _ref16 = groups.length ? getColumnGroupedColumns(tableStore, groups, customizedColumns) : [[], [], [], false],
      _ref17 = _slicedToArray(_ref16, 4),
      leftColumnGroupedColumns = _ref17[0],
      columnGroupedColumns = _ref17[1],
      rightColumnGroupedColumns = _ref17[2],
      hasAggregationColumnGroup = _ref17[3];

  if (hasHeaderGroup) {
    var groupedColumns = [].concat(_toConsumableArray(leftColumnGroupedColumns), _toConsumableArray(columnGroupedColumns));
    return [groupedColumns, getHeaderGroupedColumns(tableStore, tableStore.groupedDataWithHeader, headerTableGroups, [].concat(_toConsumableArray(leftOriginalColumns), _toConsumableArray(originalColumns), _toConsumableArray(rightOriginalColumns)), dataSet, groupedColumns, customizedColumns), rightColumnGroupedColumns, hasAggregationColumnGroup || hasAggregationColumn];
  }

  return [[].concat(_toConsumableArray(leftColumnGroupedColumns), _toConsumableArray(leftOriginalColumns)), [].concat(_toConsumableArray(columnGroupedColumns), _toConsumableArray(originalColumns)), [].concat(_toConsumableArray(rightOriginalColumns), _toConsumableArray(rightColumnGroupedColumns)), hasAggregationColumnGroup || hasAggregationColumn];
}

function getHeaderTexts(_x, _x2, _x3) {
  return _getHeaderTexts.apply(this, arguments);
}

function _getHeaderTexts() {
  _getHeaderTexts = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee4(dataSet, columns, aggregation) {
    var headers,
        column,
        _args4 = arguments;
    return _regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            headers = _args4.length > 3 && _args4[3] !== undefined ? _args4[3] : [];
            column = columns.shift();

            if (!column) {
              _context4.next = 10;
              break;
            }

            _context4.t0 = headers;
            _context4.t1 = column.name;
            _context4.next = 7;
            return getReactNodeText(getHeader(_objectSpread(_objectSpread({}, column), {}, {
              dataSet: dataSet,
              aggregation: aggregation
            })));

          case 7:
            _context4.t2 = _context4.sent;
            _context4.t3 = {
              name: _context4.t1,
              label: _context4.t2
            };

            _context4.t0.push.call(_context4.t0, _context4.t3);

          case 10:
            if (!columns.length) {
              _context4.next = 13;
              break;
            }

            _context4.next = 13;
            return getHeaderTexts(dataSet, columns, aggregation, headers);

          case 13:
            return _context4.abrupt("return", headers);

          case 14:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4);
  }));
  return _getHeaderTexts.apply(this, arguments);
}

function autoHeightToStyle(autoHeight) {
  var parentPaddingTop = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
  var type = autoHeight.type,
      diff = autoHeight.diff;
  var height = "calc(100% - ".concat(diff + parentPaddingTop, "px)");

  if (type === TableAutoHeightType.minHeight) {
    return {
      height: height
    };
  }

  return {
    maxHeight: height
  };
}

var TableStore = /*#__PURE__*/function () {
  function TableStore(node) {
    var _this = this;

    _classCallCheck(this, TableStore);

    this.editors = new Map();
    this.mouseBatchChooseStartId = 0;
    this.mouseBatchChooseEndId = 0;
    this.inBatchExpansion = false;
    this.performanceOn = false;
    this.timing = {
      renderStart: 0,
      renderEnd: 0
    };
    this.lastMeasuredIndex = 0;
    this.handleSelectAllChange = action(function () {
      if (_this.allChecked) {
        _this.unCheckAllCurrent();

        _this.unCheckAllCached();
      } else {
        _this.checkAllCurrent();

        _this.checkAllCached();
      }
    });
    this.saveCustomizedDebounce = debounce(this.saveCustomized, 1000);
    this.stopScroll = debounce(action(function () {
      _this.scrolling = false;
    }), 300);
    runInAction(function () {
      _this.scrollPosition = ScrollPosition.left;
      _this.mouseBatchChooseIdList = [];
      _this.mouseBatchChooseState = false;
      _this.showCachedSelection = false;
      _this.lockColumnsHeadRowsHeight = {};
      _this.lockColumnsBodyRowsHeight = {};
      _this.lockColumnsFootRowsHeight = {};
      _this.node = node;
      _this.expandedRows = [];
      _this.screenHeight = typeof window === 'undefined' ? 0 : document.documentElement.clientHeight;
      _this.headerHeight = 0;
      _this.footerHeight = 0;
      _this.lastScrollTop = 0;
      _this.customizedActiveKey = ['columns'];
      _this.leftOriginalColumns = [];
      _this.originalColumns = [];
      _this.rightOriginalColumns = [];
      _this.tempCustomized = {
        columns: {}
      };
      _this.customized = {
        columns: {}
      };

      _this.setProps(node.props);

      if (_this.customizable) {
        _this.loadCustomized();
      } else {
        _this.initColumns();
      }
    });
  }

  _createClass(TableStore, [{
    key: "styleHeight",
    get: function get() {
      var autoHeight = this.autoHeight,
          style = this.props.style,
          parentPaddingTop = this.parentPaddingTop;
      return autoHeight ? autoHeightToStyle(autoHeight, parentPaddingTop).height : style && style.height;
    }
  }, {
    key: "styleMaxHeight",
    get: function get() {
      var autoHeight = this.autoHeight,
          style = this.props.style,
          parentPaddingTop = this.parentPaddingTop;
      return autoHeight ? autoHeightToStyle(autoHeight, parentPaddingTop).maxHeight : style && style.maxHeight;
    }
  }, {
    key: "styleMinHeight",
    get: function get() {
      var style = this.props.style;
      return this.styleHeight || style && toPx(style.minHeight, this.getRelationSize);
    }
  }, {
    key: "computedHeight",
    get: function get() {
      if (this.heightChangeable) {
        var _this$customized = this.customized,
            heightType = _this$customized.heightType,
            height = _this$customized.height,
            heightDiff = _this$customized.heightDiff,
            tempCustomized = this.tempCustomized;

        var tempHeightType = _get(tempCustomized, 'heightType');

        if (tempHeightType) {
          if (tempHeightType === TableHeightType.fixed) {
            return _get(tempCustomized, 'height');
          }

          if (tempHeightType === TableHeightType.flex) {
            return this.screenHeight - (_get(tempCustomized, 'heightDiff') || 0);
          }

          return undefined;
        }

        if (heightType) {
          if (heightType === TableHeightType.fixed) {
            return height;
          }

          if (heightType === TableHeightType.flex) {
            return this.screenHeight - (heightDiff || 0);
          }

          return undefined;
        }
      }

      return toPx(this.styleHeight, this.getRelationSize);
    }
  }, {
    key: "otherHeight",
    get: function get() {
      var footerHeight = this.footerHeight,
          boxSizing = this.props.boxSizing;
      var footerTotalHeight = footerHeight && this.overflowX ? measureScrollbar() + footerHeight : footerHeight;
      var otherHeight = this.headerHeight + footerTotalHeight;

      if (boxSizing === TableBoxSizing.wrapper) {
        var _ref18 = this.siblingHeihgt || {},
            _ref18$before = _ref18.before,
            before = _ref18$before === void 0 ? 0 : _ref18$before,
            _ref18$after = _ref18.after,
            after = _ref18$after === void 0 ? 0 : _ref18$after;

        return otherHeight + before + after;
      }

      return otherHeight;
    }
  }, {
    key: "height",
    get: function get() {
      if (!this.isBodyExpanded) {
        return undefined;
      }

      var computedHeight = this.computedHeight;
      var maxHeight = toPx(this.styleMaxHeight, this.getRelationSize);
      var minHeight = toPx(this.styleMinHeight, this.getRelationSize);
      var isComputedHeight = isNumber(computedHeight);

      if (isComputedHeight || isNumber(minHeight) || isNumber(maxHeight)) {
        var rowHeight = this.rowHeight,
            otherHeight = this.otherHeight;
        var rowMinHeight = (isNumber(rowHeight) ? rowHeight : 30) + otherHeight;
        var minTotalHeight = minHeight ? Math.max(rowMinHeight, minHeight) : rowMinHeight;
        var height = defaultTo(computedHeight, this.bodyHeight + otherHeight);
        var totalHeight = Math.max(minTotalHeight, maxHeight ? Math.min(maxHeight, height) : minHeight ? minTotalHeight : height);
        return isComputedHeight || totalHeight !== height ? totalHeight - otherHeight : undefined;
      }
    }
  }, {
    key: "totalHeight",
    get: function get() {
      var height = this.height,
          bodyHeight = this.bodyHeight,
          otherHeight = this.otherHeight;
      return defaultTo(height, bodyHeight) + otherHeight;
    }
  }, {
    key: "bodyHeight",
    get: function get() {
      if (this.propVirtual) {
        var virtualHeight = this.virtualHeight;

        if (virtualHeight > 0) {
          return virtualHeight;
        }
      }

      return this.calcBodyHeight || 0;
    }
  }, {
    key: "stickyLeft",
    get: function get() {
      return [ScrollPosition.right, ScrollPosition.middle].includes(this.scrollPosition);
    }
  }, {
    key: "stickyRight",
    get: function get() {
      return [ScrollPosition.left, ScrollPosition.middle].includes(this.scrollPosition);
    }
  }, {
    key: "performanceEnabled",
    get: function get() {
      var performanceEnabled = this.getConfig('performanceEnabled');
      return performanceEnabled && performanceEnabled.Table;
    }
  }, {
    key: "dataSet",
    get: function get() {
      return this.props.dataSet;
    }
  }, {
    key: "prefixCls",
    get: function get() {
      return this.node.prefixCls;
    }
  }, {
    key: "customizable",
    get: function get() {
      var customizedCode = this.props.customizedCode;
      var queryBarProps = this.props.queryBarProps;
      var isSimpleMode = queryBarProps && queryBarProps.simpleMode;

      if (customizedCode || this.queryBar === TableQueryBarType.comboBar && !isSimpleMode) {
        if ('customizable' in this.props) {
          return this.props.customizable;
        }

        return this.getConfig('tableCustomizable') || this.node.context.getCustomizable('Table');
      }

      return false;
    }
    /**
     * board 组件个性化按钮 in buttons
     */

  }, {
    key: "customizedBtn",
    get: function get() {
      var _this$props = this.props,
          customizedCode = _this$props.customizedCode,
          boardCustomized = _this$props.boardCustomized;

      if (customizedCode && boardCustomized) {
        return boardCustomized.customizedBtn;
      }

      return false;
    }
  }, {
    key: "aggregation",
    get: function get() {
      var aggregation = this.customized.aggregation;

      if (aggregation !== undefined) {
        return aggregation;
      }

      var propAggregation = this.props.aggregation;
      return propAggregation;
    }
  }, {
    key: "aggregationExpandType",
    get: function get() {
      return this.customized.aggregationExpandType || 'cell';
    }
  }, {
    key: "autoHeight",
    get: function get() {
      var autoHeight = this.props.autoHeight;

      if (autoHeight) {
        var defaultAutoHeight = {
          type: TableAutoHeightType.minHeight,
          diff: this.getConfig('tableAutoHeightDiff') || 80
        };

        if (isObject(autoHeight)) {
          return _objectSpread(_objectSpread({}, defaultAutoHeight), autoHeight);
        }

        return defaultAutoHeight;
      }

      return undefined;
    }
  }, {
    key: "heightType",
    get: function get() {
      if (this.heightChangeable) {
        var tempHeightType = _get(this.tempCustomized, 'heightType');

        if (tempHeightType !== undefined) {
          return tempHeightType;
        }

        var heightType = this.customized.heightType;

        if (heightType !== undefined) {
          return heightType;
        }
      }

      return this.originalHeightType;
    }
  }, {
    key: "originalHeightType",
    get: function get() {
      var styleHeight = this.styleHeight;

      if (styleHeight) {
        if (isString(styleHeight) && isCalcSize(styleHeight)) {
          return TableHeightType.flex;
        }

        if (isNumber(toPx(styleHeight))) {
          return TableHeightType.fixed;
        }
      }

      return TableHeightType.auto;
    }
  }, {
    key: "virtualCell",
    get: function get() {
      if ('virtualCell' in this.props) {
        return this.props.virtualCell;
      }

      return this.getConfig('tableVirtualCell');
    }
  }, {
    key: "isFixedRowHeight",
    get: function get() {
      if (!this.aggregation || !this.hasAggregationColumn) {
        return isNumber(this.rowHeight);
      }

      return false;
    }
  }, {
    key: "propVirtual",
    get: function get() {
      if (this.isTree && this.rowDraggable) {
        return true;
      }

      if ('virtual' in this.props) {
        return this.props.virtual;
      }

      var configTableVirtual = this.getConfig('tableVirtual');

      if (typeof configTableVirtual === 'function') {
        return configTableVirtual(this.currentData.length, this.columnGroups.leafs.length);
      }

      return configTableVirtual;
    }
  }, {
    key: "virtual",
    get: function get() {
      if (this.height !== undefined) {
        return this.propVirtual;
      }

      return false;
    }
  }, {
    key: "columnBuffer",
    get: function get() {
      var columnBuffer = this.props.columnBuffer;

      if ('columnBuffer' in this.props && typeof columnBuffer === 'number' && columnBuffer >= 0) {
        return columnBuffer;
      }

      var bufferConfig = this.getConfig('tableVirtualBuffer');

      if (bufferConfig && 'columnBuffer' in bufferConfig && typeof bufferConfig.columnBuffer === 'number' && bufferConfig.columnBuffer >= 0) {
        return bufferConfig.columnBuffer;
      }

      return 3;
    }
  }, {
    key: "columnThreshold",
    get: function get() {
      var columnThreshold = this.props.columnThreshold;

      if ('columnThreshold' in this.props && typeof columnThreshold === 'number' && columnThreshold >= 0) {
        return columnThreshold;
      }

      var bufferConfig = this.getConfig('tableVirtualBuffer');

      if (bufferConfig && 'columnThreshold' in bufferConfig && typeof bufferConfig.columnThreshold === 'number' && bufferConfig.columnThreshold >= 0) {
        return bufferConfig.columnThreshold;
      }

      return 3;
    }
  }, {
    key: "updateRenderZonePosition",
    value: function updateRenderZonePosition() {
      // 获取表格坐标显示范围
      if (!this.width) {
        return [0, 0];
      }

      var _this$columnGroups = this.columnGroups,
          leafs = _this$columnGroups.leafs,
          leftLeafs = _this$columnGroups.leftLeafs,
          rightLeafs = _this$columnGroups.rightLeafs,
          leftLeafColumnsWidth = _this$columnGroups.leftLeafColumnsWidth,
          rightLeafColumnsWidth = _this$columnGroups.rightLeafColumnsWidth,
          columnThreshold = this.columnThreshold,
          nextRenderColIndex = this.nextRenderColIndex;
      var scrollLeft = this.lastScrollLeft || 0;
      var visibleColumnWidth = 0;
      var firstIndex = -1;
      var lastIndex = -1;
      var centerLeafsLength = leafs.length - rightLeafs.length;

      for (var i = leftLeafs.length; i < centerLeafsLength; i++) {
        var width = leafs[i].width;
        visibleColumnWidth += width;

        if (firstIndex === -1 && visibleColumnWidth > scrollLeft) {
          firstIndex = i;
        }

        if (lastIndex === -1 && i === centerLeafsLength - 1 || this.width && visibleColumnWidth >= scrollLeft + this.width - leftLeafColumnsWidth - rightLeafColumnsWidth - (this.overflowY ? measureScrollbar() : 0)) {
          lastIndex = i;
        }

        if (lastIndex !== -1 && firstIndex !== -1) {
          break;
        }
      }

      if (!nextRenderColIndex || nextRenderColIndex && nextRenderColIndex.includes(lastIndex) || lastIndex < nextRenderColIndex[0] || lastIndex > nextRenderColIndex[1]) {
        this.nextRenderColIndex = [lastIndex - columnThreshold, Math.min(lastIndex + columnThreshold, leafs.length)];
        this.prevRenderColIndex = [firstIndex, lastIndex];
        return [firstIndex, lastIndex];
      }

      return this.prevRenderColIndex || [0, 0];
    }
  }, {
    key: "virtualColumnRange",
    get: function get() {
      var _this$columnGroups2 = this.columnGroups,
          leafs = _this$columnGroups2.leafs,
          leftLeafs = _this$columnGroups2.leftLeafs,
          rightLeafs = _this$columnGroups2.rightLeafs,
          columnBuffer = this.columnBuffer;

      if (!this.propVirtual || !this.overflowX) {
        return {
          center: [0, leafs.length]
        };
      }

      var rangeThreshold = {
        center: [0, 0]
      }; // 左右固定列的坐标范围

      var leftColLength = leftLeafs.length;
      var rightColLength = rightLeafs.length;

      if (leftColLength) {
        rangeThreshold.left = [0, leftColLength];
      }

      if (rightColLength) {
        rangeThreshold.right = [leafs.length - rightColLength, leafs.length];
      }

      var _this$updateRenderZon = this.updateRenderZonePosition(),
          _this$updateRenderZon2 = _slicedToArray(_this$updateRenderZon, 2),
          start = _this$updateRenderZon2[0],
          end = _this$updateRenderZon2[1];

      var first = Math.max(leftColLength, start - columnBuffer);
      var last = Math.min(leafs.length - rightColLength, end + columnBuffer + 1);
      rangeThreshold.center = [first, last];
      return rangeThreshold;
    }
  }, {
    key: "isRenderRange",
    value: function isRenderRange(index, isGroup) {
      var virtualColumnRange = this.virtualColumnRange,
          propVirtual = this.propVirtual;

      if (!propVirtual || isGroup) {
        return true;
      }

      if (virtualColumnRange.left && index >= virtualColumnRange.left[0] && index < virtualColumnRange.left[1]) {
        return true;
      }

      if (index >= virtualColumnRange.center[0] && index < virtualColumnRange.center[1]) {
        return true;
      }

      if (virtualColumnRange.right && index >= virtualColumnRange.right[0] && index <= virtualColumnRange.right[1]) {
        return true;
      }

      return false;
    }
  }, {
    key: "blankVirtualCell",
    get: function get() {
      var virtualColumnRange = this.virtualColumnRange;
      var left = virtualColumnRange.left,
          center = virtualColumnRange.center,
          right = virtualColumnRange.right;
      return {
        left: left ? _toConsumableArray(Array(center[0] - left[1]).keys()).map(function (key) {
          return /*#__PURE__*/React.createElement("td", {
            key: "empty-left-".concat(key)
          });
        }) : [],
        right: right ? _toConsumableArray(Array(right[0] - center[1]).keys()).map(function (key) {
          return /*#__PURE__*/React.createElement("td", {
            key: "empty-right-".concat(key)
          });
        }) : []
      };
    }
  }, {
    key: "tableColumnResizeTransition",
    get: function get() {
      return this.getConfig('tableColumnResizeTransition');
    }
  }, {
    key: "virtualRowHeight",
    get: function get() {
      var rowHeight = this.rowHeight,
          _this$cellVerticalSiz = this.cellVerticalSize,
          cellVerticalSize = _this$cellVerticalSiz === void 0 ? 3 : _this$cellVerticalSiz;
      var normalRowHeight = scaleSize(isNumber(rowHeight) ? rowHeight + cellVerticalSize : 30 + cellVerticalSize);
      return this.aggregation && this.hasAggregationColumn ? normalRowHeight * 4 : normalRowHeight;
    }
  }, {
    key: "virtualEstimatedRows",
    get: function get() {
      var rowMetaData = this.rowMetaData;

      if (rowMetaData) {
        return rowMetaData.length;
      }

      var actualRows = this.actualRows;

      if (actualRows !== undefined) {
        return actualRows;
      }

      return this.data.length;
    }
  }, {
    key: "virtualHeight",
    get: function get() {
      var virtualRowHeight = this.virtualRowHeight,
          virtualEstimatedRows = this.virtualEstimatedRows;

      if (!virtualEstimatedRows) {
        return 0;
      }

      var rowMetaData = this.rowMetaData;

      if (rowMetaData) {
        var lastMeasuredIndex = this.lastMeasuredIndex;
        var totalSizeOfMeasuredItems = 0;

        if (lastMeasuredIndex >= virtualEstimatedRows) {
          lastMeasuredIndex = virtualEstimatedRows - 1;
        }

        if (lastMeasuredIndex >= 0) {
          var itemMetadata = rowMetaData[lastMeasuredIndex];

          if (itemMetadata) {
            totalSizeOfMeasuredItems = itemMetadata.offset + itemMetadata.height;
          }
        }

        var numUnmeasuredItems = virtualEstimatedRows - lastMeasuredIndex - 1;
        var totalSizeOfUnmeasuredItems = numUnmeasuredItems * virtualRowHeight;
        return totalSizeOfMeasuredItems + totalSizeOfUnmeasuredItems;
      }

      return Math.round(virtualEstimatedRows * virtualRowHeight);
    }
  }, {
    key: "virtualVisibleStartIndex",
    get: function get() {
      return getVisibleStartIndex(this);
    }
  }, {
    key: "virtualVisibleEndIndex",
    get: function get() {
      return getVisibleEndIndex(this);
    }
  }, {
    key: "virtualStartIndex",
    get: function get() {
      return getStartIndex(this);
    }
  }, {
    key: "virtualEndIndex",
    get: function get() {
      return getEndIndex(this);
    }
  }, {
    key: "virtualTop",
    get: function get() {
      var virtualRowHeight = this.virtualRowHeight,
          virtualStartIndex = this.virtualStartIndex;
      var rowMetaData = this.rowMetaData;

      if (rowMetaData && rowMetaData.length) {
        return rowMetaData[virtualStartIndex].offset;
      }

      return virtualStartIndex * virtualRowHeight;
    }
  }, {
    key: "hidden",
    get: function get() {
      return this.styledHidden || this.props.hidden;
    }
  }, {
    key: "alwaysShowRowBox",
    get: function get() {
      if ('alwaysShowRowBox' in this.props) {
        return this.props.alwaysShowRowBox;
      }

      var alwaysShowRowBox = this.getConfig('tableAlwaysShowRowBox');

      if (typeof alwaysShowRowBox !== 'undefined') {
        return alwaysShowRowBox;
      }

      return false;
    }
  }, {
    key: "keyboard",
    get: function get() {
      if ('keyboard' in this.props) {
        return this.props.keyboard;
      }

      var keyboard = this.getConfig('tableKeyboard');

      if (typeof keyboard !== 'undefined') {
        return keyboard;
      }

      return false;
    }
  }, {
    key: "columnResizable",
    get: function get() {
      if (this.currentEditRecord) {
        return false;
      }

      if ('columnResizable' in this.props) {
        return this.props.columnResizable;
      }

      return this.getConfig('tableColumnResizable') !== false;
    }
  }, {
    key: "columnHideable",
    get: function get() {
      if ('columnHideable' in this.props) {
        return this.props.columnHideable;
      }

      return this.getConfig('tableColumnHideable') !== false;
    }
    /**
     * 表头支持编辑
     */

  }, {
    key: "columnTitleEditable",
    get: function get() {
      if ('columnTitleEditable' in this.props) {
        return this.props.columnTitleEditable;
      }

      return this.getConfig('tableColumnTitleEditable') === true;
    }
  }, {
    key: "heightChangeable",
    get: function get() {
      if ('heightChangeable' in this.props) {
        return this.props.heightChangeable;
      }

      return this.getConfig('tableHeightChangeable') === true;
    }
  }, {
    key: "pagination",
    get: function get() {
      if ('pagination' in this.props) {
        return this.props.pagination;
      }

      return this.getConfig('pagination');
    }
  }, {
    key: "dragColumnAlign",
    get: function get() {
      if ('dragColumnAlign' in this.props) {
        return this.props.dragColumnAlign;
      }

      return this.getConfig('tableDragColumnAlign');
    }
  }, {
    key: "columnDraggable",
    get: function get() {
      if ('columnDraggable' in this.props) {
        return this.props.columnDraggable;
      }

      if ('dragColumn' in this.props) {
        return this.props.dragColumn;
      }

      if (this.getConfig('tableColumnDraggable') === true) {
        return true;
      }

      return this.getConfig('tableDragColumn') === true;
    }
  }, {
    key: "rowDraggable",
    get: function get() {
      if (this.groups.length) {
        return false;
      }

      if ('rowDraggable' in this.props) {
        return this.props.rowDraggable;
      }

      if ('dragRow' in this.props) {
        return this.props.dragRow;
      }

      if (this.getConfig('tableRowDraggable') === true) {
        return true;
      }

      return this.getConfig('tableDragRow') === true;
    }
  }, {
    key: "size",
    get: function get() {
      var size = this.customized.size;

      if (size !== undefined) {
        return size;
      }

      return this.props.size || Size["default"];
    }
  }, {
    key: "rowHeight",
    get: function get() {
      var _this$props$rowHeight = this.props.rowHeight,
          rowHeight = _this$props$rowHeight === void 0 ? defaultTo(this.getConfig('tableRowHeight'), 30) : _this$props$rowHeight;
      var size = this.size;

      if (typeof rowHeight === 'function') {
        return rowHeight({
          size: size
        });
      }

      if (isNumber(rowHeight)) {
        switch (size) {
          case Size.large:
            return rowHeight + 2;

          case Size.small:
            return rowHeight - 2;

          default:
        }
      }

      return rowHeight;
    }
  }, {
    key: "headerRowHeight",
    get: function get() {
      var _this$props$headerRow = this.props.headerRowHeight,
          headerRowHeight = _this$props$headerRow === void 0 ? this.getConfig('tableHeaderRowHeight') : _this$props$headerRow;

      if (headerRowHeight === undefined) {
        return this.rowHeight;
      }

      var size = this.size;

      if (typeof headerRowHeight === 'function') {
        return headerRowHeight({
          size: size
        });
      }

      if (isNumber(headerRowHeight)) {
        switch (size) {
          case Size.large:
            return headerRowHeight + 2;

          case Size.small:
            return headerRowHeight - 2;

          default:
        }
      }

      return headerRowHeight;
    }
  }, {
    key: "footerRowHeight",
    get: function get() {
      var _this$props$footerRow = this.props.footerRowHeight,
          footerRowHeight = _this$props$footerRow === void 0 ? this.getConfig('tableFooterRowHeight') : _this$props$footerRow;

      if (footerRowHeight === undefined) {
        return this.rowHeight;
      }

      var size = this.size;

      if (typeof footerRowHeight === 'function') {
        return footerRowHeight({
          size: size
        });
      }

      if (isNumber(footerRowHeight)) {
        switch (size) {
          case Size.large:
            return footerRowHeight + 2;

          case Size.small:
            return footerRowHeight - 2;

          default:
        }
      }

      return footerRowHeight;
    }
  }, {
    key: "autoFootHeight",
    get: function get() {
      if ('autoFootHeight' in this.props) {
        return this.props.autoFootHeight;
      }

      return false;
    }
  }, {
    key: "emptyText",
    get: function get() {
      var renderEmpty = this.props.renderEmpty;

      if (renderEmpty) {
        return renderEmpty();
      }

      return this.getConfig('renderEmpty')('Table');
    }
  }, {
    key: "highLightRow",
    get: function get() {
      if ('highLightRow' in this.props) {
        return this.props.highLightRow;
      }

      return this.getConfig('tableHighLightRow');
    }
  }, {
    key: "parityRow",
    get: function get() {
      var parityRow = this.customized.parityRow;

      if (parityRow !== undefined) {
        return parityRow;
      }

      if ('parityRow' in this.props) {
        return this.props.parityRow;
      }

      return this.getConfig('tableParityRow') === true;
    }
  }, {
    key: "showRemovedRow",
    get: function get() {
      if ('showRemovedRow' in this.props) {
        return this.props.showRemovedRow;
      }

      return this.getConfig('tableShowRemovedRow') === true;
    }
  }, {
    key: "autoFocus",
    get: function get() {
      if ('autoFocus' in this.props) {
        return this.props.autoFocus;
      }

      return this.getConfig('tableAutoFocus') !== false;
    }
  }, {
    key: "selectedHighLightRow",
    get: function get() {
      if ('selectedHighLightRow' in this.props) {
        return this.props.selectedHighLightRow;
      }

      return this.getConfig('tableSelectedHighLightRow') !== false;
    }
  }, {
    key: "editorNextKeyEnterDown",
    get: function get() {
      if ('editorNextKeyEnterDown' in this.props) {
        return this.props.editorNextKeyEnterDown;
      }

      return this.getConfig('tableEditorNextKeyEnterDown') !== false;
    }
  }, {
    key: "border",
    get: function get() {
      if ('border' in this.props) {
        return this.props.border;
      }

      return this.getConfig('tableBorder') !== false;
    }
  }, {
    key: "columnEditorBorder",
    get: function get() {
      if ('columnEditorBorder' in this.props) {
        return this.props.columnEditorBorder;
      }

      var tableColumnEditorBorder = this.getConfig('tableColumnEditorBorder');

      if (tableColumnEditorBorder !== undefined) {
        return tableColumnEditorBorder;
      }

      return this.border;
    }
  }, {
    key: "queryBar",
    get: function get() {
      return this.props.queryBar || this.getConfig('queryBar');
    }
  }, {
    key: "expandIcon",
    get: function get() {
      return this.props.expandIcon || this.getConfig('tableExpandIcon');
    }
  }, {
    key: "tableColumnResizeTrigger",
    get: function get() {
      return this.getConfig('tableColumnResizeTrigger');
    } // get pristine(): boolean | undefined {
    //   return this.props.pristine;
    // }

  }, {
    key: "currentEditRecord",
    get: function get() {
      return this.dataSet.find(function (record) {
        return record.editing === true;
      });
    },
    set: function set(record) {
      var _this2 = this;

      runInAction(function () {
        var currentEditRecord = _this2.currentEditRecord,
            dataSet = _this2.dataSet;

        if (currentEditRecord) {
          if (currentEditRecord.isNew) {
            dataSet.remove(currentEditRecord);
          } else {
            currentEditRecord.reset();
            currentEditRecord.editing = false;
          }
        }

        if (record) {
          record.editing = true;
        }
      });
    }
  }, {
    key: "computedRecordCachedType",
    get: function get() {
      return this.recordCachedType || this.defaultRecordCachedType;
    }
  }, {
    key: "isTree",
    get: function get() {
      return this.props.mode === TableMode.tree;
    }
  }, {
    key: "editing",
    get: function get() {
      return this.currentEditorName !== undefined || this.currentEditRecord !== undefined;
    }
  }, {
    key: "hasRowBox",
    get: function get() {
      var _this$props2 = this.props,
          dataSet = _this$props2.dataSet,
          selectionMode = _this$props2.selectionMode;
      var alwaysShowRowBox = this.alwaysShowRowBox;

      if (dataSet) {
        var selection = dataSet.selection;
        return selection && (selectionMode === SelectionMode.rowbox || alwaysShowRowBox);
      }

      return false;
    }
  }, {
    key: "useMouseBatchChoose",
    get: function get() {
      var useMouseBatchChoose = this.props.useMouseBatchChoose;

      if (useMouseBatchChoose !== undefined) {
        return useMouseBatchChoose;
      }

      return this.getConfig('tableUseMouseBatchChoose');
    }
  }, {
    key: "showCachedTips",
    get: function get() {
      var showCachedTips = this.props.showCachedTips;

      if (showCachedTips !== undefined) {
        return showCachedTips;
      }

      var tableShowCachedTips = this.getConfig('tableShowCachedTips');

      if (tableShowCachedTips !== undefined) {
        return tableShowCachedTips;
      }

      return false;
    }
  }, {
    key: "showSelectionTips",
    get: function get() {
      var showSelectionTips = this.props.showSelectionTips;

      if (showSelectionTips !== undefined) {
        return showSelectionTips;
      }

      var tableShowSelectionTips = this.getConfig('tableShowSelectionTips');

      if (tableShowSelectionTips !== undefined) {
        return tableShowSelectionTips;
      }

      return false;
    }
  }, {
    key: "overflowX",
    get: function get() {
      var width = this.width;

      if (width !== undefined) {
        return this.columnGroups.width > width;
      }

      return false;
    }
  }, {
    key: "overflowY",
    get: function get() {
      var bodyHeight = this.bodyHeight,
          height = this.height;
      return bodyHeight !== undefined && height !== undefined && height < bodyHeight + (!this.hasFooter && this.overflowX ? measureScrollbar() : 0);
    }
  }, {
    key: "hasRowGroups",
    get: function get() {
      if (this.cachedData.length) {
        return true;
      }

      var groups = this.groups;

      if (groups) {
        return groups.some(function (_ref19) {
          var type = _ref19.type;
          return type === GroupType.row;
        });
      }

      return false;
    }
  }, {
    key: "headerTableGroups",
    get: function get() {
      var groups = this.groups;
      return groups ? groups.filter(function (_ref20) {
        var type = _ref20.type;
        return type === GroupType.header;
      }) : [];
    }
  }, {
    key: "customizedColumnHeader",
    value: function customizedColumnHeader() {
      if (this.queryBar === TableQueryBarType.comboBar) {
        return /*#__PURE__*/React.createElement(ComboCustomizationSettings, null);
      }

      return /*#__PURE__*/React.createElement(CustomizationColumnHeader, {
        customizedBtn: this.customizedBtn,
        onHeaderClick: this.openCustomizationModal
      });
    }
  }, {
    key: "customizedColumn",
    get: function get() {
      if (this.customizable && !this.customizedBtn && (!this.rowDraggable || this.dragColumnAlign !== DragColumnAlign.right)) {
        return {
          key: CUSTOMIZED_KEY,
          resizable: false,
          titleEditable: false,
          align: ColumnAlign.center,
          width: scaleSize(30),
          lock: ColumnLock.right,
          header: this.customizedColumnHeader,
          headerClassName: "".concat(this.prefixCls, "-customized-column")
        };
      }

      return undefined;
    }
  }, {
    key: "expandColumn",
    get: function get() {
      if (this.expandIconAsCell) {
        return {
          key: EXPAND_KEY,
          resizable: false,
          titleEditable: false,
          align: ColumnAlign.center,
          width: scaleSize(50),
          lock: true
        };
      }

      return undefined;
    }
  }, {
    key: "rowNumberColumn",
    get: function get() {
      var rowNumber = this.props.rowNumber;

      if (rowNumber) {
        return {
          key: ROW_NUMBER_KEY,
          resizable: true,
          titleEditable: false,
          headerClassName: "".concat(this.prefixCls, "-row-number-column"),
          renderer: this.renderRowNumber,
          tooltip: TableColumnTooltip.overflow,
          align: ColumnAlign.center,
          width: scaleSize(50),
          lock: true
        };
      }

      return undefined;
    }
  }, {
    key: "selectionColumn",
    get: function get() {
      if (this.hasRowBox) {
        var dataSet = this.dataSet,
            prefixCls = this.prefixCls;
        var rowBoxPlacement = this.props.rowBoxPlacement;
        var className = "".concat(prefixCls, "-selection-column");
        var lock = ColumnLock.left;

        if (rowBoxPlacement === RowBoxPlacement.start) {
          lock = ColumnLock.left;
        } else if (rowBoxPlacement === RowBoxPlacement.end) {
          lock = ColumnLock.right;
        } else {
          lock = false;
        }

        var selectionColumn = {
          key: SELECTION_KEY,
          resizable: false,
          titleEditable: false,
          headerClassName: className,
          className: className,
          footerClassName: className,
          renderer: this.renderSelectionBox,
          align: ColumnAlign.center,
          width: scaleSize(50),
          lock: lock
        };

        if (dataSet && dataSet.selection === DataSetSelection.multiple) {
          selectionColumn.header = this.multipleSelectionRenderer;
          selectionColumn.footer = this.multipleSelectionRenderer;
        }

        return selectionColumn;
      }

      return undefined;
    }
  }, {
    key: "comboQueryColumn",
    get: function get() {
      var _this3 = this;

      if (this.queryBar === TableQueryBarType.comboBar) {
        var prefixCls = this.prefixCls,
            queryBarProps = this.props.queryBarProps;
        var showInlineSearch = queryBarProps && queryBarProps.inlineSearch;
        var showInlineSearchRender = queryBarProps && queryBarProps.inlineSearchRender;
        var className = "".concat(prefixCls, "-inline-query");
        var lock = ColumnLock.left;
        var queryColumn = {
          key: COMBOBAR_KEY,
          header: showInlineSearch && /*#__PURE__*/React.createElement(Icon, {
            type: "manage_search",
            className: className,
            onClick: action(function () {
              _this3.comboBarStatus = !_this3.comboBarStatus;

              _this3.node.handleHeightTypeChange();
            })
          }),
          resizable: false,
          titleEditable: false,
          headerClassName: className,
          className: className,
          footerClassName: className,
          renderer: function renderer() {
            return queryBarProps && queryBarProps.inlineSearchRender;
          },
          align: ColumnAlign.center,
          width: scaleSize(50),
          lock: lock
        };
        return showInlineSearch === true || showInlineSearchRender ? queryColumn : undefined;
      }

      return undefined;
    }
  }, {
    key: "draggableColumn",
    get: function get() {
      var dragColumnAlign = this.dragColumnAlign,
          rowDraggable = this.rowDraggable,
          prefixCls = this.prefixCls;

      if (dragColumnAlign && rowDraggable) {
        var draggableColumn = {
          key: DRAG_KEY,
          resizable: false,
          titleEditable: false,
          className: "".concat(prefixCls, "-drag-column"),
          renderer: this.renderDragBox,
          align: ColumnAlign.center,
          width: scaleSize(50)
        };

        if (dragColumnAlign === DragColumnAlign.left) {
          draggableColumn.lock = ColumnLock.left;
        }

        if (dragColumnAlign === DragColumnAlign.right) {
          draggableColumn.lock = ColumnLock.right;
          draggableColumn.header = this.customizable && this.customizedColumnHeader;
        }

        return draggableColumn;
      }

      return undefined;
    }
  }, {
    key: "leftColumns",
    get: function get() {
      var dragColumnAlign = this.dragColumnAlign,
          leftOriginalColumns = this.leftOriginalColumns,
          expandColumn = this.expandColumn,
          expandIconColumnIndex = this.expandIconColumnIndex,
          draggableColumn = this.draggableColumn,
          rowNumberColumn = this.rowNumberColumn,
          selectionColumn = this.selectionColumn,
          comboQueryColumn = this.comboQueryColumn;
      return observable.array([expandIconColumnIndex ? undefined : expandColumn, dragColumnAlign === DragColumnAlign.left ? draggableColumn : undefined, rowNumberColumn, selectionColumn && selectionColumn.lock === ColumnLock.left ? selectionColumn : undefined, comboQueryColumn].concat(_toConsumableArray(leftOriginalColumns)).filter(columnFilter));
    }
  }, {
    key: "rightColumns",
    get: function get() {
      var dragColumnAlign = this.dragColumnAlign,
          rightOriginalColumns = this.rightOriginalColumns,
          draggableColumn = this.draggableColumn,
          customizedColumn = this.customizedColumn,
          selectionColumn = this.selectionColumn;
      return observable.array([].concat(_toConsumableArray(rightOriginalColumns), [dragColumnAlign === DragColumnAlign.right ? draggableColumn : undefined, selectionColumn && selectionColumn.lock === ColumnLock.right ? selectionColumn : undefined, customizedColumn]).filter(columnFilter));
    }
  }, {
    key: "columns",
    get: function get() {
      var leftColumns = this.leftColumns,
          rightColumns = this.rightColumns,
          selectionColumn = this.selectionColumn,
          expandColumn = this.expandColumn,
          expandIconColumnIndex = this.expandIconColumnIndex,
          rowBoxPlacement = this.props.rowBoxPlacement;
      var originalColumns = this.originalColumns.slice();

      if (isNumber(rowBoxPlacement) && selectionColumn) {
        originalColumns.splice(rowBoxPlacement, 0, selectionColumn);
      }

      var allColumns = Array.from([].concat(_toConsumableArray(leftColumns), _toConsumableArray(originalColumns), _toConsumableArray(rightColumns)));

      if (expandIconColumnIndex && expandColumn) {
        var lock = true;

        if (expandIconColumnIndex > leftColumns.length && expandIconColumnIndex <= leftColumns.length + originalColumns.length) {
          lock = false;
        } else if (expandIconColumnIndex > leftColumns.length + originalColumns.length) {
          lock = ColumnLock.right;
        }

        allColumns.splice(expandIconColumnIndex, 0, _objectSpread(_objectSpread({}, expandColumn), {}, {
          lock: lock
        }));
      }

      return observable.array(_toConsumableArray(allColumns));
    }
  }, {
    key: "columnGroups",
    get: function get() {
      return new ColumnGroups(this.columns, this);
    }
  }, {
    key: "leftColumnGroups",
    get: function get() {
      return new ColumnGroups(this.leftColumns, this);
    }
  }, {
    key: "rightColumnGroups",
    get: function get() {
      return new ColumnGroups(this.rightColumns, this);
    }
  }, {
    key: "leafNamedColumns",
    get: function get() {
      return this.columnGroups.allLeafs.reduce(function (list, _ref21) {
        var column = _ref21.column;
        return column.name ? list.concat(column) : list;
      }, []);
    }
  }, {
    key: "hasEmptyWidthColumn",
    get: function get() {
      return this.columnGroups.leafs.some(function (_ref22) {
        var column = _ref22.column;
        return isNil(_get(column, 'width'));
      });
    }
  }, {
    key: "getLastEmptyWidthColumn",
    get: function get() {
      var emptyWidthColumns = this.columnGroups.leafs.filter(function (_ref23) {
        var column = _ref23.column;
        return isNil(_get(column, 'width'));
      });
      return emptyWidthColumns.length ? emptyWidthColumns[emptyWidthColumns.length - 1].column : undefined;
    }
  }, {
    key: "hasCheckFieldColumn",
    get: function get() {
      var checkField = this.dataSet.props.checkField;

      if (checkField) {
        var aggregation = this.aggregation;
        return this.columnGroups.leafs.some(function (_ref24) {
          var column = _ref24.column;
          return aggregation ? treeSome([column], function (c) {
            return hasCheckField(c, checkField);
          }) : hasCheckField(column, checkField);
        });
      }

      return false;
    }
  }, {
    key: "hasFooter",
    get: function get() {
      return this.columnGroups.leafs.some(function (_ref25) {
        var column = _ref25.column;
        return !!column.footer && column.key !== SELECTION_KEY;
      });
    }
  }, {
    key: "isAnyColumnsLeftLock",
    get: function get() {
      return this.leftColumns.length > 0;
    }
  }, {
    key: "isAnyColumnsRightLock",
    get: function get() {
      return this.rightColumns.length > 0;
    }
  }, {
    key: "isAnyColumnsLock",
    get: function get() {
      return this.isAnyColumnsLeftLock || this.isAnyColumnsRightLock;
    }
  }, {
    key: "isCombinedColumn",
    get: function get() {
      var columns = this.columns;
      var combinedColumn = columns.find(function (column) {
        return !column.aggregation && column.children && column.children.length > 0;
      });
      return !isNil(combinedColumn);
    }
  }, {
    key: "cachedDataInType",
    get: function get() {
      var dataSet = this.dataSet,
          showCachedSelection = this.showCachedSelection,
          _this$computedRecordC = this.computedRecordCachedType,
          computedRecordCachedType = _this$computedRecordC === void 0 ? RecordCachedType.selected : _this$computedRecordC;

      if (showCachedSelection) {
        switch (computedRecordCachedType) {
          case RecordCachedType.selected:
            return dataSet.cachedSelected;

          case RecordCachedType.add:
            return dataSet.cachedCreated;

          case RecordCachedType.update:
            return dataSet.cachedUpdated;

          case RecordCachedType["delete"]:
            return dataSet.cachedDestroyed;

          default:
        }
      }

      return [];
    }
  }, {
    key: "cachedData",
    get: function get() {
      var dataSet = this.dataSet,
          showCachedSelection = this.showCachedSelection;

      if (showCachedSelection) {
        return dataSet.cachedRecords;
      }

      return [];
    }
  }, {
    key: "currentData",
    get: function get() {
      var _this$props3 = this.props,
          pristine = _this$props3.pristine,
          recordFilter = _this$props3.filter,
          treeFilter = _this$props3.treeFilter;
      var dataSet = this.dataSet,
          isTree = this.isTree,
          headerFilter = this.headerFilter;
      var filter = isTree ? typeof treeFilter === 'function' ? treeFilter : recordFilter : recordFilter;
      var data = isTree ? dataSet.treeRecords : dataSet.records;

      if (typeof filter === 'function') {
        data = data.filter(filter);
      }

      if (pristine) {
        data = data.filter(function (record) {
          return !record.isNew;
        });
      }

      if (headerFilter) {
        var _filter = headerFilter.filter,
            filterText = headerFilter.filterText;

        if (typeof _filter === 'function') {
          data = data.filter(function (record) {
            return _filter({
              record: record,
              filterText: filterText
            });
          });
        } else {
          var field = dataSet.getField(headerFilter.fieldName);
          var type = field && field.get('type');
          var multiple = field && field.get('multiple');
          var isLookUp = false;

          if (field && (field.get('lookupCode') || isString(field.get('lookupUrl')) || type !== FieldType.object && (field.get('lovCode') || field.getLookup() || field.get('options')) || field.get('lovCode'))) {
            isLookUp = true;
          }

          data = data.filter(function (record) {
            var recordText;

            if (multiple) {
              if (isLookUp) {
                recordText = record.get(headerFilter.fieldName).map(function (value) {
                  return field.getText(value);
                }).join('');
              } else {
                recordText = record.get(headerFilter.fieldName).join('');
              }
            } else {
              recordText = isLookUp ? String(field.getText(record.get(headerFilter.fieldName))) : String(record.get(headerFilter.fieldName));
            }

            return recordText.toLocaleLowerCase().includes(String(headerFilter.filterText).toLocaleLowerCase());
          });
        }
      }

      return data;
    }
  }, {
    key: "treeFilter",
    get: function get() {
      var treeFilter = this.props.treeFilter;
      return treeFilter;
    }
  }, {
    key: "data",
    get: function get() {
      return [].concat(_toConsumableArray(this.cachedData), _toConsumableArray(this.currentData));
    }
  }, {
    key: "cachedIndeterminate",
    get: function get() {
      var dataSet = this.dataSet,
          showCachedSelection = this.showCachedSelection;

      if (dataSet) {
        var _ref26 = showCachedSelection ? getCachedSelectableCounts(dataSet, this.computedRecordCachedType, this.showCachedTips) : [0, 0],
            _ref27 = _slicedToArray(_ref26, 2),
            cachedSelectedLength = _ref27[0],
            cachedRecordsLength = _ref27[1];

        if (cachedSelectedLength) {
          return cachedSelectedLength !== cachedRecordsLength;
        }
      }

      return false;
    }
  }, {
    key: "allCachedChecked",
    get: function get() {
      var dataSet = this.dataSet,
          showCachedSelection = this.showCachedSelection;

      if (dataSet) {
        var _ref28 = showCachedSelection ? getCachedSelectableCounts(dataSet, this.computedRecordCachedType, this.showCachedTips) : [0, 0],
            _ref29 = _slicedToArray(_ref28, 2),
            cachedSelectedLength = _ref29[0],
            cachedRecordsLength = _ref29[1];

        if (cachedSelectedLength) {
          return cachedSelectedLength === cachedRecordsLength;
        }
      }

      return false;
    }
  }, {
    key: "currentIndeterminate",
    get: function get() {
      var _this$props4 = this.props,
          dataSet = _this$props4.dataSet,
          filter = _this$props4.filter;

      if (dataSet) {
        var _getCurrentSelectable = getCurrentSelectableCounts(dataSet, filter),
            _getCurrentSelectable2 = _slicedToArray(_getCurrentSelectable, 2),
            selectedLength = _getCurrentSelectable2[0],
            currentLength = _getCurrentSelectable2[1];

        if (selectedLength) {
          return selectedLength !== currentLength;
        }
      }

      return false;
    }
  }, {
    key: "allCurrentChecked",
    get: function get() {
      var _this$props5 = this.props,
          dataSet = _this$props5.dataSet,
          filter = _this$props5.filter;

      if (dataSet) {
        var _getCurrentSelectable3 = getCurrentSelectableCounts(dataSet, filter),
            _getCurrentSelectable4 = _slicedToArray(_getCurrentSelectable3, 2),
            selectedLength = _getCurrentSelectable4[0],
            currentLength = _getCurrentSelectable4[1];

        if (selectedLength) {
          return selectedLength === currentLength;
        }
      }

      return false;
    }
  }, {
    key: "indeterminate",
    get: function get() {
      var showCachedSelection = this.showCachedSelection;

      if (showCachedSelection) {
        var dataSet = this.dataSet;

        var _getCachedSelectableC = getCachedSelectableCounts(dataSet, this.computedRecordCachedType, this.showCachedTips),
            _getCachedSelectableC2 = _slicedToArray(_getCachedSelectableC, 2),
            cachedSelectedLength = _getCachedSelectableC2[0],
            cachedRecordsLength = _getCachedSelectableC2[1];

        var allLength = cachedSelectedLength + dataSet.currentSelected.length;
        return !!allLength && allLength !== cachedRecordsLength + dataSet.records.length;
      }

      return this.currentIndeterminate;
    }
  }, {
    key: "allChecked",
    get: function get() {
      var showCachedSelection = this.showCachedSelection;

      if (showCachedSelection) {
        var dataSet = this.dataSet;

        var _getCachedSelectableC3 = getCachedSelectableCounts(dataSet, this.computedRecordCachedType, this.showCachedTips),
            _getCachedSelectableC4 = _slicedToArray(_getCachedSelectableC3, 2),
            cachedSelectedLength = _getCachedSelectableC4[0],
            cachedRecordsLength = _getCachedSelectableC4[1];

        var allLength = cachedSelectedLength + dataSet.currentSelected.length;
        return !!allLength && allLength === cachedRecordsLength + dataSet.records.length;
      }

      return this.allCurrentChecked;
    }
  }, {
    key: "expandIconAsCell",
    get: function get() {
      var _this$props6 = this.props,
          expandedRowRenderer = _this$props6.expandedRowRenderer,
          expandIconAsCell = _this$props6.expandIconAsCell;

      if (expandIconAsCell !== undefined) {
        return expandIconAsCell;
      }

      return !!expandedRowRenderer && !this.isTree;
    }
  }, {
    key: "expandIconColumnIndex",
    get: function get() {
      var dragColumnAlign = this.dragColumnAlign,
          rowDraggable = this.rowDraggable,
          _this$props7 = this.props,
          _this$props7$expandIc = _this$props7.expandIconColumnIndex,
          expandIconColumnIndex = _this$props7$expandIc === void 0 ? 0 : _this$props7$expandIc,
          rowNumber = _this$props7.rowNumber;

      if ((!expandIconColumnIndex || typeof expandIconColumnIndex !== 'number') && !this.isTree) {
        return 0;
      }

      return expandIconColumnIndex + [this.hasRowBox, rowNumber, dragColumnAlign && rowDraggable, !!this.comboQueryColumn].filter(Boolean).length;
    }
  }, {
    key: "inlineEdit",
    get: function get() {
      return this.props.editMode === TableEditMode.inline;
    }
  }, {
    key: "checkAllCurrent",
    value: function checkAllCurrent() {
      var _this$props8 = this.props,
          dataSet = _this$props8.dataSet,
          filter = _this$props8.filter;
      dataSet.selectAll(filter);
    }
  }, {
    key: "unCheckAllCurrent",
    value: function unCheckAllCurrent() {
      var dataSet = this.props.dataSet;
      dataSet.unSelectAll();
    }
  }, {
    key: "checkAllCached",
    value: function checkAllCached() {
      if (this.showCachedSelection) {
        var _this$props9 = this.props,
            dataSet = _this$props9.dataSet,
            filter = _this$props9.filter;
        dataSet.batchSelect(getCachedSelectableRecords(dataSet, this.computedRecordCachedType, this.showCachedTips, filter));
      }
    }
  }, {
    key: "unCheckAllCached",
    value: function unCheckAllCached() {
      if (this.showCachedSelection) {
        var _this$props10 = this.props,
            dataSet = _this$props10.dataSet,
            filter = _this$props10.filter;
        dataSet.batchUnSelect(getCachedSelectableRecords(dataSet, this.computedRecordCachedType, this.showCachedTips, filter));
      }
    }
  }, {
    key: "isBuiltInColumn",
    value: function isBuiltInColumn(_ref30) {
      var key = _ref30.key;

      if (isString(key)) {
        return [DRAG_KEY, SELECTION_KEY, ROW_NUMBER_KEY, CUSTOMIZED_KEY, EXPAND_KEY, COMBOBAR_KEY].includes(key);
      }
    }
  }, {
    key: "getColumnTooltip",
    value: function getColumnTooltip(column) {
      var tooltip = column.tooltip;

      if (tooltip) {
        return tooltip;
      }

      return this.node.context.getTooltip('table-cell');
    }
  }, {
    key: "getColumnHeaders",
    value: function getColumnHeaders() {
      var leafNamedColumns = this.leafNamedColumns,
          dataSet = this.dataSet;
      return getHeaderTexts(dataSet, leafNamedColumns.slice(), this.aggregation);
    }
  }, {
    key: "getColumnTagRenderer",
    value: function getColumnTagRenderer(column) {
      var tagRenderer = column.tagRenderer;
      return tagRenderer;
    }
  }, {
    key: "showEditor",
    value: function showEditor(name) {
      this.currentEditorName = name;
    }
  }, {
    key: "setLastScrollTop",
    value: function setLastScrollTop(lastScrollTop) {
      if (this.virtual) {
        this.lastScrollTop = lastScrollTop;
        this.startScroll();
      }
    }
  }, {
    key: "setLastScrollLeft",
    value: function setLastScrollLeft(lastScrollLeft) {
      if (this.propVirtual) {
        this.lastScrollLeft = lastScrollLeft;
        this.startScroll();
      }
    }
  }, {
    key: "hideEditor",
    value: function hideEditor() {
      this.currentEditorName = undefined;
    }
  }, {
    key: "changeMouseBatchChooseIdList",
    value: function changeMouseBatchChooseIdList(idList) {
      this.mouseBatchChooseIdList = idList;
    }
  }, {
    key: "showNextEditor",
    value: function showNextEditor(name, reserve) {
      var dataSet = this.dataSet;
      var currentIndex = dataSet.currentIndex;
      var record = dataSet.get(reserve ? currentIndex - 1 : currentIndex + 1);

      if (record && !isDisabledRow(record)) {
        dataSet.current = record;
        this.showEditor(name);
        return true;
      }

      return false;
    }
  }, {
    key: "setProps",
    value: function setProps(props) {
      this.props = props;
      this.initGroups();
      var showCachedSelection = props.showCachedSelection;

      if (showCachedSelection !== undefined) {
        this.showCachedSelection = showCachedSelection;
      }
    }
  }, {
    key: "updateProps",
    value: function updateProps(props) {
      var _this$props11 = this.props,
          customizedCode = _this$props11.customizedCode,
          oldAggregation = _this$props11.aggregation;
      this.setProps(props);

      if (this.customizable) {
        if (customizedCode !== props.customizedCode) {
          this.loadCustomized();
          return;
        }

        var aggregation = props.aggregation;

        if (oldAggregation !== aggregation) {
          var customized = this.customized;

          if (!isNil(aggregation) && aggregation !== customized.aggregation) {
            customized.aggregation = aggregation;
            this.saveCustomized();
          }
        }
      }

      this.initColumns();
    }
  }, {
    key: "disposeGroupReaction",
    value: function disposeGroupReaction() {
      var groupReaction = this.groupReaction;

      if (groupReaction) {
        groupReaction();
      }
    }
  }, {
    key: "dispose",
    value: function dispose() {
      this.disposeGroupReaction();
    }
  }, {
    key: "initGroups",
    value: function initGroups() {
      var _this4 = this;

      this.disposeGroupReaction();
      var _this$props$groups = this.props.groups,
          groups = _this$props$groups === void 0 ? [] : _this$props$groups;

      if (groups.length) {
        this.groupReaction = reaction(function () {
          return dataSet.records;
        }, function () {
          return _this4.initGroups();
        });
        var headerGroupNames = [];
        var rowGroupNames = [];
        var groupNames = [];
        var parentFields = new Map();
        var dataSet = this.dataSet;
        var header = [];
        var row = [];
        var column = [];
        groups.forEach(function (group) {
          var type = group.type,
              name = group.name,
              parentField = group.parentField;

          switch (type) {
            case GroupType.header:
              header.push(group);
              headerGroupNames.push(name);
              break;

            case GroupType.row:
              row.push(group);
              rowGroupNames.push(name);
              break;

            case GroupType.none:
              break;

            default:
              {
                column.push(group);
                groupNames.push(name);
              }
          }

          if (parentField) {
            parentFields.set(name, parentField);
          }
        });
        this.groupedData = mergeGroupStates(normalizeGroups(rowGroupNames.concat(groupNames), headerGroupNames, dataSet.records, parentFields), this.groupedData);
        this.groupedDataWithHeader = mergeGroupStates(normalizeGroups(headerGroupNames.concat(rowGroupNames, groupNames), [], dataSet.records), this.groupedDataWithHeader);
        this.groups = [].concat(header, row, column);
      } else {
        this.groups = [];
        this.groupedData = [];
        this.groupedDataWithHeader = [];
      }
    }
  }, {
    key: "initColumns",
    value: function initColumns() {
      var customized = this.customized,
          customizable = this.customizable,
          aggregation = this.aggregation;
      var _this$props12 = this.props,
          columns = _this$props12.columns,
          children = _this$props12.children;
      var customizedColumns = customizable ? customized.columns : undefined;

      var _normalizeGroupColumn = normalizeGroupColumns(this, columns, children, aggregation, customizedColumns),
          _normalizeGroupColumn2 = _slicedToArray(_normalizeGroupColumn, 4),
          leftOriginalColumns = _normalizeGroupColumn2[0],
          originalColumns = _normalizeGroupColumn2[1],
          rightOriginalColumns = _normalizeGroupColumn2[2],
          hasAggregationColumn = _normalizeGroupColumn2[3];

      this.leftOriginalColumns = leftOriginalColumns;
      this.originalColumns = originalColumns;
      this.rightOriginalColumns = rightOriginalColumns;
      this.hasAggregationColumn = hasAggregationColumn;
    }
  }, {
    key: "isAggregationCellExpanded",
    value: function isAggregationCellExpanded(record, key) {
      var expandedKeys = record.getState(AGGREGATION_EXPAND_CELL_KEY);

      if (expandedKeys) {
        return expandedKeys.get(key);
      }
    }
  }, {
    key: "setAggregationCellExpanded",
    value: function setAggregationCellExpanded(record, key, expanded) {
      var expandedKeys = record.getState(AGGREGATION_EXPAND_CELL_KEY) || observable.map();
      expandedKeys.set(key, expanded);
      record.setState(AGGREGATION_EXPAND_CELL_KEY, expandedKeys);
    }
  }, {
    key: "isBodyExpanded",
    get: function get() {
      if (!this.props.bodyExpandable) {
        return true;
      }

      var bodyExpanded = this.props.bodyExpanded;

      if (bodyExpanded !== undefined) {
        return bodyExpanded;
      }

      var isBodyExpanded = this.dataSet.getState(BODY_EXPANDED);

      if (isBodyExpanded !== undefined) {
        return isBodyExpanded;
      }

      return defaultTo(this.props.defaultBodyExpanded, true);
    }
  }, {
    key: "setBodyExpanded",
    value: function setBodyExpanded(isBodyExpanded) {
      var _this$props13 = this.props,
          bodyExpanded = _this$props13.bodyExpanded,
          _this$props13$onBodyE = _this$props13.onBodyExpand,
          onBodyExpand = _this$props13$onBodyE === void 0 ? noop : _this$props13$onBodyE;

      if (bodyExpanded === undefined) {
        this.dataSet.setState(BODY_EXPANDED, isBodyExpanded);
      }

      onBodyExpand(isBodyExpanded);
    }
  }, {
    key: "isGroupExpanded",
    value: function isGroupExpanded(group) {
      return group.isExpanded && (!group.parent || this.isGroupExpanded(group.parent));
    }
  }, {
    key: "setGroupExpanded",
    value: function setGroupExpanded(group, isExpanded) {
      group.isExpanded = isExpanded;
    }
  }, {
    key: "isRowExpanded",
    value: function isRowExpanded(record) {
      var _record$isExpanded = record.isExpanded,
          isExpanded = _record$isExpanded === void 0 ? this.expandedRows.indexOf(record.key) !== -1 : _record$isExpanded;
      return isExpanded && (!this.isTree || !record.parent || this.isRowExpanded(record.parent));
    }
    /**
     *
     * @param record 想修改的record
     * @param expanded 设置是否展开
     * @param disHandler 设置是否需要触发展开事件
     */

  }, {
    key: "setRowExpanded",
    value: function setRowExpanded(record, expanded, disHandler) {
      record.isExpanded = expanded;

      if (!this.inBatchExpansion) {
        var index = this.expandedRows.indexOf(record.key);

        if (expanded) {
          if (index === -1) {
            this.expandedRows.push(record.key);
          }
        } else if (index !== -1) {
          this.expandedRows.splice(index, 1);
        }
      }

      var onExpand = this.props.onExpand;

      if (onExpand && !disHandler) {
        onExpand(expanded, record);
      }

      if (expanded && this.canTreeLoadData) {
        this.onTreeNodeLoad({
          record: record
        });
      }
    }
  }, {
    key: "isRowPending",
    value: function isRowPending(record) {
      return record.pending === true;
    }
  }, {
    key: "setRowPending",
    value: function setRowPending(record, pending) {
      record.pending = pending;
    }
  }, {
    key: "isRowLoaded",
    value: function isRowLoaded(record) {
      return record.childrenLoaded === true;
    }
  }, {
    key: "setRowLoaded",
    value: function setRowLoaded(record, loaded) {
      record.childrenLoaded = loaded;
    }
  }, {
    key: "isRowHover",
    value: function isRowHover(record) {
      return this.hoverRow === record;
    }
  }, {
    key: "canTreeLoadData",
    get: function get() {
      var _this$props14 = this.props,
          treeLoadData = _this$props14.treeLoadData,
          treeAsync = _this$props14.treeAsync;
      return treeAsync || !!treeLoadData;
    }
  }, {
    key: "cellHighlightRenderer",
    get: function get() {
      var _this$props$cellHighl = this.props.cellHighlightRenderer,
          cellHighlightRenderer = _this$props$cellHighl === void 0 ? this.getConfig('highlightRenderer') : _this$props$cellHighl;
      return cellHighlightRenderer;
    }
  }, {
    key: "setRowHover",
    value: function setRowHover(record, hover) {
      this.hoverRow = hover ? record : undefined;
    }
  }, {
    key: "expandAll",
    value: function expandAll() {
      var _this5 = this;

      this.inBatchExpansion = true;
      this.expandedRows = this.dataSet.records.map(function (record) {
        _this5.setRowExpanded(record, true);

        return record.key;
      });
      this.inBatchExpansion = false;
    }
  }, {
    key: "collapseAll",
    value: function collapseAll() {
      var _this6 = this;

      this.inBatchExpansion = true;
      this.dataSet.records.forEach(function (record) {
        return _this6.setRowExpanded(record, false);
      });
      this.expandedRows = [];
      this.inBatchExpansion = false;
    }
  }, {
    key: "onTreeNodeLoad",
    value: function () {
      var _onTreeNodeLoad = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(_ref31) {
        var record, _this$props15, dataSet, treeLoadData, treeAsync, selectionMode, promises, parentSelect;

        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                record = _ref31.record;
                _this$props15 = this.props, dataSet = _this$props15.dataSet, treeLoadData = _this$props15.treeLoadData, treeAsync = _this$props15.treeAsync, selectionMode = _this$props15.selectionMode;
                promises = [];
                this.setRowPending(record, true);

                if (treeAsync && dataSet) {
                  promises.push(dataSet.queryMoreChild(record, dataSet.currentPage));
                }

                if (treeLoadData) {
                  promises.push(treeLoadData({
                    record: record,
                    dataSet: dataSet
                  }));
                } // 由子选父


                parentSelect = function parentSelect(parent) {
                  if (!parent.isSelected && parent.children && parent.children.length > 0 && parent.children.every(function (child) {
                    return child.isSelected;
                  })) {
                    dataSet.select(parent);

                    if (parent.parent) {
                      parentSelect(parent.parent);
                    }
                  }
                };

                _context.prev = 7;
                _context.next = 10;
                return Promise.all(promises);

              case 10:
                if (selectionMode === SelectionMode.treebox) {
                  // 由父选子
                  if (record.isSelected) {
                    defaultTo(record.children, []).forEach(function (child) {
                      if (!child.isSelected) {
                        dataSet.select(child);
                      }
                    });
                  } else {
                    parentSelect(record);
                  }
                }

                this.setRowLoaded(record, true);

              case 12:
                _context.prev = 12;
                this.setRowPending(record, false);
                return _context.finish(12);

              case 15:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this, [[7,, 12, 15]]);
      }));

      function onTreeNodeLoad(_x4) {
        return _onTreeNodeLoad.apply(this, arguments);
      }

      return onTreeNodeLoad;
    }()
  }, {
    key: "renderSelectionBox",
    value: function renderSelectionBox(_ref32) {
      var record = _ref32.record;
      var selectionBoxRenderer = this.props.selectionBoxRenderer;

      if (selectionBoxRenderer && isFunction(selectionBoxRenderer)) {
        var element = _renderSelectionBox({
          record: record,
          store: this
        });

        return selectionBoxRenderer({
          record: record,
          element: element
        });
      }

      return _renderSelectionBox({
        record: record,
        store: this
      });
    }
  }, {
    key: "renderRowNumber",
    value: function renderRowNumber(_ref33) {
      var record = _ref33.record,
          dataSet = _ref33.dataSet;
      var isTree = this.isTree,
          rowNumber = this.props.rowNumber;
      var numbers = getRowNumbers(record, dataSet, isTree);
      var number = numbers.join('-');

      if (typeof rowNumber === 'function') {
        return rowNumber({
          record: record,
          dataSet: dataSet,
          text: number,
          pathNumbers: numbers
        });
      }

      return number;
    }
  }, {
    key: "renderDragBox",
    value: function renderDragBox(_ref34) {
      var record = _ref34.record;
      var rowDragRender = this.props.rowDragRender;

      if (rowDragRender && isFunction(rowDragRender.renderIcon)) {
        return rowDragRender.renderIcon({
          record: record
        });
      }

      return /*#__PURE__*/React.createElement(Icon, {
        type: "baseline-drag_indicator"
      });
    }
  }, {
    key: "findColumnGroup",
    value: function findColumnGroup(indexOrKeyOrName) {
      var allLeafs = this.columnGroups.allLeafs;
      return isNumber(indexOrKeyOrName) ? allLeafs[indexOrKeyOrName] : allLeafs.find(function (_ref35) {
        var key = _ref35.key;
        return String(key) === indexOrKeyOrName;
      });
    }
  }, {
    key: "setColumnWidth",
    value: function setColumnWidth(columnGroup, width) {
      var saveToCustomization = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
      var column = columnGroup.column;

      if (width !== column.width) {
        this.changeCustomizedColumnValue(column, {
          width: width
        }, saveToCustomization);
        var onColumnResize = this.props.onColumnResize;

        if (onColumnResize) {
          var index = this.columnGroups.allLeafs.indexOf(columnGroup);
          /**
           * onColumnResize 事件回调
           * 回调参数：
           * @param column
           * @param width
           */

          onColumnResize({
            column: column,
            width: width,
            index: index
          });
        }
      }
    }
  }, {
    key: "changeCustomizedColumnValue",
    value: function changeCustomizedColumnValue(column, value) {
      var saveToCustomization = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
      var columns = this.customized.columns;
      set(column, value);
      var columnKey = getColumnKey(column).toString();

      var oldCustomized = _get(columns, columnKey);

      set(columns, columnKey, _objectSpread(_objectSpread({}, oldCustomized), value));

      if (saveToCustomization) {
        this.saveCustomizedDebounce();
      }
    }
  }, {
    key: "saveCustomized",
    value: function () {
      var _saveCustomized = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2(customized) {
        var _this$props16, customizedCode, boardCustomized, tableCustomizedSave, tableCustomizedLoad, res;

        return _regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                if (!(this.customizable && this.customizedLoaded)) {
                  _context2.next = 17;
                  break;
                }

                _this$props16 = this.props, customizedCode = _this$props16.customizedCode, boardCustomized = _this$props16.boardCustomized;

                if (customized) {
                  this.customized = customized;
                }

                if (!customizedCode) {
                  _context2.next = 17;
                  break;
                }

                tableCustomizedSave = this.getConfig('tableCustomizedSave') || this.getConfig('customizedSave');
                tableCustomizedLoad = this.getConfig('tableCustomizedLoad') || this.getConfig('customizedLoad'); // board 组件列表视图配置保存

                if (!this.customizedBtn) {
                  _context2.next = 16;
                  break;
                }

                _context2.next = 9;
                return tableCustomizedSave(customizedCode, _objectSpread(_objectSpread({
                  dataJson: JSON.stringify(omit(this.customized, ['dataJson', 'creationDate', 'createdBy', 'lastUpdateDate', 'lastUpdatedBy', '_token', 'userId', 'tenantId', 'id']))
                }, omit(this.customized, ['dataJson'])), {}, {
                  defaultFlag: 1,
                  viewType: 'table'
                }), this.customizedBtn ? 'Board' : 'Table');

              case 9:
                if (!(customizedCode && boardCustomized && boardCustomized.customizedDS)) {
                  _context2.next = 14;
                  break;
                }

                _context2.next = 12;
                return tableCustomizedLoad(customizedCode, 'Board', {
                  type: 'detail',
                  id: this.customized.id
                });

              case 12:
                res = _context2.sent;

                try {
                  // @ts-ignore
                  this.customized = _objectSpread(_objectSpread({
                    columns: {}
                  }, JSON.parse(res.dataJson)), res);
                  boardCustomized.customizedDS.current.set('viewName', res.viewName);
                } catch (error) {
                  warning(false, error.message);
                }

              case 14:
                _context2.next = 17;
                break;

              case 16:
                tableCustomizedSave(customizedCode, this.customized, 'Table');

              case 17:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function saveCustomized(_x5) {
        return _saveCustomized.apply(this, arguments);
      }

      return saveCustomized;
    }()
  }, {
    key: "openCustomizationModal",
    value: function openCustomizationModal(context) {
      var customizedCode = this.props.customizedCode;
      var modalProps = {
        key: 'TABLE_CUSTOMIZATION_MODAL',
        drawer: true,
        size: Size.small,
        title: this.customizedBtn ? '表格视图配置' : $l('Table', 'customization_settings'),
        children: /*#__PURE__*/React.createElement(CustomizationSettings, {
          context: context
        }),
        bodyStyle: {
          overflow: 'hidden auto',
          padding: 0
        }
      };

      if (customizedCode) {
        modalProps.okText = $l('Table', 'save_button');
      }

      Modal.open(modalProps);
    }
  }, {
    key: "loadCustomized",
    value: function () {
      var _loadCustomized = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee3() {
        var _this7 = this;

        var _this$props17, customizedCode, boardCustomized, queryBarProps, showSimpleMode, tableCustomizedLoad, customized;

        return _regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _this$props17 = this.props, customizedCode = _this$props17.customizedCode, boardCustomized = _this$props17.boardCustomized;
                queryBarProps = this.props.queryBarProps;
                showSimpleMode = queryBarProps && queryBarProps.simpleMode;

                if (!(this.customizable && customizedCode || this.queryBar === TableQueryBarType.comboBar && !showSimpleMode)) {
                  _context3.next = 21;
                  break;
                }

                tableCustomizedLoad = this.getConfig('tableCustomizedLoad') || this.getConfig('customizedLoad');
                runInAction(function () {
                  delete _this7.customizedLoaded;
                  _this7.loading = true;
                });
                _context3.prev = 6;

                if (!(customizedCode && !boardCustomized)) {
                  _context3.next = 11;
                  break;
                }

                _context3.next = 10;
                return tableCustomizedLoad(customizedCode, 'Table');

              case 10:
                customized = _context3.sent;

              case 11:
                if (!(customizedCode && boardCustomized && boardCustomized.customizedDS)) {
                  _context3.next = 16;
                  break;
                }

                _context3.next = 14;
                return tableCustomizedLoad(customizedCode, 'Board', {
                  type: 'detail',
                  id: boardCustomized.customizedDS.current.get('id')
                });

              case 14:
                customized = _context3.sent;

                try {
                  // @ts-ignore
                  customized = _objectSpread(_objectSpread({}, JSON.parse(customized.dataJson)), customized);
                } catch (error) {
                  warning(false, error.message);
                }

              case 16:
                runInAction(function () {
                  var newCustomized = _objectSpread({
                    columns: {}
                  }, customized);

                  _this7.customized = newCustomized;

                  _this7.initColumns();

                  var customAggregation = newCustomized.aggregation;

                  if (customAggregation !== undefined) {
                    var _this7$props = _this7.props,
                        onAggregationChange = _this7$props.onAggregationChange,
                        aggregation = _this7$props.aggregation;

                    if (onAggregationChange && customAggregation !== aggregation) {
                      onAggregationChange(customAggregation);
                    }
                  }
                });

              case 17:
                _context3.prev = 17;
                this.customizedLoaded = true;
                runInAction(function () {
                  _this7.loading = false;
                });
                return _context3.finish(17);

              case 21:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this, [[6,, 17, 21]]);
      }));

      function loadCustomized() {
        return _loadCustomized.apply(this, arguments);
      }

      return loadCustomized;
    }()
  }, {
    key: "handleAllPageSelectionMenuClick",
    value: function handleAllPageSelectionMenuClick(_ref36) {
      var key = _ref36.key;
      var dataSet = this.dataSet;
      var isAllPageSelection = dataSet.isAllPageSelection;

      switch (key) {
        case 'current':
          {
            if (this.allChecked) {
              this.unCheckAllCurrent();
              this.unCheckAllCached();
            } else {
              this.checkAllCurrent();
              this.checkAllCached();
            }

            break;
          }

        case 'all':
          dataSet.setAllPageSelection(!isAllPageSelection);
          break;

        default:
      }
    }
  }, {
    key: "renderAllPageSelectionMenu",
    value: function renderAllPageSelectionMenu() {
      var isAllPageSelection = this.dataSet.isAllPageSelection,
          allChecked = this.allChecked,
          prefixCls = this.prefixCls;
      return /*#__PURE__*/React.createElement(Menu, {
        prefixCls: "".concat(prefixCls, "-dropdown-menu"),
        onClick: this.handleAllPageSelectionMenuClick
      }, /*#__PURE__*/React.createElement(Menu.Item, {
        key: "current"
      }, $l('Table', allChecked ? 'unselect_current_page' : 'select_current_page')), /*#__PURE__*/React.createElement(Menu.Item, {
        key: "all"
      }, $l('Table', isAllPageSelection ? 'unselect_all_page' : 'select_all_page')));
    }
  }, {
    key: "getConfig",
    value: function getConfig(key) {
      return this.node.getContextConfig(key);
    }
  }, {
    key: "getProPrefixCls",
    value: function getProPrefixCls(suffixCls, customizePrefixCls) {
      return this.node.getContextProPrefixCls(suffixCls, customizePrefixCls);
    }
  }, {
    key: "multipleSelectionRenderer",
    value: function multipleSelectionRenderer() {
      var buttons = [/*#__PURE__*/React.createElement(ObserverCheckBox, {
        key: "selectAll",
        checked: this.allChecked,
        indeterminate: this.indeterminate,
        onChange: this.handleSelectAllChange,
        labelLayout: LabelLayout.none,
        value: true
      })];

      if (this.props.showAllPageSelectionButton) {
        buttons.push( /*#__PURE__*/React.createElement(Dropdown, {
          key: "selectAllPage",
          overlay: this.renderAllPageSelectionMenu
        }, /*#__PURE__*/React.createElement(Icon, {
          type: "baseline-arrow_drop_down",
          className: "".concat(this.prefixCls, "-page-all-select")
        })));
      }

      return buttons;
    }
  }, {
    key: "getRelationSize",
    value: function getRelationSize(type) {
      if (type === '%') {
        return this.parentHeight;
      }

      return this.screenHeight;
    }
  }, {
    key: "startScroll",
    value: function startScroll() {
      this.scrolling = true;
      this.stopScroll();
    }
  }, {
    key: "batchSetRowHeight",
    value: function batchSetRowHeight(key, callback) {
      var batchRunner = getIf(this, 'batchRunner', function () {
        return new BatchRunner();
      });
      batchRunner.addTask(key, callback);
    }
  }, {
    key: "isRowInView",
    value: function isRowInView(index) {
      var _this8 = this;

      var propVirtual = this.propVirtual;

      if (propVirtual) {
        return true;
      }

      if (this.height === undefined) {
        return index <= 10;
      }

      var tableBodyWrap = this.node.tableBodyWrap;
      var scrollTop = tableBodyWrap ? tableBodyWrap.scrollTop : 0;
      var visibleStartIndex = getVisibleStartIndex(this, function () {
        return scrollTop;
      });
      return index >= getStartIndex(this, function () {
        return visibleStartIndex;
      }) && index <= getEndIndex(this, function () {
        return getVisibleEndIndex(_this8, function () {
          return visibleStartIndex;
        }, function () {
          return scrollTop;
        });
      });
    }
  }, {
    key: "alignEditor",
    value: function alignEditor() {
      var currentEditorName = this.currentEditorName;

      if (currentEditorName) {
        var currentEditor = this.editors.get(currentEditorName);

        if (currentEditor) {
          currentEditor.alignEditor();
        }
      }
    }
  }, {
    key: "blurEditor",
    value: function blurEditor() {
      var currentEditorName = this.currentEditorName;

      if (currentEditorName) {
        var currentEditor = this.editors.get(currentEditorName);

        if (currentEditor) {
          currentEditor.blur();
        }
      }
    }
  }]);

  return TableStore;
}();

export { TableStore as default };

__decorate([observable], TableStore.prototype, "props", void 0);

__decorate([observable], TableStore.prototype, "customized", void 0);

__decorate([observable], TableStore.prototype, "tempCustomized", void 0);

__decorate([observable], TableStore.prototype, "loading", void 0);

__decorate([observable], TableStore.prototype, "leftOriginalColumns", void 0);

__decorate([observable], TableStore.prototype, "originalColumns", void 0);

__decorate([observable], TableStore.prototype, "rightOriginalColumns", void 0);

__decorate([observable], TableStore.prototype, "hasAggregationColumn", void 0);

__decorate([observable], TableStore.prototype, "calcBodyHeight", void 0);

__decorate([observable], TableStore.prototype, "width", void 0);

__decorate([observable], TableStore.prototype, "lastScrollTop", void 0);

__decorate([observable], TableStore.prototype, "lastScrollLeft", void 0);

__decorate([observable], TableStore.prototype, "lockColumnsBodyRowsHeight", void 0);

__decorate([observable], TableStore.prototype, "lockColumnsFootRowsHeight", void 0);

__decorate([observable], TableStore.prototype, "lockColumnsHeadRowsHeight", void 0);

__decorate([observable], TableStore.prototype, "expandedRows", void 0);

__decorate([observable], TableStore.prototype, "hoverRow", void 0);

__decorate([observable], TableStore.prototype, "rowClicked", void 0);

__decorate([observable], TableStore.prototype, "currentEditorName", void 0);

__decorate([observable], TableStore.prototype, "styledHidden", void 0);

__decorate([observable], TableStore.prototype, "customizedActiveKey", void 0);

__decorate([observable], TableStore.prototype, "mouseBatchChooseState", void 0);

__decorate([observable], TableStore.prototype, "mouseBatchChooseIdList", void 0);

__decorate([observable], TableStore.prototype, "columnResizing", void 0);

__decorate([observable], TableStore.prototype, "scrollPosition", void 0);

__decorate([observable], TableStore.prototype, "comboBarStatus", void 0);

__decorate([observable], TableStore.prototype, "siblingHeihgt", void 0);

__decorate([observable], TableStore.prototype, "parentHeight", void 0);

__decorate([observable], TableStore.prototype, "parentPaddingTop", void 0);

__decorate([observable], TableStore.prototype, "screenHeight", void 0);

__decorate([observable], TableStore.prototype, "headerHeight", void 0);

__decorate([observable], TableStore.prototype, "footerHeight", void 0);

__decorate([observable], TableStore.prototype, "headerFilter", void 0);

__decorate([computed], TableStore.prototype, "computedHeight", null);

__decorate([computed], TableStore.prototype, "height", null);

__decorate([computed], TableStore.prototype, "autoHeight", null);

__decorate([computed], TableStore.prototype, "virtualColumnRange", null);

__decorate([autobind], TableStore.prototype, "isRenderRange", null);

__decorate([observable], TableStore.prototype, "actualRows", void 0);

__decorate([observable], TableStore.prototype, "rowMetaData", void 0);

__decorate([observable], TableStore.prototype, "scrolling", void 0);

__decorate([observable], TableStore.prototype, "cellVerticalSize", void 0);

__decorate([computed], TableStore.prototype, "virtualRowHeight", null);

__decorate([computed], TableStore.prototype, "virtualVisibleStartIndex", null);

__decorate([computed], TableStore.prototype, "virtualVisibleEndIndex", null);

__decorate([computed], TableStore.prototype, "virtualStartIndex", null);

__decorate([computed], TableStore.prototype, "virtualEndIndex", null);

__decorate([computed], TableStore.prototype, "currentEditRecord", null);

__decorate([observable], TableStore.prototype, "showCachedSelection", void 0);

__decorate([observable], TableStore.prototype, "defaultRecordCachedType", void 0);

__decorate([observable], TableStore.prototype, "recordCachedType", void 0);

__decorate([computed], TableStore.prototype, "overflowX", null);

__decorate([computed], TableStore.prototype, "overflowY", null);

__decorate([computed], TableStore.prototype, "hasRowGroups", null);

__decorate([computed], TableStore.prototype, "headerTableGroups", null);

__decorate([autobind], TableStore.prototype, "customizedColumnHeader", null);

__decorate([computed], TableStore.prototype, "customizedColumn", null);

__decorate([computed], TableStore.prototype, "expandColumn", null);

__decorate([computed], TableStore.prototype, "rowNumberColumn", null);

__decorate([computed], TableStore.prototype, "selectionColumn", null);

__decorate([computed], TableStore.prototype, "comboQueryColumn", null);

__decorate([computed], TableStore.prototype, "draggableColumn", null);

__decorate([computed], TableStore.prototype, "leftColumns", null);

__decorate([computed], TableStore.prototype, "rightColumns", null);

__decorate([computed], TableStore.prototype, "columns", null);

__decorate([computed], TableStore.prototype, "columnGroups", null);

__decorate([computed], TableStore.prototype, "leftColumnGroups", null);

__decorate([computed], TableStore.prototype, "rightColumnGroups", null);

__decorate([computed], TableStore.prototype, "leafNamedColumns", null);

__decorate([computed], TableStore.prototype, "hasEmptyWidthColumn", null);

__decorate([computed], TableStore.prototype, "getLastEmptyWidthColumn", null);

__decorate([computed], TableStore.prototype, "hasCheckFieldColumn", null);

__decorate([computed], TableStore.prototype, "isCombinedColumn", null);

__decorate([observable], TableStore.prototype, "groups", void 0);

__decorate([observable], TableStore.prototype, "groupedData", void 0);

__decorate([observable], TableStore.prototype, "groupedDataWithHeader", void 0);

__decorate([computed], TableStore.prototype, "currentData", null);

__decorate([computed], TableStore.prototype, "data", null);

__decorate([computed], TableStore.prototype, "cachedIndeterminate", null);

__decorate([computed], TableStore.prototype, "allCachedChecked", null);

__decorate([computed], TableStore.prototype, "currentIndeterminate", null);

__decorate([computed], TableStore.prototype, "allCurrentChecked", null);

__decorate([computed], TableStore.prototype, "indeterminate", null);

__decorate([computed], TableStore.prototype, "allChecked", null);

__decorate([action], TableStore.prototype, "showEditor", null);

__decorate([action], TableStore.prototype, "setLastScrollTop", null);

__decorate([action], TableStore.prototype, "setLastScrollLeft", null);

__decorate([action], TableStore.prototype, "hideEditor", null);

__decorate([action], TableStore.prototype, "changeMouseBatchChooseIdList", null);

__decorate([action], TableStore.prototype, "setProps", null);

__decorate([action], TableStore.prototype, "updateProps", null);

__decorate([action], TableStore.prototype, "initGroups", null);

__decorate([action], TableStore.prototype, "initColumns", null);

__decorate([action], TableStore.prototype, "setAggregationCellExpanded", null);

__decorate([action], TableStore.prototype, "setRowExpanded", null);

__decorate([action], TableStore.prototype, "setRowPending", null);

__decorate([action], TableStore.prototype, "setRowLoaded", null);

__decorate([computed], TableStore.prototype, "cellHighlightRenderer", null);

__decorate([action], TableStore.prototype, "setRowHover", null);

__decorate([action], TableStore.prototype, "expandAll", null);

__decorate([action], TableStore.prototype, "collapseAll", null);

__decorate([autobind], TableStore.prototype, "onTreeNodeLoad", null);

__decorate([autobind], TableStore.prototype, "renderSelectionBox", null);

__decorate([autobind], TableStore.prototype, "renderRowNumber", null);

__decorate([autobind], TableStore.prototype, "renderDragBox", null);

__decorate([action], TableStore.prototype, "changeCustomizedColumnValue", null);

__decorate([action], TableStore.prototype, "saveCustomized", null);

__decorate([autobind, action], TableStore.prototype, "openCustomizationModal", null);

__decorate([autobind, action], TableStore.prototype, "handleAllPageSelectionMenuClick", null);

__decorate([autobind], TableStore.prototype, "renderAllPageSelectionMenu", null);

__decorate([autobind], TableStore.prototype, "getConfig", null);

__decorate([autobind], TableStore.prototype, "getProPrefixCls", null);

__decorate([autobind], TableStore.prototype, "multipleSelectionRenderer", null);

__decorate([autobind], TableStore.prototype, "getRelationSize", null);

__decorate([action], TableStore.prototype, "startScroll", null);
//# sourceMappingURL=TableStore.js.map
