import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _createForOfIteratorHelper from "@babel/runtime/helpers/createForOfIteratorHelper";
import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
import _objectSpread from "@babel/runtime/helpers/objectSpread2";
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";
import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _classCallCheck from "@babel/runtime/helpers/classCallCheck";
import _createClass from "@babel/runtime/helpers/createClass";
import _inherits from "@babel/runtime/helpers/inherits";
import _createSuper from "@babel/runtime/helpers/createSuper";
import _typeof from "@babel/runtime/helpers/typeof";
var _excluded = ["fieldName", "value", "conditionType", "comparator"];
import _regeneratorRuntime from "@babel/runtime/regenerator";
import { __decorate } from "tslib";
import React, { cloneElement, Component, isValidElement } from 'react';
import { observer } from 'mobx-react';
import { action, autorun, isArrayLike, observable, runInAction, toJS } from 'mobx';
import uniq from 'lodash/uniq';
import pull from 'lodash/pull';
import noop from 'lodash/noop';
import map from 'lodash/map';
import isObject from 'lodash/isObject';
import isEnumEmpty from 'lodash/isEmpty';
import isNumber from 'lodash/isNumber';
import isFunction from 'lodash/isFunction';
import isEqual from 'lodash/isEqual';
import isArray from 'lodash/isArray';
import isString from 'lodash/isString';
import omit from 'lodash/omit';
import defer from 'lodash/defer';
import difference from 'lodash/difference';
import isUndefined from 'lodash/isUndefined';
import isNil from 'lodash/isNil';
import classNames from 'classnames';
import ConfigContext from '../../../../es/config-provider/ConfigContext';
import { getProPrefixCls as getProPrefixClsDefault } from '../../../../es/configure/utils';
import { pxToRem } from '../../../../es/_util/UnitConvertor';
import Icon from '../../../../es/icon';
import { Action } from '../../../../es/trigger/enum';
import Popover from '../../../../es/popover';
import Tag from '../../../../es/tag';
import DataSet from '../../data-set/DataSet';
import { DataSetEvents, DataSetSelection, DataSetStatus, FieldIgnore, FieldType, RecordStatus } from '../../data-set/enum';
import Button from '../../button';
import Dropdown from '../../dropdown';
import TextField from '../../text-field';
import { ValueChangeAction } from '../../text-field/enum';
import Tooltip from '../../tooltip';
import { WaitType, Tooltip as OptionTooltip } from '../../core/enum';
import { $l } from '../../locale-context';
import autobind from '../../_util/autobind';
import _isEmpty from '../../_util/isEmpty';
import FieldList from './FieldList';
import TableButtons from './TableButtons';
import ColumnFilter from './ColumnFilter';
import QuickFilterMenu from './quick-filter/QuickFilterMenu';
import QuickFilterMenuContext from './quick-filter/QuickFilterMenuContext';
import { ConditionDataSet, QuickFilterDataSet, NewFilterDataSet, AdvancedFieldSet } from './quick-filter/QuickFilterDataSet';
import { hide, show } from '../../tooltip/singleton';
import { ShowHelp } from '../../field/enum';
import { renderValidationMessage as utilRenderValidationMessage } from '../../field/utils';
import Select from '../../select';
import Tree from '../../tree';
import { ButtonColor, FuncType } from '../../button/enum';
import { OPERATOR, OPERATOR_TYPE } from './quick-filter/Operator';
import { getEditorByField } from '../utils';
import { ShowValidation } from '../../form/enum';
import { TriggerViewMode } from '../../trigger-field/enum';
/**
 * 当前数据是否有值并需要选中
 * @param data
 */

export function isSelect(data) {
  if (isObject(data[1])) {
    return !isEnumEmpty(data[1]);
  }

  return data[0] !== '__dirty' && !_isEmpty(data[1]);
}
export function isEqualDynamicProps(originalValue, newValue, dataSet, record, name) {
  if (isEqual(newValue, originalValue)) {
    return true;
  }

  if (isObject(newValue) && isObject(originalValue) && Object.keys(newValue).length) {
    var combineKeys = uniq(Object.keys(newValue).concat(Object.keys(originalValue)));
    return combineKeys.every(function (key) {
      var value = toJS(newValue[key]);
      var oldValue = toJS(originalValue[key]);

      if (oldValue === value) {
        return true;
      }

      if (_isEmpty(oldValue) && _isEmpty(value)) {
        return true;
      }

      if (name && name.includes('.')) {
        return _isEmpty(oldValue) && _isEmpty(record.get(name));
      }

      if (isNumber(oldValue) || isNumber(value)) {
        var oEp = isNumber(oldValue) ? _isEmpty(oldValue) : isEnumEmpty(oldValue);
        var nEp = isNumber(value) ? _isEmpty(value) : isEnumEmpty(value);

        if (oEp && nEp) {
          return true;
        }

        return String(oldValue) === String(value);
      }

      var field = dataSet.getField(key);

      if (field && field.get('range', record)) {
        var rangeValue = value ? isArray(value) ? value.join('') : Object.values(value).join('') : '';
        var rangeOldValue = oldValue ? isArray(oldValue) ? oldValue.join('') : Object.values(oldValue).join('') : '';
        return rangeValue === rangeOldValue;
      }

      if (field && field.get('lovCode') && oldValue && value) {
        var valueField = dataSet.getField(key).get('valueField', record);
        var textField = dataSet.getField(key).get('textField', record);
        return value[valueField] === oldValue[valueField] && value[textField] === oldValue[textField];
      }

      return isEqual(oldValue, value);
    });
  }

  return isEqual(newValue, originalValue);
}
/**
 * 提交副本数据
 * @param data
 */

export function omitData(data) {
  return omit(data, 'creationDate', 'createdBy', 'lastUpdateDate', 'lastUpdatedBy', 'objectVersionNumber', '_token', 'searchId', 'searchConditionId', 'searchQueryId', 'searchOrderId');
}
/**
 * obj 值使用 JSON 保存、获取赋值转化
 * @param value
 */

export function parseValue(value) {
  try {
    var res = JSON.parse(value);

    if (_typeof(res) === 'object') {
      return res;
    }

    return value;
  } catch (e) {
    return value;
  }
}
export function stringifyValue(value) {
  if (_typeof(value) === 'object') {
    return JSON.stringify(value);
  }

  return value;
}
/**
 * 处理查询过滤条件
 * @params
 */

export function processFilterParam(dataSet) {
  var newFilterDataSet = dataSet.getState(NEWFILTERDATASET); // 处理公式参数

  var searchExp = '';
  var expType = dataSet.getState(EXPTYPE);

  if (expType !== 'customize') {
    var joinStr = expType === 'all' || expType === undefined ? 'AND' : 'OR';

    for (var i = 1; i <= newFilterDataSet.length; i++) {
      if (i === newFilterDataSet.length) {
        searchExp += i;
      } else {
        searchExp += i + joinStr;
      }
    }
  } else {
    searchExp = dataSet.getState(SEARCHEXP).toLocaleUpperCase().replace(/\s+/g, '');
  }

  dataSet.setQueryParameter('search.exp', searchExp); // 处理字段参数

  var searchFld = [];
  newFilterDataSet.map(function (record, index) {
    var dataObj = record.toData();
    var name = dataObj[AdvancedFieldSet.fieldName];
    var comparator = dataObj[AdvancedFieldSet.comparator];
    var tableName = newFilterDataSet.getField(name).get('help', record);
    var value = '';

    if (isObject(dataObj[name])) {
      if (!isArray(dataObj[name])) {
        value = dataObj[name][name];
      } else {
        value = dataObj[name].map(function (arr) {
          return isObject(arr) ? arr[name] : arr;
        });
      }
    } else {
      value = dataObj[name];
    }

    var paramFieldName = tableName ? "".concat(tableName, ".").concat(name) : name;
    var searchValue = [];

    if ([OPERATOR.IS_NULL.value, OPERATOR.IS_NOT_NULL.value].includes(comparator) || value) {
      var enCodeValue = isNumber(value) ? value : value ? encodeURIComponent(value) : value;
      searchValue = [index + 1, paramFieldName, dataObj[AdvancedFieldSet.comparator], enCodeValue].filter(Boolean);
    }

    if (searchValue && searchValue.length) {
      searchFld.push(searchValue.join('@'));
    }

    return null;
  });
  var searchFilParam = searchFld.join(',');
  dataSet.setQueryParameter('search.fld', searchFilParam);
}
export var CONDITIONSTATUS = '__CONDITIONSTATUS__';
export var SELECTFIELDS = '__SELECTFIELDS__';
export var MENUDATASET = '__MENUDATASET__';
export var CONDITIONDATASET = '__CONDITIONDATASET__';
export var OPTIONDATASET = '__OPTIONDATASET__';
export var FILTERMENUDATASET = '__FILTERMENUDATASET__';
export var MENURESULT = '__MENURESULT__';
export var SEARCHTEXT = '__SEARCHTEXT__';
export var SELECTCHANGE = '__SELECTCHANGE__';
export var EXPTYPE = '__EXPTYPE__';
export var SEARCHEXP = '__SEARCHEXP__';
export var NEWFILTERDATASET = '__NEWFILTERDATASET__';
export var ORIGINALVALUEOBJ = '__ORIGINALVALUEOBJ__';

var TableDynamicFilterBar = /*#__PURE__*/function (_Component) {
  _inherits(TableDynamicFilterBar, _Component);

  var _super = _createSuper(TableDynamicFilterBar);

  function TableDynamicFilterBar(props, context) {
    var _this;

    _classCallCheck(this, TableDynamicFilterBar);

    _this = _super.call(this, props, context);
    _this.refAdvancedFilterContainer = null;
    _this.refDropdown = null;
    _this.refSingleWrapper = null;
    _this.refEditors = new Map();
    _this.refFilterItems = new Map();
    _this.refCustomizeExpTypeEditor = null;
    _this.originalConditionFields = [];

    _this.handleMouseDownOut = function (e) {
      if (_this.refDropdown && !_this.refDropdown.contains(e.target)) {
        _this.fieldSelectHidden = true;
      }
    };

    _this.setOriginalConditionFields = function (code) {
      var _this$props = _this.props,
          queryDataSet = _this$props.queryDataSet,
          dataSet = _this$props.dataSet;

      if (!code) {
        if (queryDataSet) {
          _this.initConditionFields({
            dataSet: queryDataSet,
            record: queryDataSet.get(0)
          });
        }
      } else {
        _this.originalConditionFields = Array.isArray(code) ? code : [code];
      }

      dataSet.setState(SELECTFIELDS, _toConsumableArray(_this.originalConditionFields));
    };
    /**
     * 勾选
     * @param code
     * @param record
     */


    _this.handleSelect = function (code, record) {
      var dataSet = _this.props.dataSet;
      var codes = Array.isArray(code) ? code : [code];
      var selectFields = dataSet.getState(SELECTFIELDS) || [];
      dataSet.setState(SELECTFIELDS, uniq([].concat(_toConsumableArray(selectFields), _toConsumableArray(codes))));
      var shouldUpdate = dataSet.getState(SELECTFIELDS).length !== _this.originalConditionFields.length || !!difference(toJS(dataSet.getState(SELECTFIELDS)), toJS(_this.originalConditionFields)).length;
      var isDirty = record ? record.dirty : false;

      _this.setConditionStatus(shouldUpdate || isDirty ? RecordStatus.update : RecordStatus.sync);

      dataSet.setState(SELECTCHANGE, shouldUpdate);
    };
    /**
     * 取消勾选
     * @param code
     */


    _this.handleUnSelect = function (code) {
      var _this$props2 = _this.props,
          queryDataSet = _this$props2.queryDataSet,
          dataSet = _this$props2.dataSet;
      var codes = Array.isArray(code) ? code : [code];
      var isDirty = false;

      if (queryDataSet) {
        var current = queryDataSet.current;

        if (current) {
          codes.forEach(function (name) {
            return current.set(name, undefined);
          });
          isDirty = current.dirty;
        }
      }

      var selectFields = dataSet.getState(SELECTFIELDS) || [];
      dataSet.setState(SELECTFIELDS, pull.apply(void 0, [_toConsumableArray(selectFields)].concat(_toConsumableArray(codes))));
      var shouldUpdate = dataSet.getState(SELECTFIELDS).length !== _this.originalConditionFields.length || !!difference(toJS(dataSet.getState(SELECTFIELDS)), toJS(_this.originalConditionFields)).length;

      _this.setConditionStatus(shouldUpdate || isDirty ? RecordStatus.update : RecordStatus.sync);

      dataSet.setState(SELECTCHANGE, shouldUpdate);
    };

    runInAction(function () {
      _this.fieldSelectHidden = true;
      _this.expand = true;
    });
    return _this;
  }

  _createClass(TableDynamicFilterBar, [{
    key: "prefixCls",
    get: function get() {
      var prefixCls = this.props.prefixCls;
      var _this$context$getProP = this.context.getProPrefixCls,
          getProPrefixCls = _this$context$getProP === void 0 ? getProPrefixClsDefault : _this$context$getProP;
      return getProPrefixCls('table', prefixCls);
    }
  }, {
    key: "queryFields",
    get: function get() {
      var _this$props3 = this.props,
          queryFields = _this$props3.queryFields,
          queryDataSet = _this$props3.queryDataSet,
          dataSet = _this$props3.dataSet;
      var menuDataSet = dataSet.getState(MENUDATASET);
      var isTenant = menuDataSet && menuDataSet.current && menuDataSet.current.get('isTenant');
      return queryFields.filter(function (component) {
        if (component.props.hidden) {
          return !component.props.hidden;
        }

        if (isTenant && queryDataSet && queryDataSet.getField(component.props.name)) {
          return queryDataSet.getField(component.props.name).get('fieldVisible') !== 0;
        }

        return !component.props.hidden;
      });
    }
  }, {
    key: "componentDidMount",
    value: function componentDidMount() {
      var _this2 = this;

      var _this$props4 = this.props,
          fuzzyQueryOnly = _this$props4.fuzzyQueryOnly,
          queryDataSet = _this$props4.queryDataSet,
          dataSet = _this$props4.dataSet;

      if (!fuzzyQueryOnly) {
        this.processDataSetListener(true);
        document.addEventListener('mousedown', this.handleMouseDownOut);

        if (this.isSingleLineOpt() && this.refSingleWrapper) {
          var _this$refSingleWrappe = this.refSingleWrapper.getBoundingClientRect(),
              height = _this$refSingleWrappe.height;

          var _this$refSingleWrappe2 = this.refSingleWrapper.children[0].children[0].getBoundingClientRect(),
              childHeight = _this$refSingleWrappe2.height;

          runInAction(function () {
            _this2.showExpandIcon = height > childHeight + 18;
          });
        }

        var autoQuery = dataSet.props.autoQuery;

        if (!autoQuery || autoQuery && dataSet.status === DataSetStatus.ready) {
          this.handleDataSetQuery({
            dataSet: dataSet
          });
        }
      }

      var shouldInit = dataSet.getState(ORIGINALVALUEOBJ) ? dataSet.getState(ORIGINALVALUEOBJ).query === undefined : true;

      if (shouldInit && queryDataSet && queryDataSet.current) {
        this.initConditionFields({
          dataSet: queryDataSet,
          record: queryDataSet.current
        });
      }

      if (queryDataSet && queryDataSet.current && !this.tableFilterAdapter) {
        dataSet.setState(CONDITIONSTATUS, queryDataSet.current.dirty ? RecordStatus.update : RecordStatus.sync);
      }

      this.setFilterVisiable();
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      var fuzzyQueryOnly = this.props.fuzzyQueryOnly;

      if (!fuzzyQueryOnly) {
        document.removeEventListener('mousedown', this.handleMouseDownOut);
        this.processDataSetListener(false);
      }

      if (this.isTooltipShown) {
        hide();
        delete this.isTooltipShown;
      }
    }
  }, {
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(nextProps) {
      var _this3 = this;

      var dataSet = nextProps.dataSet,
          fuzzyQueryOnly = nextProps.fuzzyQueryOnly,
          queryDataSet = nextProps.queryDataSet; // eslint-disable-next-line react/destructuring-assignment

      if (dataSet !== this.props.dataSet || fuzzyQueryOnly !== this.props.fuzzyQueryOnly) {
        runInAction(function () {
          _this3.fieldSelectHidden = true;
          _this3.expand = true;
        });

        if (!fuzzyQueryOnly) {
          // 移除原有实例监听
          this.processDataSetListener(false);
          this.processDataSetListener(true, nextProps);

          if (this.isSingleLineOpt() && this.refSingleWrapper) {
            var _this$refSingleWrappe3 = this.refSingleWrapper.getBoundingClientRect(),
                height = _this$refSingleWrappe3.height;

            var _this$refSingleWrappe4 = this.refSingleWrapper.children[0].children[0].getBoundingClientRect(),
                childHeight = _this$refSingleWrappe4.height;

            runInAction(function () {
              _this3.showExpandIcon = height > childHeight + 18;
            });
          }
        }

        var shouldInit = dataSet.getState(ORIGINALVALUEOBJ) ? dataSet.getState(ORIGINALVALUEOBJ).query === undefined : true;

        if (shouldInit && queryDataSet && queryDataSet.current) {
          this.initConditionFields({
            dataSet: queryDataSet,
            record: queryDataSet.current
          });
        }

        if (queryDataSet && queryDataSet.current && !this.tableFilterAdapter) {
          dataSet.setState(CONDITIONSTATUS, queryDataSet.current.dirty ? RecordStatus.update : RecordStatus.sync);
        }
      }
    }
  }, {
    key: "processDataSetListener",
    value: function processDataSetListener(flag, nextProps) {
      var _ref = nextProps || this.props,
          queryDataSet = _ref.queryDataSet,
          dataSet = _ref.dataSet;

      if (queryDataSet) {
        var handler = flag ? queryDataSet.addEventListener : queryDataSet.removeEventListener;
        var dsHandler = flag ? dataSet.addEventListener : dataSet.removeEventListener;
        handler.call(queryDataSet, DataSetEvents.validate, this.handleDataSetValidate);
        handler.call(queryDataSet, DataSetEvents.update, this.handleDataSetUpdate);
        handler.call(queryDataSet, DataSetEvents.create, this.handleDataSetCreate);
        handler.call(queryDataSet, DataSetEvents.reset, this.handleDataSetReset);
        handler.call(queryDataSet, DataSetEvents.load, this.handleDataSetLoad);
        dsHandler.call(dataSet, DataSetEvents.query, this.handleDataSetQuery);
      }
    }
  }, {
    key: "handleDataSetQuery",
    value: function () {
      var _handleDataSetQuery = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(_ref2) {
        var dataSet, res, defaultMenus, _ref3, conditionList, initQueryData, queryDataSet;

        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                dataSet = _ref2.dataSet;

                if (!(!dataSet.getState(MENURESULT) && this.tableFilterAdapter)) {
                  _context.next = 16;
                  break;
                }

                _context.next = 4;
                return this.initMenuDataSet();

              case 4:
                res = dataSet.getState(MENURESULT);

                if (!(res && res.length && res.filter(function (menu) {
                  return menu.defaultFlag;
                }).length)) {
                  _context.next = 16;
                  break;
                }

                defaultMenus = res.filter(function (menu) {
                  return menu.defaultFlag;
                });
                _ref3 = defaultMenus.length > 1 ? defaultMenus.find(function (menu) {
                  return menu.isTenant !== 1;
                }) : defaultMenus[0], conditionList = _ref3.conditionList;
                initQueryData = {};

                if (!(conditionList && conditionList.length)) {
                  _context.next = 16;
                  break;
                }

                map(conditionList, function (condition) {
                  if (condition.comparator === 'EQUAL') {
                    var fieldName = condition.fieldName,
                        value = condition.value;
                    initQueryData[fieldName] = parseValue(value);
                  }
                });
                queryDataSet = this.props.queryDataSet;

                if (!(queryDataSet && queryDataSet.current && dataSet.props.autoQuery)) {
                  _context.next = 16;
                  break;
                }

                if (!Object.keys(initQueryData).length) {
                  _context.next = 16;
                  break;
                }

                dataSet.query();
                return _context.abrupt("return", false);

              case 16:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function handleDataSetQuery(_x) {
        return _handleDataSetQuery.apply(this, arguments);
      }

      return handleDataSetQuery;
    }()
    /**
     * queryDataSet 查询前校验事件 触发展开动态字段
     * @param dataSet 查询DS
     * @param result
     */

  }, {
    key: "handleDataSetValidate",
    value: function () {
      var _handleDataSetValidate = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2(_ref4) {
        var _this4 = this;

        var dataSet, result, refSingleWrapper;
        return _regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                dataSet = _ref4.dataSet, result = _ref4.result;
                _context2.next = 3;
                return result;

              case 3:
                if (_context2.sent) {
                  _context2.next = 7;
                  break;
                }

                runInAction(function () {
                  var current = dataSet.current;
                  dataSet.fields.forEach(function (field, key) {
                    if (!field.isValid(current)) {
                      _this4.handleSelect(key);
                    }
                  });
                  _this4.expand = true;
                });
                refSingleWrapper = this.refSingleWrapper;

                if (refSingleWrapper) {
                  refSingleWrapper.style.height = '';
                  refSingleWrapper.style.overflow = '';
                }

              case 7:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function handleDataSetValidate(_x2) {
        return _handleDataSetValidate.apply(this, arguments);
      }

      return handleDataSetValidate;
    }()
  }, {
    key: "setConditionStatus",
    value: function setConditionStatus(value, orglValue) {
      var dataSet = this.props.dataSet;
      dataSet.setState(CONDITIONSTATUS, value);

      if (value === RecordStatus.sync && orglValue) {
        var oriObj = dataSet.getState(ORIGINALVALUEOBJ);
        dataSet.setState(ORIGINALVALUEOBJ, _objectSpread(_objectSpread({}, oriObj), {}, {
          query: orglValue
        }));
      }
    }
    /**
     * 筛选条件更新 触发表格查询
     */

  }, {
    key: "handleDataSetUpdate",
    value: function () {
      var _handleDataSetUpdate = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee3(_ref5) {
        var record, name, oldValue, value, _this$props5, dataSet, queryDataSet, _this$props5$onQuery, onQuery, autoQuery, field, shouldQuery, rangeKeys, _record$set, _record$set2, rangeValue, rangeOldValue, status, hasFocus, i, queryField, editor;

        return _regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                record = _ref5.record, name = _ref5.name, oldValue = _ref5.oldValue, value = _ref5.value;
                _this$props5 = this.props, dataSet = _this$props5.dataSet, queryDataSet = _this$props5.queryDataSet, _this$props5$onQuery = _this$props5.onQuery, onQuery = _this$props5$onQuery === void 0 ? noop : _this$props5$onQuery, autoQuery = _this$props5.autoQuery;
                field = queryDataSet && queryDataSet.getField(name);
                shouldQuery = true;

                if (field && field.get('range', record)) {
                  // 处理 range 失焦对象 undefined，bind 字段无处理的情况
                  if (value) {
                    rangeKeys = Object.keys(value);

                    if (rangeKeys.length) {
                      if (value[rangeKeys[0]] === undefined && value[rangeKeys[1]] === undefined) {
                        record.set(name, null);
                      } else if (value[rangeKeys[1]] === undefined) {
                        record.set(name, (_record$set = {}, _defineProperty(_record$set, rangeKeys[0], value[rangeKeys[0]]), _defineProperty(_record$set, rangeKeys[1], null), _record$set));
                      } else if (value[rangeKeys[0]] === undefined) {
                        record.set(name, (_record$set2 = {}, _defineProperty(_record$set2, rangeKeys[0], null), _defineProperty(_record$set2, rangeKeys[1], value[rangeKeys[1]]), _record$set2));
                      }
                    }
                  }

                  rangeValue = value ? isArray(value) ? value.join('') : Object.values(value).join('') : '';
                  rangeOldValue = oldValue ? isArray(oldValue) ? oldValue.join('') : Object.values(oldValue).join('') : '';
                  shouldQuery = rangeValue !== rangeOldValue;
                }

                status = RecordStatus.update;

                if (record) {
                  this.handleSelect(name, record);
                  status = isEqualDynamicProps(dataSet.getState(ORIGINALVALUEOBJ).query, omit(record.toData(), ['__dirty']), queryDataSet, record, name) ? RecordStatus.sync : RecordStatus.update;
                }

                this.setConditionStatus(status);

                if (!(autoQuery && shouldQuery)) {
                  _context3.next = 27;
                  break;
                }

                _context3.next = 11;
                return dataSet.modifiedCheck(undefined, dataSet, 'query');

              case 11:
                if (!_context3.sent) {
                  _context3.next = 26;
                  break;
                }

                _context3.t0 = queryDataSet && queryDataSet.current;

                if (!_context3.t0) {
                  _context3.next = 17;
                  break;
                }

                _context3.next = 16;
                return queryDataSet.current.validate();

              case 16:
                _context3.t0 = _context3.sent;

              case 17:
                if (!_context3.t0) {
                  _context3.next = 22;
                  break;
                }

                dataSet.query();
                onQuery();
                _context3.next = 24;
                break;

              case 22:
                hasFocus = false;

                for (i = 0; i < this.queryFields.length; i++) {
                  queryField = this.queryFields[i];
                  editor = this.refEditors.get(String(queryField.key));

                  if (editor && !editor.valid && !hasFocus && field && !field.get('multiple', record)) {
                    editor.focus();
                    hasFocus = true;
                  }
                }

              case 24:
                _context3.next = 27;
                break;

              case 26:
                record.init(name, oldValue);

              case 27:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function handleDataSetUpdate(_x3) {
        return _handleDataSetUpdate.apply(this, arguments);
      }

      return handleDataSetUpdate;
    }()
    /**
     * queryDS 新建，初始勾选值
     */

  }, {
    key: "handleDataSetCreate",
    value: function handleDataSetCreate() {
      var queryDataSet = this.props.queryDataSet;

      if (queryDataSet) {
        this.initConditionFields({
          dataSet: queryDataSet,
          record: queryDataSet.current
        });
      }
    }
    /**
     * queryDS reset，初始记录
     */

  }, {
    key: "handleDataSetReset",
    value: function handleDataSetReset() {
      var queryDataSet = this.props.queryDataSet;

      if (queryDataSet && !this.tableFilterAdapter) {
        queryDataSet.create();
      }
    }
    /**
     * queryDS load，兼容项目loadData([])的处理，初始化记录
     */

  }, {
    key: "handleDataSetLoad",
    value: function handleDataSetLoad(_ref6) {
      var dataSet = _ref6.dataSet;

      if (!this.tableFilterAdapter && !dataSet.length) {
        dataSet.create();
      }
    }
    /**
     * 初始化勾选值、条件字段
     * @param props
     */

  }, {
    key: "initConditionFields",
    value: function initConditionFields(props) {
      var _this5 = this;

      var dataSet = props.dataSet,
          record = props.record;
      var tableDS = this.props.dataSet;
      var originalValue = omit(record.toData(), ['__dirty']);
      var conditionData = Object.entries(originalValue);
      var newObj = tableDS.getState(ORIGINALVALUEOBJ) || {};
      tableDS.setState(ORIGINALVALUEOBJ, _objectSpread(_objectSpread({}, newObj), {}, {
        query: originalValue
      }));
      this.originalConditionFields = [];
      var fields = dataSet.fields;
      map(conditionData, function (data) {
        var name = data[0];

        if (!fields.has(data[0]) && isObject(data[1]) && !isEnumEmpty(data[1]) && !isArray(data[1])) {
          name = "".concat(data[0], ".").concat(Object.keys(data[1])[0]);
        }

        if (isSelect(data) && !(dataSet.getState(SELECTFIELDS) || []).includes(name)) {
          var field = dataSet.getField(name);

          if (!field || !field.get('bind', record) || field.get('usedFlag')) {
            _this5.originalConditionFields.push(name);

            _this5.handleSelect(name, record);
          }
        }
      });
    }
    /**
     * 高级搜索字段集
     */

  }, {
    key: "advancedSearchFieldProps",
    get: function get() {
      var _this$props6 = this.props,
          queryDataSet = _this$props6.queryDataSet,
          dataSet = _this$props6.dataSet,
          advancedSearchFields = _this$props6.advancedSearchFields;

      if (advancedSearchFields && advancedSearchFields.length) {
        var omitPropsArr = ['dynamicProps', 'computedProps', 'order', 'pattern', 'maxLength', 'minLength', 'max', 'min', 'validator', 'required', 'readOnly', 'disabled', 'defaultValue', 'cascadeMap', 'ignore'];
        return advancedSearchFields.map(function (field) {
          var name = field.name,
              source = field.source,
              alias = field.alias,
              tableName = field.tableName,
              _field$fieldProps = field.fieldProps,
              fieldProps = _field$fieldProps === void 0 ? {} : _field$fieldProps;
          var fieldName = alias || name;

          var dynamicRequired = function dynamicRequired(_ref7) {
            var record = _ref7.record;
            var isNullOperater = [OPERATOR.IS_NULL.value, OPERATOR.IS_NOT_NULL.value].includes(record.get(AdvancedFieldSet.comparator));

            if (fieldName === record.get(AdvancedFieldSet.fieldName) && !isNullOperater) {
              return true;
            }

            return false;
          };

          var required = fieldProps.required,
              computedProps = fieldProps.computedProps;
          if (required) delete fieldProps.required;
          if (computedProps && 'required' in computedProps) delete computedProps.required;

          if (fieldProps.dynamicProps) {
            fieldProps.dynamicProps.required = dynamicRequired;
          } else {
            fieldProps.dynamicProps = {
              required: dynamicRequired
            };
          }

          if (source === 'fields' && dataSet.props.fields) {
            var dsFiedlsProps = dataSet.props.fields.find(function (f) {
              return f.name === name;
            });
            return _objectSpread(_objectSpread(_objectSpread({}, omit(dsFiedlsProps, omitPropsArr)), fieldProps), {}, {
              help: tableName,
              name: fieldName
            });
          }

          if (source === 'queryFields' && queryDataSet && queryDataSet.props.fields) {
            var _dsFiedlsProps = queryDataSet.props.fields.find(function (f) {
              return f.name === name;
            });

            return _objectSpread(_objectSpread(_objectSpread({}, omit(_dsFiedlsProps, omitPropsArr)), fieldProps), {}, {
              help: tableName,
              name: fieldName
            });
          }

          return _objectSpread(_objectSpread({}, fieldProps), {}, {
            // multiple: fieldProps && fieldProps.multiple ? '|' : false,
            help: tableName,
            name: fieldName
          });
        });
      }

      return [];
    }
    /**
    * 加载动态筛选条相关初始数据 & 存储初始值
    *  1.筛选条
    *  2.高级搜索
    *  3.模糊搜索
    * @param param
    */

  }, {
    key: "loadConditionData",
    value: function loadConditionData(_ref8) {
      var conditionDataSet = _ref8.conditionDataSet,
          newFilterDataSet = _ref8.newFilterDataSet,
          menuRecord = _ref8.menuRecord,
          dataSet = _ref8.dataSet,
          searchText = _ref8.searchText;
      var comparatorData = [];
      var regularData = [];
      var newObj = dataSet.getState(ORIGINALVALUEOBJ) || {};
      var searchTextValue = null;
      menuRecord.get('conditionList').forEach(function (condition) {
        if (condition.conditionType !== 'comparator') {
          regularData.push(condition);
        } else {
          var _objectSpread2;

          var fieldName = condition.fieldName,
              value = condition.value,
              conditionType = condition.conditionType,
              comparator = condition.comparator,
              rest = _objectWithoutProperties(condition, _excluded);

          comparatorData.push(_objectSpread((_objectSpread2 = {}, _defineProperty(_objectSpread2, AdvancedFieldSet.fieldName, fieldName), _defineProperty(_objectSpread2, fieldName, value), _defineProperty(_objectSpread2, AdvancedFieldSet.conditionType, conditionType), _defineProperty(_objectSpread2, AdvancedFieldSet.comparator, comparator), _objectSpread2), rest));
        }
      }); // 加载筛选条数据

      conditionDataSet.loadData(regularData); // 加载高级搜索面板数据

      newFilterDataSet.loadData(comparatorData); // 初始高级搜索

      dataSet.setState(ORIGINALVALUEOBJ, _objectSpread(_objectSpread({}, newObj), {}, {
        advance: comparatorData
      })); // 加载模糊搜索数据

      if (menuRecord && menuRecord.get('queryList') && menuRecord.get('queryList').length) {
        var searchObj = menuRecord.get('queryList').find(function (ql) {
          return ql.fieldName === SEARCHTEXT;
        });
        searchTextValue = searchObj ? searchObj.value : null;
        dataSet.setState(SEARCHTEXT, searchTextValue); // 初始模糊搜索

        dataSet.setState(ORIGINALVALUEOBJ, _objectSpread(_objectSpread({}, newObj), {}, {
          advance: comparatorData,
          fuzzy: searchTextValue
        }));
        dataSet.setQueryParameter(searchText, searchTextValue);
      }
    }
    /**
     * 初始筛选条数据源状态
     */

  }, {
    key: "initMenuDataSet",
    value: function () {
      var _initMenuDataSet = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee4() {
        var _this6 = this;

        var _this$props7, queryDataSet, dataSet, dynamicFilterBar, searchCode, getConfig, searchCodes, menuDataSet, conditionDataSet, newFilterDataSet, optionDataSet, filterMenuDataSet, status, result, menuRecord, current;

        return _regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                _this$props7 = this.props, queryDataSet = _this$props7.queryDataSet, dataSet = _this$props7.dataSet, dynamicFilterBar = _this$props7.dynamicFilterBar, searchCode = _this$props7.searchCode;
                getConfig = this.context.getConfig;
                searchCodes = dynamicFilterBar && dynamicFilterBar.searchCode || searchCode;

                if (!this.tableFilterAdapter) {
                  _context4.next = 25;
                  break;
                }

                menuDataSet = new DataSet(QuickFilterDataSet({
                  searchCode: searchCodes,
                  queryDataSet: queryDataSet,
                  tableFilterAdapter: this.tableFilterAdapter
                }), {
                  getConfig: getConfig
                });
                conditionDataSet = new DataSet(ConditionDataSet(), {
                  getConfig: getConfig
                });
                newFilterDataSet = new DataSet(NewFilterDataSet({
                  propFields: this.advancedSearchFieldProps
                }), {
                  getConfig: getConfig
                });
                optionDataSet = new DataSet({
                  paging: false,
                  selection: DataSetSelection.single,
                  fields: [{
                    // 是否租户默认配置
                    name: 'isTenant',
                    type: FieldType.string,
                    transformResponse: function transformResponse(value) {
                      return value ? $l('Table', 'preset') : $l('Table', 'user');
                    },
                    group: true
                  }]
                }, {
                  getConfig: getConfig
                });
                filterMenuDataSet = new DataSet({
                  autoCreate: true,
                  fields: [{
                    name: 'filterName',
                    type: FieldType.string,
                    textField: 'searchName',
                    valueField: 'searchId',
                    options: optionDataSet,
                    ignore: FieldIgnore.always
                  }]
                }, {
                  getConfig: getConfig
                });
                status = RecordStatus.update;

                if (queryDataSet && queryDataSet.current && dataSet.getState(ORIGINALVALUEOBJ)) {
                  status = isEqualDynamicProps(dataSet.getState(ORIGINALVALUEOBJ).query, omit(queryDataSet.current.toData(), ['__dirty']), queryDataSet, queryDataSet.current) ? RecordStatus.sync : RecordStatus.update;
                } else {
                  status = RecordStatus.sync;
                } // 初始化状态


                dataSet.setState(MENUDATASET, menuDataSet);
                dataSet.setState(CONDITIONDATASET, conditionDataSet);
                dataSet.setState(NEWFILTERDATASET, newFilterDataSet);
                dataSet.setState(OPTIONDATASET, optionDataSet);
                dataSet.setState(FILTERMENUDATASET, filterMenuDataSet);
                dataSet.setState(CONDITIONSTATUS, status); // dataSet.setState(SEARCHTEXT, null);

                _context4.next = 19;
                return menuDataSet.query();

              case 19:
                result = _context4.sent;
                dataSet.setState(MENURESULT, result);

                if (optionDataSet) {
                  optionDataSet.loadData(result);
                }

                menuRecord = menuDataSet.current;

                if (menuRecord) {
                  this.loadConditionData({
                    menuRecord: menuRecord,
                    conditionDataSet: conditionDataSet,
                    newFilterDataSet: newFilterDataSet,
                    dataSet: dataSet,
                    searchText: this.searchText
                  });
                }

                if (result && result.length) {
                  runInAction(function () {
                    _this6.shouldLocateData = true;
                  });

                  if (queryDataSet && queryDataSet.fields && !this.tempFields) {
                    this.tempFields = queryDataSet.snapshot().dataSet.fields;
                  }
                } else {
                  current = filterMenuDataSet.current;
                  if (current) current.set('filterName', undefined);
                  runInAction(function () {
                    _this6.shouldLocateData = true;
                  });
                }

              case 25:
                return _context4.abrupt("return", true);

              case 26:
              case "end":
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function initMenuDataSet() {
        return _initMenuDataSet.apply(this, arguments);
      }

      return initMenuDataSet;
    }()
    /**
     * tableFilterSuffix 预留自定义区域
     */

  }, {
    key: "renderSuffix",
    value: function renderSuffix() {
      var _this$props8 = this.props,
          dynamicFilterBar = _this$props8.dynamicFilterBar,
          queryDataSet = _this$props8.queryDataSet,
          dataSet = _this$props8.dataSet,
          _this$props8$buttons = _this$props8.buttons,
          buttons = _this$props8$buttons === void 0 ? [] : _this$props8$buttons;
      var getConfig = this.context.getConfig;
      var prefixCls = this.prefixCls;
      var suffixes = dynamicFilterBar && dynamicFilterBar.suffixes || getConfig('tableFilterSuffix');
      var children = [];
      var suffixesDom = null;
      var tableButtons = buttons.length ? /*#__PURE__*/React.createElement(TableButtons, {
        key: "".concat(buttons.length, "-toolbar"),
        prefixCls: "".concat(prefixCls, "-dynamic-filter"),
        buttons: buttons
      }) : null;

      if (suffixes && suffixes.length) {
        suffixes.forEach(function (suffix) {
          if (suffix === 'filter') {
            children.push( /*#__PURE__*/React.createElement(ColumnFilter, {
              prefixCls: prefixCls
            }));
          } else if ( /*#__PURE__*/isValidElement(suffix)) {
            children.push(suffix);
          } else if (isFunction(suffix)) {
            children.push(suffix({
              queryDataSet: queryDataSet,
              dataSet: dataSet
            }));
          }
        });
        suffixesDom = /*#__PURE__*/React.createElement("div", {
          className: "".concat(prefixCls, "-dynamic-filter-bar-suffix")
        }, children);
      }

      return [tableButtons, suffixesDom];
    }
  }, {
    key: "getPrefix",
    value: function getPrefix() {
      var _this$props9 = this.props,
          dynamicFilterBar = _this$props9.dynamicFilterBar,
          queryDataSet = _this$props9.queryDataSet,
          dataSet = _this$props9.dataSet;
      var prefixCls = this.prefixCls;
      var prefixes = dynamicFilterBar && dynamicFilterBar.prefixes;
      var children = [];

      if (prefixes && prefixes.length) {
        prefixes.forEach(function (prefix) {
          if (isString(prefix) && prefix === 'filter') {
            children.push( /*#__PURE__*/React.createElement(ColumnFilter, {
              prefixCls: prefixCls
            }));
          } else if ( /*#__PURE__*/isValidElement(prefix)) {
            children.push(prefix);
          } else if (isFunction(prefix)) {
            children.push(prefix({
              queryDataSet: queryDataSet,
              dataSet: dataSet
            }));
          }
        });
        return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
          className: "".concat(prefixCls, "-dynamic-filter-bar-prefix")
        }, children), /*#__PURE__*/React.createElement("span", {
          className: "".concat(prefixCls, "-filter-search-divide")
        }));
      }

      return null;
    }
    /**
     * 注入 refEditors
     * @param element
     * @param name
     */

  }, {
    key: "createFields",
    value: function createFields(element, name) {
      var _this7 = this;

      var props = {
        ref: function ref(node) {
          return _this7.refEditors.set(name, node);
        },
        border: false,
        clearButton: true,
        _inTable: true,
        showValidation: 'tooltip'
      };

      if (isUndefined(element.props.suffix) && ['Currency', 'ObserverNumberField', 'EmailField', 'UrlField', 'ObserverTextField'].includes(element.type.name)) {
        _extends(props, {
          suffix: /*#__PURE__*/React.createElement(Icon, {
            type: "search"
          })
        });
      }

      return /*#__PURE__*/cloneElement(element, props);
    }
    /**
     * 判断查询值是否为空
     * @param value
     */

  }, {
    key: "isEmpty",
    value: function isEmpty(value) {
      return isArrayLike(value) ? !value.length : _isEmpty(value);
    }
  }, {
    key: "tableFilterAdapter",
    get: function get() {
      var _this$props10 = this.props,
          dynamicFilterBar = _this$props10.dynamicFilterBar,
          searchCode = _this$props10.searchCode;
      var getConfig = this.context.getConfig;
      var searchCodes = dynamicFilterBar && dynamicFilterBar.searchCode || searchCode;
      return searchCodes ? dynamicFilterBar && dynamicFilterBar.tableFilterAdapter || getConfig('tableFilterAdapter') : null;
    }
    /**
     * 是否单行操作
     */

  }, {
    key: "isSingleLineOpt",
    value: function isSingleLineOpt() {
      var fuzzyQuery = this.props.fuzzyQuery;
      return !(fuzzyQuery || this.tableFilterAdapter);
    }
    /**
     * 查询前修改提示及校验定位
     */

  }, {
    key: "modifiedCheckQuery",
    value: function () {
      var _modifiedCheckQuery = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee5(fuzzyValue, fuzzyOldValue, refresh) {
        var _this$props11, dataSet, queryDataSet, fuzzyQueryOnly, _this$props11$onRefre, onRefresh, hasFocus, _iterator, _step, _step$value, key, value;

        return _regeneratorRuntime.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                _this$props11 = this.props, dataSet = _this$props11.dataSet, queryDataSet = _this$props11.queryDataSet, fuzzyQueryOnly = _this$props11.fuzzyQueryOnly, _this$props11$onRefre = _this$props11.onRefresh, onRefresh = _this$props11$onRefre === void 0 ? noop : _this$props11$onRefre;
                _context5.next = 3;
                return dataSet.modifiedCheck(undefined, dataSet, 'query');

              case 3:
                if (!_context5.sent) {
                  _context5.next = 30;
                  break;
                }

                _context5.t1 = queryDataSet && queryDataSet.current;

                if (!_context5.t1) {
                  _context5.next = 9;
                  break;
                }

                _context5.next = 8;
                return queryDataSet.current.validate();

              case 8:
                _context5.t1 = _context5.sent;

              case 9:
                _context5.t0 = _context5.t1;

                if (_context5.t0) {
                  _context5.next = 12;
                  break;
                }

                _context5.t0 = fuzzyQueryOnly;

              case 12:
                if (!_context5.t0) {
                  _context5.next = 25;
                  break;
                }

                if (fuzzyValue) {
                  runInAction(function () {
                    dataSet.setState(SEARCHTEXT, fuzzyValue || null);
                  });
                  dataSet.setQueryParameter(this.searchText, fuzzyValue);
                }

                if (!refresh) {
                  _context5.next = 22;
                  break;
                }

                _context5.next = 17;
                return onRefresh();

              case 17:
                _context5.t2 = _context5.sent;

                if (!(_context5.t2 !== false)) {
                  _context5.next = 20;
                  break;
                }

                dataSet.query();

              case 20:
                _context5.next = 23;
                break;

              case 22:
                dataSet.query();

              case 23:
                _context5.next = 28;
                break;

              case 25:
                hasFocus = false;
                _iterator = _createForOfIteratorHelper(this.refEditors.entries());

                try {
                  for (_iterator.s(); !(_step = _iterator.n()).done;) {
                    _step$value = _slicedToArray(_step.value, 2), key = _step$value[0], value = _step$value[1];

                    if (value && !value.valid && !hasFocus) {
                      this.refEditors.get(key).focus();
                      hasFocus = true;
                    }
                  }
                } catch (err) {
                  _iterator.e(err);
                } finally {
                  _iterator.f();
                }

              case 28:
                _context5.next = 32;
                break;

              case 30:
                runInAction(function () {
                  dataSet.setState(SEARCHTEXT, fuzzyOldValue || null);
                });
                dataSet.setQueryParameter(this.searchText, fuzzyOldValue);

              case 32:
              case "end":
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      function modifiedCheckQuery(_x4, _x5, _x6) {
        return _modifiedCheckQuery.apply(this, arguments);
      }

      return modifiedCheckQuery;
    }()
  }, {
    key: "renderRefreshBtn",
    value: function renderRefreshBtn() {
      var _this8 = this;

      var prefixCls = this.prefixCls;
      return /*#__PURE__*/React.createElement("span", {
        className: "".concat(prefixCls, "-filter-menu-query"),
        onClick: function () {
          var _onClick = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee6(e) {
            return _regeneratorRuntime.wrap(function _callee6$(_context6) {
              while (1) {
                switch (_context6.prev = _context6.next) {
                  case 0:
                    e.stopPropagation();
                    _context6.next = 3;
                    return _this8.modifiedCheckQuery(undefined, undefined, true);

                  case 3:
                  case "end":
                    return _context6.stop();
                }
              }
            }, _callee6);
          }));

          function onClick(_x7) {
            return _onClick.apply(this, arguments);
          }

          return onClick;
        }()
      }, /*#__PURE__*/React.createElement(Tooltip, {
        title: $l('Table', 'refresh')
      }, /*#__PURE__*/React.createElement(Icon, {
        type: "refresh"
      })));
    }
    /**
     * 渲染展开逻辑
     * @param hidden 是否隐藏全部
     */

  }, {
    key: "getExpandNode",
    value: function getExpandNode(hidden) {
      var _this9 = this;

      var prefixCls = this.prefixCls,
          refreshBtn = this.props.refreshBtn;
      if (!this.showExpandIcon && !hidden) return refreshBtn ? this.renderRefreshBtn() : null;
      return /*#__PURE__*/React.createElement("span", {
        className: "".concat(prefixCls, "-filter-menu-expand"),
        onClick: function onClick() {
          var refSingleWrapper = _this9.refSingleWrapper;

          if (refSingleWrapper) {
            var _refSingleWrapper$get = refSingleWrapper.getBoundingClientRect(),
                height = _refSingleWrapper$get.height;

            var _refSingleWrapper$chi = refSingleWrapper.children[0].children[0].getBoundingClientRect(),
                childHeight = _refSingleWrapper$chi.height;

            runInAction(function () {
              _this9.expand = hidden ? height <= 0 : height <= childHeight + 18;
            });

            if (hidden && height) {
              // 收起全部
              refSingleWrapper.style.display = 'none';
            } else {
              refSingleWrapper.style.display = 'flex';
              refSingleWrapper.style.height = '';
              refSingleWrapper.style.overflow = '';
            }

            if (height > childHeight + 18 && !hidden) {
              // 收起留一行高度
              refSingleWrapper.style.height = pxToRem(childHeight + 18, true) || '';
              refSingleWrapper.style.overflow = 'hidden';
            } else {
              refSingleWrapper.style.height = '';
              refSingleWrapper.style.overflow = '';
            }
          }
        }
      }, refreshBtn ? this.renderRefreshBtn() : null, this.expand ? /*#__PURE__*/React.createElement(Tooltip, {
        title: $l('Table', 'collapse')
      }, /*#__PURE__*/React.createElement(Icon, {
        type: "baseline-arrow_drop_up"
      })) : /*#__PURE__*/React.createElement(Tooltip, {
        title: $l('Table', 'expand_button')
      }, /*#__PURE__*/React.createElement(Icon, {
        type: "baseline-arrow_drop_down"
      })));
    }
    /**
     * 过滤比较符
     * @param record
     * @param optionRecord
     * @returns
     */

  }, {
    key: "optionsFilter",
    value: function optionsFilter(record, optionRecord) {
      if (!record.get(AdvancedFieldSet.fieldName)) return false;
      var dataSet = this.props.dataSet;
      var newFilterDataSet = dataSet.getState(NEWFILTERDATASET);
      var field = newFilterDataSet.getField(record.get(AdvancedFieldSet.fieldName), record);
      var type = field.get('type', record);
      var value = OPERATOR[optionRecord.get('value').toUpperCase()];

      if (!OPERATOR_TYPE[type.toUpperCase()]) {
        return OPERATOR_TYPE.STRING.includes(value);
      }

      if (field.get('lookupCode', record) || isString(field.get('lookupUrl', record)) || type !== FieldType.object && (field.get('lovCode', record) || field.getLookup(record) || field.get('options', record))) {
        return OPERATOR_TYPE.LOOKUP.includes(value);
      }

      if (field.get('lovCode', record)) {
        return OPERATOR_TYPE.LOV.includes(value);
      }

      return OPERATOR_TYPE[type.toUpperCase()].includes(value);
    }
    /**
     * 高级筛选弹窗确认
     */

  }, {
    key: "handleFilterOk",
    value: function () {
      var _handleFilterOk = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee8() {
        var _this10 = this;

        var _this$props12, dataSet, autoQuery, newFilterDataSet, res, customizeExpTypeValidation;

        return _regeneratorRuntime.wrap(function _callee8$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                _this$props12 = this.props, dataSet = _this$props12.dataSet, autoQuery = _this$props12.autoQuery;
                newFilterDataSet = dataSet.getState(NEWFILTERDATASET);
                _context8.next = 4;
                return newFilterDataSet.validate();

              case 4:
                res = _context8.sent;
                _context8.t0 = !this.refCustomizeExpTypeEditor;

                if (_context8.t0) {
                  _context8.next = 10;
                  break;
                }

                _context8.next = 9;
                return this.refCustomizeExpTypeEditor.checkValidity();

              case 9:
                _context8.t0 = _context8.sent;

              case 10:
                customizeExpTypeValidation = _context8.t0;

                if (res && customizeExpTypeValidation) {
                  runInAction( /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee7() {
                    var jsonData, status;
                    return _regeneratorRuntime.wrap(function _callee7$(_context7) {
                      while (1) {
                        switch (_context7.prev = _context7.next) {
                          case 0:
                            // 初始状态下移除空行
                            if (newFilterDataSet.length !== 1) {
                              newFilterDataSet.map(function (r) {
                                if (!r.get(AdvancedFieldSet.fieldName)) {
                                  newFilterDataSet.remove(r, true);
                                }

                                return null;
                              });
                            }

                            jsonData = newFilterDataSet.toJSONData();
                            processFilterParam(dataSet);
                            status = isEqualDynamicProps(toJS(dataSet.getState(ORIGINALVALUEOBJ).advance), jsonData, newFilterDataSet) ? dataSet.getState(CONDITIONSTATUS) : RecordStatus.update; // 更新筛选条状态

                            _this10.setConditionStatus(status);

                            if (autoQuery) dataSet.query();
                            _this10.visible = false;

                          case 7:
                          case "end":
                            return _context7.stop();
                        }
                      }
                    }, _callee7);
                  })));
                }

              case 12:
              case "end":
                return _context8.stop();
            }
          }
        }, _callee8, this);
      }));

      function handleFilterOk() {
        return _handleFilterOk.apply(this, arguments);
      }

      return handleFilterOk;
    }()
    /**
     * 高级筛选弹窗取消
     */

  }, {
    key: "handleFilterCancel",
    value: function handleFilterCancel() {
      var _this11 = this;

      var dataSet = this.props.dataSet;
      var newFilterDataSet = dataSet.getState(NEWFILTERDATASET);
      var orgValue = dataSet.getState(ORIGINALVALUEOBJ).advance;
      newFilterDataSet.loadData(orgValue);
      this.setConditionStatus(dataSet.getState(CONDITIONSTATUS));
      runInAction(function () {
        if (_this11.refCustomizeExpTypeEditor && _this11.refCustomizeExpTypeEditor.validationResults && _this11.refCustomizeExpTypeEditor.validationResults.length > 0) {
          _this11.refCustomizeExpTypeEditor.validationResults = undefined;
        }

        _this11.visible = false;
      });
    }
  }, {
    key: "setFilterVisiable",
    value: function setFilterVisiable() {
      var _this12 = this;

      autorun(function () {
        if (_this12.visible) {
          document.addEventListener('mousedown', _this12.handleEmitFilterCancel);
          document.addEventListener('keydown', _this12.handleEmitFilterCancel);
        } else {
          document.removeEventListener('mousedown', _this12.handleEmitFilterCancel);
          document.removeEventListener('keydown', _this12.handleEmitFilterCancel);
        }
      });
    }
  }, {
    key: "isPopupChildren",
    value: function isPopupChildren(element) {
      var getConfig = this.context.getConfig;
      var proPreCls = getConfig('proPrefixCls');
      var popupClasses = ["".concat(proPreCls, "-popup"), "".concat(proPreCls, "-modal")];
      var hasPopupClass = popupClasses.some(function (cls) {
        return element && element.className.includes(cls);
      });

      if (hasPopupClass) {
        return true;
      }

      if (!hasPopupClass && element && element.parentElement && element.parentElement !== document.body) {
        return this.isPopupChildren(element.parentElement);
      }

      return false;
    }
  }, {
    key: "handleEmitFilterCancel",
    value: function handleEmitFilterCancel(e) {
      var target = e.target,
          type = e.type;

      switch (type) {
        case 'mousedown':
          {
            var _this$refAdvancedFilt;

            var container = (_this$refAdvancedFilt = this.refAdvancedFilterContainer) === null || _this$refAdvancedFilt === void 0 ? void 0 : _this$refAdvancedFilt.getPopupDomNode();
            var keepVisible = target === container || container.contains(target) || this.isPopupChildren(target);

            if (!keepVisible) {
              this.handleFilterCancel();
            }

            break;
          }

        case 'keydown':
          {
            if (e instanceof KeyboardEvent && (e.ctrlKey || e.metaKey) && e.key === '/') {
              this.handleFilterCancel();
            }

            break;
          }

        default:
          break;
      }
    }
  }, {
    key: "saveCustomizeExpTypeEditorRef",
    value: function saveCustomizeExpTypeEditorRef(node) {
      this.refCustomizeExpTypeEditor = node;
    }
  }, {
    key: "saveAdvancedFilterContainerRef",
    value: function saveAdvancedFilterContainerRef(node) {
      this.refAdvancedFilterContainer = node;
    }
    /**
     * 渲染高级筛选面板
     */

  }, {
    key: "getAdvancedFilter",
    value: function getAdvancedFilter() {
      var _this13 = this;

      var prefixCls = this.prefixCls,
          getConfig = this.context.getConfig;
      var _this$props13 = this.props,
          dataSet = _this$props13.dataSet,
          advancedSearchFields = _this$props13.advancedSearchFields;
      var tlsKey = getConfig('tlsKey');
      var newFilterDataSet = dataSet.getState(NEWFILTERDATASET);
      var menuDataSet = dataSet.getState(MENUDATASET);

      if (advancedSearchFields && advancedSearchFields.length && newFilterDataSet) {
        var _getConfig = this.context.getConfig;
        var selectOptions = this.advancedSearchFieldProps.map(function (fieldProps) {
          var name = fieldProps.name,
              label = fieldProps.label,
              bind = fieldProps.bind;
          if (bind) return null;
          var field = newFilterDataSet.getField(name);

          var hasBindProps = function hasBindProps(propsName) {
            return field && field.get(propsName) && field.get(propsName).bind;
          };

          if (field.get('bind') || hasBindProps('computedProps') || hasBindProps('dynamicProps') || field.get('name').includes(tlsKey)) {
            return null;
          }

          return /*#__PURE__*/React.createElement(Select.Option, {
            key: "".concat(name, "_select_option"),
            value: name
          }, label || name);
        });
        var PopoverContent = /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(Tooltip, {
          theme: "light",
          title: dataSet.getState(EXPTYPE) === 'customize' ? $l('Table', 'ad_search_help') : null
        }, /*#__PURE__*/React.createElement(Select, {
          isFlat: true,
          border: true,
          // dataSet={menuDataSet.current}
          // name="conExpression"
          value: dataSet.getState(EXPTYPE) ? dataSet.getState(EXPTYPE) : ['all', 'either'].includes(menuDataSet.current && menuDataSet.current.get('conExpression')) ? menuDataSet.current.get('conExpression') : "all",
          onChange: function onChange(value) {
            dataSet.setState(EXPTYPE, value);

            if (value !== 'customize') {
              dataSet.setState(SEARCHEXP, undefined);
            }

            if (menuDataSet.current) {
              menuDataSet.current.set('conExpression', value);
            }
          },
          clearButton: false,
          className: "".concat(prefixCls, "-new-filter-popover-select"),
          trigger: [Action.click, Action.focus]
        }, /*#__PURE__*/React.createElement(Select.Option, {
          value: "all"
        }, $l('Table', 'ad_search_all')), /*#__PURE__*/React.createElement(Select.Option, {
          value: "either"
        }, $l('Table', 'ad_search_any')), /*#__PURE__*/React.createElement(Select.Option, {
          value: "customize"
        }, $l('Table', 'ad_search_custom'), /*#__PURE__*/React.createElement(Tooltip, {
          theme: "light",
          title: $l('Table', 'ad_search_help')
        }, /*#__PURE__*/React.createElement(Icon, {
          className: "".concat(prefixCls, "-new-filter-popover-select-icon"),
          type: "help_outline"
        }))))), dataSet.getState(EXPTYPE) === 'customize' ? /*#__PURE__*/React.createElement(TextField, {
          ref: this.saveCustomizeExpTypeEditorRef,
          clearButton: true,
          validator: function validator(value) {
            return isNil(value) ? $l('Table', 'ad_search_validation') : true;
          },
          value: dataSet.getState(SEARCHEXP) ? dataSet.getState(SEARCHEXP) : menuDataSet.current && menuDataSet.current.get('conExpression') !== 'customize' ? menuDataSet.current.get('conExpression') : undefined,
          onChange: function onChange(value) {
            dataSet.setState(SEARCHEXP, value);

            if (menuDataSet.current) {
              menuDataSet.current.set('conExpression', value);
            }
          },
          className: classNames("".concat(prefixCls, "-new-filter-popover-input"), "".concat(_getConfig('proPrefixCls'), "-input-required"), "".concat(_getConfig('proPrefixCls'), "-input-required-colors")),
          placeholder: $l('Table', 'ad_search_placeholder'),
          showValidation: ShowValidation.tooltip
        }) : null, newFilterDataSet.length > 0 && /*#__PURE__*/React.createElement("div", {
          className: "".concat(prefixCls, "-new-filter-popover-line")
        }), /*#__PURE__*/React.createElement(Tree, {
          key: "__filter",
          dataSet: newFilterDataSet,
          showLine: {
            showLeafIcon: false
          },
          className: "".concat(prefixCls, "-new-filter-popover-tree"),
          onTreeNode: function onTreeNode() {
            return {
              className: "".concat(prefixCls, "-new-filter-popover-tree-node")
            };
          },
          showIcon: true,
          icon: function icon(props) {
            return /*#__PURE__*/React.createElement(Tag, {
              className: "".concat(prefixCls, "-new-filter-popover-item-tag")
            }, Number(props.pos.split('0-')[1]) + 1);
          },
          renderer: function renderer(_ref10) {
            var record = _ref10.record;
            if (!record) return null;
            var comparator = record.get(AdvancedFieldSet.comparator);
            var fieldName = record.get(AdvancedFieldSet.fieldName);
            var field = record.getField(fieldName);
            var disabled = [OPERATOR.IS_NULL.value, OPERATOR.IS_NOT_NULL.value].includes(comparator);
            var valueFieldDom = /*#__PURE__*/React.createElement(TextField, {
              style: {
                width: 192
              },
              disabled: true
            });

            if (field && !field.get('bind', record) && !fieldName.includes(tlsKey)) {
              var editor = getEditorByField(field, record, true);
              var shouldUseClick = false;

              if ('viewMode' in editor.props && editor.props.viewMode === TriggerViewMode.popup) {
                shouldUseClick = true;
              }

              var fieldValue = record.get(fieldName);
              var maxTagTextLength = field.get('multiple') && fieldValue && fieldValue.length > 1 ? 3 : 7;
              valueFieldDom = /*#__PURE__*/cloneElement(editor, {
                help: '',
                key: fieldName,
                name: fieldName,
                record: record,
                style: {
                  width: 192
                },
                disabled: disabled,
                showValidation: ShowValidation.tooltip,
                trigger: shouldUseClick ? [Action.click, Action.focus] : undefined,
                maxTagCount: 1,
                maxTagTextLength: maxTagTextLength,
                maxTagPlaceholder: function maxTagPlaceholder(restValues) {
                  return "+".concat(restValues.length);
                }
              });
            }

            return /*#__PURE__*/React.createElement("div", {
              className: "".concat(prefixCls, "-new-filter-popover-item"),
              key: "".concat(record.index, "_option")
            }, /*#__PURE__*/React.createElement(Select, {
              name: AdvancedFieldSet.fieldName,
              style: {
                width: 120
              },
              record: record,
              placeholder: $l('Lov', 'choose'),
              optionTooltip: OptionTooltip.overflow,
              trigger: [Action.click, Action.focus],
              showValidation: ShowValidation.tooltip
            }, selectOptions), /*#__PURE__*/React.createElement(Select, {
              name: AdvancedFieldSet.comparator,
              renderer: function renderer(_ref11) {
                var value = _ref11.value,
                    text = _ref11.text;
                return value ? $l('Operator', value.toLowerCase()) : text;
              },
              optionRenderer: function optionRenderer(_ref12) {
                var value = _ref12.value,
                    text = _ref12.text;
                return value ? $l('Operator', value.toLowerCase()) : text;
              },
              style: {
                width: 88
              },
              dropdownMatchSelectWidth: false,
              optionTooltip: OptionTooltip.overflow,
              record: record,
              disabled: !record.get(AdvancedFieldSet.fieldName),
              optionsFilter: function optionsFilter(optionRecord) {
                return _this13.optionsFilter(record, optionRecord);
              },
              trigger: [Action.click, Action.focus],
              showValidation: ShowValidation.tooltip
            }), valueFieldDom, /*#__PURE__*/React.createElement(Button, {
              funcType: FuncType.link,
              icon: "delete_black-o",
              color: ButtonColor.primary,
              onClick: function onClick() {
                if (newFilterDataSet.length === 1) {
                  newFilterDataSet.removeAll();
                } else {
                  newFilterDataSet.remove(record);
                }
              }
            }));
          }
        }), /*#__PURE__*/React.createElement(Button, {
          funcType: FuncType.link,
          icon: "add",
          color: ButtonColor.primary,
          onClick: function onClick() {
            newFilterDataSet.create();
          }
        }, $l('Table', 'ad_search_add')), /*#__PURE__*/React.createElement("div", {
          className: "".concat(prefixCls, "-new-filter-popover-footer")
        }, /*#__PURE__*/React.createElement(Button, {
          onClick: this.handleFilterCancel
        }, $l('Modal', 'cancel')), /*#__PURE__*/React.createElement(Button, {
          onClick: this.handleFilterOk,
          color: ButtonColor.primary
        }, $l('Modal', 'ok'))));
        return /*#__PURE__*/React.createElement(Popover, {
          title: $l('Table', 'ad_search_title'),
          content: PopoverContent,
          overlayClassName: "".concat(prefixCls, "-new-filter-popover"),
          trigger: "click",
          visible: this.visible,
          ref: this.saveAdvancedFilterContainerRef
        }, /*#__PURE__*/React.createElement(Button, {
          funcType: FuncType.link,
          className: "".concat(prefixCls, "-new-filter-popover-icon"),
          icon: "filter2",
          color: newFilterDataSet && newFilterDataSet.some(function (r) {
            return r.status !== 'sync';
          }) ? ButtonColor.primary : ButtonColor.dark,
          onClick: function onClick() {
            if (!newFilterDataSet.length) {
              newFilterDataSet.create();
            }

            runInAction(function () {
              _this13.visible = true;
            });
          }
        }));
      }

      return null;
    }
    /**
     * 渲染模糊搜索
     */

  }, {
    key: "getFuzzyQuery",
    value: function getFuzzyQuery() {
      var _this14 = this;

      var _this$props14 = this.props,
          dataSet = _this$props14.dataSet,
          fuzzyQuery = _this$props14.fuzzyQuery,
          fuzzyQueryPlaceholder = _this$props14.fuzzyQueryPlaceholder,
          _this$props14$onReset = _this$props14.onReset,
          onReset = _this$props14$onReset === void 0 ? noop : _this$props14$onReset;
      var prefixCls = this.prefixCls;
      var placeholder = fuzzyQueryPlaceholder || $l('Table', 'enter_search_content');
      var fuzzyValue = dataSet.getState(ORIGINALVALUEOBJ) && dataSet.getState(ORIGINALVALUEOBJ).fuzzy;
      if (!fuzzyQuery) return null;
      return /*#__PURE__*/React.createElement("div", {
        className: "".concat(prefixCls, "-filter-search")
      }, /*#__PURE__*/React.createElement(TextField, {
        style: {
          width: 280
        },
        clearButton: true,
        placeholder: placeholder,
        prefix: /*#__PURE__*/React.createElement(Icon, {
          type: "search"
        }),
        value: dataSet.getState(SEARCHTEXT),
        valueChangeAction: ValueChangeAction.input,
        waitType: WaitType.debounce,
        wait: 500,
        onChange: function onChange(value, oldValue) {
          _this14.handleQuery(undefined, value, oldValue);

          dataSet.setState(SEARCHTEXT, value);
          dataSet.setQueryParameter(_this14.searchText, value);

          _this14.setConditionStatus(value === fuzzyValue ? RecordStatus.sync : RecordStatus.update);
        },
        onClear: function onClear() {
          runInAction(function () {
            dataSet.setState(SEARCHTEXT, null);
          });
          onReset();
        }
      }));
    }
    /**
     * 渲染重置按钮
     */

  }, {
    key: "getResetButton",
    value: function getResetButton() {
      var _this15 = this;

      var _this$props15 = this.props,
          queryDataSet = _this$props15.queryDataSet,
          dataSet = _this$props15.dataSet,
          autoQueryAfterReset = _this$props15.autoQueryAfterReset,
          _this$props15$onReset = _this$props15.onReset,
          onReset = _this$props15$onReset === void 0 ? noop : _this$props15$onReset;
      var prefixCls = this.prefixCls;
      return /*#__PURE__*/React.createElement("div", {
        className: "".concat(prefixCls, "-filter-buttons")
      }, dataSet.getState(CONDITIONSTATUS) === RecordStatus.update && /*#__PURE__*/React.createElement(Button, {
        onClick: function onClick() {
          var shouldQuery = false;

          if (queryDataSet) {
            var current = queryDataSet.current;

            if (current) {
              var orgObj = dataSet.getState(ORIGINALVALUEOBJ);
              shouldQuery = dataSet.getState(SEARCHTEXT) !== orgObj.fuzzy || !isEqualDynamicProps(orgObj.query, omit(current.toData(), ['__dirty']), queryDataSet, current);
              dataSet.setState(SEARCHTEXT, orgObj.fuzzy);
              dataSet.setQueryParameter(_this15.searchText, orgObj.fuzzy);
              current.reset();
              dataSet.setState(SELECTFIELDS, _toConsumableArray(_this15.originalConditionFields));
            }
          }

          _this15.setConditionStatus(RecordStatus.sync);

          onReset();

          if (autoQueryAfterReset && shouldQuery) {
            _this15.handleQuery();
          }
        }
      }, $l('Table', 'reset_button')));
    }
    /**
     * 筛选头
     * fuzzyQuery + quickFilterMenu + resetButton + buttons
     */

  }, {
    key: "getFilterMenu",
    value: function getFilterMenu() {
      var _this$props16 = this.props,
          defaultActiveKey = _this$props16.defaultActiveKey,
          queryFields = _this$props16.queryFields,
          queryDataSet = _this$props16.queryDataSet,
          dataSet = _this$props16.dataSet,
          dynamicFilterBar = _this$props16.dynamicFilterBar,
          searchCode = _this$props16.searchCode,
          autoQuery = _this$props16.autoQuery,
          fuzzyQueryOnly = _this$props16.fuzzyQueryOnly,
          sortableFieldNames = _this$props16.sortableFieldNames;
      var prefixCls = this.prefixCls;
      var prefix = this.getPrefix();
      var suffix = this.renderSuffix();
      var fuzzyQuery = this.getFuzzyQuery();
      var advancedFilter = this.getAdvancedFilter();

      if (fuzzyQueryOnly) {
        return /*#__PURE__*/React.createElement("div", {
          className: "".concat(prefixCls, "-filter-menu")
        }, prefix, advancedFilter, fuzzyQuery, suffix);
      }

      if (queryDataSet && queryFields.length) {
        var searchCodes = dynamicFilterBar && dynamicFilterBar.searchCode || searchCode;
        var quickFilterMenu = this.tableFilterAdapter && searchCodes ? /*#__PURE__*/React.createElement(QuickFilterMenuContext.Provider, {
          value: {
            tempQueryFields: this.tempFields,
            autoQuery: autoQuery,
            prefixCls: prefixCls,
            expand: this.expand,
            dataSet: dataSet,
            queryDataSet: queryDataSet,
            refEditors: this.refEditors,
            onChange: this.handleSelect,
            conditionStatus: dataSet.getState(CONDITIONSTATUS),
            onStatusChange: this.setConditionStatus,
            selectFields: dataSet.getState(SELECTFIELDS),
            onOriginalChange: this.setOriginalConditionFields,
            menuDataSet: dataSet.getState(MENUDATASET),
            filterMenuDataSet: dataSet.getState(FILTERMENUDATASET),
            conditionDataSet: dataSet.getState(CONDITIONDATASET),
            newFilterDataSet: dataSet.getState(NEWFILTERDATASET),
            optionDataSet: dataSet.getState(OPTIONDATASET),
            shouldLocateData: this.shouldLocateData,
            initConditionFields: this.initConditionFields,
            sortableFieldNames: sortableFieldNames,
            searchText: this.searchText,
            loadConditionData: this.loadConditionData,
            defaultActiveKey: defaultActiveKey
          }
        }, /*#__PURE__*/React.createElement(QuickFilterMenu, null)) : null;
        var resetButton = this.isSingleLineOpt() || this.tableFilterAdapter ? null : this.getResetButton();
        return /*#__PURE__*/React.createElement("div", {
          className: "".concat(prefixCls, "-filter-menu")
        }, prefix, fuzzyQuery, advancedFilter, quickFilterMenu, resetButton, this.isSingleLineOpt() ? null : /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("span", {
          className: "".concat(prefixCls, "-filter-search-divide")
        }), this.getExpandNode(true)), suffix);
      }
    }
    /**
     * 查询字段初始顺序
     * 排除动态属性影响
     */

  }, {
    key: "originOrderQueryFields",
    get: function get() {
      var queryDataSet = this.props.queryDataSet;
      var getConfig = this.context.getConfig;
      var result = [];
      var tlsKey = getConfig('tlsKey');

      if (queryDataSet) {
        var fields = queryDataSet.fields,
            _queryDataSet$props$f = queryDataSet.props.fields,
            propFields = _queryDataSet$props$f === void 0 ? [] : _queryDataSet$props$f;
        var cloneFields = fields.toJS();
        propFields.forEach(function (_ref13) {
          var name = _ref13.name;

          if (name) {
            var field = cloneFields.get(name);

            var hasBindProps = function hasBindProps(propsName) {
              return field && field.get(propsName) && field.get(propsName).bind;
            };

            if (field && !field.get('bind') && !hasBindProps('computedProps') && !hasBindProps('dynamicProps') && !field.get('name').includes(tlsKey)) {
              result.push(field);
            }
          }
        });
      }

      return result;
    }
    /**
     * 获取模糊搜索字段参数名
     */

  }, {
    key: "searchText",
    get: function get() {
      var dynamicFilterBar = this.props.dynamicFilterBar;
      var getConfig = this.context.getConfig;
      return dynamicFilterBar && dynamicFilterBar.searchText || getConfig('tableFilterSearchText') || 'params';
    }
  }, {
    key: "handleHelpMouseEnter",
    value: function handleHelpMouseEnter(e, help) {
      var target = e.target;
      var _this$context = this.context,
          getTooltipTheme = _this$context.getTooltipTheme,
          getTooltipPlacement = _this$context.getTooltipPlacement;
      show(target, {
        title: help,
        theme: getTooltipTheme('help'),
        placement: getTooltipPlacement('help')
      });
      this.isTooltipShown = true;
    }
  }, {
    key: "handleHelpMouseLeave",
    value: function handleHelpMouseLeave() {
      hide();
    }
  }, {
    key: "renderTooltipHelp",
    value: function renderTooltipHelp(help) {
      var _this16 = this;

      if (help) {
        return /*#__PURE__*/React.createElement(Icon, {
          type: "help",
          onMouseEnter: function onMouseEnter(e) {
            return _this16.handleHelpMouseEnter(e, help);
          },
          onMouseLeave: this.handleHelpMouseLeave
        });
      }
    }
    /**
     * 渲染查询项 label
     * @param param
     * @returns
     */

  }, {
    key: "getLabel",
    value: function getLabel(_ref14) {
      var field = _ref14.field,
          value = _ref14.value,
          record = _ref14.record,
          placeholder = _ref14.placeholder;

      if (value) {
        return field.get('label', record);
      }

      return field.get('label', record) || placeholder;
    }
    /**
     * 渲染查询条
     */

  }, {
    key: "getQueryBar",
    value: function getQueryBar() {
      var _this17 = this;

      var getConfig = this.context.getConfig;
      var _this$props17 = this.props,
          _this$props17$queryFi = _this$props17.queryFieldsLimit,
          queryFieldsLimit = _this$props17$queryFi === void 0 ? 3 : _this$props17$queryFi,
          queryFields = _this$props17.queryFields,
          queryDataSet = _this$props17.queryDataSet,
          dataSet = _this$props17.dataSet,
          fuzzyQueryOnly = _this$props17.fuzzyQueryOnly;
      var menuDataSet = dataSet.getState(MENUDATASET);
      var isTenant = menuDataSet && menuDataSet.current && menuDataSet.current.get('isTenant');
      var fieldsLimit = queryFieldsLimit;

      if (isTenant) {
        fieldsLimit = 0;
      }

      var prefixCls = this.prefixCls;
      var selectFields = dataSet.getState(SELECTFIELDS) || [];

      if (fuzzyQueryOnly) {
        return /*#__PURE__*/React.createElement("div", {
          key: "query_bar",
          className: "".concat(prefixCls, "-dynamic-filter-bar")
        }, this.getFilterMenu());
      }

      if (queryDataSet && queryFields.length) {
        var singleLineModeAction = this.isSingleLineOpt() ? /*#__PURE__*/React.createElement("div", {
          className: "".concat(prefixCls, "-dynamic-filter-bar-single-action")
        }, this.getResetButton(), this.getExpandNode(false)) : null;
        return /*#__PURE__*/React.createElement("div", {
          key: "query_bar",
          className: "".concat(prefixCls, "-dynamic-filter-bar")
        }, this.getFilterMenu(), /*#__PURE__*/React.createElement("div", {
          className: "".concat(prefixCls, "-dynamic-filter-single-wrapper"),
          ref: function ref(node) {
            return _this17.refSingleWrapper = node;
          }
        }, /*#__PURE__*/React.createElement("div", {
          className: "".concat(prefixCls, "-filter-wrapper")
        }, this.queryFields.slice(0, fieldsLimit).map(function (element) {
          var _classNames;

          var _element$props = element.props,
              name = _element$props.name,
              hidden = _element$props.hidden,
              showHelp = _element$props.showHelp,
              disabled = _element$props.disabled,
              help = _element$props.help,
              placeholder = _element$props.placeholder;
          var isLabelShowHelp = (showHelp || getConfig('showHelp')) === ShowHelp.label;
          if (hidden) return null;
          var queryField = queryDataSet.getField(name);
          if (!queryField) return null;
          var isRequired = queryField && queryField.get('required');
          var validationMessage = queryField && queryField.getValidationMessage(queryDataSet.current);
          var hasValue = !_this17.isEmpty(queryDataSet.current && queryDataSet.current.get(name));

          var label = _this17.getLabel({
            field: queryField,
            value: hasValue,
            placeholder: placeholder,
            record: queryDataSet.current
          });

          var isDisabled = disabled || queryField && queryField.get('disabled', queryDataSet.current);
          var itemContentClassName = classNames("".concat(prefixCls, "-filter-content"), (_classNames = {}, _defineProperty(_classNames, "".concat(prefixCls, "-filter-content-disabled"), isDisabled), _defineProperty(_classNames, "".concat(prefixCls, "-filter-content-required"), isRequired), _defineProperty(_classNames, "".concat(prefixCls, "-filter-content-has-value"), hasValue), _defineProperty(_classNames, "".concat(prefixCls, "-filter-content-invalid"), validationMessage), _classNames));
          return /*#__PURE__*/React.createElement("div", {
            className: itemContentClassName,
            key: name,
            onMouseDown: function onMouseDown() {
              if (!isDisabled) {
                var editor = _this17.refEditors.get(name);

                if (editor) {
                  defer(function () {
                    _this17.refEditors.get(name).focus();
                  }, 50);
                }
              }
            },
            onClick: function onClick() {
              if (!isDisabled) {
                var filterItem = _this17.refFilterItems.get(name);

                if (filterItem) {
                  if (!filterItem.className.includes("c7n-pro-lov-click")) {
                    filterItem.className += ' c7n-pro-lov-click';
                  }
                }
              }
            },
            onBlur: function onBlur() {
              var filterItem = _this17.refFilterItems.get(name);

              if (filterItem && filterItem.className.includes("c7n-pro-lov-click")) {
                filterItem.className = filterItem.className.split(" c7n-pro-lov-click")[0];
              }
            },
            onMouseEnter: function onMouseEnter(e) {
              if (validationMessage) {
                var currentTarget = e.currentTarget;
                show(currentTarget, {
                  title: utilRenderValidationMessage(validationMessage, true),
                  theme: 'light',
                  placement: 'top'
                });
                _this17.isTooltipShown = true;
              }
            },
            onMouseLeave: function onMouseLeave() {
              hide();
            }
          }, /*#__PURE__*/React.createElement("span", {
            className: "".concat(prefixCls, "-filter-label")
          }, label, isLabelShowHelp ? _this17.renderTooltipHelp(help || queryField && queryField.get('help', queryDataSet.current)) : null), /*#__PURE__*/React.createElement("span", {
            className: classNames("".concat(prefixCls, "-filter-item"), _defineProperty({}, "".concat(prefixCls, "-filter-item-has-value"), hasValue)),
            ref: function ref(node) {
              return _this17.refFilterItems.set(name, node);
            }
          }, _this17.createFields(element, name)));
        }), this.queryFields.slice(fieldsLimit).map(function (element) {
          var _classNames3;

          var _element$props2 = element.props,
              name = _element$props2.name,
              hidden = _element$props2.hidden,
              showHelp = _element$props2.showHelp,
              disabled = _element$props2.disabled,
              help = _element$props2.help,
              placeholder = _element$props2.placeholder;
          var isLabelShowHelp = (showHelp || getConfig('showHelp')) === ShowHelp.label;
          if (hidden) return null;
          var queryField = queryDataSet.getField(name);
          if (!queryField) return null;
          var isRequired = queryField && queryField.get('required');
          var validationMessage = queryField && queryField.getValidationMessage(queryDataSet.current);
          var hasValue = !_this17.isEmpty(queryDataSet.current && queryDataSet.current.get(name));

          var label = _this17.getLabel({
            field: queryField,
            value: hasValue,
            placeholder: placeholder,
            record: queryDataSet.current
          });

          var isDisabled = disabled || queryField && queryField.get('disabled', queryDataSet.current);
          var itemContentClassName = classNames("".concat(prefixCls, "-filter-content"), (_classNames3 = {}, _defineProperty(_classNames3, "".concat(prefixCls, "-filter-content-disabled"), isDisabled), _defineProperty(_classNames3, "".concat(prefixCls, "-filter-content-required"), isRequired), _defineProperty(_classNames3, "".concat(prefixCls, "-filter-content-has-value"), hasValue), _defineProperty(_classNames3, "".concat(prefixCls, "-filter-content-invalid"), validationMessage), _classNames3));

          if (selectFields.includes(name)) {
            return /*#__PURE__*/React.createElement("div", {
              className: itemContentClassName,
              key: name,
              onMouseDown: function onMouseDown() {
                if (!isDisabled) {
                  var editor = _this17.refEditors.get(name);

                  if (editor) {
                    defer(function () {
                      _this17.refEditors.get(name).focus();
                    }, 50);
                  }
                }
              },
              onClick: function onClick() {
                if (!isDisabled) {
                  var filterItem = _this17.refFilterItems.get(name);

                  if (filterItem) {
                    if (!filterItem.className.includes("c7n-pro-lov-click")) {
                      filterItem.className += ' c7n-pro-lov-click';
                    }
                  }
                }
              },
              onBlur: function onBlur() {
                var filterItem = _this17.refFilterItems.get(name);

                if (filterItem && filterItem.className.includes("c7n-pro-lov-click")) {
                  filterItem.className = filterItem.className.split(" c7n-pro-lov-click")[0];
                }
              },
              onMouseEnter: function onMouseEnter(e) {
                if (validationMessage) {
                  var currentTarget = e.currentTarget;
                  show(currentTarget, {
                    title: utilRenderValidationMessage(validationMessage, true),
                    theme: 'light',
                    placement: 'top'
                  });
                  _this17.isTooltipShown = true;
                }
              },
              onMouseLeave: function onMouseLeave() {
                hide();
              }
            }, /*#__PURE__*/React.createElement(Icon, {
              type: "cancel",
              className: "".concat(prefixCls, "-filter-item-close"),
              onMouseDown: function onMouseDown(e) {
                e.stopPropagation();
              },
              onClick: function onClick() {
                _this17.handleUnSelect([name]);
              }
            }), /*#__PURE__*/React.createElement("span", {
              className: "".concat(prefixCls, "-filter-label")
            }, label, isLabelShowHelp ? _this17.renderTooltipHelp(help || queryField && queryField.get('help', queryDataSet.current)) : null), /*#__PURE__*/React.createElement("span", {
              className: classNames("".concat(prefixCls, "-filter-item"), _defineProperty({}, "".concat(prefixCls, "-filter-item-has-value"), hasValue)),
              ref: function ref(node) {
                return _this17.refFilterItems.set(name, node);
              }
            }, _this17.createFields(element, name)));
          }

          return null;
        }), fieldsLimit < this.queryFields.length && /*#__PURE__*/React.createElement("div", {
          className: "".concat(prefixCls, "-filter-item")
        }, /*#__PURE__*/React.createElement(Dropdown, {
          visible: !this.fieldSelectHidden,
          overlay: /*#__PURE__*/React.createElement("div", {
            role: "none",
            ref: function ref(node) {
              return _this17.refDropdown = node;
            },
            onClick: function onClick(e) {
              e.stopPropagation();
            }
          }, /*#__PURE__*/React.createElement(FieldList, {
            groups: [{
              title: $l('Table', 'predefined_fields'),
              fields: isTenant ? _toConsumableArray(queryDataSet.fields.values()) : this.originOrderQueryFields.slice(fieldsLimit)
            }],
            prefixCls: "".concat(prefixCls, "-filter-list") || 'c7n-pro-table-filter-list',
            closeMenu: function closeMenu() {
              return runInAction(function () {
                return _this17.fieldSelectHidden = true;
              });
            },
            value: selectFields,
            onSelect: this.handleSelect,
            onUnSelect: this.handleUnSelect
          })),
          trigger: [Action.click]
        }, /*#__PURE__*/React.createElement("span", {
          className: "".concat(prefixCls, "-add-fields"),
          onMouseDown: function onMouseDown(e) {
            e.nativeEvent.stopImmediatePropagation();
            runInAction(function () {
              _this17.fieldSelectHidden = false;
            });
          }
        }, /*#__PURE__*/React.createElement(Icon, {
          type: "add"
        }), $l('Table', 'add_filter'))))), singleLineModeAction));
      }

      return null;
    }
    /**
     *
     * @param collapse
     * @param value 模糊查询参数
     */

  }, {
    key: "handleQuery",
    value: function () {
      var _handleQuery = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee9(collapse, value, oldValue) {
        var _this$props18, _this$props18$onQuery, onQuery, autoQuery;

        return _regeneratorRuntime.wrap(function _callee9$(_context9) {
          while (1) {
            switch (_context9.prev = _context9.next) {
              case 0:
                _this$props18 = this.props, _this$props18$onQuery = _this$props18.onQuery, onQuery = _this$props18$onQuery === void 0 ? noop : _this$props18$onQuery, autoQuery = _this$props18.autoQuery;

                if (!autoQuery) {
                  _context9.next = 4;
                  break;
                }

                _context9.next = 4;
                return this.modifiedCheckQuery(value, oldValue);

              case 4:
                if (!collapse) {
                  onQuery();
                }

              case 5:
              case "end":
                return _context9.stop();
            }
          }
        }, _callee9, this);
      }));

      function handleQuery(_x8, _x9, _x10) {
        return _handleQuery.apply(this, arguments);
      }

      return handleQuery;
    }()
  }, {
    key: "render",
    value: function render() {
      var _this$props19 = this.props,
          summaryBar = _this$props19.summaryBar,
          buttons = _this$props19.buttons;
      var prefixCls = this.prefixCls;
      var queryBar = this.getQueryBar();

      if (queryBar) {
        return [queryBar, summaryBar];
      }

      return /*#__PURE__*/React.createElement(TableButtons, {
        key: "toolbar",
        prefixCls: "".concat(prefixCls, "-dynamic-filter-buttons"),
        buttons: buttons
      }, summaryBar);
    }
  }], [{
    key: "contextType",
    get: function get() {
      return ConfigContext;
    }
  }]);

  return TableDynamicFilterBar;
}(Component);

TableDynamicFilterBar.defaultProps = {
  queryFieldsLimit: 3,
  autoQueryAfterReset: true,
  fuzzyQuery: true,
  fuzzyQueryOnly: false,
  autoQuery: true,
  refreshBtn: true,
  buttons: []
};

__decorate([observable], TableDynamicFilterBar.prototype, "moreFields", void 0);

__decorate([observable], TableDynamicFilterBar.prototype, "fieldSelectHidden", void 0);

__decorate([observable], TableDynamicFilterBar.prototype, "expand", void 0);

__decorate([observable], TableDynamicFilterBar.prototype, "visible", void 0);

__decorate([observable], TableDynamicFilterBar.prototype, "shouldLocateData", void 0);

__decorate([observable], TableDynamicFilterBar.prototype, "showExpandIcon", void 0);

__decorate([action], TableDynamicFilterBar.prototype, "handleMouseDownOut", void 0);

__decorate([autobind], TableDynamicFilterBar.prototype, "handleDataSetQuery", null);

__decorate([autobind], TableDynamicFilterBar.prototype, "handleDataSetValidate", null);

__decorate([autobind, action], TableDynamicFilterBar.prototype, "setConditionStatus", null);

__decorate([action], TableDynamicFilterBar.prototype, "setOriginalConditionFields", void 0);

__decorate([autobind], TableDynamicFilterBar.prototype, "handleDataSetUpdate", null);

__decorate([autobind], TableDynamicFilterBar.prototype, "handleDataSetCreate", null);

__decorate([autobind], TableDynamicFilterBar.prototype, "handleDataSetReset", null);

__decorate([autobind], TableDynamicFilterBar.prototype, "handleDataSetLoad", null);

__decorate([autobind, action], TableDynamicFilterBar.prototype, "initConditionFields", null);

__decorate([autobind], TableDynamicFilterBar.prototype, "initMenuDataSet", null);

__decorate([action], TableDynamicFilterBar.prototype, "handleSelect", void 0);

__decorate([action], TableDynamicFilterBar.prototype, "handleUnSelect", void 0);

__decorate([autobind], TableDynamicFilterBar.prototype, "handleFilterOk", null);

__decorate([autobind], TableDynamicFilterBar.prototype, "handleFilterCancel", null);

__decorate([autobind], TableDynamicFilterBar.prototype, "setFilterVisiable", null);

__decorate([autobind], TableDynamicFilterBar.prototype, "isPopupChildren", null);

__decorate([autobind], TableDynamicFilterBar.prototype, "handleEmitFilterCancel", null);

__decorate([autobind], TableDynamicFilterBar.prototype, "saveCustomizeExpTypeEditorRef", null);

__decorate([autobind], TableDynamicFilterBar.prototype, "saveAdvancedFilterContainerRef", null);

__decorate([autobind], TableDynamicFilterBar.prototype, "handleHelpMouseEnter", null);

__decorate([autobind], TableDynamicFilterBar.prototype, "handleHelpMouseLeave", null);

__decorate([autobind], TableDynamicFilterBar.prototype, "handleQuery", null);

TableDynamicFilterBar = __decorate([observer], TableDynamicFilterBar);
export default TableDynamicFilterBar;
//# sourceMappingURL=TableDynamicFilterBar.js.map
